/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.auxml;

import java.io.*;
import org.xml.sax.*;

/** 
 * Creates Entity resolver for XML files related to AU - info.xml, modules catalog
 * TODO consider usage of a XMLCatalog
 */
public class AuEntityResolver implements EntityResolver {

    private String infoDtd;
    private String catalogDtd;
    private String moduleStatusDtd;
    private static AuEntityResolver instance;

    /**
     * Creates Entity Resolver object
     */
    private AuEntityResolver(String infoDtd, String catalogDtd, String moduleStatusDtd) {
        this.infoDtd = infoDtd;
        this.catalogDtd = catalogDtd;
        this.moduleStatusDtd = moduleStatusDtd;
    }

    public static void initialize(String infoDtd, String catalogDtd, String moduleStatusDtd) {
        instance = new AuEntityResolver(infoDtd, catalogDtd, moduleStatusDtd);
    }

    public static AuEntityResolver getInstance() {
        if (instance == null) {
            throw new IllegalStateException("Not initialized yet");
        }
        return instance;
    }

    /**
     * Resolves entity tag
     */
    public InputSource resolveEntity(String publicId, String systemId) throws SAXException, IOException {
        String dtdFile;
        if (systemId.startsWith("http://www.netbeans.org/dtds/autoupdate-info")) {
            dtdFile = infoDtd;
        } else if (systemId.startsWith("http://www.netbeans.org/dtds/autoupdate-catalog")) {
            dtdFile = catalogDtd;
        } else if (systemId.startsWith("http://www.netbeans.org/dtds/module-status")) {
            dtdFile = moduleStatusDtd;
        } else {
            return null;
        }
        return new InputSource(new FileReader(dtdFile));
    }
}
