/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.auxml;

import java.util.Enumeration;
import java.util.Properties;
import org.w3c.dom.Attr;
import org.w3c.dom.CDATASection;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

/** Creates 
 *
 * @author  rnovak
 * @version 1.0
 */
public class Licenses extends Object {

    static private Properties setOfLicenses = new Properties();
    public static final String LICENSE = "license";

    public Licenses() {
    }

    public static void setLicense(String licenseName, String licenseText) {
        setOfLicenses.setProperty(licenseName, licenseText);
    }

    public static void setLicense(String licenseName, String licenseText, String module, String version) {

        String oneParam = ModulesSetup.getParam(module, version, LICENSE);
        if (oneParam != null) {
            licenseName = oneParam;
        }

        setOfLicenses.setProperty(licenseName, licenseText);
    }

    public static String getLicense(String licenseName) {
        return (setOfLicenses.getProperty(licenseName));
    }

    public static void loadLicense(Element element) {
        String name = null;

        NamedNodeMap attrs = element.getAttributes();
        for (int i = 0; i < attrs.getLength(); i++) {
            Attr attr = (Attr) attrs.item(i);
            if (attr.getName().equals("name")) {
                name = attr.getValue();
            }
        }

        NodeList nodes = element.getChildNodes();

        for (int i = 0; i < nodes.getLength(); i++) {
            Node node = nodes.item(i);
            switch (node.getNodeType()) {
                case Node.TEXT_NODE:
                    String desc = ((Text) node).getData();
                    setOfLicenses.setProperty(name, desc);
                    break;
            }
        }
    }

    public static void toElement(Document resDoc, Element parElement) {
        Element license_element;
        String licName, licText;
        CDATASection licCdataSection;

        for (Enumeration e = setOfLicenses.propertyNames(); e.hasMoreElements();) {
            license_element = resDoc.createElement("license");

            licName = (String) e.nextElement();
            license_element.setAttribute("name", licName);
            parElement.appendChild(license_element);

            licText = setOfLicenses.getProperty(licName);
            licCdataSection = resDoc.createCDATASection(licText);
            license_element.appendChild(licCdataSection);
        }
    }
}
    