/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.auxml;

import java.io.*;
import java.util.*;

import org.w3c.dom.*;

/** 
 * Represents all information about one Module Group
 */
public class ModuleGroup extends Object {

    String name = null;
    String title = null;
    String description = null;
    String spec_version = new String();
    String impl_version = null;
    String distribution_dir = null;
    String license = null;
    boolean is_new = false;
    private boolean staging_group = false;
    List module_groups = new ArrayList();
    List modules = new ArrayList();
    private Set all_modules = new HashSet();
    private Set module_deps = new HashSet();
    private Set version_deps = new HashSet();
    private ModuleGroup parent_group = null;
    public static final String STAGING_DIR = "_staging_";
    public static final String TITLE = "title";
    public static final String IMPL_VERSION = "impl-version";
    public static final String DESCRIPTION = "description";
    public static final String DISTRIBUTION_DIR = "distribution_dir";
    public static final String LICENSE = "license";

    /**
     * Creates new Module Group with name "name"
     */
    public ModuleGroup(String name, ModuleGroup parentGroup) {
        this.parent_group = parentGroup;
        this.name = name;
        title = name;
        description = name;

    }

    /**
     * Creates new Module Group from directory with NBMs
     */
    public ModuleGroup(File nbmsDir, ModuleGroup parentGroup) {
        this.parent_group = parentGroup;
        String dir_name = nbmsDir.getName();


        int j = dir_name.indexOf('.');

        spec_version = "1.0";
        name = dir_name;
        title = name.replace('_', ' ');
        description = title;

        if (dir_name.equals(STAGING_DIR)) {
            System.out.println("STAGING group");
            staging_group = true;
        } else {
            if (parentGroup != null && parentGroup.isStaging()) {
                staging_group = true;
            }
        }


        applyModulesSetup();
        is_new = isNew();

        System.out.print(AppSetup.odsadenie);
        System.out.println("|= Module group: " + name + " " + spec_version + " " + is_new);
        AppSetup.odsadenie.append(" ");

        AppSetup.new_modules.add(name);
        AppSetup.new_versions.add(name + " " + spec_version);

        File one_file;
        ModuleGroup new_group;
        Module new_module;

        for (Iterator i = new NBMiterator(nbmsDir); i.hasNext();) {
            one_file = (File) i.next();
            if (one_file.isDirectory()) {
                new_group = new ModuleGroup(one_file, this);
                module_groups.add(new_group);
                module_deps.addAll(new_group.getModuleDeps());
                version_deps.addAll(new_group.getVersionDeps());
                all_modules.addAll(new_group.getAllModules());
            } else {
                NBMfile nbm_file = new NBMfile(one_file);
                new_module = new Module(nbm_file, this);
                modules.add(new_module);
                module_deps.addAll(new_module.getModuleDeps());
                version_deps.addAll(new_module.getVersionDeps());
                all_modules.add(new_module);
            }
        }

        AppSetup.odsadenie.delete(0, 1);
    }
    
    public String getName() {
        return name;
    }

    /**
     * Returns list of Module Groups that are under 
     */
    public List getModuleGroups() {
        return (module_groups);
    }

    /**
     * Returns list of Modules that are under 
     */
    public List getModules() {
        return (modules);
    }

    /**
     * Returns list of all Modules that are under, including all subfolders
     */
    public Set getAllModules() {
        return (all_modules);
    }

    /**
     * Returns set of module dependencies 
     */
    public Set getModuleDeps() {
        return (module_deps);
    }

    /**
     * Returns set of module dependencies with concrete version
     */
    public Set getVersionDeps() {
        return (version_deps);
    }

    /**
     * Returns parent module group
     */
    public ModuleGroup getParentGroup() {
        return (parent_group);
    }

    /**
     * Returns true if the group is staging group
     */
    public boolean isStaging() {
        return (staging_group);
    }

    /**
     * Stores Module Group as an element of Module Catalog - "resDoc", under
     * parent element "par Element"
     */
    public void toCatalogElement(Element parElement, boolean recursive) {
        if (!ModulesCatalog.staging && staging_group) {
            return;
        }

        Element group_element = createGroupElement(parElement, true);

        ModuleGroup one_module_group;
        for (Iterator i = module_groups.listIterator(); i.hasNext();) {
            one_module_group = (ModuleGroup) i.next();
            if (recursive) {
                one_module_group.toCatalogElement(group_element, true);
            } else {
                one_module_group.createGroupElement(group_element, false);
            }
        }

        Module one_module;
        for (Iterator i = modules.listIterator(); i.hasNext();) {
            one_module = (Module) i.next();
            one_module.toCatalogElement(group_element);
        }
    }

    /**
     * 
     */
    public Element createGroupElement(Element parElement, boolean recursive) {
        Document result_doc = parElement.getOwnerDocument();
        Element group_element;

        System.out.println(" |- saving group: " + name);

        if (recursive) {
            group_element = result_doc.createElement("module_group");
        } else {
            group_element = result_doc.createElement("module2");
        }

        if (name != null) {
            group_element.setAttribute("name", name);
        }

        parElement.appendChild(group_element);

        addExtraInfo(group_element);
        return (group_element);
    }

    /**
     * Stores additional group information to <group_extra_info> element 
     * input:
     * parElement - parent element
     */
    private void addExtraInfo(Element parElement) {
        Document resDoc = parElement.getOwnerDocument();

        Element extra_info_element = resDoc.createElement("group_extra_info");
        parElement.appendChild(extra_info_element);

        if (title != null) {
            extra_info_element.setAttribute("title", title);
        }

        if (spec_version != null) {
            extra_info_element.setAttribute("spec-version", spec_version);
        }

        if (impl_version != null) {
            extra_info_element.setAttribute("impl-version", impl_version);
        }

        if (description != null) {
            extra_info_element.setAttribute("description", description);
        }

        Element request_element = resDoc.createElement("request");
        extra_info_element.appendChild(request_element);
        if (is_new) {
            request_element.setAttribute("type", "create");
        } else {
            request_element.setAttribute("type", "update");
        }

        Element desc_element = resDoc.createElement("group_description");
        extra_info_element.appendChild(desc_element);

        Text desc_text = resDoc.createTextNode(description);
        desc_element.appendChild(desc_text);
    }

    /**
     * Prints list of Modules and Module Groups that belong to the Module Group
     */
    public void printModuleGroup() {

        System.out.println("Module group " + name + ":");
        Module one_module;
        for (Iterator i = modules.listIterator(); i.hasNext();) {
            one_module = (Module) i.next();
            System.out.println("module -> " + one_module.getName());
        }

        ModuleGroup one_module_group;
        for (Iterator i = modules.listIterator(); i.hasNext();) {
            one_module_group = (ModuleGroup) i.next();
            System.out.println("module -> " + one_module_group.name);
        }

    }

    /**
     * Sets params according to External Parameters
     */
    private void applyModulesSetup() {
        String t_title, t_impl_version, t_description;

        if (spec_version == null) {
            spec_version = "1.0";
        }
        impl_version = spec_version;

        t_title = ModulesSetup.getParam(name, spec_version, TITLE);
        t_impl_version = ModulesSetup.getParam(name, spec_version, IMPL_VERSION);
        t_description = ModulesSetup.getParam(name, spec_version, DESCRIPTION);

        if (t_title != null) {
            title = t_title;
        }
        if (t_impl_version != null) {
            impl_version = t_impl_version;
        }
        if (t_description != null) {
            description = t_description;
        }

        distribution_dir = ModulesSetup.getParam(name, spec_version, DISTRIBUTION_DIR);
        if (distribution_dir == null && parent_group != null) {
            distribution_dir = parent_group.distribution_dir;
        }

        license = ModulesSetup.getParam(name, spec_version, LICENSE);
        if (license == null && parent_group != null) {
            license = parent_group.license;
        }
    }

    /**
     * Test if the module group should be created or updated
     */
    private boolean isNew() {
        return (!AppSetup.loaded_modules.contains(name));
    }

    /**
     * Sets params according to External Parameters
     */
    public void printDeps() {
        System.out.println();
        System.out.println("Module deps for group " + name);
        for (Iterator i = module_deps.iterator(); i.hasNext();) {
            System.out.println("- dep: " + i.next());
        }

        System.out.println("Version deps for group " + name);
        for (Iterator i = version_deps.iterator(); i.hasNext();) {
            System.out.println("- dep: " + i.next());
        }
    }
}


