/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.auxml;

import java.io.*;
import java.util.Map;
import java.util.HashMap;
import java.util.Properties;
import java.util.logging.Logger;

/** 
 * Represents module parameters 
 */
public class ModulesSetup extends Object {

    private static final Logger LOG = Logger.getLogger(ModulesSetup.class.getName());
    private static Map params = new HashMap();

    /**
     * Creates Dependency object from dependency string and type
     */
    public ModulesSetup(String initFile) throws IOException {
        String one_line;
        int i = 1;
        LOG.info("Loading modules setup from " + initFile);
        BufferedReader init_stream = new BufferedReader(new FileReader(initFile));
        while ((one_line = init_stream.readLine()) != null) {
            extractParams(one_line.trim(), i++);
        }
    }

    /**
     * Extracts and stores parameters 
     */
    private static void extractParams(String allParams, int line) {
        int i, j;
        String module_name, version, param, value;

        if (allParams.length() == 0) {
            return;
        }
        if (allParams.startsWith("#")) {
            return;
        }

        i = allParams.indexOf(' ');
        if (i == -1) {
            throw new IllegalStateException("Failure #1. Incorrect format of Params file on line " + line + ".");
        }
        module_name = allParams.substring(0, i);

        j = allParams.indexOf(' ', i + 1);
        if (j == -1) {
            throw new IllegalStateException("Failure #2. Incorrect format of Params file on line " + line + ".");
        }
        version = allParams.substring(i + 1, j);

        i = j;
        if (i == -1) {
            throw new IllegalStateException("Failure #3. Incorrect format of Params file on line " + line + ".");
        }
        j = allParams.indexOf(' ', i + 1);
        param = allParams.substring(i + 1, j);

        i = j;
        j = allParams.length();
        value = allParams.substring(i + 1, j);


        storeParam(module_name, version, param, value);

    }

    /**
     * Stores one parameter for module
     */
    public static void storeParam(String moduleName, String version, String param, String value) {
        Properties module_props;

        module_props = (Properties) params.get(module(moduleName, version));
        if (module_props == null) {
            module_props = new Properties();
            params.put(module(moduleName, version), module_props);
        }

        module_props.setProperty(param, value);
    }

    /**
     * Creates internal module name. It is "(module_name} {version}"
     */
    private static String module(String moduleName, String version) {
        return (moduleName + " " + version);
    }

    /**
     * Returns all params for a module
     */
    public static Properties getAllParams(String moduleName, String version) {
        return ((Properties) params.get(module(moduleName, version)));
    }

    /**
     * Returns value of requested param for a module
     */
    public static String getParam(String moduleName, String version, String param) {

        Properties module_props = (Properties) params.get(module(moduleName, version));
        if (module_props == null) {
            module_props = (Properties) params.get(module(moduleName, "*"));
        }
        if (module_props == null) {
            return (null);
        }

        String param_value = (String) module_props.get(param);

        if (param_value == null) {
            module_props = (Properties) params.get(module(moduleName, "*"));
            if (module_props == null) {
                return (null);
            }
        }
        param_value = (String) module_props.get(param);

        return (param_value);
    }
}
