/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.auxml;

import java.io.*;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.w3c.dom.*;

/**
 * represents generator of Quantum XML Catalog 
 */
public class UpdateCenter {
    //XXX this fields should not be static
    private static String notifPath;

    public static String getNotifPath() {
        return notifPath;
    }
    private static String distributionBaseURL;

    public static String getDistributionBaseURL() {
        return distributionBaseURL;
    }
    private File ucRoot;
    private String ucVersion;
    private String ucType;
    private ModulesCatalog catalog;
    private File modulesSetupFile;
    private File catalogDir;
    private File catalogXSLTemplate;
    private List<String> languages;

    /**
     * Creates new Quantum Catalog generator
     */
    public UpdateCenter(String ucRootPath, String ucVersion, String ucType) throws IOException {
        ucRoot = new File(ucRootPath);
        if (!ucRoot.isDirectory()) {
            throw new IllegalArgumentException(ucRootPath + " is not a valid directory!");
        }
        this.ucVersion = ucVersion;
        this.ucType = ucType;
        // TODO remaining static fields, should be eliminated later
        notifPath = new File(ucRoot, "notifs").getAbsolutePath();
        // initialize AuEntityResolver TODO: should be AuManager responsibility
        String infoDTD = new File(ucRoot, "dtd/autoupdate-info.dtd").getAbsolutePath();
        String catalogDTD = new File(ucRoot, "dtd/autoupdate-catalog.dtd").getAbsolutePath();
        String moduleStatusDTD = new File(ucRoot, "dtd/module-status.dtd").getAbsolutePath();
        AuEntityResolver.initialize(infoDTD, catalogDTD, moduleStatusDTD);
        // fields computed from constructor parameters (def setup)
        modulesSetupFile = new File(ucRoot, "catalogs.setup/" + ucVersion + "/modules.setup");
        catalogDir = new File(ucRoot, "nbms/" + ucVersion + "/" + ucType);
        catalog = new ModulesCatalog(getCatalogDir());
        // fields obatained from UC Setup file
        File setupFile = new File(ucRoot, "catalogs.setup/" + ucVersion + "/setup." + ucVersion + "." + ucType);
        Properties properties = new Properties();
        FileInputStream fis = new FileInputStream(setupFile);
        try {
            properties.load(fis);
        } finally {
            fis.close();
        }
        String template = properties.getProperty("catalog_template");
        if (template == null) {
            throw new IllegalStateException("Mising 'catalog_template' property in: " + setupFile.getAbsolutePath());
        }
        catalogXSLTemplate = new File(ucRoot, template);
        distributionBaseURL = properties.getProperty("distribution_base_url");
        String sl = properties.getProperty("supported_languages", "").trim();
        if (sl.length() > 0) {
            languages = Collections.unmodifiableList(Arrays.asList(sl.split("[, ]+")));
        } else {
            languages = Collections.emptyList();
        }
        new ModulesSetup(getModulesSetupFile().getAbsolutePath());
        for (File file : new File[]{ucRoot, modulesSetupFile, catalogDir, catalogXSLTemplate}) {
            if (!file.canRead()) {
                throw new IllegalArgumentException("Cannot read: " + file);
            }
        }

    }

    /**
     * MAIN for Quantum script generating
     */
    public static void main(String args[]) {

        System.out.println("\n------- Starting catalog generator -------");
        UpdateCenter uc = null;
        try {
            uc = new UpdateCenter(args[0], args[1], args[2]);
        } catch (IOException ex) {
            Logger.getLogger(UpdateCenter.class.getName()).log(Level.SEVERE, "Cannot create Update Center instance.", ex);
        }
        uc.generateCatalog();
    }

    public File getRoot() {
        return ucRoot;
    }

    public File getCatalogFile() {
        return getCatalogFile(false);
    }

    public File getCatalogFile(boolean stage) {
        return getCatalogFile(stage, null);
    }

    public File getCatalogFile(boolean stage, String locale) {
        StringBuilder sb = new StringBuilder("catalog");
        if (stage) {
            sb.append("_stage");
        }
        if (locale != null) {
            sb.append("_").append(locale);
        }
        sb.append(".xml");
        String name = sb.toString();
        return new File(getCatalogDir(), name);
    }

    public void generateCatalog() {
        catalog = new ModulesCatalog(getCatalogDir());
        catalog.setStagingFlag(false);
        Document catalog_doc = catalog.toCatalogDocument();
        storeTofile(catalog_doc, getCatalogFile().getAbsolutePath());

        catalog.setStagingFlag(true);
        catalog_doc = catalog.toCatalogDocument();
        storeTofile(catalog_doc, getCatalogFile(true).getAbsolutePath());

        // modules_updates.
        for (String lang : languages) {
            System.out.println("\nLanguage: " + lang);
            catalog.setLanguage(lang);
            catalog.setStagingFlag(false);
            catalog_doc = catalog.toCatalogDocument();
            storeTofile(catalog_doc, getCatalogFile(false, lang).getAbsolutePath());
            catalog.setStagingFlag(true);
            catalog_doc = catalog.toCatalogDocument();
            storeTofile(catalog_doc, getCatalogFile(true, lang).getAbsolutePath());
        }
        System.out.println("------- Ending catalog generator -------");
    }

    /**
     * Stores generated Master Catalog to file
     */
    private void storeTofile(Document catalogDoc, String catalogFile) {
        System.out.println("\nCatalog file: " + catalogFile);
        XMLutils.XMLtransform(catalogDoc, catalogFile, getCatalogXSLTemplate().getAbsolutePath());
    }

    /**
     * Prints list of module - for debug
     */
    public void printModules() {
        Module one_module;

        System.out.println();
        System.out.println("List of loaded modules: ");
        for (Iterator i = catalog.getModulesIterator(); i.hasNext();) {
            one_module = (Module) i.next();
            System.out.println(one_module.getName());
        }
    }

    public String getVersion() {
        return ucVersion;
    }

    public String getType() {
        return ucType;
    }

    public File getCatalogDir() {
        return catalogDir;
    }

    public File getModulesSetupFile() {
        return modulesSetupFile;
    }

    public File getCatalogXSLTemplate() {
        return catalogXSLTemplate;
    }

    public List<String> getLanguages() {
        return languages;
    }

    public List<Module> getModules() {
        return catalog.getModules();
    }
}
