/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.auxml;

import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.regex.Pattern;

/**
 *
 * @author Martin.Schovanek@sun.com
 */
public class Version implements Serializable, Comparable {

    private String name;
    private File ucRoot;
    private File setupDir;
    private File modulessDir;

    public Version(String name, File ucRoot) {
        this.name = name;
        this.ucRoot = ucRoot;
        setupDir = new File(ucRoot, AuManager.SETUP_PATH + "/" + name);
        modulessDir = new File(ucRoot, AuManager.NBMS_PATH + "/" + name);
        for (File file : new File[]{setupDir, modulessDir}) {
            if (!file.canRead()) {
                throw new IllegalArgumentException("Cannot read: " + file);
            }
        }
    }

    public String getName() {
        return name;
    }

    public List<UpdateCenter> getUpdateCenters() throws IOException {
        List<UpdateCenter> ucs;
        File[] files = getSetupDir().listFiles(new FilenameFilter() {

            Pattern setupName = Pattern.compile("^setup." + name + ".[^.]+");

            public boolean accept(File dir, String name) {
                return setupName.matcher(name).matches();
            }
        });
        if (files == null || files.length == 0) {
            ucs = Collections.EMPTY_LIST;
        } else {
            ucs = new ArrayList<UpdateCenter>();
            Pattern prefix = Pattern.compile("^setup." + name + ".");
            for (File file : files) {
                String type = prefix.matcher(file.getName()).replaceFirst("");
                ucs.add(getUpdateCenter(type));
            }
        }
        return ucs;
    }

    public UpdateCenter getUpdateCenter(String type) throws IOException {
        return new UpdateCenter(ucRoot.getAbsolutePath(), name, type);
    }
    
    public File getSetupDir() {
        return setupDir;
    }

    public File getModulesDir() {
        return modulessDir;
    }

    public int compareTo(Object version) {
        Version ver = (Version) version;
        return getName().compareTo(ver.getName());
    }
}