/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.auxml;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author Martin.Schovanek@sun.com
 */
public class AuManagerTest {

    private String testUCPath;

    public AuManagerTest() {
        testUCPath = this.getClass().getResource("testuc").getPath();
    }

    /**
     * Test of getVersions method, of class AuManager.
     */
    @Test
    public void testGetVersions() {
        System.out.println("===== getVersions =====");
        AuManager instance = AuManager.getInstance(testUCPath);
        List<String> expResult = Arrays.asList("60", "61");
        List<String> result = new ArrayList(2);
        for (Version version : instance.getVersions()) {
            result.add(version.getName());
        }
        assertEquals(expResult, result);
    }

    /**
     * Test of createVersion method, of class AuManager.
     */
    @Test
    public void testCreateVersion() {
        System.out.println("===== createVersion =====");
        String name = "77";
        AuManager instance = AuManager.getInstance(testUCPath);
        instance.createVersion(name);
        String setupPath = AuManager.SETUP_PATH + "/" + name;
        String modulesSetupPath = setupPath + "/modules.setup";
        String nbmsPath = AuManager.NBMS_PATH + "/" + name;
        File ucRoot = new File(testUCPath);
        for (String path : new String[]{setupPath, modulesSetupPath, nbmsPath}) {
            File file = new File(ucRoot, path);
            assertTrue("Missing file or folder: " + file.getAbsolutePath(), file.canRead());
        }
    }

    /**
     * Test of createUpdateCenter method, of class AuManager.
     */
    @Test
    public void testCreateUpdateCenter() throws IOException {
        System.out.println("===== createUpdateCenter =====");
        // create UC
        String version = "61";
        String type = "test2";
        AuManager instance = AuManager.getInstance(testUCPath);
        instance.createUpdatCenter(version, type, "catalog60.xsl");
        // verify newly created file and folders
        String ucSetupPath = AuManager.SETUP_PATH + "/61/setup.61.test2";
        String stagingPath = AuManager.NBMS_PATH + "/61/test2/_staging_";
        String modulesPath = AuManager.NBMS_PATH + "/61/test2/modules";
        File ucRoot = new File(testUCPath);
        for (String path : new String[]{ucSetupPath, stagingPath, modulesPath}) {
            File file = new File(ucRoot, path);
            assertTrue("Missing file or folder: " + file.getAbsolutePath(), file.canRead());
        }
        // verify catalogs' files
        UpdateCenter uc = AuManager.getInstance(testUCPath).getCatalog(version, type);
        Utils.verifyCatalogs(uc);
    }
    
    /**
     * Test of createUpdateCenter method, of class AuManager.
     */
    @Test
    public void testGetTemplates() throws IOException {
        System.out.println("===== getTemplates =====");
        AuManager instance = AuManager.getInstance(testUCPath);
        File[] templates = instance.getTemplates();
        assertEquals(templates.length, 1);
        assertTrue(templates[0].isFile());
        assertTrue(templates[0].getName().equals("catalog60.xsl"));
    }
    
    /**
     * Test of createUpdateCenter method, of class AuManager.
     */
    @Test
    public void testStageModule() throws IOException {
        System.out.println("===== stageModule =====");
        String version = "61";
        String type = "test3";
        String template = "catalog60.xsl";
        File nbm = new File(testUCPath, "testData/org-netbeans-modules-apisupport-kit-3.nbm");
        // stage nbm
        AuManager manager = AuManager.getInstance(testUCPath);
        manager.createUpdatCenter(version, type, template);
        manager.stageModule(version, type, nbm);
        // verify catalogs' files
        UpdateCenter uc = manager.getCatalog(version, type);
        Utils.verifyCatalogs(uc);
    }
}
