/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.coverage.data.local;

import com.vladium.emma.EMMAProperties;
import com.vladium.emma.EMMARuntimeException;
import com.vladium.emma.IAppConstants;
import com.vladium.emma.data.DataFactory;
import com.vladium.emma.data.ICoverageData;
import com.vladium.emma.data.IMergeable;
import com.vladium.emma.data.IMetaData;
import com.vladium.emma.report.IReportGenerator;
import com.vladium.emma.report.SourcePathCache;
import com.vladium.util.IProperties;
import java.io.File;
import java.io.IOException;
import org.netbeans.modules.coverage.data.*;
import org.netbeans.modules.coverage.exceptions.UnableToLoadCoverageDataException;

/**
 * The class is used to obtain the coverage data object based on the EMMA's coverage data file
 * @author mikk@netbeans.org
 */
public class CoverageDataFactory implements ICoverageDataFactory {
    File coverageFile = null;
    File templateFile = null;

    private CoverageDataFactory(){}

    /**
     * Factory constructor
     * @param coverageFile EMMA coverage data file to read
     * @param templateFile EMMA template file to read
     */
    public CoverageDataFactory(File coverageFile, File templateFile){
        this.coverageFile = coverageFile;
        this.templateFile= templateFile;
    }
    
    /**
     * 
     */
    public IProjectCoverageData getCoverageData() throws UnableToLoadCoverageDataException {
        IProperties toolProperties = EMMAProperties.getAppProperties();
        RuntimeException failure = null;
        ProjectCoverageData nbCoverageData = new ProjectCoverageData();
        
        try {
            IMetaData mdata = null;
            ICoverageData cdata = null;
            
            try {
                //gathering and merging meta data
                IMergeable [] fileData = DataFactory.load(templateFile);
                final IMetaData _mdata = (IMetaData) fileData [DataFactory.TYPE_METADATA];
                if (_mdata != null) {
                    mdata = _mdata;
                }
                
                //getting the coverage data
                fileData = DataFactory.load(coverageFile);
                final ICoverageData _cdata = (ICoverageData) fileData [DataFactory.TYPE_COVERAGEDATA];
                if (_cdata != null) {
                    cdata = _cdata;
                }
                
                //creating dummy source path cache
                SourcePathCache srcpathCache = null;
                
                //processing the report
                final IReportGenerator generator = new ReportGenerator(nbCoverageData);
                try {
                    generator.process(mdata, cdata, srcpathCache, toolProperties);
                } catch (Exception e) {
                    // TODO: handle and continue
                    e.printStackTrace(System.out);
                    // TODO: continue here
                    throw new UnableToLoadCoverageDataException();
                } finally {
                    try { generator.cleanup(); } catch (Throwable ignore) {}
                }
                
            } catch (IOException ioe) {
                ioe.printStackTrace(System.out);
            }
        } catch (SecurityException se) {
            failure = new EMMARuntimeException("SECURITY_RESTRICTION", new String [] {IAppConstants.APP_NAME}, se);
        } catch (RuntimeException re) {
            failure = re;
        }
        
        if (failure != null) {
            throw failure;
        }
        
        return nbCoverageData;
        
    }
    
    
    
    
}