/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.coverage.ui.statistics;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import org.jdesktop.layout.GroupLayout;
import org.netbeans.api.project.Project;
import org.netbeans.api.project.ProjectUtils;
import org.netbeans.modules.coverage.data.IClassCoverageData;
import org.netbeans.modules.coverage.data.IProjectCoverageData;
import org.netbeans.modules.coverage.data.IPackageCoverageData;
import org.netbeans.modules.coverage.ui.statistics.model.ClassNameColumn;
import org.netbeans.modules.coverage.ui.statistics.model.ClassCoverageDataProvider;
import org.netbeans.modules.coverage.ui.statistics.model.ClassLinesCoverageColumn;
import org.netbeans.modules.coverage.ui.statistics.model.PackageClassesCoverageColumn;
import org.netbeans.modules.coverage.ui.statistics.model.PackageCoverageDataProvider;
import org.netbeans.modules.coverage.ui.statistics.model.PackageLinesCoverageColumn;
import org.netbeans.modules.coverage.ui.statistics.model.PackageNameColumn;
import org.netbeans.modules.coverage.utils.ui.LayoutUtils;
import org.netbeans.modules.coverage.utils.ui.table.ProviderTableModel;
import org.netbeans.modules.coverage.utils.ui.table.column.ColumnsPolicy;
import org.netbeans.modules.coverage.utils.ui.table.column.impl.ColumnsPolicyBuilderImpl;
import org.netbeans.swing.etable.ETable;
import org.netbeans.swing.etable.ETableColumnModel;
import org.openide.util.NbBundle;
import org.openide.windows.TopComponent;

/**
 *
 * @author Lyadzhin
 */
public class CoverageStatisticsWindow extends TopComponent {
    private static final String COMPONENT_ID = "project_coverage_statistics";
            
    private final Project project;
    private IProjectCoverageData data;
    private PackageCoverageDataProvider packageDataProvider;
    private ClassCoverageDataProvider classDataProvider;
    private boolean hideCoveredClasses = false;
    private boolean hideCoveredPackages = false;
    
    private ProjectInformationPanel informationPanel = new ProjectInformationPanel();
    private ETable packageTable = new ETable();
    private ETable classTable = new ETable();
    
    public static boolean isStatisticsWindow(TopComponent component, Project project) {
        return getComponentName(project).equals(component.getName());
    }
    
    private static String getComponentName(Project project) {
        return COMPONENT_ID + "_" + ProjectUtils.getInformation(project).getName(); 
    }

    public CoverageStatisticsWindow(Project project, IProjectCoverageData coverageData) {
        //packageTable.setAutoCreateRowSorter(true);
        //classTable.setAutoCreateRowSorter(true);
        this.project = project;
        this.setData(coverageData);
        this.setName(getComponentName(project));
        this.setHtmlDisplayName(buildHtmlName());
        this.setLayout(new BorderLayout());
        this.add(informationPanel, BorderLayout.NORTH);
        this.add(LayoutUtils.createVerticalDock(true, createPackageCoveragePane(), createClassCoveragePane()));
    }
    
    public void setData(IProjectCoverageData data) {
        this.data = data;
        informationPanel.refresh();
        packageDataProvider = new PackageCoverageDataProvider(data, hideCoveredPackages);
        final ProviderTableModel packageModel = new ProviderTableModel(getPackageCoverageColumnsPolicy(), packageDataProvider);
        packageTable.setModel(packageModel);
        packageTable.setColumnHidingAllowed(false);
        classDataProvider = new ClassCoverageDataProvider(data, hideCoveredClasses);
        final ProviderTableModel classModel = new ProviderTableModel(getClassCoverageColumnsPolicy(), classDataProvider);
        classTable.setModel(classModel);
        classTable.setColumnHidingAllowed(false);
        configureColumns();
    }
    
    public void setHideCoveredClasses(boolean hide) {
        this.hideCoveredClasses = hide;
        classDataProvider.setHideCovered(hide);
    }
    
    public void setHideCoveredPackages(boolean hide) {
        this.hideCoveredPackages = hide;
        packageDataProvider.setHideCovered(hide);
    }
    
    protected void configureColumns() {
        //packageTable.getRowSorter().toggleSortOrder(0);
        //classTable.getRowSorter().toggleSortOrder(0);
    }

    @Override
    protected String preferredID() {
        return COMPONENT_ID;
    }
    
    protected JPanel createPackageCoveragePane() {
        final JPanel panel = new JPanel(new BorderLayout());
        panel.setBorder(BorderFactory.createTitledBorder(getMessage("PackageCoverage")));
        final JCheckBox hideCheckBox = new JCheckBox(getMessage("HideCoveredPackages"));
        hideCheckBox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setHideCoveredPackages(hideCheckBox.isSelected());
            }
        });
        panel.add(hideCheckBox, BorderLayout.NORTH);
        panel.add(new JScrollPane(packageTable, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER));
        return panel;
    }
    
    protected JPanel createClassCoveragePane() {
        final JPanel panel = new JPanel(new BorderLayout());
        panel.setBorder(BorderFactory.createTitledBorder(getMessage("ClassCoverage")));
        final JCheckBox hideCheckBox = new JCheckBox(getMessage("HideCoveredClasses"));
        hideCheckBox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setHideCoveredClasses(hideCheckBox.isSelected());
            }
        });
        panel.add(hideCheckBox, BorderLayout.NORTH);
        panel.add(new JScrollPane(classTable, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER));
        return panel;
    }
    
    protected String getMessage(String key, Object... args) {
        return NbBundle.getMessage(this.getClass(), key, args);
    }
    
    private String buildHtmlName() {
        return getMessage("ProjectStatisticsTabName",  ProjectUtils.getInformation(project).getDisplayName());
    }
    
    private ColumnsPolicy<IClassCoverageData> getClassCoverageColumnsPolicy() {
        return new ColumnsPolicyBuilderImpl<IClassCoverageData>()
                .addColumn(ClassNameColumn.COL)
                .addColumn(ClassLinesCoverageColumn.COL)
                .buildPolicy();
    }
    
    private ColumnsPolicy<IPackageCoverageData> getPackageCoverageColumnsPolicy() {
        return new ColumnsPolicyBuilderImpl<IPackageCoverageData>()
                .addColumn(PackageNameColumn.COL)
                .addColumn(PackageClassesCoverageColumn.COL)
                .addColumn(PackageLinesCoverageColumn.COL)
                .buildPolicy();
    }

    @Override
    public int getPersistenceType() {
        return PERSISTENCE_NEVER;
    }
    
    private class ProjectInformationPanel extends JPanel {
        private JLabel nameLabel = new JLabel();
        private JLabel isCoveredLabel = new JLabel();
        private JLabel totalClassesLabel = new JLabel();
        private JLabel totalLinesLabel = new JLabel();
        private JLabel totalPackagesLabel = new JLabel();
        
        public ProjectInformationPanel() {
            final GroupLayout layout = LayoutUtils.setGroupLayout(this);
            layout.setHorizontalGroup(layout.createParallelGroup()
                    .add(nameLabel)
                    .add(isCoveredLabel)
                    .add(totalClassesLabel)
                    .add(totalLinesLabel)
                    .add(totalPackagesLabel)
            );
            layout.setVerticalGroup(layout.createSequentialGroup()
                    .add(nameLabel)
                    .add(isCoveredLabel)
                    .add(totalClassesLabel)
                    .add(totalLinesLabel)
                    .add(totalPackagesLabel)
            );
        }
        
        public void refresh() {
            nameLabel.setText(getMessage("ProjectName", ProjectUtils.getInformation(project).getDisplayName()));
            isCoveredLabel.setText(data.isCovered() ? getMessage("ProjectIsCovered") : getMessage("ProjectIsntCovered"));
            totalClassesLabel.setText(getMessage("TotalClassesCovered", data.getOverallClassCoverage().toString()));
            totalLinesLabel.setText(getMessage("TotalLinesCovered", data.getOverallLineCoverage().toString()));
            totalPackagesLabel.setText(getMessage("TotalPackagesCovered", data.getOverallPackageCoverage().toString()));
        }
    }
}
