/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.jasm.parser;

import java.io.*;
import java.util.ArrayList;

public class Attributes {

	Object parent;
	ConstantPool constPool;
	int count;
	Attribute[] array;
	
	public Attributes (Object p, ConstantPool cp) {
		parent = p;
		constPool = cp;
	}

    public AttrLineNumberTable getAttrLineNumberTable () {
        for (int a = 0; a < count; a ++) {
            if (array[a] instanceof AttrLineNumberTable) {
                return (AttrLineNumberTable) array[a];
            }
        }
        return null;
    }

	public void fromClass (DataInputStream dis) throws Exception {
		count = dis.readUnsignedShort ();
		array = new Attribute[count];
		for (int a = 0; a < count; a ++) {
			int iName = dis.readUnsignedShort ();
			String str = constPool.getUTF8Value (iName);
			int len = dis.readInt ();
			Attribute attr;
			if ("ConstantValue".equals (str))
				attr = new AttrConstant (this, constPool, iName, len);
			else if ("Code".equals (str))
				attr = new AttrCode (this, constPool, iName, len);
			else if ("Exceptions".equals (str))
				attr = new AttrExceptions (this, constPool, iName, len);
			else if ("InnerClasses".equals (str))
				attr = new AttrInnerClasses (this, constPool, iName, len);
			else if ("Synthetic".equals (str))
				attr = new AttrSynthetic (this, constPool, iName, len);
			else if ("SourceFile".equals (str))
				attr = new AttrSourceFile (this, constPool, iName, len);
			else if ("LineNumberTable".equals (str))
				attr = new AttrLineNumberTable (this, constPool, iName, len);
			else if ("LocalVariableTable".equals (str))
				attr = new AttrLocalVariableTable (this, constPool, iName, len);
			else if ("Deprecated".equals (str))
				attr = new AttrDeprecated (this, constPool, iName, len);
			else
				attr = new AttrUnknown (this, constPool, iName, len);
			attr.fromClass (dis);
			array[a] = attr;
		}
	}
	
	public void toClass (DataOutputStream dos) throws Exception {
		dos.writeShort (count);
		for (int a = 0; a < count; a ++) {
			dos.writeShort (array[a].iName);
			dos.writeInt (array[a].len);
			array[a].toClass (dos);
		}
	}
	
	public void fromJRAW (Lexan lex) throws Exception {
		lex.cmpLexel ("Attributes");
		lex.cmpLexel ("{");
		ArrayList al = new ArrayList ();
		for (;;) {
			String s = lex.getLexel ();
			if ("}".equals (s))
				break;
			Attribute attr;
			lex.cmpLexel ("Name");
			int iName;
			try {
				iName = (int) lex.getLong ();
			} catch (Exception e) {
				iName = constPool.getLabelPos (lex.getLexel ());
			}
			String str = constPool.getUTF8Value (iName);
			lex.cmpLexel ();
			lex.cmpLexel ("Len");
			int len = (int) lex.getLong ();
			lex.cmpLexel ();
			lex.cmpLexel ("{");
			if ("ConstantValue".equals (str))
				attr = new AttrConstant (this, constPool, iName, len);
			else if ("Code".equals (str))
				attr = new AttrCode (this, constPool, iName, len);
			else if ("Exceptions".equals (str))
				attr = new AttrExceptions (this, constPool, iName, len);
			else if ("InnerClasses".equals (str))
				attr = new AttrInnerClasses (this, constPool, iName, len);
			else if ("Synthetic".equals (str))
				attr = new AttrSynthetic (this, constPool, iName, len);
			else if ("SourceFile".equals (str))
				attr = new AttrSourceFile (this, constPool, iName, len);
			else if ("LineNumberTable".equals (str))
				attr = new AttrLineNumberTable (this, constPool, iName, len);
			else if ("LocalVariableTable".equals (str))
				attr = new AttrLocalVariableTable (this, constPool, iName, len);
			else if ("Deprecated".equals (str))
				attr = new AttrDeprecated (this, constPool, iName, len);
			else
				attr = new AttrUnknown (this, constPool, iName, len);
			attr.fromJRAW (lex);
			lex.cmpLexel ("}");
			al.add (attr);
		}
		count = al.size ();
		array = new Attribute[count];
		for (int a = 0; a < count; a ++)
			array[a] = (Attribute) al.get (a);
		lex.cmpLexel ();
	}
	
	public void toJRAW (ExtStringBuffer esb) {
		esb.println ("Attributes {");
		esb.incIndent ();
		for (int a = 0; a < count; a ++) {
			esb.println ("Name #cp" + array[a].iName + " // " + constPool.getUTF8String (array[a].iName));
			esb.println ("Len " + array[a].len + " {");
			esb.incIndent ();
			array[a].toJRAW (esb);
			esb.decIndent ();
			esb.println ("}");
		}
		esb.decIndent ();
		esb.println ("}");
	}

}
