/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.jasm.parser;

import java.io.*;
import java.util.ArrayList;

public class Methods {

	Parser parent;
	ConstantPool constPool;
	int count;
	Method[] array;
	
	public Methods (Parser p, ConstantPool cp) {
		parent = p;
		constPool = cp;
		count = 0;
	}
	
	public void fromClass (DataInputStream dis) throws Exception {
		count = dis.readUnsignedShort ();
		array = new Method[count];
		for (int a = 0; a < count; a ++) {
			Method m = new Method (this, constPool);
			m.fromClass (dis);
			array[a] = m;
		}
	}
	
	public void toClass (DataOutputStream dos) throws Exception {
		dos.writeShort (count);
		for (int a = 0; a < count; a ++)
			array[a].toClass (dos);
	}
    
	public void fromJRAW (Lexan lex) throws Exception {
		lex.cmpLexel ("Methods");
		lex.cmpLexel ("{");
		ArrayList al = new ArrayList ();
		for (;;) {
			String s = lex.getLexel ();
			if ("}".equals (s))
				break;
			Method m = new Method (this, constPool);
			m.fromJRAW (lex);
			al.add (m);
		}
		count = al.size ();
		array = new Method[count];
		for (int a = 0; a < count; a ++)
			array[a] = (Method) al.get (a);
		lex.cmpLexel ();
	}
	
    public void toJRAW (ExtStringBuffer esb) {
		esb.println ("Methods {");
		esb.incIndent ();
		for (int a = 0; a < count; a ++)
			array[a].toJRAW (esb);
		esb.decIndent ();
		esb.println ("}");
	}

}
