/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package generatestubs;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.EnumSet;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * Generate the set of JavaScript stub files for the builtin libraries and the DOM
 * libraries. It bases its information on documentation available from the 
 * Mozilla DEveloper Connection as well as the W3C standard documents themselves.
 * <h2>Setup</h2>
 * <h3>Mozilla Developer Connection</h3>
 * I first gathered the documentation from
 * <pre>
 *   wget -m -k -K -Ddeveloper.mozilla.org --no-parent -e robots=off \
 *    -I/en/docs --accept="Core_JavaScript*" \
 *    http://developer.mozilla.org/en/docs/Core_JavaScript_1.5_Reference
 * </pre>
 * The DOM docs were generated using almost the same:
 * wget -m -k -K -Ddeveloper.mozilla.org --no-parent -e robots=off -I/en/docs --accept "Gecko_DOM*" --accept="DOM*" http://developer.mozilla.org/en/docs/Gecko_DOM_Reference -o ./wget.log
 *
 * (I also added in http://developer.mozilla.org/en/docs/XMLHttpRequest and renamed it DOM:XMLHttpRequest)
 * 
 * This will generate files like this:
 * <pre>
 * developer.mozilla.org/en/docs/Core_JavaScript_1.5_Reference:Global_Objects:Array:slice
 * </pre>
 * 
 * This script will take a developer.mozilla.org tree like the above and
 * emit a set of javascript sources that includes that rough JavaScript
 * necessary to capture the libraries with documentation, methods, etc.
 * 
 * <h3>W3C</h3>
 * For the DOM documentation I also downloaded
 *  <ul>
 *   <li>  http://www.w3.org/TR/REC-DOM-Level-1/ecma-script-language-binding.html 
 *   <li>  http://www.w3.org/TR/DOM-Level-2-Core/ecma-script-binding.html
 *   <li>  http://www.w3.org/TR/DOM-Level-2-HTML/ecma-script-binding.html
 *   <li>  http://www.w3.org/TR/2000/REC-DOM-Level-2-Events-20001113/ecma-script-binding.html
 *   <li>  http://www.w3.org/TR/2000/REC-DOM-Level-2-Style-20001113/ecma-script-binding.html
 *   <li>  http://www.w3.org/TR/2000/REC-DOM-Level-2-Traversal-Range-20001113/ecma-script-binding.html
 *   <li>  http://www.w3.org/TR/2000/REC-DOM-Level-2-Views-20001113/ecma-script-binding.html
 *   <li>  http://www.w3.org/TR/DOM-Level-3-Core/ecma-script-binding.html
 *  </ul>
 * The script also processes these and uses them to emit skeleton classes for the DOM stuff.
 * (These are augmented by Mozilla documentation when available.)
 * 
 * @todo Handle optional parameters in Syntax items correctly (foo[,bar])
 * @todo Create permutations for methods that have multiple meanings for the params,
 *    such as String.replace (regexp/substr should create two versions of the same method?)
 * @todo Syntax validate the generated JavaScript; just parse with Rhino?
 * @todo What about properties, like ReferenceError.name? Should these
 *    be created as functions? Probably not... but what?
 * @todo Handle
 *   String.concat = function(string2, string3, ..., stringN) {
 * @todo Must also handle Global Properties (such as :Infinity):
 *    Core_JavaScript_1.5_Reference:Global_Properties
 *   as well as the GlobalFunctions
 * @todo Handle DOM:
 *    Level 0, 1, 2 core: See http://www.w3.org/DOM/
 *    Style: http://www.w3.org/TR/DOM-Level-2-Style/
 *    JS: http://www.w3.org/TR/DOM-Level-2-Style/ecma-script-binding.html
 *    Events: http://www.w3.org/TR/DOM-Level-2-Events/
 *    JS: http://www.w3.org/TR/DOM-Level-2-Events/ecma-script-binding.html
 * @todo For DOM, pull in the data in
 *       file:///home/tor/dev/mozdom/developer.mozilla.org/en/docs/DOM_Client_Object_Cross-Reference
 *    to populate the browser compliance section
 * @todo Syntax highlight the code examples in the docs
 * @todo Insert the Copyright sections into the articles
 * @todo For prototype.js code, go to www.prototypejs.org/api/element/ etc.
 *    It looks like it's creative commons
 * @todo Generate something for the Microsoft extensions, such as those listed here:
 *    http://www.quirksmode.org/dom/w3c_core.html
 *    The question is how I can get any documentation for these.
 * @todo What's with all the CAPITALIZED names in Math? LOG10E, SQRT2, LN2, etc?
 * @todo Use "const" for the constants in Number -- NEGATIVE_INFINITY, MIN_VALUE, etc.
 * @todo Window.screen and Window.navigator -- these are properties I don't handle right
 * @todo Look for these sections: 
 *   "The following methods were introduced in JavaScript 1.8. See New in JavaScript 1.8 for more information."
 *   (and ditto for other versions) and annotate the methods somehow? 
 * @todo Add "prototype" completion for all classes!!! Ditto for "arguments" within functions!
 * @todo Pull out the documentation for the keywords as well! (under "statements")
 * @todo Bugs: Array.indexOf,lastIndexOf,reduce,reduceRight is missing. There are "No type located for"
 *    warnings which pertain to these.
 * @todo An example of inheritance is: http://developer.mozilla.org/en/docs/Core_JavaScript_1.5_Reference:Global_Objects:Number
 * @todo For DOM classes, turn off the constructor generation - you can't create new Elements or Documents or Windows - they are handed to you
 * @todo Hook up the javascript version stamps I'm computing for some attributes, as well as return value mechanism stuff and store as attributes
 * @todo Fix computeSignatures such that it computes MULTIPLE signatures (and diffs them if necessary to avoid conflicts - must differ in arity somehow)
 * @todo Make sure I emit the right thing for Type.CONSTANT
 * @todo Use the DOM origin to determine which browsers are supported and add in support flags
 * @todo Insert a @link reference to the HTTP documentation URL for more on each method?
 * @todo GENERATE_SDOC isn't fully implemented; it needs to convert signatures into a section of @param, @return etc. statements.
 *     Perhaps I can generate a @signature item -- as well as a @browsercompat item?
 * 
 * @author Tor Norbye
 */
public class Main {
    /** 
     * Generating the docs can be really time consuming; set to true when
     * only working on the browser compat or code generator portions unrelated
     * to comments/docs.
     */
    private boolean SKIP_DOCS = false;
    /**
     * Skip reading in the Mozilla Developer Connection files - during development of the W3C scanning code
     */
    private boolean SKIP_MDC = false;

    /**
     * Skip reading in the W3C files - during development of the MDC scanning code
     */
    private boolean SKIP_W3C = false;

    /**
     * Generate sdoc files instead of javascript files
     */
    private boolean GENERATE_SDOC = false; // NOT YET FULLY IMPLEMENTED!
    private boolean GENERATE_JAVASCRIPT = !GENERATE_SDOC;
    
    /**
     * Generate browser maps? Used for quickfixes for illegal calls
     */
    private boolean GENERATE_BROWSER_MAPS = true;
    
    static final String[] PREFIXES = { 
        "Core_JavaScript_1.5_Reference:Global_Objects:", 
        "Core_JavaScript_1.5_Reference:Objects:", 
        "Core_JavaScript_1.5_Reference:Functions:", 
        "Core_JavaScript_1.5_Reference:Global_Functions:", 
        "Core_JavaScript_1.5_Reference:Global_Objects:", 
        "Core_JavaScript_1.5_Reference:Global_Properties:", 
        "Core_JavaScript_1.5_Reference:Properties:", 
        // Skipped: Core_JavaScript_1.5_Reference:{Statements:,Operators:,Comments:, etc.}
        "DOM:" 
    };

    private void addComment(Main.AttributeDesc attribute, String string) {
        if (attribute.comment != null) {
            String s = attribute.comment;
            int end = s.lastIndexOf("*/");
            assert string.startsWith("/**") : string;
            assert string.endsWith("*/\n") : string;
            s = s.substring(0, end) + string.substring("/**".length(), string.length()-"*/\n".length()) + "\n" + s.substring(end);
            attribute.comment = s;
        } else {
            attribute.comment = string;
        }
    }

    enum Type { FUNCTION, METHOD, PROPERTY, EVENT, CLASS, CONSTANT, CONSTRUCTOR };
    enum Origin { CORE, DOM1, DOM2, DOM3, DOM, CSS, HTML, RANGE, EVENTS, CANVAS };
    
    private Map<String,ClassDesc> classMap = new HashMap<String,ClassDesc>();

    private static final Map<String,String> CORE_TYPES = new HashMap<String,String>() {{
        put("Array.concat", "@type Array");
        put("Array.join", "@type String");
        put("Array.length", "@type Number");
        put("Array.reverse", "@type void");
        put("Array.shift", "@type void");
        put("Array.slice", "@type Array");
        put("Array.sort", "@type Array");
        put("Array.splice", "@type Array");
        put("Array.toLocaleString", "@type String");
        put("Array.toString", "@type String");
        put("Array.unshift", "@type Number");
        put("Boolean.toString", "@type String");
        put("Date.getDate", "@type Number");
        put("Date.getDay", "@type Number");
        put("Date.getFullYear", "@type Number");
        put("Date.getHours", "@type Number");
        put("Date.getMilliseconds", "@type Number");
        put("Date.getMinutes", "@type Number");
        put("Date.getMonth", "@type Number");
        put("Date.getSeconds", "@type Number");
        put("Date.getTime", "@type Number");
        put("Date.getTimezoneOffset", "@type Number");
        put("Date.getUTCDate", "@type Number");
        put("Date.getUTCDay", "@type Number");
        put("Date.getUTCFullYear", "@type Number");
        put("Date.getUTCHours", "@type Number");
        put("Date.getUTCMilliseconds", "@type Number");
        put("Date.getUTCMonth", "@type Number");
        put("Date.getUTCSeconds", "@type Number");
        put("Date.getYear", "@type Number");
        put("Date.parse", "@type Number");
        //put("Date.setDate", "@param {Number} date_of_month");
        // TODO - set the various setter signatures here. No return types but relevant for parameter types,
        // but I have to match to parameter names with what I have in the Mozilla documentation
        put("Date.toDateString", "@type String");
        put("Date.toGMTString", "@type String");
        put("Date.toLocaleDateString", "@type String");
        put("Date.toLocaleString", "@type String");
        put("Date.toLocaleTimeString", "@type String");
        put("Date.toString", "@type String");
        put("Date.toTimeString", "@type String");
        put("Date.toUTCString", "@type String");
        put("Date.UTC", "@type Number");
        put("Date.valueOf", "@type Number");
        put("Date.decodeURI", "@type String\n@param {String} uri");
        put("Date.decodeURIComponent", "@type String\n@param {String} s");
        put("Date.encodeURI", "@type String\n@param {String} uri");
        put("Date.encodeURIComponent", "@type String\n@param {String} s");
        put("Error.message", "@type String");
        put("Error.name", "@type String");
        put("Error.toString", "@type String");
        put("escape", "@type String"); // XXX This is missing
        put("eval", "@param {String} code");
        put("Function.length", "@type Number");
        put("Function.toString", "@type String");
        put("Function.getClass", "@type JavaClass");
        put("Infinity", "@type Number");
        put("isFinite", "@type Boolean");
        put("isNaN", "@type Boolean");
        put("Math.E", "@type Number");
        put("Math.LN10", "@type Number");
        put("Math.LN2", "@type Number");
        put("Math.LOG10E", "@type Number");
        put("Math.LOG2E", "@type Number");
        put("Math.PI", "@type Number");
        put("Math.SQRT1_2", "@type Number");
        put("Math.SQRT2", "@type Number");
        put("Math.abs", "@type Number");
        put("Math.acos", "@type Number");
        put("Math.asin", "@type Number");
        put("Math.atan", "@type Number");
        put("Math.atan2", "@type Number");
        put("Math.ceil", "@type Number");
        put("Math.cos", "@type Number");
        put("Math.exp", "@type Number");
        put("Math.floor", "@type Number");
        put("Math.log", "@type Number");
        put("Math.max", "@type Number");
        put("Math.min", "@type Number");
        put("Math.pow", "@type Number");
        put("Math.random", "@type Number");
        put("Math.round", "@type Number");
        put("Math.sin", "@type Number");
        put("Math.sqrt", "@type Number");
        put("Math.tan", "@type Number");
        put("Math.NaN", "@type Number");
        put("Number.MAX_VALUE", "@type Number");
        put("Number.MIN_VALUE", "@type Number");
        put("Number.NaN", "@type Number");
        put("Number.NEGATIVE_INFINITY", "@type Number");
        put("Number.POSITIVE_INFINITY", "@type Number");
        put("Number.toExponential", "@type String");
        put("Number.toFixed", "@type String");
        put("Number.toLocaleString", "@type String");
        put("Number.toPrecision", "@type String");
        put("Number.toString", "@type String");
        put("Number.valueOf", "@type Number");
        put("Object.hasOwnProperty", "@type Boolean");
        put("Object.isPrototypeOf", "@type Boolean");
        put("Object.propertyIsEnumerable", "@type Boolean");
        put("Object.toLocaleString", "@type String");
        put("Object.toString", "@type String");
        put("parseFloat", "@type Number");
        put("parseInt", "@type Number");
        put("RegExp.global", "@type Boolean");
        put("RegExp.ignoreCase", "@type Boolean");
        put("RegExp.lastIndex", "@type Number");
        put("RegExp.source", "@type String");
        put("RegExp.test", "@type Boolean");
        put("RegExp.toString", "@type String");
        put("String.charAt", "@type String");
        put("String.charCodeAt", "@type Number");
        put("String.concat", "@type String");
        put("String.fromCharCode", "@type String");
        put("String.indexOf", "@type Number");
        put("String.lastIndexOf", "@type Number");
        put("String.length", "@type Number");
        put("String.localeCompare", "@type Number");
        put("String.match", "@type String[]");
        put("String.replace", "@type String");
        put("String.search", "@type Number");
        put("String.slice", "@type String");
        put("String.split", "@type String[]");
        put("String.substr", "@type String");
        put("String.substring", "@type String");
        put("String.toLocaleLowerCase", "@type String");
        put("String.toLocaleUpperCase", "@type String");
        put("String.toLowerCase", "@type String");
        put("String.toString", "@type String");
        put("String.toUpperCase", "@type String");
        put("String.valueOf", "@type String");
        put("unescape", "@type String\n@param {String} s");
    }};

    private static final Map<String,String> DOM_TYPES = new HashMap<String,String>() {{
        put("Canvas.getContext", "@type CanvasRenderingContext2D");
        // TODO - Canvas
        
        put("CharacterData.substringData", "@type String");
        put("Document.createAttribute", "@type Attr");
        put("Document.createAttributeNS", "@type Attr");
        put("Document.createCDATASection", "@type CDATASection");
        put("Document.createComment", "@type Comment");
        put("Document.createDocumentFragment", "@type DocumentFragment");
        put("Document.createElement", "@type Element");
        put("Document.createElementNS", "@type Element");
        put("Document.createEvent", "@type Event");
        put("Document.createExpression", "@type XPathExpression");
        put("Document.createProcessingInstruction", "@type ProcessingExpression");
        put("Document.createRange", "@type Range");
        put("Document.createTextNode", "@type Text");
        put("Document.getElementById", "@type Element");
        put("Document.getElementsByTagName", "@type Element[]");
        put("Document.getElementsByTagNameNS", "@type Element[]");
        put("Document.importNode", "@type Node");
        //put("Document.loadXML"  IE only
        put("DOMImplementation.createDocumentType", "@type DocumentType");
        put("DOMImplementation.hasFeature", "@type Boolean");
        put("DOMParser.parseFromString", "@type Document");
        put("Element.addEventListener", "@type void");
        put("Element.attachEvent", "@type void");
        put("Element.detachEvent", "@type void");
        put("Element.dispatchEvent", "@type void");
        put("Element.getAttribute", "@type String");
        put("Element.getAttributeNode", "@type Attr");
        put("Element.getAttributeNodeNS", "@type Attr");
        put("Element.getAttributeNS", "@type String");
        put("Element.getElementsByTagName", "@type Element[]");
        put("Element.getElementsByTagNameNS", "@type Element[]");
        put("Element.hasAttribute", "@type Boolean");
        put("Element.hasAttributeNS", "@type Boolean");
        put("Element.removeAttribute", "@type void");
        put("Element.removeAttributeNS", "@type void");
        put("Element.removeAttributeNode", "@type Attr");
        put("Element.removeEventListener", "@type void");
        put("Element.setAttribute", "@type void");
        put("Element.setAttributeNS", "@type void");
        put("Element.setAttributeNode", "@type Attr");
        put("Element.setAttributeNodeNS", "@type Attr");
        put("Event.initEvent", "@type void");
        put("Event.preventDefault", "@type void");
        put("Event.stopPropagation", "@type void");
        put("ExternalInterface.addCallBack", "@type Boolean");
        //put("ExternalInterface.call", "@type Object");
        put("FlashPlayer.GetVariable(variableName)", "Get a movie variable\n@type String\n@param {String} variableName The name of the variable");
        put("FlashPlayer.GotoFrame(frameNumber)", "Jump to a specified frame\n@type void\n@param {Number} frameNumber");
        put("FlashPlayer.IsPlaying()", "Return true if the movie is playing\n@type Boolean");
        put("FlashPlayer.LoadMovie(layer,url)", "Load a given movie into a given layer\n@type void\n@param {Number} layer Layer to display in\n@param {String} url The URL of the movie");
        put("FlashPlayer.Pan(dx,dy,mode)", "Move the movie viewport\n@type void\n@param {Number} dx Horizontal pan\n@param {Number} dy Vertical pan\n@param {Number} mode 0 for pixels, 1 for percentages");
        put("FlashPlayer.PercentLoaded()", "Return the percentage of the movie that has been loaded\n@type Number");
        put("FlashPlayer.Play()", "Play the movie\n@type void");
        put("FlashPlayer.Rewind()", "Rewind the movie\n@type void");
        put("FlashPlayer.SetVariable(name,value)", "Set a movie variable\n@type void\n@param {String} name The name of the variable\n@param {String} value The value of the vairable");
        put("FlashPlayer.SetZoomRect(left,top,right,bottom)", "Set the viewport\n@type void\n@param {Number} left Left edge of viewport in twips\n@param {Number} top Top edge of viewport in twips\n@param {Number} right Right edge of viewport in twips\n@param {Number} bottom Bottom edge of viewport in twips\n");
        put("FlashPlayer.StopPlay()", "Stop playing the movie\n@type void");
        put("FlashPlayer.TotalFrames()", "Return the framecount for the movie\n@type Number");
        put("FlashPlayer.Zoom(percentage)", "Zoom in or out\n@param {Number} percentage Scale amount, or 0 to resize to full");
        put("Form.elements", "@type HTMLCollection");
        put("Form.reserve", "@type void");
        put("Form.submit", "@type void");
        // Stopped at Hidden p811
        put("Window.alert", "@type void\n@param {String} message");
        put("Window.blur", "@type void");
        put("Window.clearInterval", "@type void");
        put("Window.clearTimeout", "@type void");
        put("Window.close", "@type void");
        put("Window.confirm", "@param {String} message\n@type void");
        put("Window.defaultStatus", "@type String");
        put("Window.focus", "@type void");
        put("Window.getComputedStyle", "@type CSS2Properties");
        put("Window.moveBy", "@type void");
        put("Window.moveTo", "@type void");
        put("Window.print", "@type void");
        put("Window.prompt", "@type String\n@param {String} text\n@param {String} value");
        put("Window.resizeBy", "@type void");
        put("Window.resizeTo", "@type void");
        put("Window.scrollBy", "@type void");
        put("Window.scrollTo", "@type void");
        put("Window.setInterval", "@param {Number} delay\n@param {Function} func");
        put("Window.setTimeout", "@param {Number} delay\n@param {Function} func");
    }};
    
    public static void main(String[] args) {
        if (args.length < 2) {
            System.err.println("Usage: " + Main.class.getName() + " <inputdir> <outputdir> [w3cdir]");
            System.exit(-1);
        }
        
        try {
            File mozdev = new File(args[0]);
            if (!mozdev.exists()) {
                System.err.println(mozdev.getPath() + " does not exist");
                System.exit(1);
            }
            File docs = new File(mozdev, "en" + File.separator + "docs");
            if (!docs.exists()) {
                System.err.println("Can't find " + docs.getPath());
                System.err.println("Is this a valid input directory? See the javadoc header for\ninstructions on how to generate it.\n");
                System.exit(2);
            }

            File input = docs;
            File output = new File(args[1]);
            if (output.exists() && output.isFile()) {
                System.err.println("Output directory must be a directory!");
                System.exit(3);
            }
            
            File w3cInput = null;
            if (args.length >= 3) {
                w3cInput = new File(args[2]);
                if (!w3cInput.exists()) {
                    System.err.println("Can't find " + w3cInput.getPath());
                    System.exit(2);
                }
            }
            new Main().process(input, output, w3cInput);
        } catch (IOException ioe) {
            ioe.printStackTrace();
        }
    }

    /** Compute a set of classes*/
    private void findMDCClasses(File input) {
        if (SKIP_MDC) {
            return;
        }
        
        for (File f : input.listFiles()) {
            String fileName = f.getName();
            for (String prefix : PREFIXES) {
                if (fileName.endsWith(".orig")) {
                    continue;
                }
                if (fileName.startsWith(prefix)) {

//if (!fileName.startsWith("DOM:element")) {
//    continue;
//}
                    
                    if (fileName.equals("DOM:WRONG_DOCUMENT_ERR_note")) {
                        // Not a real class document
                        continue;
                    }
                    
                    if (fileName.startsWith("DOM:node")) {
                        // Skip -- this is just a placeholder in the current version of the documentation
                        continue;
                    }

                    if (fileName.equals("Core_JavaScript_1.5_Reference:Functions:arguments:length")) {
                        // Special case: arguments is a local var!
                        // See http://developer.mozilla.org/en/docs/Core_JavaScript_1.5_Reference:Functions:arguments
                        continue;
                    }

                    int prefixLength = prefix.length();
                    String clz = fileName.substring(prefixLength);
                    int colonIndex = fileName.indexOf(':', prefixLength);
                    if (colonIndex == -1) {
                        colonIndex = fileName.indexOf('.', prefixLength);
                    }
                    String rest = "";
                    if (colonIndex != -1) {
                        clz = fileName.substring(prefixLength, colonIndex);
                        rest = fileName.substring(colonIndex+1);
                        int restDot = rest.indexOf('.');
                        if (restDot != -1) {
                            // Multiple dots -- it's a compound class
                            clz = rest.substring(0, restDot);
                            assert rest.indexOf('.', restDot+1) == -1; // Can't handle more levels!!
                            colonIndex = colonIndex+1+restDot;
                        }
                    }

                    // Make sure we don't have multiple
                    // Special cases
                    if (clz.equals("dispatchEvent_example")) { // NOI18N
                        // Not a real class
                        continue;
                    }

                    Origin origin = Origin.CORE;
                    if (prefix.startsWith("DOM:")) {
                        origin = Origin.DOM;
                    }

                    String property = null;
                    if (Character.isLowerCase(clz.charAt(0))) {
                        if (origin == Origin.DOM) {
                            property = clz;
                        }
                        clz = Character.toUpperCase(clz.charAt(0)) + clz.substring(1);
                    }
                    
                    ClassDesc desc = classMap.get(clz);
                    if (desc == null) {
                        desc = new ClassDesc(clz, origin);
                        desc.property = property;
                        classMap.put(clz, desc);
                    }
                    if (colonIndex != -1) {
                        String prop = fileName.substring(colonIndex+1);
                        AttributeDesc attribute = desc.getAttribute(prop);
                        if (attribute == null) {
                            attribute = new AttributeDesc(prop, f, desc);
                            desc.addAttribute(attribute);
                        } else {
                            attribute.files.add(f);
                        }
                    } else {
                        if (desc.descFiles == null) {
                            desc.descFiles = new ArrayList<File>(2);
                        }
                        desc.descFiles.add(f);
                    }
                }
            }
        }
    }
    
    private void assignMDCComments() {
        Map<String,String> map = classMap.get("Array") == null ? DOM_TYPES : CORE_TYPES;
        for (Map.Entry<String,String> entry : map.entrySet()) {
            String key = entry.getKey();
            String value = entry.getValue();
            
            int index = key.indexOf('.');
            String clz, attr;
            if (index == -1) {
                clz = "Object";
                attr = key;
            } else {
                clz = key.substring(0, index);
                attr = key.substring(index+1);
            }
            ClassDesc desc = classMap.get(clz);
            int sigStart = attr.indexOf('(');
            String attrName = sigStart == -1 ? attr : attr.substring(0, sigStart);
            
            //assert desc != null : clz + " from " + entry;
            if (desc == null) {
                if (sigStart == -1) {
                    System.err.println("WARNING: No class desc found for class " + clz + " in entry " + key);
                    continue;
                } else {
                    // Create this class
                    Origin origin = map == DOM_TYPES ? Origin.DOM : Origin.CORE;
                    desc = new ClassDesc(clz, origin);
                    classMap.put(clz, desc);
                }
            }

            AttributeDesc a = desc.getAttribute(attrName);
            if (a == null) {
                if (sigStart == -1) {
                    System.err.println("WARNING: No attribute " + attr + " found in class " + clz);
                    continue;
                }
                a = new AttributeDesc(attrName, null, desc);
                a.signatures = Collections.singletonList(attr);
                a.type = Type.METHOD;
                desc.addAttribute(a);
            }

            StringBuilder sb = new StringBuilder();
            for (String s : value.split("\n")) {
                sb.append(" * ");
                sb.append(s);
                sb.append("\n");
            }
            String comment = sb.toString();
            
            if (a.comment != null) {
                // Append the comment
                int endIndex = a.comment.lastIndexOf("*/");
                assert endIndex != -1 : a.comment;
                a.comment = a.comment.substring(0, endIndex) + "\n" + comment + "*/\n";
            } else {
                a.comment = "/**\n" + comment + "*/\n";
            }
        }
    }

    private void findW3CClasses(File input) throws IOException {
        if (SKIP_W3C) {
            return;
        }
        for (File f : input.listFiles()) {
            String fileName = f.getName();
            if (!fileName.endsWith(".html") || !fileName.contains("ecma")) {
                continue;
            }
            
            Origin origin = null;
            if (fileName.indexOf("-Core") != -1) {
                if (fileName.indexOf("Level-3") != -1) {
                    origin = Origin.DOM3;
                } else if (fileName.indexOf("Level-1") != -1) {
                    origin = Origin.DOM1;
                } else {
                    origin = Origin.DOM2;
                }
            } else if (fileName.indexOf("-Style") != -1) {
                origin = Origin.CSS;
            } else if (fileName.indexOf("-HTML") != -1) {
                origin = Origin.HTML;
            } else if (fileName.indexOf("-Traversal") != -1) {
                origin = Origin.RANGE;
            } else if (fileName.indexOf("-Events") != -1) {
                origin = Origin.EVENTS;
            } else {
                // ?
                System.err.println("No known origin for " + fileName);
                origin = Origin.DOM2;
            }

            Document html = parse(f);
            
            // Find documentation files, if any
            List<Document> docList = new ArrayList<Document>();
            Map<String,String> documentation = new HashMap<String,String>();
            int dot = fileName.indexOf("ecma");
            String base = fileName.substring(0, dot);
            for (File otherFile : input.listFiles()) {
                String other = otherFile.getName();
                if (other.indexOf("ecma") == -1 && other.startsWith(base)) {
                    Document doc = parse(otherFile);
                    if (doc != null) {
                        initDoc(documentation, doc);
                        docList.add(doc);
                    }
                }
            }
            
            List<Element> divs = findElementsByClass(html, "div", "ecma-block");
            for (Element div : divs) {
                List<Element> dls = findDirectDescendantElements(div, "dl");
                // Each dl corresponds to a class
                for (Element dl : dls) {
                    List<Element> dtdds = findDirectDescendantElements(dl, null);
                    for (int elementIndex = 0; elementIndex < dtdds.size(); elementIndex++) {
                        Element dt = dtdds.get(elementIndex);
                        assert dt.getTagName().equals("dt") : dt;
                        Element dd = null;
                        if (elementIndex < dtdds.size()-1) {
                            dd = dtdds.get(elementIndex+1);
                            if (!dd.getTagName().equals("dd")) {
                                dd = null;
                            } else {
                                elementIndex++;
                            }
                        }
                        
                        String clz = null;
                        String dtText = getText(dt).replace('\n', ' ').trim();
                        
                        if (dtText.matches(".*Properties of the (\\S+) Constructor function.*")) {
                            // Known problem - we probably don't need these yet. TODO FIXME.
                            //elementIndex++;
                            continue;
                        }
                        
                        Matcher m2 = CLASS_PATTERN.matcher(dtText);
                        if (m2.find()) {
                            clz = m2.group(1);
                        }
                        
                        if (clz == null) {
                            List<Element> bs = findDirectDescendantElements(dt, "b");
                            assert bs.size() == 1 : dtText;
                            Element b = bs.get(0);
                            clz = getText(b).trim();
                        }
                        
                        ClassDesc desc = classMap.get(clz);
                        if (desc == null) {
                            desc = new ClassDesc(clz, origin);
                            classMap.put(clz, desc);
                        } else if (origin.ordinal() < desc.origin.ordinal()) {
                            desc.origin = origin;
                        }
                        
                        // TODO - scan the spec for actual documentation for these classes
                        if (dd != null) {
                            List<Element> ddch = findDirectDescendantElements(dd, "dl");
                            if (ddch.size() == 0) {
                                // Probably a tidied document
                                continue;
                            }
                            assert ddch.size() == 1 : ddch.size();
                            List<Element> defs = findDirectDescendantElements(ddch.get(0), null);
                            int index = 0;
                            while (index < defs.size()) {
                                Element e = defs.get(index);
                                assert e.getTagName().equals("dt") : e;
                                String text = getText(e).trim();
                                text = text.replace('\n',' ');
                                if (text.startsWith("Objects that implement the")) {
                                    // Inheritance information
                                    index++; 
                                } else if (text.startsWith("Properties of objects") || text.endsWith("properties:")) {
                                    // It's a property section
                                    index++;
                                    Element section = defs.get(index);
                                    index++;
                                    addProperties(desc, Type.PROPERTY, section, origin, documentation);
                                } else if (text.startsWith("Functions of objects") || text.endsWith("methods:") || text.endsWith("functions:")) {
                                    // It's a function section
                                    index++;
                                    Element section = defs.get(index);
                                    index++;
                                    addProperties(desc, Type.METHOD, section, origin, documentation);
                                } else if (text.endsWith("class has the following constants:")) {
                                    // It's a function section
                                    index++;
                                    Element section = defs.get(index);
                                    index++;
                                    addProperties(desc, Type.CONSTANT, section, origin, documentation);
                                } else {
                                    Matcher m = EXTENDS_PATTERN.matcher(text);
                                    if (m.find()) {
                                        String child = m.group(1);
                                        assert child.equals(desc.name);
                                        String superClass = m.group(2);
                                        desc.superClass = superClass;
                                        index++;
                                        continue;
                                    } else {
                                        m = EXTENDS_PATTERN2.matcher(text);
                                        if (m.find()) {
                                            String child = m.group(1);
                                            assert child.equals(desc.name);
                                            String superClass = m.group(2);
                                            desc.superClass = superClass;
                                            index++;
                                            continue;
                                        }
                                    }

                                    assert false : text;
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    
    private void initDoc(Map<String,String> map, Document doc) {
        List<Element> methods = findElementsByClass(doc, "code", "method-name");
        process(map, methods);
        List<Element> constants = findElementsByClass(doc, "code", "constant-name");
        process(map, constants);
        List<Element> attributes = findElementsByClass(doc, "code", "attribute-name");
        process(map, attributes);
        // and parameters, parameter-name, paramtable
    }
    
    private void process(Map<String,String> map, List<Element> elements) {
        for (Element element : elements) {
            String property = getText(element);
            String s = getDocument(element);
            assert property.indexOf(".") == -1 : s;
            assert property.length() > 0 : s;
            assert s.length() > 0 :  property + " has empty docs"; 
            
            //assert !map.containsKey(property) : property + ":" + s + map.get(property);
            if (map.containsKey(property)) {
                System.err.println("WARNING - duplicate doc keys: property=" +  property + ":" + s + map.get(property));
            }
            
            map.put(property, s);
        }
    }
    
    private void addMethodNodeText(Node n, StringBuilder sb, boolean add, String cls) {
        // This doesn't pick up text in comments, which is usually the correct thing to do,
        // but for things like JavaScript code in HTML pages, it's sometimes embedded in
        // comments for backwards compatibility reasons
        if (n.getNodeType() == Node.TEXT_NODE) {
            if (add) {
                String v = n.getNodeValue();
                v = v.trim();
                if (v.length() > 0) {
                    if (sb.length() > 0 && !Character.isWhitespace(sb.charAt(sb.length()-1))) {
                        sb.append(' ');
                    }
                    sb.append(v);
                }
            }
        } else {
            if (n.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element)n;
                if (element.getTagName().equals("div")) {
                    cls = element.getAttribute("class");
                    if ("parameters".equals(cls) || "return".equals("cls")) {
                        add = false;
                    } else if ("attribute-name".equals(cls)) {
                        add = true;
                    } else if ("method".equals(cls)) {
                        add = true;
                    } else if ("returntable".equals(cls)) {
                        add = true;
                        if (sb.length() > 0 && sb.charAt(sb.length()-1) != '\n') {
                            sb.append("\n");
                        }
                        sb.append("@return ");
                    } else if ("paramtable".equals(cls)) {
                        add = true;
                    } else {
                        add = false;
                    }
                } else if (element.getTagName().equals("dt")) {
                    if ("paramtable".equals(cls)) {
                        if (sb.length() > 0 && sb.charAt(sb.length()-1) != '\n') {
                            sb.append("\n");
                        }
                        sb.append("@param ");
                    } else if ("method".equals(cls)) {
                        String s = getText(element).trim();
                        if ("Parameters".equals(s)) {
                            add = false;
                        }
                    }
                }
            }
            if (n.hasChildNodes()) {
                NodeList nl = n.getChildNodes();
                for (int i = 0; i < nl.getLength(); i++) {
                    addMethodNodeText(nl.item(i), sb, add, cls);
                }
            }
        }
    }

    private String getDocument(Element labelElement) {
        assert labelElement.getTagName().equals("code") : labelElement.getTagName();
        Element dt = (Element) labelElement.getParentNode();
        if (dt.getTagName().equals("a")) {
            dt = (Element)dt.getParentNode();
        }
        assert dt.getTagName().equals("dt") : dt.getTagName();
        Element dd = getSiblingElement(dt);
        assert dd.getTagName().equals("dd") : dt.getTagName();
    
        String desc;
        // See if it's a method node - we know how to handle these specially
        List<Element> divs = findElementsByClass(dd, "div", "method");
        if (divs.size() == 1) {
            StringBuilder sb = new StringBuilder();
            addMethodNodeText(dd, sb, false, "");
            desc = sb.toString();
        } else {
            desc = getText(dd).trim();

            // Chop off to first paragraph
            int dot = -1;
            while (true) {
                dot = desc.indexOf('\n', dot+1);
                if (dot == -1) {
                    break;
                }
                // Scan forward and see if we have a blank line
                boolean foundChar = false;
                for (int i = dot+1; i < desc.length(); i++) {
                    char c = desc.charAt(i);
                    if (c == '\n') {
                        break;
                    }
                    if (!Character.isWhitespace(c)) {
                        foundChar = true;
                        break;
                    }
                }
                if (!foundChar) {
                    desc = desc.substring(0, dot);
                    break;
                }
            }
        }
        
        return desc;
    }
    
    final static Pattern PARAMETER_PATTERN = Pattern.compile("The (\\S+) parameter is a (\\S+)");
    final static Pattern PARAMETER_PATTERN2 = Pattern.compile("The (\\S+) parameter is an object that implements the (\\S+)");
    final static Pattern PARAMETER_PATTERN3 = Pattern.compile("The (\\S+) parameter is of type (\\S+)");
    final static Pattern CLASS_PATTERN = Pattern.compile("Objects that implements the (\\S+)");
    final static Pattern EXTENDS_PATTERN = Pattern.compile("(\\S+) has the all the properties and methods of the (\\S+) object as well as");
    final static Pattern EXTENDS_PATTERN2 = Pattern.compile("(\\S+) has the all the properties and methods of (\\S+) as well as");
    final static Pattern RETURN_CLASS_PATTERN = Pattern.compile("This function returns an object that implements the (\\S+)");
    final static Pattern RETURN_CLASS_PATTERN2 = Pattern.compile("This method returns a (\\S+)");
    
    private String fixType(String type) {
        type = type.trim();
        if ("any".equals(type)) {
            return "Object";
        } else if ("long".equals(type) || "short".equals(type) || "float".equals(type)) {
            return "Number";
        }
        
        return type;
    }
    
    private void addProperties(ClassDesc clz, Type type, Element dd, Origin origin, Map<String,String> documentation) {
        List<Element> ddch = findDirectDescendantElements(dd, "dl");
        assert ddch.size() == 1;
        List<Element> props = findDirectDescendantElements(ddch.get(0), null);
        int index = 0;
        while (index < props.size()) {
            Element item = props.get(index);
            assert item.getTagName().equals("dt");
            
            String propName = null;
            List<Element> bs = findDirectDescendantElements(item, "b");
            if (bs.size() == 0) {
                propName = getText(item).trim();
            } else {
                assert bs.size() == 1 : bs.size() + ":" + getText(item).trim();
                Element b = bs.get(0);
                propName = getText(b).trim();
            }
            if (propName.startsWith(clz.name+".")) {
                propName = propName.substring(clz.name.length()+1);
            }
            
            String signature = getCleanString(propName);
            assert signature != null;
            int parenIndex = propName.indexOf('(');
            if (parenIndex != -1) {
                propName = propName.substring(0, parenIndex).trim();
            }
            
            index++;
            
            AttributeDesc a = clz.getAttribute(propName);
            if (a == null) {
                a = new AttributeDesc(propName, null, clz);
                clz.addAttribute(a);
            }
            boolean replaceComment = false;
            if (a.origin != null && origin != null && a.origin != origin) {
                if (origin.ordinal() < a.origin.ordinal()) {
                    a.origin = origin;
                } else {
                    replaceComment = true;
                }
            }
            a.origin = origin;
            a.type = type;
            a.signatures = Collections.singletonList(signature);
            StringBuilder sb = new StringBuilder();
            sb.append("/**\n");
            
            String doc = documentation.get(propName);
            if (doc != null) {
                for (String d : doc.split("\n")) {
                    if (!d.startsWith(" * ")) {
                        sb.append(" * ");
                    }
                    sb.append(d);
                    sb.append("\n");
                }
            }
            
            Element expl = props.get(index);
            String[] valueText = getFlattenedText(expl).split("\n");
            String propType = null;
            for (String s : valueText) {
                int from = 0;
                if (s.indexOf("This function has no return value") != -1 ||
                        s.indexOf("This method has no return value") != -1) {
                    propType = "void";
                }
                while (from < s.length()) {
                    Matcher m = PARAMETER_PATTERN.matcher(s);
                    if (m.find(from)) {
                        String paramName = m.group(1);
                        String paramType = fixType(m.group(2));
                        int exists = sb.indexOf("@param " + paramName);
                        if (exists != -1) {
                            sb.insert(exists+"@param ".length(), "{" + paramType+"} ");
                        } else {
                            sb.append(" * @param {" + paramType + "} " + paramName + "\n");
                        }
                        from = m.end();
                        continue;
                    } else {
                        m = RETURN_CLASS_PATTERN.matcher(s);
                        if (m.find(from)) {
                            String paramType = m.group(1);
                            propType = fixType(paramType);
                            if (propType != null) {
                                sb.append(" * @type " + propType + "\n");
                            }
                            from = m.end();
                            continue;
                        } else {
                            m = RETURN_CLASS_PATTERN2.matcher(s);
                            if (m.find(from)) {
                                String paramType = m.group(1);
                                propType = paramType;
                                if (propType != null) {
                                    sb.append(" * @type " + propType + "\n");
                                }
                                from = m.end();
                                continue;
                            } else {
                                m = PARAMETER_PATTERN2.matcher(s);
                                if (m.find(from)) {
                                    String paramName = m.group(1);
                                    String paramType = fixType(m.group(2));
                                    int exists = sb.indexOf("@param " + paramName);
                                    if (exists != -1) {
                                        sb.insert(exists+"@param ".length(), "{" + paramType+"} ");
                                    } else {
                                        sb.append(" * @param {" + paramType + "} " + paramName + "\n");
                                    }
                                    from = m.end();
                                    continue;
                                } else {
                                    m = PARAMETER_PATTERN3.matcher(s);
                                    if (m.find(from)) {
                                        String paramName = m.group(1);
                                        String paramType = fixType(m.group(2));
                                        int exists = sb.indexOf("@param " + paramName);
                                        if (exists != -1) {
                                            sb.insert(exists+"@param ".length(), "{" + paramType+"} ");
                                        } else {
                                            sb.append(" * @param {" + paramType + "} " + paramName + "\n");
                                        }
                                        from = m.end();
                                        continue;
                                    }
                                }
                            }
                        }
                    }
                    from = s.length();
                }
            }
            if (propType == null && (a.type != Type.FUNCTION && a.type != Type.METHOD)) {
                List<Element> typeBs = findDirectDescendantElements(expl, "b");
                if (typeBs.size() > 0) {
                    assert typeBs.size() >= 1;
                    propType = getText(typeBs.get(0)).trim();
                }
            }
            index++;
            
            if (propType != null && sb.indexOf("@type " + propType) == -1) {
                propType = fixType(propType);
                if (propType != null) {
                    sb.append(" * @type " + propType + "\n");
                }
            }
            sb.append(" */\n");
            if (a.comment == null || replaceComment) {
                a.comment = sb.toString();
            }
        }
    }
    
    private void generateCode(Collection<ClassDesc> classes, File outputDir) throws IOException {
        outputDir.mkdirs();
        List<ClassDesc> list = new ArrayList<Main.ClassDesc>(classes);
        Collections.sort(list);
        for (ClassDesc desc : list) {
            String origin = desc.origin.name().toLowerCase();
            String extension = GENERATE_SDOC ? ".sdoc" : ".js";
            File outputFile = new File(outputDir, "stub_" + origin + "_" + desc.name + extension);
            FileWriter writer = new FileWriter(outputFile);
            String code = desc.toCode();
            writer.write(code);
            writer.close();
            
            checkValidJs(outputFile, desc);
        }
    }
    
    /** Make sure the given file contains valid JavaScript */
    void checkValidJs(File file, ClassDesc desc) {
        StringBuilder sb = new StringBuilder();
        try {
            BufferedReader f = new BufferedReader(new FileReader(file));
            while (true) {
                String line = f.readLine();
                if (line == null) {
                    break;
                }
                sb.append(line);
                sb.append("\n");
            }
            f.close();
        } catch (IOException e) {
            e.printStackTrace();
            System.exit(1);
        }
        
//        if ("Function".equals(desc.name)) {
//            // XXX I don't know why parsing this file fails in the internal interpreter.
//            return;
//        }
//        ScriptEngineManager mgr = new ScriptEngineManager();
//        ScriptEngine jsEngine = mgr.getEngineByName("JavaScript");
//        if (jsEngine == null) {
//            System.err.println("No JavaScript scripting engine is available");
//            return;
//        }
//        try {
//            jsEngine.eval(sb.toString());
//        } catch (ScriptException ex) {
//            System.err.println("WARNING: Parse error for file " + file.getPath() + " in class " + desc.name);
//            System.err.println(ex.getFileName() + ":" + ex.getLineNumber() + ":"+ ex.getColumnNumber() + ": " + ex.getMessage());
//            //            ex.printStackTrace();
//            //            System.exit(2);
//        }        
        
        boolean ok = Parser.parse(sb.toString(), file.getPath(), 0);
        assert ok : file;
    }

    private boolean retried = false;
    
    private Document parse(File file) throws IOException {
        try {
            DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
            docFactory.setValidating(false);
            
            DocumentBuilder docBuilder = docFactory.newDocumentBuilder();
//            docBuilder.setEntityResolver(new EntityResolver() {
//                      public InputSource resolveEntity(java.lang.String publicId, java.lang.String systemId)
//                             throws SAXException, java.io.IOException
//                      {
//                        if (publicId.equals("--myDTDpublicID--"))
//                          // this deactivates the open office DTD
//                          return new InputSource(new ByteArrayInputStream("<?xml version='1.0' encoding='UTF-8'?>".getBytes()));
//                        else return null;
//                          return null;
//                      }
//            });            
            //Document doc = docBuilder.parse(file);
            // Read file in and skip DOCTYPE
            BufferedReader f = new BufferedReader(new FileReader(file));
            StringBuilder sb = new StringBuilder();
            while (true) {
                String line = f.readLine();
                if (line == null) {
                    break;
                }
                if (line.equals("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\">")) {
                    line = "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">";
                }
//                if (line.startsWith("<!DOCTYPE html PUBLIC")) {
//                    continue;
//                }
                sb.append(line);
                sb.append("\n");
            }
            f.close();
            String buffer = sb.toString();
//            if (!buffer.startsWith("<?xml")) {
//                buffer = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" + buffer;
//            }
            InputSource is = new InputSource(new StringReader(buffer));
            Document doc = docBuilder.parse(is);

            if (retried) {
                System.err.println("Dodged a bullet - tidying the source file succeeded.");
            }

            return doc;
        } catch (SAXException ex) {
            if (retried) {
                System.err.println("Parsing tidied file " + file.getAbsolutePath() + " somehow failed...");
                System.exit(0);
            }
            // Try reparsing!!
            if (!new File("/usr/bin/tidy").exists()) {
                System.err.println("There's a fatal parsing error for " + file.getPath());
                System.err.println("I'd like to fix it, but you need to have /usr/bin/tidy installed");
                System.exit(0);
            }
            
            System.err.println("Warning: There's a fatal parsing error for " + file.getPath());
            File output = new File("/tmp", file.getName());
            if (output.exists()) {
                output.delete();
            }
            ProcessBuilder pb = new ProcessBuilder("/usr/bin/tidy", "-asxhtml", "-o", output.getAbsolutePath(), file.getAbsolutePath());
            Process start = pb.start();
            try {
                start.waitFor();
                retried = true;
                return parse(output);
            } catch (InterruptedException ex1) {
                Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex1);
                System.exit(0);
            } finally {
                retried = false;
            }
            System.err.println("The document is not well formed.");
            System.err.println("Cannot continue - exiting");
            System.exit(0);

            Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
            IOException ioe = new IOException();
            ioe.initCause(ex);
            throw ioe;
        } catch (ParserConfigurationException ex) {
            Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
            IOException ioe = new IOException();
            ioe.initCause(ex);
            throw ioe;
        }
    }
    
    private List<Element> findElementsByClass(Document doc, String tagName, String className) {
        List<Element> elements = new ArrayList<Element>();
        
        NodeList nl = doc.getElementsByTagName(tagName);
        for (int i = 0; i < nl.getLength(); i++) {
            Element element = (Element) nl.item(i);
            if (tagName.equals(element.getTagName())) { // NOI18N
                if (className == null || className.equals(element.getAttribute("class"))) { // NOI18N
                    elements.add(element);
                }
            }
        }
        
        return elements;
    }

    private List<Element> findElementsByClass(Element root, String tagName, String className) {
        List<Element> elements = new ArrayList<Element>();
        
        NodeList nl = root.getElementsByTagName(tagName);
        for (int i = 0; i < nl.getLength(); i++) {
            Element element = (Element) nl.item(i);
            if (tagName.equals(element.getTagName())) { // NOI18N
                if (className == null || className.equals(element.getAttribute("class"))) { // NOI18N
                    elements.add(element);
                }
            }
        }
        
        return elements;
    }
    
    private Element getSiblingElement(Element element) {
        Node n = element.getNextSibling();
        while (n != null && n.getNodeType() != Node.ELEMENT_NODE) {
            n = n.getNextSibling();
        }
        return (Element)n;
    }

    private List<Element> findDirectDescendantElements(Element parent, String tagName) {
        List<Element> elements = new ArrayList<Element>();
        
        NodeList nl = parent.getChildNodes();
        for (int i = 0; i < nl.getLength(); i++) {
            Node child = nl.item(i);
            if (child.getNodeType() != Node.ELEMENT_NODE) {
                continue;
            }
            Element element = (Element)child;
            if (tagName == null || tagName.equals(element.getTagName())) { // NOI18N
                elements.add(element);
            }
        }
        
        return elements;
    }
    
    private void removeGeckoDomHeaders(Document doc) {
        NodeList nl = doc.getElementsByTagName("p");
        for (int i = 0, len = nl.getLength(); i < len; i++) {
            Element e = (Element) nl.item(i);
            
            // See if it's the DOM title node
            boolean found = false;
            NodeList childList = e.getChildNodes();
            for (int j = 0; j < childList.getLength(); j++) {
                Node child = childList.item(j);
                if (child.getNodeType() == Node.ELEMENT_NODE) {
                    Element childElement = (Element)child;
                    if (childElement.getTagName().equals("a")) {
                        if (childElement.getAttribute("href").equals("Gecko_DOM_Reference")) {
                            if (childList.getLength() == 3) {
                                e.getParentNode().removeChild(e);
                                return;
                            } else {
                                // Could be some other content in there - keep it
                                childElement.getParentNode().removeChild(childElement);
                                return;
                            }
                        }
                    }
                }
            }
            
            if (found) {
                e.getParentNode().removeChild(e);
            }
        }
    }
    
    private String serialize(Node node) throws IOException {
        try {
            Transformer transformer = TransformerFactory.newInstance().newTransformer();
            //transformer.setOutputProperty(OutputKeys.INDENT, "yes");
            transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
            StringWriter sw = new StringWriter();
            StreamResult result = new StreamResult(sw);
            NodeList nl = node.getChildNodes();
            // Serialize only the CHILDREN of the <div class="article">
            for (int i = 0; i < nl.getLength(); i++) {
                Node child = nl.item(i);
//                DOMSource source = new DOMSource(node);
                DOMSource source = new DOMSource(child);
                transformer.transform(source, result);
            }
            String xmlString = result.getWriter().toString().trim();
//            final String XML_HEADER = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
//            if (xmlString.startsWith(XML_HEADER)) {
//                xmlString = xmlString.substring(XML_HEADER.length());
//            }
            return xmlString;
        } catch (TransformerException ex) {
            Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
            IOException ioe = new IOException();
            ioe.initCause(ex);
            throw ioe;
        }
    }

    private void removeNodes(Document doc, String tag, String cls) {
        List<Element> edits = findElementsByClass(doc, tag, cls);
        for (Element element : edits) {
            element.getParentNode().removeChild(element);
        }
    }
    
    private void inlineStyle(Document doc) {
        // Copy in certain styles, since I don't have a way to
        // include a stylesheet in the middle of code completion documentation
        // strings
        NodeList nl = doc.getElementsByTagName("table");
        for (int i = 0; i < nl.getLength(); i++) {
            Element child = (Element) nl.item(i);
            String style = "background:#FFFFFF none repeat scroll 0%;border: 1px solid #666666;margin-bottom:10px;margin-top:10px";
            child.setAttribute("style", style);
            child.setAttribute("border", "1");
            child.setAttribute("width", "100%");
        }

        nl = doc.getElementsByTagName("pre");
        for (int i = 0; i < nl.getLength(); i++) {
            Element child = (Element) nl.item(i);
            String style = "background:#EEEEEE none repeat scroll 0% 50%;border:1px solid #666666;padding:5px 5px";
            child.setAttribute("style", style);
            child.removeAttribute("xml:space");
        }

        nl = doc.getElementsByTagName("td");
        for (int i = 0; i < nl.getLength(); i++) {
            Element child = (Element) nl.item(i);
            if ("header".equals(child.getAttribute("class"))) {
                String style = "background:#DDDDDD none repeat scroll 0%; border:1px solid #BBBBBB;";
                child.setAttribute("style", style);
            }
        }
        nl = doc.getElementsByTagName("dt");
        for (int i = 0; i < nl.getLength(); i++) {
            Element child = (Element) nl.item(i);
            String style = "font-weight:bold";
            child.setAttribute("style", style);
        }
    }
    
    /** Remove anchors and update URLs in links */
    private void cleanAnchors(Document doc) {
        NodeList nl = doc.getElementsByTagName("a");
        for (int i = 0; i < nl.getLength(); i++) {
            Element child = (Element) nl.item(i);
            if (!child.hasAttribute("href")) {
                child.getParentNode().removeChild(child);
            } else {
                String href = child.getAttribute("href");
                for (String prefix : PREFIXES) {
                    href = href.replace(prefix, "");
                }
                href = href.replace("Core_JavaScript_1.5_Reference:", "");
                child.setAttribute("href", href);
            }
        }
    }

    private void removeEmptyElements(Node node) {
        if (node.getNodeType() == Node.ELEMENT_NODE && !node.hasChildNodes()) {
            node.getParentNode().removeChild(node);
        } else {
            NodeList nl = node.getChildNodes();
            for (int i = 0; i < nl.getLength(); i++) {
                Node child = nl.item(i);
                removeEmptyElements(child);
            }
        }
    }

    private void removeClasses(Node node) {
        if (node.getNodeType() == Node.ELEMENT_NODE) {
            Element element = (Element)node;
            if (element.hasAttribute("class")) {
                element.removeAttribute("class");
            }
        }
        NodeList nl = node.getChildNodes();
        for (int i = 0; i < nl.getLength(); i++) {
            Node child = nl.item(i);
            removeClasses(child);
        }
    }
    
    private void moveCopyright(Document doc, Element target) {
        // Find the copyright notice, and insert it at the bottom
        // of the div
        Element copyright = doc.getElementById("f-copyright");
        assert copyright != null;
        copyright = (Element) copyright.getParentNode(); // The surrounding <ul>
        assert copyright.getTagName().equals("ul");
        
        copyright.getParentNode().removeChild(copyright);
        target.appendChild(copyright);
        String style = "list-style-type:none;font-size:0.9em;text-align:center";
        copyright.setAttribute("style", style);

//        Element about = doc.getElementById("f-about");
//        assert about != null;
//        
//        copyright.getParentNode().removeChild(about);
//        target.appendChild(about);
//        about.setAttribute("style", style);
    }
    
    private void removeComments(Node node) {
        if (node.getNodeType() == Node.COMMENT_NODE) {
            node.getParentNode().removeChild(node);
        } else {
            NodeList nl = node.getChildNodes();
            for (int i = 0; i < nl.getLength(); i++) {
                Node child = nl.item(i);
                removeComments(child);
            }
        }
    }
    
    private void removeNode(Document doc, String id) {
        Element element = doc.getElementById(id);
        element.getParentNode().removeChild(element);
    }
    
    private void scrubPage(Document doc) {
        // Remove various sections from the downloaded wiki mirror: edit-sections, javascript executable
        // sections, the table of contents, access counts for the page, etc.
        removeNodes(doc, "span", "editsection");
        removeNodes(doc, "div", "breadcrumbs");
        removeNodes(doc, "table", "toc");
        removeNodes(doc, "li", "f-lastmod");
        removeNodes(doc, "li", "f-viewcount");
        removeNodes(doc, "script", null);
        removeNodes(doc, "div", "printfooter");
        removeNode(doc, "siteSub");
        removeNodes(doc, "h1", "firstHeading");
        
        removeGeckoDomHeaders(doc);
        
        cleanAnchors(doc);
        removeEmptyElements(doc.getDocumentElement());
        removeComments(doc.getDocumentElement());
        
        inlineStyle(doc);
    }
    
    private String comment(String s) {
        StringBuilder sb = new StringBuilder(s.length()*2);
        String[] lines = s.split("\n");
        sb.append("/**\n");
        boolean wasEmpty = true;
        for (String line : lines) {
            line = line.trim();
            if (line.length() == 0) {
                if (wasEmpty) {
                    continue;
                }
                wasEmpty = true;
            } else {
                wasEmpty = false;
            }
            if (line.indexOf("*/") != -1) {
                line = line.replace("*/", "* /");
            }
            if (line.indexOf("/*") != -1) {
                line = line.replace("/*", "/ *");
            }
            sb.append("* ");
            sb.append(line);
            sb.append("\n");
        }
        sb.append("*/\n");
        
        return sb.toString();
    }

    @SuppressWarnings("empty-statement")
    private void analyzeClass(ClassDesc clz, Document doc, File classFile) {
        List<String> signatures = computeSignatures(doc, clz.name, true);
        if (signatures.size() > 0) {
            // Add in constructors
            clz.constructors.addAll(signatures);
        }

        // Read all the properties and methods; store the type for each,
        // and ensure that I have a 1-1 correspondence between the declared classdoc
        // and the available detail files! (I should given that each should link
        // to a document!! I shouldn't even need to go looking for these guys!)
        NodeList nl = doc.getElementsByTagName("a");
        for (int i = 0; i < nl.getLength(); i++) {
            Element child = (Element) nl.item(i);
            String name = child.getAttribute("name");
            Type type = null;
            EnumSet<BrowserCompat> compatibility = null;
            if ("Properties".equals(name)) {
                type = Type.PROPERTY;
            } else if ("Methods".equals(name) || "Methods_unrelated_to_HTML".equals(name)) {
                type = Type.METHOD;
            } else if ("Events".equals(name) || "Event_Handlers".equals(name)) {
                type = Type.EVENT;
            } else if ("Gecko_Methods".equals(name)) {
                compatibility = EnumSet.of(BrowserCompat.FF1, BrowserCompat.FF2, BrowserCompat.FF3);
                type = Type.METHOD;
            } else if (name.equals("Other_Properties_and_Methods")) {
                continue;
            } else if (name.endsWith("_methods") || name.endsWith("_Methods")) {
                if ("Generic_methods".equals(name)) { // From Array; these aren't method defs
                    continue;
                }
                if (!name.equals("Iteration_methods") && !name.equals("Mutator_methods") && !name.equals("Accessor_methods") &&
                        !name.equals("Static_methods") && !name.equals("Positioning_Methods") &&
                        !name.equals("Editing_Methods") &&
                        !name.equals("Other_Methods") &&
                        !name.equals("HTML_wrapper_methods")) {
                    if (name.indexOf("Example:") != -1) {
                        continue; // Skip things like Example:_Accessing_inherited_methods
                    }
                    assert false : "Unexpected " + name + "; file=" + classFile;
                }
                type = Type.METHOD;
            }
            if (type != null) {
                // One of the above
                // Find <h2>
                Node n = child.getNextSibling();
                while (n != null && n.getNodeType() != Node.ELEMENT_NODE) {
                    n = n.getNextSibling();
                }
                Element e = (Element)n;
                assert e != null && (e.getTagName().equals("h2") || e.getTagName().equals("h3") || e.getTagName().equals("h4")) : e + " in " + classFile;
                // Find <dl>
                n = n.getNextSibling();
                while (n != null && n.getNodeType() != Node.ELEMENT_NODE) {
                    n = n.getNextSibling();
                }
                e = (Element)n;
                assert e != null;
                
                if (e.getTagName().equals("a")) {
                    // Empty section
                    continue;
                }
                String versionString = null;
                if (e.getTagName().equals("div")) {
                    String cls = e.getAttribute("class");
                    if ("note".equals(cls)) {
                        // Compatibility -- record
                        String s = getText(e);
                        int version = s.indexOf("JavaScript 1.");
                        if (version != -1) {
                            versionString = s.substring(version+"JavaScript ".length());
                        }
                        n = n.getNextSibling();
                        while (n != null && n.getNodeType() != Node.ELEMENT_NODE) {
                            n = n.getNextSibling();
                        }
                        e = (Element)n;
                        assert e != null;
                    } else {
                        continue;
                    }
                }
                
                if ("HTML_wrapper_methods".equals(name)) {
                    // Skip the "Nonstandard" warning div
                    Node no = e.getNextSibling();
                    for (; no.getNodeType() != Node.ELEMENT_NODE || !"ul".equals(no.getNodeName());
                        no = no.getNextSibling()) {
                        ;
                    }
                    e = (Element)no;
                }
                
                assert (e.getTagName().equals("dl") || e.getTagName().equals("p") || e.getTagName().equals("ul") || e.getTagName().equals("table")) : e + " in section " + name + " in " + classFile;
                while (true) {
                    if (e.getTagName().equals("table")) {
                        processTable(clz, doc, classFile, e, type, name, versionString, compatibility);
                    } else {
                        handleAnchors(clz, doc, classFile, e, type, name, versionString, compatibility);
                    }

                    // Find inheritance <div>
//                    n = n.getNextSibling();
//                    while (n != null && n.getNodeType() != Node.ELEMENT_NODE) {
//                        n = n.getNextSibling();
//                    }
//                    e = (Element)n;
//                    if (e != null && e.getTagName().equals("div")) {
//                        as = e.getElementsByTagName("a");
//                        for (int j = 0; j < as.getLength(); j++) {
//                            Element a = (Element)as.item(j);
//                            if (a.getParentNode().getParentNode().getParentNode() != e) {
//                                // It's some <a> at a different nesting level than the one I'm after
//                                continue;
//                            }
//
//                            NodeList nla = e.getChildNodes();
//                            String href = a.getAttribute("href");
//                            assert nla.getLength() == 1 : href;
//                            String prototype = nla.item(0).getNodeValue();
//                            assert prototype.length() > 0;
//                            assert prototype.endsWith(".prototype");
//                            String superName = prototype.substring(0, prototype.length()-".prototype".length());
//                            assert clz.superClass == null : clz.superClass + "<=" + superName;
//                            clz.superClass = superName;
//                        }
//                    }

                    n = n.getNextSibling();
                    while (n != null && n.getNodeType() != Node.ELEMENT_NODE) {
                        n = n.getNextSibling();
                    }
                    e = (Element)n;
                    if (e == null || !(e.getTagName().equals("p") || e.getTagName().equals("dl") || e.getTagName().equals("ul"))) {
                        break;
                    }
                }
            }
        }
    }

    private void processTable(ClassDesc clz, Document doc, File classFile, Element table, Type type, String name, String version, EnumSet<BrowserCompat> compatibility) {
        NodeList nl = table.getElementsByTagName("tr");
        for (int i = 0; i < nl.getLength(); i++) {
            //boolean isFirst = isFirst(a);
            Element tr = (Element)nl.item(i);
            Element td = getFirstChild(tr);
            if (!td.getTagName().equals("td")) {
                continue;
            }

            if (type == Type.PROPERTY) {
                // Look up the property type in column 3 ("-" means unknown)
            } else if (type == Type.METHOD) {
                // Look up the return type in column 2 ("-" means unknown)
            }

            // TODO - check for the fourth column
            handleAnchors(clz, doc, classFile, td, type, name, version, compatibility);

        }
    }

    private Element getFirstChild(Element element) {
        NodeList nl = element.getChildNodes();
        for (int l = 0; l < nl.getLength(); l++) {
            Node childItem = nl.item(l);
            if (childItem.getNodeType() == Node.ELEMENT_NODE) {
                return (Element)childItem;
            }
        }

        return null;
    }

    private boolean isFirstChild(Element element) {
        boolean isFirst = false;
        NodeList nl = element.getParentNode().getChildNodes();
        for (int l = 0; l < nl.getLength(); l++) {
            Node childItem = nl.item(l);
            if (childItem.getNodeType() == Node.ELEMENT_NODE) {
                return childItem == element;
            }
        }

        return false;
    }

    private boolean handleAnchors(ClassDesc clz, Document doc, File classFile, Element e, Type type, String name, String version, EnumSet<BrowserCompat> compatibility) {
        Node n = e;
        NodeList as = e.getElementsByTagName("a");
        if (as.getLength() == 0 && e.getTagName().equals("p")) {
            // See Array for an example of this -- we have explanatory text
            n = n.getNextSibling();
            while (n != null && n.getNodeType() != Node.ELEMENT_NODE) {
                n = n.getNextSibling();
            }
            e = (Element)n;
            if (e == null) {
                return false;
            }
            as = e.getElementsByTagName("a");
            if (as.getLength() == 0 && (e.getTagName().equals("p") || (e.getTagName().equals("pre")))) {
                // See Array for an example of this -- we have explanatory text
                n = n.getNextSibling();
                while (n != null && n.getNodeType() != Node.ELEMENT_NODE) {
                    n = n.getNextSibling();
                }
                e = (Element)n;
                if (e == null) {
                    return false;
                }
                as = e.getElementsByTagName("a");
            }
            if (!(e.getTagName().equals("dl") || e.getTagName().equals("p"))) {
                return false;
            }
            assert e != null && (e.getTagName().equals("dl") || e.getTagName().equals("p")) : e + ";" + name;
        }

        //boolean grandChild = !e.getTagName().equals("dl");
        for (int j = 0; j < as.getLength(); j++) {
            Element a = (Element)as.item(j);

            // It must be the FIRST "a" in its parent. This happens because
            // sometimes there's a description next to each definition that
            // includes a hyperlink.
//                        boolean isFirst = false;
//                        NodeList pnl = a.getParentNode().getChildNodes();
//                        for (int l = 0; l < pnl.getLength(); l++) {
//                            Node childItem = pnl.item(l);
//                            if (childItem.getNodeType() == Node.ELEMENT_NODE &&
//                                    childItem.getNodeName().equals("a")) {
//                                if (childItem == a) {
//                                    isFirst = true;
//                                    break;
//                                } else {
//                                    break;
//                                }
//                            }
//                        }
            boolean isFirst = isFirstChild(a);
            if (!isFirst) {
                continue;
            }

            if (a.getParentNode().getNodeType() == Node.ELEMENT_NODE &&
                    "dd".equals(a.getParentNode().getNodeName())) {
                // Also eliminate links appearing within <dd> lists - these
                // are definitions
                continue;
            }
            if (a.getParentNode().getNodeType() == Node.ELEMENT_NODE &&
                    "code".equals(a.getParentNode().getNodeName()) &&
                    a.getParentNode().getParentNode().getNodeType() == Node.ELEMENT_NODE &&
                    "dd".equals(a.getParentNode().getParentNode().getNodeName())) {
                continue;
            }

//                    if (grandChild && a.getParentNode().getParentNode() != e) {
//                        // Skip - it's some other anchor
//                        continue;
//                    } else if (!grandChild && a.getParentNode() != e) {
//                        // Skip - it's some other anchor
//                        continue;
//                    }
            NodeList nla = a.getChildNodes();
            String href = a.getAttribute("href");
//                    if (nla.getLength() != 1) {
//                        StringBuilder sb = new StringBuilder();
//                        for (int k = 0; k < nla.getLength(); k++) {
//                            sb.append("Element " + k + " = " + nla.item(k));
//                        }
//                        assert false : sb;
//                    }
            assert nla.getLength() == 1 : href +"; length=" + nla.getLength() + " in file " + classFile;
            boolean mustHaveFile = true;
            if (href.startsWith("http")) {
                if (classFile.getName().equals("Core_JavaScript_1.5_Reference:Global_Objects:Object") &&
                        href.indexOf("index.php?title") != -1) {
                    // Known exception
                    continue;
                }
                if (href.startsWith("http://www.w3.org")) {
                    continue;
                }
                if ("http://developer.mozilla.org/en/docs/Online_and_offline_events".equals(href)) {
                    continue; // From the document class
                }
                if ("http://developer.mozilla.org/en/docs/Components_object".equals(href)) {
                    // window.Components -- XPCOM entry point
                    continue;
                }
                if (href.startsWith("http://developer.mozilla.org/en/docs/Firefox")) {
                    continue;
                }
                if (href.startsWith("http://developer.mozilla.org/en/docs/nsIURI")) {
                    continue;
                }
                boolean okay = false;
                if (href.startsWith("http://developer.mozilla.org/en/docs/Rich-Text_Editing_in_Mozilla")) {
                    mustHaveFile = false;
                    okay = true;
                }
                if (href.indexOf("&action=edit") != -1) {
                    // Probably a pointer to a property whose documentation has not yet been written (edit links in wiki)
                    mustHaveFile = false;
                    okay = true;
                }

                //: href + " in " + classFile;
                assert okay : href + " in " + classFile;
            }
            if (href.startsWith("/en/docs/")) {
                href = href.substring("/en/docs/".length());
            } else if (href.startsWith("/")) {
                assert false : href;
            }
            String prop = nla.item(0).getNodeValue();
            prop = prop.trim();
            if (prop.startsWith("arguments.")) {
                // The documentation for "arguments" for some reason includes
                // the prefix in the links -- remove 
                prop = prop.substring("arguments.".length());
            } else if (prop.startsWith(clz.name.toLowerCase()+".")) {
                // The Table DOM documentation for example starts with "table." on every attribute
                prop = prop.substring(clz.name.length()+1);
            } else if (prop.startsWith("row.")) {
                // The TableRow class uses "row." instead of "tableRow."
                prop = prop.substring("row.".length());
            }
            if (prop.startsWith("screen.")) {
                // In the window class, indirect properties are documented: window.screen.availTop etc
                prop = prop.substring("screen.".length());
            }
            assert prop.indexOf('.') == -1 : prop + " from " + classFile;
            AttributeDesc attribute = clz.getAttribute(prop);
            if (attribute == null) {
                File file = new File(classFile.getParentFile(), href);
                if (href.startsWith("DOM:Storage#")) {
                    // Known exception
                    file = null;
                } else {
                    assert !mustHaveFile || file.exists() : file;
                    if (!mustHaveFile) {
                        file = null;
                    }
                }
                attribute = new AttributeDesc(prop, file, clz);
                clz.addAttribute(attribute);
            } else {
                assert attribute.files != null : attribute + " in " + classFile;
            }
            attribute.compatibility = compatibility;
            attribute.version = version;
            attribute.type = type;

            // TODO - there is a brief one-line summary for each <dt>
            // as a <dd>. These can be useful for tooltips etc since they don't
            // contain code snippets etc. as the Summary sections do.
        }

        // The first <div> after the <dl> contains the immediate parent
        // or superclass for the prototype chain
        return true;
    }
    
    private void readClassDocs(Main.ClassDesc clz) throws IOException {
//        if (SKIP_DOCS) {
//            clz.comment = "";
//            return;
//        }        
        if (clz.descFiles == null) {
            clz.comment = "";
            return;
        }
        assert clz.descFiles != null;

        // Process all the files in case they overlap in which aspects they describe
        assert clz.descFiles.size() > 0 : clz;
        Document doc = null;
        for (File file : clz.descFiles) {
            doc = parse(file);
            analyzeClass(clz, doc, file);
        }
        
    if (SKIP_DOCS) {
        clz.comment = "";
        return;
    }
        // XXX Uhm, which ONE do we choose for documentation?
        scrubPage(doc);
        List<Element> articles = findElementsByClass(doc, "div", "article");
        assert articles.size() == 1;
        Element div = articles.get(0);
        moveCopyright(doc, div);
        removeClasses(doc);
        String s = serialize(div);
        String comment = comment(s);
        clz.comment = comment;
    }
    
    private void readAttributeDocs(AttributeDesc desc, ClassDesc forClass) throws IOException {
        // TODO - handle the fact that there may be multiple files here?
        if (SKIP_DOCS || desc.files.size() == 0) {
            if (desc.comment == null) {
                desc.comment = "";
            }
            if (desc.signatures == null || desc.signatures.size() == 0) {
                String sig = desc.type == Type.FUNCTION ? desc.name+"()" : desc.name;
                desc.signatures = Collections.singletonList(sig);
            }
            if (desc.files.size() == 0) {
                System.err.println("WARNING: There was no description file for " + desc);
            }
            return;
        }        
        File file = desc.files.get(0);
        Document doc = parse(file);
        computeSignatures(doc, desc, forClass);
        scrubPage(doc);
        List<Element> articles = findElementsByClass(doc, "div", "article");
        assert articles.size() == 1;
        Element div = articles.get(0);
        moveCopyright(doc, div);
        removeClasses(doc);
        String s = serialize(div);
        String comment = comment(s);
        
        if (desc.comment == null) {
            // For subclasses ensure that we copy applicable documentation, if any
            String attrSig = desc.signatures != null && desc.signatures.size() > 0 ? desc.signatures.get(0).trim() : null;
            if (forClass.superClass != null && attrSig != null) {
                ClassDesc superClass = classMap.get(forClass.superClass);
                for (ClassDesc c = superClass; c != null; c = classMap.get(c.superClass)) {
                    AttributeDesc superAttr = c.getAttribute(desc.name);
                    if (superAttr != null && superAttr.signatures != null && superAttr.signatures.size() > 0) {
                        // Make sure the signatures match
                        boolean match = false;
                        for (String sig : superAttr.signatures) {
                            if (sig.indexOf(attrSig) != -1) {
                                match = true;
                                break;
                            }
                        }
                        if (match && c.comment != null) {
                            desc.comment = c.comment;
                            break;
                        }
                    }
                    
                    if (c.superClass == null) {
                        break;
                    }
                }
            }
        }
        
        if (desc.comment != null) {
            // Add in the old comment first:
            assert comment.endsWith("*/\n") : comment;
            String begin = comment.substring(0, comment.length()-3);
            assert desc.comment.startsWith("/**") : desc.comment;
            String end = desc.comment.substring(3);
            // Don't add a "Original documentation" label on stuff that is just a type annotation.
            String e = end.trim();
            if (e.endsWith("\n*/")) {
                e = e.substring(0, e.length()-3);
            } else if (e.endsWith("\n */")) {
                e = e.substring(0, e.length()-4);
            }
            if (e.startsWith("/**\n")) {
                e = e.substring(4);
            }
            boolean isOnlyType = e.indexOf("* @type") != -1 && e.indexOf("\n") == -1;
            if (isOnlyType) {
                comment = begin.trim() + "\n " + end.trim() + "\n";
            } else {
                comment = begin.trim() + "\n * <p>Original " + ((desc.origin != Origin.CORE) ? "W3C " : "") +  "documentation:</p>\n " + end.trim() + "\n";
            }
        }
        desc.comment = comment;
    }
    
    /** Compute the text under the given element */
    private String getText(Element element) {
        StringBuilder sb = new StringBuilder();
        addNodeText(element, sb);
        return sb.toString();
    }
    
    private void addNodeText(Node n, StringBuilder sb) {
        // This doesn't pick up text in comments, which is usually the correct thing to do,
        // but for things like JavaScript code in HTML pages, it's sometimes embedded in
        // comments for backwards compatibility reasons
        if (n.getNodeType() == Node.TEXT_NODE) {
            String v = n.getNodeValue();
            sb.append(v);
        } else if (n.hasChildNodes()) {
            NodeList nl = n.getChildNodes();
            for (int i = 0; i < nl.getLength(); i++) {
                addNodeText(nl.item(i), sb);
            }
        }
    }

    /** Compute the text under the given element */
    private String getFlattenedText(Element element) {
        StringBuilder sb = new StringBuilder();
        addFlattenedNodeText(element, sb);
        return sb.toString();
    }
    
    private void addFlattenedNodeText(Node n, StringBuilder sb) {
        // This doesn't pick up text in comments, which is usually the correct thing to do,
        // but for things like JavaScript code in HTML pages, it's sometimes embedded in
        // comments for backwards compatibility reasons
        if (n.getNodeType() == Node.TEXT_NODE) {
            String v = n.getNodeValue();
            v = v.trim();
            if (v.length() > 0) {
                boolean lastWasSpace = sb.length() == 0 || sb.charAt(sb.length()-1) == '\n';
                if (!lastWasSpace) {
                    sb.append(' ');
                }
                for (int i = 0; i < v.length(); i++) {
                    char c = v.charAt(i);
                    boolean space = Character.isWhitespace(c);
                    if (space && !lastWasSpace) {
                        sb.append(' ');
                    } else if (!space) {
                        sb.append(c);
                    }
                    lastWasSpace = space;
                }
            }
        } else if (n.getNodeType() == Node.ELEMENT_NODE) {
            Element e = (Element)n;
            if (e.getTagName().equals("br")) {
                sb.append("\n");
            }
        } 
        
        if (n.hasChildNodes()) {
            NodeList nl = n.getChildNodes();
            for (int i = 0; i < nl.getLength(); i++) {
                addFlattenedNodeText(nl.item(i), sb);
            }
        }
    }
    
    private Element findHeader(Document doc, String header) {
        // Locate parameter list
        List<Element> headlines = findElementsByClass(doc, "span", "mw-headline");
        for (Element element : headlines) {
            String s = getText(element).trim();
            if (header.equals(s)) {
                return element;
            }
        }
        
        return null;
    }
    
    private String getCleanString(String s) {
        boolean replace = false;
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < s.length(); i++) {
            int c = s.charAt(i);
            // Convert nonbreaking space to space
            if (c == 160) { 
                c = 32;
                replace = true;
            }
            if (c <= 127 && c >= 32) {
                char ch = (char)c;
                sb.append(ch);
            } else {
                replace = true;
            }
        }

        if (replace) {
            s = sb.toString();
        }
        
        return s;
    }

    // Compute all the permutations of signatures from a syntax description such
    // as 
    //  var newString = str.replace(regexp/substr, newSubStr/function[, flags]);
    // which would be
    //   replace(regexp, newSubStr)
    //   replace(regexp, newSubStr, flags)
    //   replace(regexp, function)
    //   replace(regexp, function, flags)
    //   replace(substr, newSubStr)
    //   replace(substr, newSubStr, flags)
    //   replace(substr, function)
    //   replace(regexp, function, flags)
    private List<String> computeSignatures(Document doc, String name, boolean findConstructors) {
        List<String> signatures = new ArrayList<String>();

        Element pre = null;
        Element header = findHeader(doc, "Syntax");
        if (header == null) {
            header = findHeader(doc, "Usage");
        }
        if (header == null && findConstructors) {
            header = findHeader(doc, "Created by");
            if (header != null) {
                // Look for the next pre, not interrupted by an a
                //Node n = header.getNextSibling();
                assert header.getParentNode().getNodeName().equals("h2") : header.getParentNode().getNodeName();
                Node n = header.getParentNode().getNextSibling();
                for (; n != null; n = n.getNextSibling()) {
                    if (n.getNodeType() != Node.ELEMENT_NODE) {
                        continue;
                    }
                    Element e = (Element)n;
                    if (e.getTagName().equals("pre")) {
                        pre = e;
                        break;
                    } else if (e.getTagName().equals("a")) {
                        return Collections.emptyList();
//                        assert false : "No comment found for " + name;
//                        header = null;
//                        break;
                    }
                }
            }
        } else if (header == null) {
            // Probably a property
            //signatures.add(name+"()");
            if (!findConstructors) {
                signatures.add(name);
            }
            return signatures;
        } else {
            assert header != null;
            // I'm looking for the <pre> that is a sibling to the <h2> containing the header line <span>
            Node next = header.getParentNode().getNextSibling();
            if (next.getNodeType() == Node.TEXT_NODE) {
                next = next.getNextSibling();
            }
            assert next.getNodeType() == Node.ELEMENT_NODE;
            pre = (Element)next;
            assert pre.getTagName().equals("pre") || pre.getTagName().equals("p") : pre.getTagName();
        }
        
        if (pre == null) {
            if (!findConstructors) {
                signatures.add(name);
            }
            return signatures;
        }

        String allText = getText(pre).trim();
        String[] allSigs = findConstructors ? allText.split("\n") : new String[] { allText };

        for (String signature : allSigs) {

            // Remove special chars, such as line 3782 in stub_windows.js where there are some bogus
            // characters in the name
            signature = getCleanString(signature);

            // Strip out stuff like "var newString = str." in the following
            // var newString = str.replace(regexp/substr, newSubStr/function[, flags]);
            int start = signature.indexOf(name);
            if (start == -1) {
                start = signature.toLowerCase().indexOf(name.toLowerCase());
            }
            if (start == -1 && findConstructors) {
                System.err.println("Warning: Constructor search for class " + name + " didn't find a good signature. TODO: Look for multiple pre elements.");
                return signatures;
            }
            assert start != -1 : "No " + name + " in " + signature;

            signature = signature.substring(start);

    //        int openParen = signature.indexOf('(');
    //        assert openParen != -1 : signature + " contains no (";
    //
    //        int closeParen = signature.indexOf(')', openParen);
    //        assert closeParen != -1 : signature + " contains no )";
    //        
    //        String argStr = signature.substring(openParen+1, closeParen);
    //        
    //        String[] args = argStr.split(",");

            if (signature.startsWith("new ")) {
                // For constructors
                signature = signature.substring(4);
            }

            // TODO: FIXME
            // For now, instead of permutations, turn "/" into "_or_", e.g.
            // regexp/substr becomes "regexp_or_substr
            signature = signature.replace("/", "_or_");

            // TODO: FIXME
            // For now, treat optional args as mandatory
            signature = signature.replace("[", "");
            signature = signature.replace("]", "");

            // TODO: Handle multiple arguments
            signature = signature.replace("...,", "");
            signature = signature.replace("...", "");

            // Remove invalid chars in parameter names, like "x-coord"
            signature = signature.replace("-", "_");

            // Can't have quoted strings in the args, like the arg example to window.updateCommands
            signature = signature.replace("\"", "");

            // Remove trailing commas
            signature = signature.replace(",)", ")");
            signature = signature.replace(", )", ")");

            // Cleanup whitespace resulting from the above replacemenets
            signature = signature.replace("   ", "");
            signature = signature.replace("  ", "");
            signature = signature.replace(";", "");

            if (!signatures.contains(signature)) {
                signatures.add(signature);
            }
        }
        
        return signatures;
    }
    
    private void computeSignatures(Document doc, AttributeDesc desc, ClassDesc forClass) {
        List<String> signatures = computeSignatures(doc, desc.name, false);
        desc.signatures = signatures;
    }
    
    private void readDocumentation(Collection<ClassDesc> classes) throws IOException {
        for (ClassDesc clz : classes) {
            System.out.println("Processing file for class " + clz.name);
            readClassDocs(clz);
            
            for (AttributeDesc attr : clz.attributes) {
                System.out.println("    Processing attribute " + attr);
                readAttributeDocs(attr, clz);
            }
        }
    }
    
    private void process(File input, File output, File w3cInput) throws IOException {
        if (w3cInput != null) {
            findW3CClasses(w3cInput);
        }
        findMDCClasses(input);
        addCanvasClasses();
        addIteratorClasses();
        assignMDCComments();
        
        Collection<ClassDesc> classes = classMap.values();
        
        readDocumentation(classes);
        fixSpecialCases(classes);
        //printClasses(classes);
        
        generateCode(classes, output);
        
        emitBrowserMaps(classes);
    }

    private void addCanvasClasses() {
        // Based on http://www.whatwg.org/specs/web-apps/current-work/#the-canvas

        List<ClassDesc> canvasClasses = new ArrayList<ClassDesc>();

        ClassDesc clz = new ClassDesc("CanvasRenderingContext2D", Origin.CANVAS);
        canvasClasses.add(clz);
        classMap.put(clz.name, clz);

        // Compatibility: http://developer.mozilla.org/en/docs/HTML:Canvas
        //  "Mozilla applications gained support for <canvas> starting with
        // Gecko 1.8 (i.e. Firefox 1.5). The element was originally introduced
        // by Apple for the OS X Dashboard and Safari. Internet Explorer does
        // not natively support <canvas>, but a page can effectively add support
        // for it by including a script from Google's Explorer Canvas project.
        // Opera 9 also supports <canvas>."
        EnumSet<BrowserCompat> canvasCompat = EnumSet.of(BrowserCompat.FF3, BrowserCompat.FF2, BrowserCompat.OPERA,
                BrowserCompat.SAFARI2, BrowserCompat.SAFARI3, BrowserCompat.KONQ);
        // Actually FireFox 1.5 supports it too - I don't have an enum for that!

        AttributeDesc a = new AttributeDesc("canvas", null, clz);
        a.type = Type.PROPERTY;
        a.comment = "/** @type HTMLCanvasElement */";
        clz.addAttribute(a);

        a = new AttributeDesc("save", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** push state on state stack\n* @type void\n*/";
        a.signatures = Collections.singletonList("save()");
        clz.addAttribute(a);

        a = new AttributeDesc("load", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** pop state stack and restore state\n* @type void\n*/";
        a.signatures = Collections.singletonList("load()");
        clz.addAttribute(a);

        // transformations (default transform is the identity matrix)

        a = new AttributeDesc("scale", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** The scale(x, y) method will add the scaling transformation described by the arguments to the transformation matrix. The x argument represents the scale factor in the horizontal direction and the y argument represents the scale factor in the vertical direction. The factors are multiples.\n  @param {Number} x\n@param {Number} y\n* @type void\n*/";
        a.signatures = Collections.singletonList("scale(x,y)");
        clz.addAttribute(a);

        a = new AttributeDesc("rotate", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** The rotate(angle) method will add the rotation transformation described by the argument to the transformation matrix. The angle argument represents a clockwise rotation angle expressed in radians. If the angle argument is infinite, the method call must be ignored.\n@param {Number} angle\n* @type void\n*/";
        a.signatures = Collections.singletonList("rotate(x,y)");
        clz.addAttribute(a);

        a = new AttributeDesc("translate", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** The translate(x, y) method will add the translation transformation described by the arguments to the transformation matrix. The x argument represents the translation distance in the horizontal direction and the y argument represents the translation distance in the vertical direction. The arguments are in coordinate space units.\n@param {Number} x\n@param {Number} y\n* @type void\n*/";
        a.signatures = Collections.singletonList("translate(x,y)");
        clz.addAttribute(a);

        a = new AttributeDesc("transform", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** The transform(m11, m12, m21, m22, dx, dy) method will multiply the current transformation matrix with the given matrix.\n@param {Number} m11\n @param {Number} m12\n @param {Number} m21\n @param {Number} m22\n@param {Number} dx\n@param {Number} dy\n* @type void\n*/";
        a.signatures = Collections.singletonList("transform(m11, m12, m21, m22, dx, dy)");
        clz.addAttribute(a);

        a = new AttributeDesc("setTransform", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** The setTransform(m11, m12, m21, m22, dx, dy) method will reset the current transform to the identity matrix, and then invoke the transform(m11, m12, m21, m22, dx, dy) method with the same arguments.\n@param {Number} m11\n @param {Number} m12\n @param {Number} m21\n @param {Number} m22\n@param {Number} dx\n@param {Number} dy\n* @type void\n*/";
        a.signatures = Collections.singletonList("setTransform(m11, m12, m21, m22, dx, dy)");
        clz.addAttribute(a);

        // Compositing

        a = new AttributeDesc("globalAlpha", null, clz);
        a.type = Type.PROPERTY;
        a.comment = "/** The globalAlpha attribute gives an alpha value that is applied to shapes and images before they are composited onto the canvas. The value must be in the range from 0.0 (fully transparent) to 1.0 (no additional transparency). If an attempt is made to set the attribute to a value outside this range, the attribute must retain its previous value. When the context is created, the globalAlpha attribute must initially have the value 1.0.\n@type Number\n@default 1.0 */";
        clz.addAttribute(a);

        a = new AttributeDesc("globalCompositeOperation", null, clz);
        a.type = Type.PROPERTY;
        a.comment = "/** The globalCompositeOperation  attribute sets how shapes and images are drawn onto the existing bitmap, once they have had globalAlpha and the current transformation matrix applied. It must be set to a value from the following list. In the descriptions below, the source image, A, is the shape or image being rendered, and the destination image, B, is the current state of the bitmap.\n@type String\n@default source-over */";
        clz.addAttribute(a);

        // Colors and styles

        a = new AttributeDesc("strokeStyle", null, clz);
        a.type = Type.PROPERTY;
        a.comment = "/** The strokeStyle attribute represents the color or style to use for the lines around shapes\n@default black */";
        clz.addAttribute(a);

        a = new AttributeDesc("fillStyle", null, clz);
        a.type = Type.PROPERTY;
        a.comment = "/** The fillStyle attribute represents the color or style to use inside the shapes.\n@default black */";
        clz.addAttribute(a);

        a = new AttributeDesc("createLinearGradient", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** The createLinearGradient(x0, y0, x1, y1) method takes four arguments that, after being subjected to the current transformation matrix, represent the start point (x0, y0) and end point (x1, y1) of the gradient. If any of the arguments to createLinearGradient() are infinite or NaN, the method will raise an INDEX_SIZE_ERR  exception. Otherwise, the method will return a linear CanvasGradient initialised with the specified line.\n@param {Number} x0\n @param {Number} y0\n @param {Number} x1\n @param {Number} y1\n* @type CanvasGradient\n*/";
        a.signatures = Collections.singletonList("createLinearGradient(x0, y0, x1, y1)");
        clz.addAttribute(a);

        a = new AttributeDesc("createRadialGradient", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** The createRadialGradient(x0, y0, r0, x1, y1, r1) method takes six arguments, the first three representing the start circle with origin (x0, y0) and radius r0, and the last three representing the end circle with origin (x1, y1) and radius r1. The values are in coordinate space units. If either of r0 or r1 are negative, or if any of the arguments are infinite or NaN, an INDEX_SIZE_ERR exception must be raised. Otherwise, the method will return a radial CanvasGradient initialised with the two specified circles, after transforming them according to the current transformation matrix.\n@param {Number} x0\n @param {Number} y0\n @param {Number} x1\n @param {Number} y1\n@param {Number} r0\n@param {Number} r1\n* @type CanvasGradient\n*/";
        a.signatures = Collections.singletonList("createRadialGradient(x0, y0, r0, x1, y1, r1)");
        clz.addAttribute(a);

        a = new AttributeDesc("createPattern", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** Patterns are represented by objects implementing the opaque CanvasPattern interface. To create objects of this type, the createPattern(image, repetition) method is used. The first argument gives the image to use as the pattern (either an HTMLImageElement or an HTMLCanvasElement). Modifying this image after calling the createPattern() method will not affect the pattern. The second argument must be a string with one of the following values: repeat, repeat-x, repeat-y, no-repeat. If the empty string or null is specified, repeat must be assumed. If an unrecognised value is given, then the user agent must raise a SYNTAX_ERR exception. User agents must recognise the four values described above exactly (e.g. they must not do case folding). The method will return a CanvasPattern object suitably initialised. \n@param {HTMLImageElement|HTMLCanvasElement} image\n @param {String} repetition\n* @type CanvasPattern\n*/";
        a.signatures = Collections.singletonList("createPattern(image, repetition)");
        clz.addAttribute(a);

        // line caps/joins
        a = new AttributeDesc("lineWidth", null, clz);
        a.type = Type.PROPERTY;
        a.comment = "/** The lineWidth attribute gives the width of lines, in coordinate space units. On setting, zero, negative, infinite, and NaN values must be ignored, leaving the value unchanged.\n @type Number\n@default 1 */";
        clz.addAttribute(a);

        a = new AttributeDesc("lineCap", null, clz);
        a.type = Type.PROPERTY;
        a.comment = "/** The lineCap attribute defines the type of endings that UAs will place on the end of lines. The three valid values are butt, round, and square. The butt value means that the end of each line is a flat edge perpendicular to the direction of the line. The round value means that a semi-circle with the diameter equal to the width of the line must then be added on to the end of the line. The square value means that a rectangle with the length of the line width and the width of half the line width, placed flat against the edge perpendicular to the direction of the line, must be added at the end of each line. On setting, any other value than the literal strings butt, round, and square must be ignored, leaving the value unchanged.\n\"butt\", \"round\", \"square\"\n@type String\n@default \"butt\" */";
        clz.addAttribute(a);

        a = new AttributeDesc("lineJoin", null, clz);
        a.type = Type.PROPERTY;
        a.comment = "/** The lineJoin attribute defines the type of corners that UAs will place where two lines meet. The three valid values are bevel, round, and miter.\n\"round\", \"bevel\", \"miter\"\n @type String\n@default \"miter\" */";
        clz.addAttribute(a);

        a = new AttributeDesc("miterLimit", null, clz);
        a.type = Type.PROPERTY;
        a.comment = "/** @type Number\n@default 10 */";
        clz.addAttribute(a);


        // shadows
        a = new AttributeDesc("shadowOffsetX", null, clz);
        a.type = Type.PROPERTY;
        a.comment = "/** The shadowOffsetX and shadowOffsetY  attributes specify the distance that the shadow will be offset in the positive horizontal and positive vertical distance respectively. Their values are in coordinate space units. They are not affected by the current transformation matrix.\n@type Number\n@default 0 */";
        clz.addAttribute(a);

        a = new AttributeDesc("shadowOffsetY", null, clz);
        a.type = Type.PROPERTY;
        a.comment = "/** The shadowOffsetX and shadowOffsetY  attributes specify the distance that the shadow will be offset in the positive horizontal and positive vertical distance respectively. Their values are in coordinate space units. They are not affected by the current transformation matrix.\n@type Number\n@default 0 */";
        clz.addAttribute(a);

        a = new AttributeDesc("shadowBlur", null, clz);
        a.type = Type.PROPERTY;
        a.comment = "/** The shadowBlur attribute specifies the size of the blurring effect. (The units do not map to coordinate space units, and are not affected by the current transformation matrix.)\n@type Number\n@default 0 */";
        clz.addAttribute(a);

        a = new AttributeDesc("shadowColor", null, clz);
        a.type = Type.PROPERTY;
        a.comment = "/** The shadowColor attribute sets the color of the shadow.\n@type String\n@default transparent black */";
        clz.addAttribute(a);

        // rects
        a = new AttributeDesc("clearRect", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** The clearRect(x, y, w, h) method will clear the pixels in the specified rectangle that also intersect the current clipping region to a fully transparent black, erasing any previous image. If either height or width are zero, this method has no effect.\n@param {Number} x\n @param {Number} y\n @param {Number} w\n @param {Number} h\n* @type void\n*/";
        a.signatures = Collections.singletonList("clearRect(x, y, w, h)");
        clz.addAttribute(a);

        a = new AttributeDesc("fillRect", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** The fillRect(x, y, w, h) method will paint the specified rectangular area using the fillStyle. If either height or width are zero, this method has no effect.\n@param {Number} x\n @param {Number} y\n @param {Number} w\n @param {Number} h\n* @type void\n*/";
        a.signatures = Collections.singletonList("fillRect(x,y,w,h)");
        clz.addAttribute(a);

        a = new AttributeDesc("strokeRect", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** The strokeRect(x, y, w, h) method will stroke the specified rectangle's path using the strokeStyle, lineWidth, lineJoin, and (if appropriate) miterLimit attributes. If both height and width are zero, this method has no effect, since there is no path to stroke (it's a point). If only one of the two is zero, then the method will draw a line instead (the path for the outline is just a straight line along the non-zero dimension).\n@param {Number} x\n @param {Number} y\n @param {Number} w\n @param {Number} h\n* @type void\n*/";
        a.signatures = Collections.singletonList("strokeRect(x,y,w,h)");
        clz.addAttribute(a);


        // path API
        a = new AttributeDesc("beginPath", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** The beginPath() method will empty the list of subpaths so that the context once again has zero subpaths.\n@type void\n*/";
        a.signatures = Collections.singletonList("beginPath()");
        clz.addAttribute(a);

        a = new AttributeDesc("closePath", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** The closePath() method will do nothing if the context has no subpaths. Otherwise, it must mark the last subpath as closed, create a new subpath whose first point is the same as the previous subpath's first point, and finally add this new subpath to the path. (If the last subpath had more than one point in its list of points, then this is equivalent to adding a straight line connecting the last point back to the first point, thus \"closing\" the shape, and then repeating the last moveTo() call.)\n@type void\n*/";
        a.signatures = Collections.singletonList("closePath()");
        clz.addAttribute(a);

        a = new AttributeDesc("moveTo", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** The moveTo(x, y) method will create a new subpath with the specified point as its first (and only) point.\n@param {Number} x\n @param {Number} y\n* @type void\n*/";
        a.signatures = Collections.singletonList("moveTo(x, y)");
        clz.addAttribute(a);

        a = new AttributeDesc("lineTo", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** The lineTo(x, y) method will do nothing if the context has no subpaths. Otherwise, it must connect the last point in the subpath to the given point (x, y) using a straight line, and must then add the given point (x, y) to the subpath.\n@param {Number} x\n @param {Number} y\n* @type void\n*/";
        a.signatures = Collections.singletonList("lineTo(x, y)");
        clz.addAttribute(a);

        a = new AttributeDesc("quadraticCurveTo", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** The quadraticCurveTo(cpx, cpy, x, y) method will do nothing if the context has no subpaths. Otherwise it must connect the last point in the subpath to the given point (x, y) using a quadratic Bézier curve with control point (cpx, cpy), and must then add the given point (x, y) to the subpath.\n@param {Number} cpx\n @param {Number} cpy\n@param {Number} x\n @param {Number} y\n* @type void\n*/";
        a.signatures = Collections.singletonList("quadraticCurveTo(cpx, cpy, x, y)");
        clz.addAttribute(a);

        a = new AttributeDesc("bezierCurveTo", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** The bezierCurveTo(cp1x, cp1y, cp2x, cp2y, x, y) method will do nothing if the context has no subpaths. Otherwise, it must connect the last point in the subpath to the given point (x, y) using a cubic Bézier curve with control points (cp1x, cp1y) and (cp2x, cp2y). Then, it must add the point (x, y) to the subpath.\n@param {Number} cp1x\n @param {Number} cp1y\n@param {Number} cp2x\n @param {Number} cp2y\n@param {Number} x\n @param {Number} y\n* @type void\n*/";
        a.signatures = Collections.singletonList("bezierCurveTo(cp1x, cp1y, cp2x, cp2y, x, y)");
        clz.addAttribute(a);

        a = new AttributeDesc("arcTo", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** The arcTo(x1, y1, x2, y2, radius) method will do nothing if the context has no subpaths. If the context does have a subpath, then the behavior depends on the arguments and the last point in the subpath.\n@param {Number} x1\n @param {Number} y1\n@param {Number} x2\n @param {Number} y2\n@param {Number} radius\n* @type void\n*/";
        a.signatures = Collections.singletonList("arcTo(x1,y1,x2,y2,radius)");
        clz.addAttribute(a);

        a = new AttributeDesc("rect", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** The rect(x, y, w, h) method will create a new subpath containing just the four points (x, y), (x+w, y), (x+w, y+h), (x, y+h), with those four points connected by straight lines, and must then mark the subpath as closed. It must then create a new subpath with the point (x, y) as the only point in the subpath.\n@param {Number} x\n @param {Number} y\n @param {Number} w\n @param {Number} h\n* @type void\n*/";
        a.signatures = Collections.singletonList("rect(x,y,w,h)");
        clz.addAttribute(a);

        a = new AttributeDesc("arc", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** The arc(x, y, radius, startAngle, endAngle, anticlockwise) method draws an arc. If the context has any subpaths, then the method will add a straight line from the last point in the subpath to the start point of the arc. In any case, it must draw the arc between the start point of the arc and the end point of the arc, and add the start and end points of the arc to the subpath.\n@param {Number} x\n @param {Number} y\n@param {Number} radius\n @param {Number} startAngle\n@param {Number} endAngle\n@param {Boolean} anticlockwise\n* @type void\n*/";
        a.signatures = Collections.singletonList("arc(x,y,radius,startAngle,endAngle,anticlockwise)");
        clz.addAttribute(a);

        a = new AttributeDesc("fill", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** The fill()  method will fill all the subpaths of the current path, using fillStyle, and using the non-zero winding number rule. Open subpaths must be implicitly closed when being filled (without affecting the actual subpaths).\n@type void\n*/";
        a.signatures = Collections.singletonList("fill()");
        clz.addAttribute(a);

        a = new AttributeDesc("stroke", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** The stroke() method will calculate the strokes of all the subpaths of the current path, using the lineWidth, lineCap, lineJoin, and (if appropriate) miterLimit attributes, and then fill the combined stroke area using the strokeStyle, attribute.\n@type void\n*/";
        a.signatures = Collections.singletonList("stroke()");
        clz.addAttribute(a);

        a = new AttributeDesc("clip", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** The clip()  method will create a new clipping region by calculating the intersection of the current clipping region and the area described by the current path, using the non-zero winding number rule. Open subpaths must be implicitly closed when computing the clipping region, without affecting the actual subpaths. The new clipping region replaces the current clipping region.\n@type void\n*/";
        a.signatures = Collections.singletonList("clip()");
        clz.addAttribute(a);

        a = new AttributeDesc("isPointInPath", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** The isPointInPath(x, y) method will return true if the point given by the x and y coordinates passed to the method, when treated as coordinates in the canvas coordinate space unaffected by the current transformation, is inside the current path; and must return false otherwise. Points on the path itself are considered to be inside the path. If either of the arguments is infinite or NaN, then the method will return false.\n@param {Number} x\n@param {Number} y\n @type Boolean\n*/";
        a.signatures = Collections.singletonList("isPointInPath(x,y)");
        clz.addAttribute(a);


         // drawing images
        a = new AttributeDesc("drawImage", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** To draw images onto the canvas, the drawImage method can be used.\n @param {HTMLImageElement|HTMLCanvasElement} image\n @param {Number} dx\n@param {Number} dy\n* @type void\n*/";
        a.signatures = Collections.singletonList("drawImage(image,dx,dy)");
        clz.addAttribute(a);

        a = new AttributeDesc("drawImage", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** To draw images onto the canvas, the drawImage method can be used.\nThe destination rectangle is the rectangle whose corners are the four points (dx, dy), (dx+dw, dy), (dx+dw, dy+dh), (dx, dy+dh).\n@param {HTMLImageElement|HTMLCanvasElement} image\n @param {Number} dx\n@param {Number} dy\n @param {Number} dw\n@param {Number} dh\n* @type void\n*/";
        a.signatures = Collections.singletonList("drawImage(image,dx,dy,dw,dh)");
        clz.addAttribute(a);

        a = new AttributeDesc("drawImage", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** To draw images onto the canvas, the drawImage method can be used.\nThe source rectangle is the rectangle whose corners are the four points (sx, sy), (sx+sw, sy), (sx+sw, sy+sh), (sx, sy+sh).\nThe destination rectangle is the rectangle whose corners are the four points (dx, dy), (dx+dw, dy), (dx+dw, dy+dh), (dx, dy+dh).\n@param {HTMLImageElement|HTMLCanvasElement} image\n @param {Number} sx\n@param {Number} sy\n @param {Number} sw\n@param {Number} sh\n@param {Number} dx\n@param {Number} dy\n @param {Number} dw\n@param {Number} dh\n* @type void\n*/";
        a.signatures = Collections.singletonList("drawImage(image,sx,sy,sw,sh,dx,dy,dw,dh)");
        clz.addAttribute(a);


         // pixel manipulation
        a = new AttributeDesc("createImageData", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** The createImageData(sw, sh) method will return an ImageData object representing a rectangle with a width in CSS pixels equal to the absolute magnitude of sw and a height in CSS pixels equal to the absolute magnitude of sh, filled with transparent black.\n@param {Number} sw\n@param {Number} sh\n* @type ImageData\n*/";
        a.signatures = Collections.singletonList("createImageData(sw,sh)");
        clz.addAttribute(a);

        a = new AttributeDesc("getImageData", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** The getImageData(sx, sy, sw, sh) method will return an ImageData object representing the underlying pixel data for the area of the canvas denoted by the rectanglewhose corners are the four points (sx, sy), (sx+sw, sy), (sx+sw, sy+sh), (sx, sy+sh), in canvas coordinate space units. Pixels outside the canvas must be returned as transparent black. Pixels must be returned as non-premultiplied alpha values.\n@param {Number} sx\n@param {Number} sy\n@param {Number} sw\n@param {Number} sh\n* @type ImageData\n*/";
        a.signatures = Collections.singletonList("getImageData(sx,sy,sw,sh)");
        clz.addAttribute(a);

        a = new AttributeDesc("putImageData", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** @param {ImageData} imagedata\n@param {Number} dx\n@param {Number} dy\n* @type void\n*/";
        a.signatures = Collections.singletonList("putImageData(imagedata,dx,dy)");
        clz.addAttribute(a);

        a = new AttributeDesc("putImageData", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** The putImageData(imagedata, dx, dy, dirtyX, dirtyY, dirtyWidth, dirtyHeight)  method writes data from ImageData  structures back to the canvas.\n@param {ImageData} imagedata\n@param {Number} dx\n@param {Number} dy\n@param {Number} dirtyX\n@param {Number} dirtyY\n@param {Number} dirtyWidth\n@param {Number} dirtyHeight\n* @type void\n*/";
        a.signatures = Collections.singletonList("putImageData(imagedata,dx,dy,dirtyX,dirtyY,dirtyWidth,dirtyHeight)");
        clz.addAttribute(a);



        // The CanvasGradient class
        clz = new ClassDesc("CanvasGradient", Origin.CANVAS);
        classMap.put(clz.name, clz);
        canvasClasses.add(clz);

        a = new AttributeDesc("addColorStop", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** The addColorStop(offset, color) method on the CanvasGradient interface adds a new stop to a gradient. If the offset is less than 0, greater than 1, infinite, or NaN, then an INDEX_SIZE_ERR exception must be raised. If the color cannot be parsed as a CSS color, then a SYNTAX_ERR exception must be raised. Otherwise, the gradient must have a new stop placed, at offset offset relative to the whole gradient, and with the color obtained by parsing color as a CSS <color> value. If multiple stops are added at the same offset on a gradient, they must be placed in the order added, with the first one closest to the start of the gradient, and each subsequent one infinitesimally further along towards the end point (in effect causing all but the first and last stop added at each point to be ignored).\n@param {Number} offset\n@param {String} color\n* @type void\n*/";
        a.signatures = Collections.singletonList("addColorStop(offset,  color)");
        clz.addAttribute(a);

        // The CanvasPattern class
        clz = new ClassDesc("CanvasPattern", Origin.CANVAS);
        classMap.put(clz.name, clz);
        canvasClasses.add(clz);

        // The ImageData class
        clz = new ClassDesc("ImageData", Origin.CANVAS);
        classMap.put(clz.name, clz);
        canvasClasses.add(clz);

        a = new AttributeDesc("width", null, clz);
        a.type = Type.PROPERTY;
        a.comment = "/** @type Number */";
        clz.addAttribute(a);

        a = new AttributeDesc("height", null, clz);
        a.type = Type.PROPERTY;
        a.comment = "/** @type Number */";
        clz.addAttribute(a);

        a = new AttributeDesc("data", null, clz);
        a.type = Type.PROPERTY;
        a.comment = "/** int[] data @type int[] */";
        clz.addAttribute(a);

        // TODO HTMLCanvasElement, and clean up ActiveX XHR alternative

        // The ImageData class
        clz = new ClassDesc("HTMLCanvasElement", Origin.CANVAS);
        classMap.put(clz.name, clz);
        canvasClasses.add(clz);
        clz.superClass = "HTMLElement";

        a = new AttributeDesc("width", null, clz);
        a.type = Type.PROPERTY;
        a.comment = "/** @type Number */";
        clz.addAttribute(a);

        a = new AttributeDesc("height", null, clz);
        a.type = Type.PROPERTY;
        a.comment = "/** @type Number */";
        clz.addAttribute(a);

        a = new AttributeDesc("toDataURL", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** @type String\n*/";
        a.signatures = Collections.singletonList("toDataURL()");
        clz.addAttribute(a);

        a = new AttributeDesc("toDataURL", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** @param {String} type\n@type String\n*/";
        a.signatures = Collections.singletonList("toDataURL(type)");
        clz.addAttribute(a);

        a = new AttributeDesc("getContext", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** @param {String} contextId\n*/";
        a.signatures = Collections.singletonList("getContext(contextId)");
        clz.addAttribute(a);

        // Update the attributes
        for (ClassDesc cd : canvasClasses) {
            for (AttributeDesc desc : cd.attributes) {
                desc.compatibility = canvasCompat;
                desc.origin = Origin.CANVAS;
                if (desc.type == Type.METHOD) {
                    assert desc.signatures.get(0).startsWith(desc.name) : desc;
                }
                if (desc.comment != null && !desc.comment.endsWith("\n")) {
                    desc.comment = desc.comment+"\n";
                }
            }
        }
    }

    private void addIteratorClasses() {
        // Based on https://developer.mozilla.org/en/Core_JavaScript_1.5_Guide/Iterators_and_Generators

        List<ClassDesc> iteratorClasses = new ArrayList<ClassDesc>();

        ClassDesc clz = new ClassDesc("Iterator", Origin.CORE);
        clz.comment = "/** An Iterator is a JavaScript 1.7 object which can iterate through collections */";
        iteratorClasses.add(clz);
        classMap.put(clz.name, clz);

        AttributeDesc a;

        a = new AttributeDesc("next", null, clz);
        a.type = Type.METHOD;
        a.comment = "/** Return the next item from the iterator\n*/";
        a.signatures = Collections.singletonList("next()");
        clz.addAttribute(a);

        a = new AttributeDesc("Iterator", null, clz);
        a.type = Type.CONSTRUCTOR;
        a.comment = "/** Create an iterator for the given collection\n* @type Iterator\n*/";
        a.signatures = Collections.singletonList("Iterator(collection)");
        clz.addAttribute(a);

        a = new AttributeDesc("Iterator", null, clz);
        a.type = Type.CONSTRUCTOR;
        a.comment = "/** Create an iterator for the given collection\n* @type Iterator\n*/";
        a.signatures = Collections.singletonList("Iterator(collection,keysOnly)");
        clz.addAttribute(a);

        // Compatibility:
        // New in JavaScript 1.7 - which is supported by:
        // Firefox 2+, Safari 3+, and -maybe- Opera - not sure.
        // See http://en.wikipedia.org/wiki/JavaScript and  http://ejohn.org/blog/versions-of-javascript and
        //
        EnumSet<BrowserCompat> iteratorCompat = EnumSet.of(BrowserCompat.FF3, BrowserCompat.FF2, BrowserCompat.OPERA,
                BrowserCompat.SAFARI3, BrowserCompat.KONQ);

        // Update the attributes
        for (ClassDesc cd : iteratorClasses) {
            for (AttributeDesc desc : cd.attributes) {
                desc.compatibility = iteratorCompat;
                desc.origin = Origin.CORE;
                if (desc.type == Type.METHOD) {
                    assert desc.signatures.get(0).startsWith(desc.name) : desc;
                }
                if (desc.comment != null && !desc.comment.endsWith("\n")) {
                    desc.comment = desc.comment+"\n";
                }
            }
        }
    }

    private void fixSpecialCases(Collection<ClassDesc> classes) throws IOException {
        // The wiki documentation isn't always consistent which trips up the documentation scanner.
        // This code emits various warnings when it doesn't find information it needs to.
        // The below fixes up specific cases listed as warnings or noticed to be incorrect.
        for (ClassDesc clz : classes) {
            String name = clz.name;
            if ("Array".equals(name)) {
                String[] constructors = { 
                    "Array(arrayLength)",
                    //"Array(element0, element1, ..., elementN)", // TODO - invent some kind of syntax to include ...'s
                    "Array(element0, element1, elementN)",
                }; // TODO - invent some kind of syntax to include ...'s
                clz.constructors = new HashSet<String>(Arrays.asList(constructors));
                clz.getAttribute("indexOf").type = Type.METHOD;
                clz.getAttribute("lastIndexOf").type = Type.METHOD;
                clz.getAttribute("reduce").type = Type.METHOD;
                clz.getAttribute("reduceRight").type = Type.METHOD;
            } else if ("CssRule".equals(name)) {
                clz.name = "CSSRule"; // Different capitalization than the one I inferred
                clz.getAttribute("cssText").type = Type.PROPERTY;
                AttributeDesc a = clz.getAttribute("parentRule");
                if (a != null) {
                    a.type = Type.PROPERTY;
                }
                a = clz.getAttribute("parentStyleSheet");
                if (a != null) {
                    a.type = Type.PROPERTY;
                }
                a = clz.getAttribute("type");
                if (a != null) {
                    a.type = Type.PROPERTY;
                }
                // TODO -  CSSStyleRule! with 2 properties: selectorText and rule
            } else if ("Date".equals(name)) {
                // Fix constructors
                String[] constructors = { 
                    "Date()", 
                    "Date(milliseconds)",
                    "Date(dateString)",
                    "Date(yr_num, mo_num, day_num)",
                    "Date(yr_num, mo_num, day_num, hr_num, min_num, sec_num, ms_num)",
                };
                clz.constructors = new HashSet<String>(Arrays.asList(constructors));
            } else if ("Document".equals(name)) {
                AttributeDesc a = clz.getAttribute("createCDATASection");
                if (a != null) {
                    a.signatures = Collections.singletonList("createCDATASection(data)");
                    clz.getAttribute("createComment").signatures = Collections.singletonList("createComment(comment)");
                    clz.getAttribute("createEntityReference").signatures = Collections.singletonList("createEntityReference(name)");
                    clz.getAttribute("createProcessingInstruction").signatures = Collections.singletonList("createProcessingInstruction(target,data)");
                    a = clz.getAttribute("execCommand");
                    if (a != null) {
                        a.signatures = Collections.singletonList("execCommand(aCommandName, aShowDefaultUI, aValueArgument)");
                        //clz.getAttribute("loadOverlay").signatures = Collections.singletonList("loadOverlay(url, observer)");
                        clz.getAttribute("queryCommandEnabled").signatures = Collections.singletonList("queryCommandEnabled(commandIdentifier)");
                        clz.getAttribute("queryCommandIndeterm").signatures = Collections.singletonList("queryCommandIndeterm(commandIdentifier)");
                        clz.getAttribute("queryCommandState").signatures = Collections.singletonList("queryCommandState(commandIdentifier)");
                        clz.getAttribute("queryCommandValue").signatures = Collections.singletonList("queryCommandValue(commandIdentifier)");
                    }
                }
                a = clz.getAttribute("getElementById");
                if (a != null) {
                    a.compatibility = EnumSet.allOf(BrowserCompat.class);
                }
                
            } else if ("Element".equals(name)) {
                for (AttributeDesc a : clz.attributes) {
                    if (a.name.startsWith("on") && a.type == null) {
                        a.type = Type.EVENT;
                    }
                }
                AttributeDesc a = clz.getAttribute("getBoundingClientRect");
                if (a != null) {
                    a.type = Type.METHOD;
                    clz.getAttribute("getBoundingClientRect").signatures = Collections.singletonList("getBoundingClientRect()");
                    clz.getAttribute("getBoundingClientRect").compatibility = EnumSet.of(BrowserCompat.FF3);
                    clz.getAttribute("getClientRects").type = Type.METHOD;
                    clz.getAttribute("getClientRects").signatures = Collections.singletonList("getClientRects()");
                    clz.getAttribute("getClientRects").compatibility = EnumSet.of(BrowserCompat.FF3);
                    clz.getAttribute("item").type = Type.METHOD;
                    clz.getAttribute("item").signatures = Collections.singletonList("item(index)");
                    clz.getAttribute("length").type = Type.PROPERTY;
                }
            } else if ("Error".equals(name)) {
                String[] constructors = { 
                    "Error()",
                    "Error(message)",
                };
                clz.constructors = new HashSet<String>(Arrays.asList(constructors));

                clz.getAttribute("constructor").type = Type.PROPERTY;
                clz.getAttribute("message").type = Type.PROPERTY;
                clz.getAttribute("name").type = Type.PROPERTY;
                clz.getAttribute("toString").type = Type.METHOD;
            } else if ("Event".equals(name)) {
                clz.removeAttribute("Comparison_of_Event_Targets");
                AttributeDesc a = clz.getAttribute("preventBubble");
                if (a != null) {
                    a.signatures = Collections.singletonList("preventBubble()");
                    clz.getAttribute("preventCapture").signatures = Collections.singletonList("preventCapture()");
                }
            } else if ("EvalError".equals(name)) {
                clz.getAttribute("constructor").type = Type.PROPERTY;
                clz.getAttribute("name").type = Type.PROPERTY;
            } else if ("Function".equals(name)) {
                String[] constructors = { 
                    "Function(arg1, arg2, argN, functionBody)",
                };
                clz.constructors = new HashSet<String>(Arrays.asList(constructors));
                clz.getAttribute("arguments").type = Type.PROPERTY;
                clz.removeAttribute("arguments:length");
            } else if ("JavaArray".equals(name)) {
                clz.constructors = Collections.emptySet(); // No constructor, created from Java
                clz.getAttribute("toString").signatures = Collections.singletonList("toString()");
            } else if ("Math".equals(name)) {
                // No constructors!
                clz.constructors = Collections.emptySet();
                clz.getAttribute("random").signatures = Collections.singletonList("random()");
            } else if ("Number".equals(name)) {
                clz.getAttribute("toLocaleString").signatures = Collections.singletonList("toLocaleString()");
            } else if ("Navigator".equals(name)) {
                clz.getAttribute("isLocallyAvailable").type = Type.METHOD;
                clz.getAttribute("onLine").type = Type.PROPERTY;
                clz.getAttribute("plugins").type = Type.PROPERTY;
                clz.getAttribute("registerContentHandler").type = Type.PROPERTY;
                clz.getAttribute("registerProtocolHandler").signatures = Collections.singletonList("registerProtocolHandler(protocol, uri, title)");
                clz.getAttribute("registerProtocolHandler").type = Type.PROPERTY;
                clz.getAttribute("registerContentHandler").signatures = Collections.singletonList("registerContentHandler(mimeType, uri, title)");
            } else if ("Object".equals(name)) {
                String[] constructors = { 
                    "Object()",
                };
                clz.constructors = new HashSet<String>(Arrays.asList(constructors));
                clz.getAttribute("isPrototypeOf").signatures = Collections.singletonList("isPrototypeOf(object)");
                clz.getAttribute("toLocaleString").type = Type.METHOD;
            } else if ("RangeError".equals(name)) {
                clz.constructors = Collections.emptySet();
                clz.getAttribute("constructor").type = Type.PROPERTY;
                clz.getAttribute("name").type = Type.PROPERTY;
            } else if ("ReferenceError".equals(name)) {
                clz.constructors = Collections.emptySet();
                clz.getAttribute("constructor").type = Type.PROPERTY;
                clz.getAttribute("name").type = Type.PROPERTY;
            } else if ("RegExp".equals(name)) {
                String[] constructors = { 
                    "RegExp(pattern)",
                    "RegExp(pattern,flags)",
                };
                clz.constructors = new HashSet<String>(Arrays.asList(constructors));
            } else if ("Selection".equals(name)) {
                AttributeDesc a = clz.getAttribute("selectionLanguageChange");
                if (a != null) {
                    a.signatures = Collections.singletonList("selectionLanguageChange(rtlflag)");
                }
            } else if ("String".equals(name)) {
                clz.getAttribute("anchor").type = Type.METHOD;
                addComment(clz.getAttribute("anchor"), "/**\n @type String */\n");
                clz.getAttribute("big").type = Type.METHOD;
                addComment(clz.getAttribute("big"),"/**\n @type String */\n");
                clz.getAttribute("blink").type = Type.METHOD;
                addComment(clz.getAttribute("blink"), "/**\n @type String */\n");
                clz.getAttribute("bold").type = Type.METHOD;
                addComment(clz.getAttribute("bold"), "/**\n @type String */\n");
                clz.getAttribute("fixed").type = Type.METHOD;
                addComment(clz.getAttribute("fixed"), "/**\n @type String */\n");
                clz.getAttribute("fontcolor").type = Type.METHOD;
                addComment(clz.getAttribute("fontcolor"), "/**\n @type String */\n");
                clz.getAttribute("fontsize").type = Type.METHOD;
                addComment(clz.getAttribute("fontsize"), "/**\n @type String */\n");
                clz.getAttribute("italics").type = Type.METHOD;
                addComment(clz.getAttribute("italics"), "/**\n @type String */\n");
                clz.getAttribute("link").type = Type.METHOD;
                addComment(clz.getAttribute("link"), "/**\n @type String */\n");
                clz.getAttribute("small").type = Type.METHOD;
                addComment(clz.getAttribute("small"), "/**\n @type String */\n");
                clz.getAttribute("strike").type = Type.METHOD;
                addComment(clz.getAttribute("strike"), "/**\n @type String */\n");
                clz.getAttribute("sub").type = Type.METHOD;
                addComment(clz.getAttribute("sub"), "/**\n @type String */\n");
                clz.getAttribute("sup").type = Type.METHOD;
                addComment(clz.getAttribute("sup"), "/**\n @type String */\n");
            } else if ("SyntaxError".equals(name)) {
                clz.constructors = Collections.emptySet();
                clz.getAttribute("constructor").type = Type.PROPERTY;
                clz.getAttribute("name").type = Type.PROPERTY;
            } else if ("TableRow".equals(name)) {
                clz.getAttribute("deleteCell").signatures = Collections.singletonList("deleteCell(index)");
            } else if ("TreeWalker".equals(name)) {
                AttributeDesc a = clz.getAttribute("firstChild");
                if (a != null) {
                    a.signatures = Collections.singletonList("firstChild()");
                    clz.getAttribute("lastChild").signatures = Collections.singletonList("lastChild()");
                    clz.getAttribute("nextNode").signatures = Collections.singletonList("nextNode()");
                    clz.getAttribute("nextSibling").signatures = Collections.singletonList("nextSibling()");
                    clz.getAttribute("parentNode").signatures = Collections.singletonList("parentNode()");
                    clz.getAttribute("previousNode").signatures = Collections.singletonList("previousNode()");
                    clz.getAttribute("previousSibling").signatures = Collections.singletonList("previousSibling()");
                }
            } else if ("TypeError".equals(name)) {
                clz.constructors = Collections.emptySet();
                clz.getAttribute("constructor").type = Type.PROPERTY;
                clz.getAttribute("name").type = Type.PROPERTY;
            } else if ("Window".equals(name)) {
                clz.getAttribute("escape").signatures = Collections.singletonList("escape(regular)");
            } else if ("URIError".equals(name)) {
                clz.constructors = Collections.emptySet();
                clz.getAttribute("constructor").type = Type.PROPERTY;
                clz.getAttribute("name").type = Type.PROPERTY;
            } else if ("XMLHttpRequest".equals(name)) {
                EnumSet<BrowserCompat> notIe = EnumSet.complementOf(EnumSet.of(BrowserCompat.IE55, BrowserCompat.IE6));

                // The constructor isn't valid on IE prior to IE7; you had to use new ActiveXObject("MSXML2.XMLHTTP"
                //clz.constructors = Collections.singleton("XMLHttpRequest()");
                AttributeDesc a = new AttributeDesc("XMLHttpRequest", null, clz);
                a.comment = "/** Constructs a new XMLHttpRequest object.\n@type XMLHttpRequest */\n";
                a.type = Type.CONSTRUCTOR;
                a.signatures = Collections.singletonList(("XMLHttpRequest()"));
                a.compatibility = notIe;
                clz.addAttribute(a);
                
                
                a = new AttributeDesc("onreadystatechange", null, clz);
                a.comment = "/** Specifies a reference to an event handler for an event that fires at every state change. */\n";
                a.type = Type.PROPERTY;
                a.signatures = Collections.singletonList(("onreadystatechange"));
                //a.compatibility = notIe;
                clz.addAttribute(a);

                a = new AttributeDesc("readyState", null, clz);
                a.comment = "/** The XMLHttpRequest  object can be in five states: UNSENT (0), OPENED (1), HEADERS_RECEIVED (2), LOADING (3) and DONE (4). The current state is exposed through the readyState " +
                        "attribute.\n */\n";
                a.type = Type.PROPERTY;
                a.signatures = Collections.singletonList(("readyState"));
                //a.compatibility = notIe;
                clz.addAttribute(a);

                a = new AttributeDesc("responseText", null, clz);
                a.comment = "/** Returns the response as a String\n@type String\n */\n";
                a.type = Type.PROPERTY;
                a.signatures = Collections.singletonList(("responseText"));
                //a.compatibility = notIe;
                clz.addAttribute(a);

                a = new AttributeDesc("responseXML", null, clz);
                a.comment = "/** Returns the response as XML. This property returns an XML document object, which can be examined and parsed using W3C DOM node tree methods and properties. */\n";
                a.type = Type.PROPERTY;
                a.signatures = Collections.singletonList(("responseXML"));
                //a.compatibility = notIe;
                clz.addAttribute(a);

                a = new AttributeDesc("responseBody", null, clz);
                a.comment = "/** Returns the response as a binary encoded string (an array of unsigned bytes) */\n";
                a.type = Type.PROPERTY;
                a.signatures = Collections.singletonList(("responseBody"));
                //a.compatibility = notIe;
                clz.addAttribute(a);

                a = new AttributeDesc("status", null, clz);
                a.comment = "/** Returns the HTTP status code (such as 404 for not found and 200 for OK)\n@type Number */\n";
                a.type = Type.PROPERTY;
                a.signatures = Collections.singletonList(("status"));
                //a.compatibility = notIe;
                clz.addAttribute(a);

                a = new AttributeDesc("statusText", null, clz);
                a.comment = "/** Returns the HTTP status as a String (such as \"OK\" or \"Not Found\")\n@type String\n */\n";
                a.type = Type.PROPERTY;
                a.signatures = Collections.singletonList(("statusText"));
                //a.compatibility = notIe;
                clz.addAttribute(a);

                a = new AttributeDesc("abort", null, clz);
                a.comment = "/** Cancel the current HTTP request */\n";
                a.type = Type.METHOD;
                a.signatures = Collections.singletonList(("abort()"));
                //a.compatibility = notIe;
                clz.addAttribute(a);

                a = new AttributeDesc("getAllResponseHeaders", null, clz);
                a.comment = "/** Returns the complete set of HTTP headers as a string.\n@type String\n */\n";
                a.type = Type.METHOD;
                a.signatures = Collections.singletonList(("getAllResponseHeaders()"));
                //a.compatibility = notIe;
                clz.addAttribute(a);

                a = new AttributeDesc("getResponseHeader", null, clz);
                a.comment = "/** Returns the value of the specified header.\n* @param header The name of the header\n@type String\n */\n";
                a.type = Type.METHOD;
                a.signatures = Collections.singletonList(("getResponseHeader(header)"));
                //a.compatibility = notIe;
                clz.addAttribute(a);

                a = new AttributeDesc("send", null, clz);
                a.comment = "/** Send the request\n* @param content The content to be sent\n */\n";
                a.type = Type.METHOD;
                a.signatures = Collections.singletonList(("send(content)"));
                //a.compatibility = notIe;
                clz.addAttribute(a);

                a = new AttributeDesc("setRequestHeader", null, clz);
                a.comment = "/** Sets the given header with the given value.\n* @param header The name of the header\n* @param value The value of the header */\n";
                a.type = Type.METHOD;
                a.signatures = Collections.singletonList(("getResponseHeader(header,value)"));
                //a.compatibility = notIe;
                clz.addAttribute(a);

                a = new AttributeDesc("open", null, clz);
                a.comment = "/** Specifies the method, URL, and other optional attributes of a request.\n* @param method HTTP method such " +
                        "as GET or POST.\n* @param url A relative or complete URL.\n* @param async \"true\" for asynchronous requests.\n" +
                        "* @param user Authentication user name\n* @param password Authentication password. */\n";
                a.type = Type.METHOD;
                a.signatures = Arrays.asList(new String[] {"open(method,url)", "open(method,url,async)",
                "open(method,url,async,user)","open(method,url,async,user,password)"});
                //a.compatibility = notIe;
                clz.addAttribute(a);
            }
            
            // TODO: ActiveXObject("MSXML2.XMLHTTP.3.0");
        }
    }
    
    private String insertJsDoc(String comment, String jsdoc) {
        int index = comment.indexOf("* @");
        if (index == -1) {
            index = comment.lastIndexOf("*/");
            index--;
            for (; index >= 0; index--) {
                char c = comment.charAt(index);
                if (c == '\n') {
                    // Found new line which just ends with */
                    return comment.substring(0, index+1) + " * " + jsdoc.trim() + comment.substring(index);
                } else if (c == '*') {
                    // Found line which possibly ends with * */
                    return comment.substring(0, index+1) + "\n * " + jsdoc.trim() + "\n" + comment.substring(index+1);
                } else if (!Character.isWhitespace(c)) {
                    // Some other character - create a newline before resuming with our comment
                    return comment.substring(0, index+1) + "\n * " + jsdoc.trim() + "\n" + comment.substring(index+1);
                }
            }
            assert false : comment + " for jsdoc " + jsdoc;
            return comment;
        } else {
            return comment.substring(0, index) + " * " + jsdoc.trim() + "\n" + comment.substring(index);
        }
    }

    private class AttributeDesc implements Comparable<AttributeDesc> {
        private String name;
        private List<File> files;
        private String comment;
        private List<String> signatures;
        private ClassDesc memberOf;
        private Type type;
        private String version;
        private EnumSet<BrowserCompat> compatibility;
        private Origin origin;
        
        AttributeDesc(String name, File file, ClassDesc memberOf) {
            assert name.indexOf('.') == -1 : name + " in " + memberOf + " from " + file;
            this.name = name;
            files = new ArrayList<File>(2);
            if (file != null) {
                files.add(file);
            }
            this.memberOf = memberOf;
        }

        @Override
        public String toString() {
            return name + ":" + type + " from class " + memberOf;
        }

        @Override
        public int compareTo(Main.AttributeDesc o) {
            return name.compareTo(o.name);
        }
    }
    
    private class ClassDesc implements Comparable<ClassDesc> {
        private String property;
        private String name;
        private List<File> descFiles;
        private String comment;
        private String superClass;
        private Set<AttributeDesc> attributes = new HashSet<AttributeDesc>();
        private Set<String> constructors = new HashSet<String>();
        private Origin origin;

        ClassDesc(String name, Origin origin) {
//            if (Character.isLowerCase(name.charAt(0))) {
//                property = name;
//                this.name = Character.toUpperCase(name.charAt(0)) + name.substring(1);
//            } else {
            assert Character.isUpperCase(name.charAt(0));
                this.name = name;
//            }
            this.origin = origin;
        }
        
        void addAttribute(AttributeDesc desc) {
//            // Skip prototype attributes
//            if (desc.name.indexOf("prototype") != -1) {
//                return;
//            }
            attributes.add(desc);
        }

        @Override
        public String toString() {
            return name;// + ",super=" + superClass;
        }

        @Override
        public int compareTo(Main.ClassDesc o) {
            return name.compareTo(o.name);
        }
        
        public String toCode() {
            StringBuilder sb = new StringBuilder();
            sb.append(comment);

            List<AttributeDesc> sortedAttributes = new ArrayList<AttributeDesc>(attributes);
            Collections.sort(sortedAttributes);
            
            String compatibility = getCompatibilityFlags(name);

            if (GENERATE_JAVASCRIPT) {
                sb.append("var ");
                sb.append(name);
                if (superClass != null) {
                    sb.append(" = Object.extend(new ");
                    sb.append(superClass);
                    sb.append("(), {");
                } else {
                    sb.append(" = {");
                }

                if (compatibility == null) {
                    for (AttributeDesc attribute : sortedAttributes) {
                        if (attribute.type == Type.CONSTRUCTOR) {
                            String fqn = name + "." + attribute.name;
                            compatibility = getCompatibilityFlags(fqn);
                            if (compatibility == null && attribute.compatibility != null && attribute.compatibility.size() < BrowserCompat.getBrowserCount()) {
                                compatibility = BrowserCompat.toFlags(attribute.compatibility);
                            }
                            if (attribute.compatibility == null) {
                                if (compatibility == null && attribute.origin == Origin.DOM3) {
                                    compatibility = BrowserCompat.toFlags(EnumSet.complementOf(EnumSet.of(BrowserCompat.IE55, BrowserCompat.IE6)));
                                }
                                if (compatibility == null && attribute.origin == Origin.RANGE) {
                                    compatibility = BrowserCompat.toFlags(EnumSet.complementOf(EnumSet.of(BrowserCompat.IE55, BrowserCompat.IE6,BrowserCompat.IE7)));
                                }
                            }
                        }
                        if (compatibility != null) {
                            break;
                        }
                    }
                }
                
                if (compatibility != null) {
                    sb.append(" // COMPAT=");
                    sb.append(compatibility);
                }
                sb.append("\n");
                sb.append("  // This is just a stub for a builtin native JavaScript object.\n");
            }
            
            for (AttributeDesc attribute : sortedAttributes) {
                if (attribute.type == Type.CONSTRUCTOR) {
                    continue;
                }
                if (attribute.signatures == null) {
                    System.err.println("WARNING - no signatures for attribute " + attribute);
                }
                for (String signature : attribute.signatures) {
                    //assert attribute.type != null : attribute;
                    if (attribute.type == null) {
                        System.err.println("WARNING: No type located for " + attribute);
                    }
                    int openParen = signature.indexOf('(');
                    int equal = signature.indexOf('=');
                    int newline = signature.indexOf('\n');
                    if ((equal != -1 && equal < openParen) ||
                            ((newline != -1 && newline < openParen))) {
                        openParen = -1;
                    }

                    String argStr = null;
                    
                    boolean isProperty = false;
                    if (openParen != -1) {
                        assert openParen != -1 : signature + " contains no (";
                        int closeParen = signature.indexOf(')', openParen);
                        assert closeParen != -1 : signature + " contains no )";
                        argStr = signature.substring(openParen+1, closeParen);
//                    } else if (signature.indexOf('=') == -1) {
//                        System.err.println("Skipping method declaration for " + name + "." + attribute.name + " - no parens");
//                        continue;
                        // Perhaps something like a Selection.anchorNode
                        // e.g. a property
                    } else {
                        // Probably an event handler like "onload"
                        isProperty = true;
                    }

                    if (attribute.type != Type.METHOD && attribute.type != Type.FUNCTION) {
                        isProperty = true;
                    }
                    
                    //assert isProperty == (attribute.type != Type.METHOD && attribute.type != Type.FUNCTION);
                    if (isProperty != (attribute.type != Type.METHOD && attribute.type != Type.FUNCTION)) {
                        System.err.println("Warning: for attribute " + attribute + " isProperty=" + isProperty);
                    }
                    
                    if (attribute.comment != null) {
                        sb.append(attribute.comment);
                    }
                    
                    sb.append(attribute.name);
                    sb.append(": ");
                    
                    if (isProperty) {
                        sb.append("undefined,");
                    } else {
                        sb.append("function(");

                        if (argStr != null) {
                            sb.append(argStr);
                        }

                        sb.append(") {");
                    }
                    
                    String fqn = name + "." + attribute.name;
                    compatibility = getCompatibilityFlags(fqn);
                    if (compatibility == null && attribute.compatibility != null && attribute.compatibility.size() < BrowserCompat.getBrowserCount()) {
                        compatibility = BrowserCompat.toFlags(attribute.compatibility);
                    }
                    if (attribute.compatibility == null) {
                        if (compatibility == null && attribute.origin == Origin.DOM3) {
                            compatibility = BrowserCompat.toFlags(EnumSet.complementOf(EnumSet.of(BrowserCompat.IE55, BrowserCompat.IE6)));
                        }
                        if (compatibility == null && attribute.origin == Origin.RANGE) {
                            compatibility = BrowserCompat.toFlags(EnumSet.complementOf(EnumSet.of(BrowserCompat.IE55, BrowserCompat.IE6,BrowserCompat.IE7)));
                        }
                    }
                    if (compatibility != null) {
                        sb.append(" // COMPAT=");
                        sb.append(compatibility);
                    }
                    
                    sb.append("\n");
                    if (!isProperty) {
                        sb.append("  // This is just a stub for a builtin native JavaScript object.\n");
                        sb.append("},\n");
                    }
                }
            }

            sb.append("}");
            if (superClass != null) {
                sb.append(")");
            }
            sb.append(";\n");

            
            // Add constructors
            if (constructors != null) {
                for (String signature : constructors) {
                    sb.append(comment);
                    sb.append("function ");
                    sb.append(signature);
                    if (signature.indexOf('(') == -1) {
                        sb.append("()");
                    }
                    sb.append(" {};");
                    sb.append("\n");
                }
            } else if (origin == Origin.CORE) {
                // Just create a default constructor (but not for DOM objects - these are usually not creatable)
                sb.append(comment);
                sb.append("function ");
                sb.append(name);
                sb.append("() {};");
                sb.append("\n");
            }

            for (AttributeDesc attribute : sortedAttributes) {
                if (attribute.type == Type.CONSTRUCTOR) {
                    String fqn = name + "." + attribute.name;
                    compatibility = getCompatibilityFlags(fqn);
                    if (compatibility == null && attribute.compatibility != null && attribute.compatibility.size() < BrowserCompat.getBrowserCount()) {
                        compatibility = BrowserCompat.toFlags(attribute.compatibility);
                    }
                    if (attribute.compatibility == null) {
                        if (compatibility == null && attribute.origin == Origin.DOM3) {
                            compatibility = BrowserCompat.toFlags(EnumSet.complementOf(EnumSet.of(BrowserCompat.IE55, BrowserCompat.IE6)));
                        }
                        if (compatibility == null && attribute.origin == Origin.RANGE) {
                            compatibility = BrowserCompat.toFlags(EnumSet.complementOf(EnumSet.of(BrowserCompat.IE55, BrowserCompat.IE6,BrowserCompat.IE7)));
                        }
                    }
                    
                    sb.append(comment);
                    for (String signature : attribute.signatures) {
                        sb.append("function ");
                        
                        sb.append(signature);
                        if (signature.indexOf('(') == -1) {
                            sb.append("()");
                        }
                        sb.append(" {};");
                        if (compatibility != null) {
                            sb.append(" // COMPAT=");
                            sb.append(compatibility);
                        }
                        sb.append("\n");
                    }
                }
            }
            
            sb.append("\n");
            if (property != null) {
                sb.append(comment);
                sb.append("var ");
                sb.append(property);
                sb.append(" = new ");
                sb.append(name);
                sb.append("()");
                sb.append(";\n");
            }

            return sb.toString();
        }

        private AttributeDesc getAttribute(String prop) {
            for (AttributeDesc desc : attributes) {
                if (desc.name.equals(prop)) {
                    return desc;
                }
            }
            
            return null;
        }

        private void removeAttribute(String prop) {
            for (AttributeDesc desc : attributes) {
                if (desc.name.equals(prop)) {
                    attributes.remove(desc);
                    return;
                }
            }
        }
    }
    
    private Document browserCompatibility;
    private Map<String,Element> compatEntries = new HashMap<String, Element>();
    
    private void emitBrowserMaps(Collection<ClassDesc> classes) throws IOException {
        if (!GENERATE_BROWSER_MAPS) {
            return;
        }
        initializeCompatibilityTable();
        
        Map<String,String> e = new HashMap<String,String>();
        
        int browserCount = BrowserCompat.getBrowserCount();
        for (ClassDesc clz : classes) {
            if (clz.name.startsWith("DOM")) {
                // Don't bother with the DOMImplementation stuff
                continue;
            }
            for (AttributeDesc attr : clz.attributes) {
                if (attr.type != Type.FUNCTION && attr.type != Type.METHOD && attr.type != Type.CONSTRUCTOR) {
                    // I don't want compatibility maps for properties right now since
                    // I'm only checking compatibility on calls...
                    continue;
                }
                if (attr.name.equals("toString")) { // NOI18N
                    // This is just too common
                    continue;
                }
                EnumSet<BrowserCompat> compat = attr.compatibility;
                if (compat == null && attr.origin == Origin.DOM3) {
                    compat = EnumSet.complementOf(EnumSet.of(BrowserCompat.IE55, BrowserCompat.IE6));
                }
                if (compat == null && attr.origin == Origin.RANGE) {
                    compat = EnumSet.complementOf(EnumSet.of(BrowserCompat.IE55, BrowserCompat.IE6,BrowserCompat.IE7));
                }
                if (compat == null) {
                    continue;
                }
                if (compat.size() < browserCount) {
                    // Not supported by all browser... add to map if necessary
                    String fqn = attr.type == Type.CONSTRUCTOR ? attr.name : clz.name + "." + attr.name;
                    if (clz.name.indexOf('.') != -1 || clz.name.indexOf('.') != -1) {
                        System.err.println("WARNING: Skipping " + fqn + " from browser maps");
                        continue;
                    }
                    String flags = BrowserCompat.toFlags(compat);
                    e.put(fqn, flags);
                }
            }
        }

        for (String fqn : compatEntries.keySet()) {
            if (fqn.endsWith("()")) {
                fqn = fqn.substring(0, fqn.length()-2);
            } else if (fqn.endsWith("[]")) {
                fqn = fqn.substring(0, fqn.length()-2);
            }
            if (Character.isLowerCase(fqn.charAt(0))) {
                fqn = Character.toUpperCase(fqn.charAt(0)) + fqn.substring(1);
            }
            String flags = getCompatibilityFlags(fqn);
            if (flags != null) {
                e.put(fqn, flags);
            }
        }

        List<String> keys = new ArrayList<String>(e.keySet());
        Collections.sort(keys);
        
        File browserMaps = new File("browser-maps.java");
        FileWriter fw = new FileWriter(browserMaps);
        
        fw.write("    // This code is automatically generated by #emitBrowserMaps()\n" +
                 "    // in the javascript.generatestubs project in the misc repository.\n");
        fw.write("    private static final Set<String> NAME_SET = new HashSet<String>(" + 2*keys.size() + ");\n");
        fw.write("    private static final Map<String,Boolean> MUST_CHECK_FQN = new HashMap<String,Boolean>(" + 2*keys.size() + ");\n");
        fw.write("    private static final Map<String,String> NAME_TO_FQN = new HashMap<String,String>(" + 2*keys.size() + ");\n");
        fw.write("    private static final Map<String,EnumSet<BrowserVersion>> COMPAT_MAP = new HashMap<String,EnumSet<BrowserVersion>>(" + 2*keys.size() + ");\n");
        
        // Compute constant versions
        // Figure out unique ones
        Map<String,EnumSet<BrowserCompat>> compats = new HashMap<String,EnumSet<BrowserCompat>>();
        for (String key : keys) {
            String flags = e.get(key);
            EnumSet<BrowserCompat> compat = compats.get(flags);
            if (compat == null) {
                compat = BrowserCompat.fromFlags(flags);
                compats.put(flags, compat);
            }
        }
        
        Set<String> duplicates = new HashSet<String>();
        Set<String> uniqueNames = new HashSet<String>();
        Set<String> nonduplicatedNames = new HashSet<String>();
        Map<String,String> nameToFqn = new HashMap<String,String>();
        for (String fqn : e.keySet()) {
            String name = fqn.substring(fqn.lastIndexOf('.')+1);
            if (uniqueNames.contains(name)) {
                nonduplicatedNames.remove(name);
                duplicates.add(fqn);
                nameToFqn.remove(name);
                continue;
            }
            
            nameToFqn.put(name, fqn);
            nonduplicatedNames.add(name);
            uniqueNames.add(name);
        }
        
        Map<String,EnumSet<BrowserCompat>> compatNames = new HashMap<String,EnumSet<BrowserCompat>>();
        Map<String,String> flagToName = new HashMap<String,String>();
        for (String flag : compats.keySet()) {
            EnumSet<BrowserCompat> compat = compats.get(flag);
            assert compat != null;
            String name = flag.replace("|", "");
            if (compat.size() == browserCount) {
                name = "ALL";
            } else if (compat.size() == browserCount-3 && flag.indexOf("IE") == -1) {
                name = "NOT_IE";
            } else if (compat.size() == 3 && flag.equals("FF1|FF2|FF3")) {
                name = "FIREFOX";
            } else if (compat.size() == 3 && flag.equals("IE55|IE6|IE7")) {
                name = "IE";
            } else if (compat.size() == browserCount-1 && flag.indexOf("IE55") == -1) {
                name = "NOT_IE55";
            } else if (compat.size() == browserCount-2 && flag.indexOf("IE55") == -1 && flag.indexOf("IE6") == -1) {
                name = "NOT_IE55_OR_6";
            }
            compatNames.put(name, compat);
            flagToName.put(flag,name);
        }
        
        for (String name : compatNames.keySet()) {
            fw.write("    private static final EnumSet<BrowserVersion> ");
            fw.write(name);
            fw.write(" = BrowserVersion.fromFlags(\"");
            EnumSet<BrowserCompat> compat = compatNames.get(name);
            fw.write(BrowserCompat.toFlags(compat));
            fw.write("\"); // NOI18N\n");
        }

        fw.write("    static {\n");
        //List<String> names = new ArrayList<String>();
        for (String fqn : keys) {
            //int dot = fqn.lastIndexOf('.');
            //assert dot != -1;
            //String name = fqn.substring(dot+1);
            //names.add(name);
            String flags = e.get(fqn);
            
            if (fqn.endsWith(".hasAttribute")) {
                fw.write("// This one is overridden in prototype... Element.hasAttribute may be referring to it!\n// For now, don't warn about this one.\n");
                fw.write("//");
            }

            fw.write("        COMPAT_MAP.put(\"");
            fw.write(fqn);
            
            fw.write("\", ");
            fw.write(flagToName.get(flags));
            fw.write("); // NOI18N\n"); // NOI18N
        }
        
        fw.write("\n");
        
        List<String> names = new ArrayList<String>(uniqueNames);
        Collections.sort(names);
        for (String name : names) {
            if (name.equals("hasAttribute")) {
                fw.write("//");
            }
            fw.write("        NAME_SET.add(\"");
            fw.write(name);
            fw.write("\"); // NOI18N\n");
        }

        names = new ArrayList<String>(nonduplicatedNames);
        Collections.sort(names);
        fw.write("\n");
        for (String name : names) {
            if (name.equals("hasAttribute")) {
                fw.write("//");
            }
            fw.write("        NAME_TO_FQN.put(\"");
            fw.write(name);
            fw.write("\", \"");
            fw.write(nameToFqn.get(name));
            fw.write("\"); // NOI18N\n");
        }
        
        if (duplicates.size() > 0) {
            fw.write("\n        // Initialize items we know we can't search by duplicate\n");
            for (String fqn : duplicates) {
                fw.write("        MUST_CHECK_FQN.put(\"");
                fw.write(fqn);
                fw.write("\", Boolean.FALSE); // NOI18N\n");
            }
        }
        
        fw.write("    }\n");
        
        fw.close();
        
        System.err.println("WROTE BROWSER COMPATIBILITY MAPS TO " + browserMaps.getAbsolutePath());
    }

    private void initializeCompatibilityTable() {
        try {
            if (browserCompatibility == null) {
                File browserFile = new File("browser-compatibility.xml");
                if (!browserFile.exists()) {
                    System.err.println("Can't find " + browserFile.getAbsolutePath());
                    System.exit(0);
                }
                browserCompatibility = parse(browserFile);
                if (browserCompatibility == null || browserCompatibility.getDocumentElement() == null) {
                    System.err.println("No compatibility entries?");
                    System.exit(0);
                }

                NodeList nl = browserCompatibility.getDocumentElement().getChildNodes();
                for (int i = 0, n = nl.getLength(); i < n; i++) {
                    Node node = nl.item(i);
                    if (node.getNodeType() == Node.ELEMENT_NODE) {
                        Element e = (Element) node;
                        String name = e.getAttribute("name");
                        String in = e.getAttribute("in");
                        assert name != null && in != null;
                        String sig = in != null ? in + "." + name : name;
                        compatEntries.put(sig, e);
                    }
                }
            }
        } catch (IOException ioe) {
            ioe.printStackTrace();
            System.exit(0);
        }
    }
    
    private String getCompatibilityFlags(String fqn) {
        initializeCompatibilityTable();
        
        Element element = compatEntries.get(fqn);
        
        if (element == null) {
            fqn = fqn + "()";
            element = compatEntries.get(fqn);
            if (element == null) {
                // Instead of Element.hasAttribute, look for element.hasAttribute
                element = compatEntries.get(Character.toLowerCase(fqn.charAt(0))+fqn.substring(1));
            }
        }
        
        if (element != null) {
            List<Element> includes = new ArrayList<Element>();
            List<Element> excludes = new ArrayList<Element>();
            List<Element> limitations = new ArrayList<Element>();
            
            NodeList nl = element.getChildNodes();
            for (int i = 0, n = nl.getLength(); i < n; i++) {
                Node node = nl.item(i);
                if (node.getNodeType() == Node.ELEMENT_NODE) {
                    Element e = (Element) node;
                    String tag = e.getTagName();
                    if ("include".equals(tag)) {
                        includes.add(e);
                    } else if ("exclude".equals(tag)) {
                        excludes.add(e);
                    } else if ("limitation".equals(tag)) {
                        limitations.add(e);
                    } else {
                        assert false : "Unexpected element " + tag;
                    }
                }
            }

            EnumSet es;
            if (excludes.size() > 0) {
                assert includes.size() == 0 : "Can't specify both includes and excludes!";
                es = EnumSet.allOf(BrowserCompat.class);
                for (Element e : excludes) {
                    Set<BrowserCompat> browsers = BrowserCompat.get(e);
                    es.removeAll(browsers);
                }
            } else if (includes.size() > 0) {
                assert excludes.size() == 0 : "Can't specify both includes and excludes!";
                es = EnumSet.noneOf(BrowserCompat.class);
                for (Element e : includes) {
                    Set<BrowserCompat> browsers = BrowserCompat.get(e);
                    es.addAll(browsers);
                }
            } else {
                // What do I do about stuff with just limitations?
                // For now, ignored
                return null;
            }

            // Compute browser flags from the given enum set
            // TODO
            String flags = BrowserCompat.toFlags(es);
            
            return flags;
        }
        
        return null;
    }
    
    enum BrowserCompat {
        // Keep synced with version in javascript/editing
        IE55, IE6, IE7, FF1, FF2, FF3, OPERA, SAFARI2, SAFARI3, KONQ;
        
        public static int getBrowserCount() {
            return values().length;
        }
        
        static Set<BrowserCompat> get(Element e) {
            String browser = e.getAttribute("browser");
            assert browser != null;
            
            String versions = e.getAttribute("versions");
            if (versions == null) {
                versions = "all";
            }

            if ("firefox".equals(browser)) {
                if ("all".equals(versions)) {
                    return EnumSet.of(IE55, IE6, IE7);
                }
                String[] v = versions.split(",");
                EnumSet es = EnumSet.noneOf(BrowserCompat.class);
                for (String version : v) {
                    BrowserCompat b;
                    if ("1".equals(version)) {
                        b = FF1;
                    } else if ("2".equals(version)) {
                        b = FF2;
                    } else if ("3".equals(version)) {
                        b = FF3;
                    } else {
                        System.err.println("Unknown version " + version + " for browser " + browser + " in " + e);
                        System.exit(0);
                        return null;
                    }
                    es.add(b);
                }
                
                return es;
            } else if ("ie".equals(browser)) {
                if ("all".equals(versions)) {
                    return EnumSet.of(IE55, IE6, IE7);
                }
                String[] v = versions.split(",");
                EnumSet es = EnumSet.noneOf(BrowserCompat.class);
                for (String version : v) {
                    BrowserCompat b;
                    if ("5.5".equals(version)) {
                        b = IE55;
                    } else if ("6".equals(version)) {
                        b = IE6;
                    } else if ("7".equals(version)) {
                        b = IE7;
                    } else {
                        System.err.println("Unknown version " + version + " for browser " + browser + " in " + e);
                        System.exit(0);
                        return null;
                    }
                    es.add(b);
                }
                
                return es;
            } else if ("safari".equals(browser)) {
                if ("all".equals(versions)) {
                    return EnumSet.of(SAFARI2, SAFARI3);
                }
                String[] v = versions.split(",");
                EnumSet es = EnumSet.noneOf(BrowserCompat.class);
                for (String version : v) {
                    BrowserCompat b;
                    if ("2".equals(version)) {
                        b = SAFARI2;
                    } else if ("3".equals(version)) {
                        b = SAFARI3;
                    } else {
                        System.err.println("Unknown version " + version + " for browser " + browser + " in " + e);
                        System.exit(0);
                        return null;
                    }
                    es.add(b);
                }
                
                return es;
            } else if ("konqueror".equals(browser)) {
                return EnumSet.of(KONQ);
            } else if ("opera".equals(browser)) {
                return EnumSet.of(OPERA);
            } else {
                throw new RuntimeException("No browsers found for " + e);
            }
        }
        
        static String toFlags(EnumSet<BrowserCompat> es) {
            StringBuilder sb = new StringBuilder();
            for (BrowserCompat b : es) {
                if (sb.length() > 0) {
                    sb.append("|");
                }
                sb.append(b.name());
            }
            return sb.toString();
        }
        
        static EnumSet<BrowserCompat> fromFlags(String flags) {
            EnumSet<BrowserCompat> es = EnumSet.noneOf(BrowserCompat.class);

            String[] versions = flags.split("\\|"); // NOI18N
            for (String version : versions) {
                BrowserCompat b = BrowserCompat.valueOf(version);

                if (b != null) {
                    es.add(b);
                } else {
                    // What do I do here?
                    assert false : version;
                }
            }

            return es;
        }
    }
}


