/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.netbeans.jsstubsgenerator.core.mdn;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 *
 * @author marfous
 */
public class ReturnType {

    private static final Map<String, List<ReturnHolder>> RETURN_TYPES = new HashMap<String, List<ReturnHolder>>();

    static {
        ReturnHolder[] globalHolders = new ReturnHolder[]{
            new ReturnHolder("Array", "Array"),
            new ReturnHolder("ArrayBuffer", "ArrayBuffer"),
            new ReturnHolder("Boolean", "Boolean"),
            new ReturnHolder("Date", "Date"),
            new ReturnHolder("decodeURI", "String"),
            new ReturnHolder("decodeURIComponent", "String"),
            new ReturnHolder("encodeURI", "String"),
            new ReturnHolder("encodeURIComponent", "String"),
            new ReturnHolder("Error", "Error"),
            new ReturnHolder("eval", "Object"),
            new ReturnHolder("EvalError", "Error"),
            new ReturnHolder("Float32Array", "Array"),
            new ReturnHolder("Float64Array", "Array"),
            new ReturnHolder("Function", "Function"),
            new ReturnHolder("Infinity", "Number"),
            new ReturnHolder("Int16Array", "Array"),
            new ReturnHolder("Int32Array", "Array"),
            new ReturnHolder("Int8Array", "Array"),
            new ReturnHolder("isFinite", "Boolean"),
            new ReturnHolder("isNaN", "Boolean"),
            new ReturnHolder("Iterator", "Iterator"),
            new ReturnHolder("JSON", "JSON"),
            new ReturnHolder("Math", "Math"),
            new ReturnHolder("NaN", "Number"),
            new ReturnHolder("Number", "Number"),
            new ReturnHolder("Object", "Object"),
            new ReturnHolder("parseFloat", "Number"),
            new ReturnHolder("parseInt", "Number"),
            new ReturnHolder("RangeError", "Error"),
            new ReturnHolder("ReferenceError", "Error"),
            new ReturnHolder("RegExp", "RegExp"),
            new ReturnHolder("StopIteration", "StopIteration"),
            new ReturnHolder("String", "String"),
            new ReturnHolder("SyntaxError", "Error"),
            new ReturnHolder("TypeError", "Error"),
            new ReturnHolder("Uint16Array", "Array"),
            new ReturnHolder("Uint32Array", "Array"),
            new ReturnHolder("Uint8Array", "Array"),
            new ReturnHolder("Uint8ClampedArray", "Array"),
            new ReturnHolder("undefined", "undefined"),
            new ReturnHolder("uneval", "Number"),
            new ReturnHolder("URIError", "Error"),
        };
        RETURN_TYPES.put("global", Arrays.asList(globalHolders));
        ReturnHolder[] EvalErrorHolders = new ReturnHolder[]{
            new ReturnHolder("name", "String"),
            new ReturnHolder("prototype", "Object"),
            new ReturnHolder("constructor", "Function"),
            new ReturnHolder("message", "String"),
            new ReturnHolder("toString", "String"),
        };
        RETURN_TYPES.put("EvalError", Arrays.asList(EvalErrorHolders));
        ReturnHolder[] MathHolders = new ReturnHolder[]{
            new ReturnHolder("toSource", "String"),
            new ReturnHolder("LN2", "Number"),
            new ReturnHolder("E", "Number"),
            new ReturnHolder("atan", "Number"),
            new ReturnHolder("pow", "Number"),
            new ReturnHolder("asin", "Number"),
            new ReturnHolder("cos", "Number"),
            new ReturnHolder("ceil", "Number"),
            new ReturnHolder("round", "Number"),
            new ReturnHolder("atan2", "Number"),
            new ReturnHolder("abs", "Number"),
            new ReturnHolder("SQRT1_2", "Number"),
            new ReturnHolder("acos", "Number"),
            new ReturnHolder("min", "Number"),
            new ReturnHolder("LN10", "Number"),
            new ReturnHolder("max", "Number"),
            new ReturnHolder("sqrt", "Number"),
            new ReturnHolder("random", "Number"),
            new ReturnHolder("SQRT2", "Number"),
            new ReturnHolder("log", "Number"),
            new ReturnHolder("exp", "Number"),
            new ReturnHolder("floor", "Number"),
            new ReturnHolder("PI", "Number"),
            new ReturnHolder("sin", "Number"),
            new ReturnHolder("tan", "Number"),
            new ReturnHolder("LOG10E", "Number"),
            new ReturnHolder("LOG2E", "Number"),
            new ReturnHolder("acosh", "Number"),
            new ReturnHolder("asinh", "Number"),
            new ReturnHolder("atanh", "Number"),
            new ReturnHolder("cbrt", "Number"),
            new ReturnHolder("clz32", "Number"),
            new ReturnHolder("cosh", "Number"),
            new ReturnHolder("tan", "Number"),
            new ReturnHolder("expm1", "Number"),
            new ReturnHolder("fround", "Number"),
            new ReturnHolder("SQRT2", "Number"),
            new ReturnHolder("hypot", "Number"),
            new ReturnHolder("imul", "Number"),
            new ReturnHolder("log10", "Number"),
            new ReturnHolder("log1p", "Number"),
            new ReturnHolder("log2", "Number"),
            new ReturnHolder("sign", "Number"),
            new ReturnHolder("sinh", "Number"),
            new ReturnHolder("tanh", "Number"),
            new ReturnHolder("trunc", "Number"),
        };
            
        RETURN_TYPES.put("Math", Arrays.asList(MathHolders));
        ReturnHolder[] ArrayHolders = new ReturnHolder[]{
            new ReturnHolder("splice", "Array"),
            new ReturnHolder("toSource", "String"),
            new ReturnHolder("sort", "Array"),
            new ReturnHolder("reduceRight", "Object"),
            new ReturnHolder("pop", "Object"),
            new ReturnHolder("indexOf", "Number"),
            new ReturnHolder("every", "Boolean"),
            new ReturnHolder("prototype", "Object"),
            new ReturnHolder("push", "Number"),
            new ReturnHolder("length", "Number"),
            new ReturnHolder("map", "Array"),
            new ReturnHolder("unshift", "Number"),
            new ReturnHolder("constructor", "Function"),
            new ReturnHolder("reverse", "Array"),
            new ReturnHolder("forEach", "undefined"),
            new ReturnHolder("lastIndexOf", "Number"),
            new ReturnHolder("join", "String"),
            new ReturnHolder("concat", "Array"),
            new ReturnHolder("reduce", "Object"),
            new ReturnHolder("slice", "Array"),
            new ReturnHolder("toString", "String"),
            new ReturnHolder("isArray", "Boolean"),
            new ReturnHolder("some", "Boolean"),
            new ReturnHolder("shift", "Object"),
            new ReturnHolder("filter", "Array"),
            new ReturnHolder("from", "Array"),
            new ReturnHolder("of", "Array"),
            new ReturnHolder("copyWithin", "undefined"),
            new ReturnHolder("fill", "undefined"),
            new ReturnHolder("find", "Object"),
            new ReturnHolder("findIndex", "Number"),
            new ReturnHolder("keys", "Array"),
            new ReturnHolder("values", "Array"),
            new ReturnHolder("includes", "Boolean"),
            new ReturnHolder("entries", "Array"),
            new ReturnHolder("toLocaleString", "String"),
            
        };
        RETURN_TYPES.put("Array", Arrays.asList(ArrayHolders));
        ReturnHolder[] RegExpHolders = new ReturnHolder[]{
            new ReturnHolder("lastIndex", "Number"),
            new ReturnHolder("toSource", "String"),
            new ReturnHolder("sticky", "Boolean"),
            new ReturnHolder("source", "String"),
            new ReturnHolder("test", "Boolean"),
            new ReturnHolder("multiline", "Boolean"),
            new ReturnHolder("global", "Boolean"),
            new ReturnHolder("toString", "String"),
            new ReturnHolder("exec", "Array"),
            new ReturnHolder("ignoreCase", "Boolean"),
            new ReturnHolder("prototype", "Object"),
            new ReturnHolder("constructor", "Function"),
            new ReturnHolder("flags", "String"),
            new ReturnHolder("unicode", "Boolean"),
        };
        RETURN_TYPES.put("RegExp", Arrays.asList(RegExpHolders));
        ReturnHolder[] DateHolders = new ReturnHolder[]{
            new ReturnHolder("toLocaleDateString", "String"),
            new ReturnHolder("getUTCSeconds", "Number"),
            new ReturnHolder("setYear", "Number"),
            new ReturnHolder("toUTCString", "String"),
            new ReturnHolder("setUTCFullYear", "Number"),
            new ReturnHolder("getTimezoneOffset", "Number"),
            new ReturnHolder("getDay", "Number"),
            new ReturnHolder("setUTCHours", "Number"),
            new ReturnHolder("getYear", "Number"),
            new ReturnHolder("prototype", "Number"),
            new ReturnHolder("setMinutes", "Number"),
            new ReturnHolder("toISOString", "String"),
            new ReturnHolder("setMilliseconds", "Number"),
            new ReturnHolder("UTC", "Number"),
            new ReturnHolder("getUTCDate", "Number"),
            new ReturnHolder("toTimeString", "String"),
            new ReturnHolder("setSeconds", "Number"),
            new ReturnHolder("setUTCSeconds", "Number"),
            new ReturnHolder("toDateString", "String"),
            new ReturnHolder("getDate", "Number"),
            new ReturnHolder("setHours", "Number"),
            new ReturnHolder("now", "Number"),
            new ReturnHolder("getUTCFullYear", "Number"),
            new ReturnHolder("getMonth", "Number"),
            new ReturnHolder("toString", "String"),
            new ReturnHolder("setUTCDate", "Number"),
            new ReturnHolder("setUTCMonth", "Number"),
            new ReturnHolder("getMinutes", "Number"),
            new ReturnHolder("toLocaleString", "String"),
            new ReturnHolder("toSource", "String"),
            new ReturnHolder("getSeconds", "Number"),
            new ReturnHolder("getUTCMonth", "Number"),
            new ReturnHolder("getFullYear", "Number"),
            new ReturnHolder("getHours", "Number"),
            new ReturnHolder("getUTCHours", "Number"),
            new ReturnHolder("toLocaleFormat", "Number"),
            new ReturnHolder("valueOf", "Number"),
            new ReturnHolder("parse", "Number"),
            new ReturnHolder("constructor", "Function"),
            new ReturnHolder("setTime", "Number"),
            new ReturnHolder("toLocaleTimeString", "String"),
            new ReturnHolder("setUTCMilliseconds", "Number"),
            new ReturnHolder("setUTCMinutes", "Number"),
            new ReturnHolder("setMonth", "Number"),
            new ReturnHolder("setDate", "Number"),
            new ReturnHolder("getUTCMinutes", "Number"),
            new ReturnHolder("setFullYear", "Number"),
            new ReturnHolder("getTime", "Number"),
            new ReturnHolder("toJSON", "Number"),
            new ReturnHolder("getUTCDay", "Number"),
            new ReturnHolder("toGMTString", "String"),
            new ReturnHolder("getUTCMilliseconds", "Number"),
            new ReturnHolder("getMilliseconds", "Number"),};
        RETURN_TYPES.put("Date", Arrays.asList(DateHolders));
        ReturnHolder[] URIErrorHolders = new ReturnHolder[]{
            new ReturnHolder("name", "String"),
            new ReturnHolder("prototype", "Object"),
            new ReturnHolder("message", "String"),
            new ReturnHolder("constructor", "Function"),
            new ReturnHolder("toString", "String"),
        };
        RETURN_TYPES.put("URIError", Arrays.asList(URIErrorHolders));
        ReturnHolder[] ObjectHolders = new ReturnHolder[]{
            new ReturnHolder("isPrototypeOf", "Boolean"),
            new ReturnHolder("toSource", "String"),
            new ReturnHolder("toLocaleString", "String"),
            new ReturnHolder("defineProperties", "undefined"),
            new ReturnHolder("unwatch", "undefined"),
            new ReturnHolder("prototype", "Object"),
            new ReturnHolder("getOwnPropertyDescriptor", "Object"),
            new ReturnHolder("getPrototypeOf", "Object"),
            new ReturnHolder("seal", "Object"),
            new ReturnHolder("propertyIsEnumerable", "Boolean"),
            new ReturnHolder("getOwnPropertyNames", "Array"),
            new ReturnHolder("valueOf", "Object"),
            new ReturnHolder("constructor", "Function"),
            new ReturnHolder("keys", "Array"),
            new ReturnHolder("watch", "undefined"),
            new ReturnHolder("eval", "Object"),
            new ReturnHolder("isFrozen", "Boolean"),
            new ReturnHolder("isExtensible", "Boolean"),
            new ReturnHolder("create", "Object"),
            new ReturnHolder("hasOwnProperty", "Boolean"),
            new ReturnHolder("isSealed", "Boolean"),
            new ReturnHolder("freeze", "Object"),
            new ReturnHolder("preventExtensions", "Object"),
            new ReturnHolder("toString", "String"),
            new ReturnHolder("defineProperty", "Object"),
            new ReturnHolder("assign", "Object"),
            new ReturnHolder("getOwnPropertySymbols", "Array"),
            new ReturnHolder("is", "Boolean"),
        };
        RETURN_TYPES.put("Object", Arrays.asList(ObjectHolders));
        ReturnHolder[] ReferenceErrorHolders = new ReturnHolder[]{
            new ReturnHolder("name", "String"),
            new ReturnHolder("prototype", "Object"),
            new ReturnHolder("constructor", "Function"),
            new ReturnHolder("message", "String"),
            new ReturnHolder("toString", "String"),
        };
        RETURN_TYPES.put("ReferenceError", Arrays.asList(ReferenceErrorHolders));
        ReturnHolder[] RangeErrorHolders = new ReturnHolder[]{
            new ReturnHolder("name", "String"),
            new ReturnHolder("prototype", "Object"),
            new ReturnHolder("constructor", "Function"),
            new ReturnHolder("message", "String"),
            new ReturnHolder("toString", "String"),
        };
        RETURN_TYPES.put("RangeError", Arrays.asList(RangeErrorHolders));
        ReturnHolder[] JSONHolders = new ReturnHolder[]{
            new ReturnHolder("stringify", "String"),
            new ReturnHolder("parse", "Array|Object"),};
        RETURN_TYPES.put("JSON", Arrays.asList(JSONHolders));
        ReturnHolder[] NumberHolders = new ReturnHolder[]{
            new ReturnHolder("toLocaleString", "String"),
            new ReturnHolder("toSource", "String"),
            new ReturnHolder("NaN", "Number"),
            new ReturnHolder("prototype", "Object"),
            new ReturnHolder("toPrecision", "Number"),
            new ReturnHolder("NEGATIVE_INFINITY", "String"),
            new ReturnHolder("toFixed", "Number"),
            new ReturnHolder("MAX_VALUE", "Number"),
            new ReturnHolder("toString", "String"),
            new ReturnHolder("valueOf", "Number"),
            new ReturnHolder("toExponential", "Number"),
            new ReturnHolder("MIN_VALUE", "Number"),
            new ReturnHolder("POSITIVE_INFINITY", "String"),
            new ReturnHolder("constructor", "Function"),
            new ReturnHolder("EPSILON", "Number"),
            new ReturnHolder("MAX_SAFE_INTEGER", "Number"),
            new ReturnHolder("MIN_SAFE_INTEGER", "Number"),
            new ReturnHolder("isFinite", "Boolean"),
            new ReturnHolder("isInteger", "Boolean"),
            new ReturnHolder("isNaN", "Boolean"),
            new ReturnHolder("isSafeInteger", "Boolean"),
            new ReturnHolder("parseFloat", "Number"),
            new ReturnHolder("parseInt", "Number"),
        };
        RETURN_TYPES.put("Number", Arrays.asList(NumberHolders));
        ReturnHolder[] FunctionHolders = new ReturnHolder[]{
            new ReturnHolder("caller", "String"),
            new ReturnHolder("apply", "Array"),
            new ReturnHolder("call", "String"),
            new ReturnHolder("toSource", "String"),
            new ReturnHolder("prototype", "Object"),
            new ReturnHolder("arity", "Number"),
            new ReturnHolder("arguments", "Array"),
            new ReturnHolder("isGenerator", "Boolean"),
            new ReturnHolder("name", "String"),
            new ReturnHolder("length", "Number"),
            new ReturnHolder("toString", "String"),
            new ReturnHolder("bind", "String"),
            new ReturnHolder("constructor", "Function"),};
        RETURN_TYPES.put("Function", Arrays.asList(FunctionHolders));
        ReturnHolder[] SyntaxErrorHolders = new ReturnHolder[]{
            new ReturnHolder("name", "String"),
            new ReturnHolder("prototype", "Object"),
            new ReturnHolder("constructor", "Function"),
            new ReturnHolder("message", "String"),
            new ReturnHolder("toString", "String"),
        };
        RETURN_TYPES.put("SyntaxError", Arrays.asList(SyntaxErrorHolders));
        ReturnHolder[] BooleanHolders = new ReturnHolder[]{
            new ReturnHolder("toSource", "String"),
            new ReturnHolder("toString", "String"),
            new ReturnHolder("valueOf", "Boolean"),
            new ReturnHolder("prototype", "Object"),
            new ReturnHolder("constructor", "Function"),};
        RETURN_TYPES.put("Boolean", Arrays.asList(BooleanHolders));
        ReturnHolder[] ErrorHolders = new ReturnHolder[]{
            new ReturnHolder("message", "String"),
            new ReturnHolder("toSource", "String"),
            new ReturnHolder("description", "String"),
            new ReturnHolder("name", "String"),
            new ReturnHolder("fileName", "String"),
            new ReturnHolder("lineNumber", "Number"),
            new ReturnHolder("toString", "String"),
            new ReturnHolder("number", "Number"),
            new ReturnHolder("stack", "String"),
            new ReturnHolder("prototype", "Object"),
            new ReturnHolder("constructor", "Function"),};
        RETURN_TYPES.put("Error", Arrays.asList(ErrorHolders));
        ReturnHolder[] TypeErrorHolders = new ReturnHolder[]{
            new ReturnHolder("name", "String"),
            new ReturnHolder("message", "String"),
            new ReturnHolder("prototype", "Object"),
            new ReturnHolder("constructor", "Function"),
            new ReturnHolder("toString", "String"),
        };
        RETURN_TYPES.put("TypeError", Arrays.asList(TypeErrorHolders));
        ReturnHolder[] StringHolders = new ReturnHolder[]{
            new ReturnHolder("toSource", "String"),
            new ReturnHolder("sub", "String"),
            new ReturnHolder("italics", "String"),
            new ReturnHolder("link", "String"),
            new ReturnHolder("fontcolor", "String"),
            new ReturnHolder("indexOf", "Number"),
            new ReturnHolder("sup", "String"),
            new ReturnHolder("fixed", "String"),
            new ReturnHolder("prototype", "Object"),
            new ReturnHolder("anchor", "String"),
            new ReturnHolder("toUpperCase", "String"),
            new ReturnHolder("strike", "String"),
            new ReturnHolder("bold", "String"),
            new ReturnHolder("search", "Number"),
            new ReturnHolder("quote", "String"),
            new ReturnHolder("substring", "String"),
            new ReturnHolder("length", "Number"),
            new ReturnHolder("valueOf", "String"),
            new ReturnHolder("toLocaleUpperCase", "String"),
            new ReturnHolder("trim", "String"),
            new ReturnHolder("charAt", "String"),
            new ReturnHolder("substr", "String"),
            new ReturnHolder("constructor", "Function"),
            new ReturnHolder("fromCharCode", "String"),
            new ReturnHolder("trimLeft", "String"),
            new ReturnHolder("localeCompare", "Number"),
            new ReturnHolder("trimRight", "String"),
            new ReturnHolder("lastIndexOf", "Number"),
            new ReturnHolder("toLocaleLowerCase", "String"),
            new ReturnHolder("concat", "String"),
            new ReturnHolder("replace", "String"),
            new ReturnHolder("blink", "String"),
            new ReturnHolder("charCodeAt", "Number"),
            new ReturnHolder("toLowerCase", "String"),
            new ReturnHolder("slice", "String"),
            new ReturnHolder("split", "Array"),
            new ReturnHolder("toString", "String"),
            new ReturnHolder("match", "Array"),
            new ReturnHolder("big", "String"),
            new ReturnHolder("small", "String"),
            new ReturnHolder("fontsize", "Number"),
            new ReturnHolder("raw", "String"),
            new ReturnHolder("startsWith", "Boolean"),
            new ReturnHolder("repeat", "String"),
            new ReturnHolder("normalize", "String"),
            new ReturnHolder("link", "String"),
            new ReturnHolder("includes", "Boolean"),
            new ReturnHolder("endsWith", "Boolean"),
            new ReturnHolder("codePointAt", "Number"),
            new ReturnHolder("fromCodePoint", "String"),
        };
        RETURN_TYPES.put("String", Arrays.asList(StringHolders));
        
        ReturnHolder[] ArrayBuffer = new ReturnHolder[]{
            new ReturnHolder("byteLength", "Number"),
            new ReturnHolder("slice", "Object"),
            new ReturnHolder("isView", "Boolean"),};
        RETURN_TYPES.put("ArrayBuffer", Arrays.asList(ArrayBuffer));
        
        ReturnHolder[] DataView = new ReturnHolder[]{
            new ReturnHolder("getUint8", "Number"),
            new ReturnHolder("setInt32", "undefined"),
            new ReturnHolder("getFloat32", "Number"),
            new ReturnHolder("getUint16", "Number"),
            new ReturnHolder("setFloat32", "undefined"),
            new ReturnHolder("getFloat64", "Number"),
            new ReturnHolder("getInt8", "Number"),
            new ReturnHolder("setInt16", "undefined"),
            new ReturnHolder("setInt8", "undefined"),
            new ReturnHolder("byteOffset", "Number"),
            new ReturnHolder("getInt32", "Number"),
            new ReturnHolder("setFloat64", "undefined"),
            new ReturnHolder("getInt16", "Number"),
            new ReturnHolder("setUint8", "undefined"),
            new ReturnHolder("getUint32", "Number"),
            new ReturnHolder("setUint32", "undefined"),
            new ReturnHolder("setUint16", "undefined"),
            new ReturnHolder("buffer", "Object"),
            new ReturnHolder("byteLength", "Number"),
        };
        RETURN_TYPES.put("DataView", Arrays.asList(DataView));
        
        ReturnHolder[] Infinity = new ReturnHolder[]{};
        RETURN_TYPES.put("Infinity", Arrays.asList(Infinity));
    
        ReturnHolder[] Map = new ReturnHolder[]{
            new ReturnHolder("size", "Number"),
            new ReturnHolder("entries", "Array"),
            new ReturnHolder("values", "Array"),
            new ReturnHolder("forEach", "undefined"),
            new ReturnHolder("get", "Object"),
            new ReturnHolder("keys", "Array"),
            new ReturnHolder("delete", "Boolean"),
            new ReturnHolder("set", "Object"),
            new ReturnHolder("has", "Boolean"),
            new ReturnHolder("clear", "undefined"),
        };
        RETURN_TYPES.put("Map", Arrays.asList(Map));
        
        ReturnHolder[] NaN = new ReturnHolder[]{};
        RETURN_TYPES.put("NaN", Arrays.asList(NaN));
        
        ReturnHolder[] Promise = new ReturnHolder[]{
            new ReturnHolder("then", "Promise"),
            new ReturnHolder("resolve", "Promise"),
            new ReturnHolder("race", "Promise"),
            new ReturnHolder("reject", "Promise"),
            new ReturnHolder("catch", "Promise"),
            new ReturnHolder("all", "Promise"),
        };
        RETURN_TYPES.put("Promise", Arrays.asList(Promise));
        
        ReturnHolder[] Proxy = new ReturnHolder[]{
            new ReturnHolder("defineProperty", "Boolean"),
            new ReturnHolder("set", "Boolean"),
            new ReturnHolder("preventExtensions", "Boolean"),
            new ReturnHolder("deleteProperty", "Boolean"),
            new ReturnHolder("get", "Object"),
            new ReturnHolder("construct", "Object"),
            new ReturnHolder("getOwnPropertyDescriptor", "Object"),
            new ReturnHolder("apply", "Object"),
            new ReturnHolder("setPrototypeOf", "Boolean"),
            new ReturnHolder("has", "Boolean"),
            new ReturnHolder("ownKeys", "Array"),
            new ReturnHolder("getPrototypeOf", "Object"),
            new ReturnHolder("isExtensible", "Boolean"),
        };
        RETURN_TYPES.put("Proxy", Arrays.asList(Proxy));
        
        ReturnHolder[] Reflect = new ReturnHolder[]{
            new ReturnHolder("getOwnPropertyDescriptor", "Object"),
            new ReturnHolder("set", "Boolean"),
            new ReturnHolder("getPrototypeOf", "Boolean"),
            new ReturnHolder("isExtensible", "Boolean"),
            new ReturnHolder("setPrototypeOf", "Boolean"),
            new ReturnHolder("construct", "Boolean"),
            new ReturnHolder("get", "Object"),
            new ReturnHolder("preventExtensions", "Boolean"),
            new ReturnHolder("deleteProperty", "Boolean"),
            new ReturnHolder("defineProperty", "Boolean"),
            new ReturnHolder("ownKeys", "Array"),
            new ReturnHolder("has", "Boolean"),
            new ReturnHolder("apply", "undefined"),
        };
        RETURN_TYPES.put("Reflect", Arrays.asList(Reflect));
        
        ReturnHolder[] Set = new ReturnHolder[]{
            new ReturnHolder("size", "Number"),
            new ReturnHolder("has", "Boolean"),
            new ReturnHolder("forEach", "undefined"),
            new ReturnHolder("clear", "undefined"),
            new ReturnHolder("add", "Object"),
            new ReturnHolder("entries", "Array"),
            new ReturnHolder("values", "Array"),
            new ReturnHolder("delete", "Boolean"),      
        };
        RETURN_TYPES.put("Set", Arrays.asList(Set));
        
        ReturnHolder[] Symbol = new ReturnHolder[]{
            new ReturnHolder("for", "Symbol"),
            new ReturnHolder("toString", "String"),
            new ReturnHolder("search", "Number"),
            new ReturnHolder("toPrimitive", "undefined"),
            new ReturnHolder("split", "Array"),
            new ReturnHolder("replace", "String"),
            new ReturnHolder("keyFor", "String"),
            new ReturnHolder("unscopables", "Object"),
            new ReturnHolder("iterator", "Array"),
            new ReturnHolder("match", "Array"),
            new ReturnHolder("isConcatSpreadable", "Boolean"),
            new ReturnHolder("species", "undefined"),
            new ReturnHolder("valueOf", "undefined"),
        };
        RETURN_TYPES.put("Symbol", Arrays.asList(Symbol));
        
        ReturnHolder[] WeakMap = new ReturnHolder[]{
            new ReturnHolder("get", "Object"),
            new ReturnHolder("set", "Object"),
            new ReturnHolder("has", "Boolean"),
            new ReturnHolder("delete", "Boolean"),
        };
        RETURN_TYPES.put("WeakMap", Arrays.asList(WeakMap));
        
        ReturnHolder[] WeakSet = new ReturnHolder[]{
            new ReturnHolder("has", "Boolean"),
            new ReturnHolder("delete", "Boolean"),
            new ReturnHolder("add", "Object"),
        };
        RETURN_TYPES.put("WeakSet", Arrays.asList(WeakSet));
        
        ReturnHolder[] TypedArray = new ReturnHolder[]{
            new ReturnHolder("splice", "Array"),
            new ReturnHolder("toSource", "String"),
            new ReturnHolder("sort", "Array"),
            new ReturnHolder("reduceRight", "Object"),
            new ReturnHolder("pop", "Object"),
            new ReturnHolder("indexOf", "Number"),
            new ReturnHolder("every", "Boolean"),
            new ReturnHolder("prototype", "Object"),
            new ReturnHolder("push", "Number"),
            new ReturnHolder("length", "Number"),
            new ReturnHolder("map", "Array"),
            new ReturnHolder("unshift", "Number"),
            new ReturnHolder("constructor", "Function"),
            new ReturnHolder("reverse", "Array"),
            new ReturnHolder("forEach", "undefined"),
            new ReturnHolder("lastIndexOf", "Number"),
            new ReturnHolder("join", "String"),
            new ReturnHolder("concat", "Array"),
            new ReturnHolder("reduce", "Object"),
            new ReturnHolder("slice", "Array"),
            new ReturnHolder("toString", "String"),
            new ReturnHolder("isArray", "Boolean"),
            new ReturnHolder("some", "Boolean"),
            new ReturnHolder("shift", "Object"),
            new ReturnHolder("filter", "Array"),
            new ReturnHolder("from", "Array"),
            new ReturnHolder("of", "Array"),
            new ReturnHolder("copyWithin", "undefined"),
            new ReturnHolder("fill", "undefined"),
            new ReturnHolder("find", "Object"),
            new ReturnHolder("findIndex", "Number"),
            new ReturnHolder("keys", "Array"),
            new ReturnHolder("values", "Array"),
            new ReturnHolder("includes", "Boolean"),
            new ReturnHolder("entries", "Array"),
            new ReturnHolder("name", "String"),
            new ReturnHolder("subarray", "Array"),
            new ReturnHolder("byteLength", "Number"),
            new ReturnHolder("buffer", "Object"),
            new ReturnHolder("byteOffset", "Number"),
            new ReturnHolder("BYTES_PER_ELEMENT", "Number"),
            new ReturnHolder("set", "undefined"),
        };
        RETURN_TYPES.put("Float32Array", Arrays.asList(TypedArray));
        RETURN_TYPES.put("Float64Array", Arrays.asList(TypedArray));
        RETURN_TYPES.put("Int8Array", Arrays.asList(TypedArray));
        RETURN_TYPES.put("Int16Array", Arrays.asList(TypedArray));
        RETURN_TYPES.put("Int32Array", Arrays.asList(TypedArray));
        RETURN_TYPES.put("Uint8Array", Arrays.asList(TypedArray));
        RETURN_TYPES.put("Uint16Array", Arrays.asList(TypedArray));
        RETURN_TYPES.put("Uint32Array", Arrays.asList(TypedArray));
        RETURN_TYPES.put("Uint8ClampedArray", Arrays.asList(TypedArray));
        
        ReturnHolder[] undefined = new ReturnHolder[]{};
        RETURN_TYPES.put("undefined", Arrays.asList(undefined));
        
        ReturnHolder[] Generator = new ReturnHolder[]{
            new ReturnHolder("throw", "Object"),
            new ReturnHolder("next", "Object"),
            new ReturnHolder("return", "undefined"),

        };
        RETURN_TYPES.put("Generator", Arrays.asList(Generator));
        
        ReturnHolder[] GeneratorFunction = new ReturnHolder[]{
        };
        RETURN_TYPES.put("GeneratorFunction", Arrays.asList(GeneratorFunction));
        
        
    }

    public static String serachFor(String parent, String name) {
        List<ReturnHolder> list = RETURN_TYPES.get(parent);
        if (list == null) {
            return null;
        }

        for (ReturnHolder returnHolder : list) {
            if (returnHolder.getName().equals(name)) {
                return returnHolder.getReturnType();
            }
        }

        return null;
    }

    public static String guessReturns(String parent, String name, String description) {
        // Function.toString case
        if ("toString".equals(parent) && "indentation".equals(name)) {
            return "Number";
        }

        // name specific
        if ("every".equals(name) || name.startsWith("is") || "some".equals(name)) {
            return "Boolean";
        }
        if ("forEach".equals(name)) {
            return "undefined";
        }
        if ("constructor".equals(name) || "callback".equals(name) || "function".equals(name)) {
            return "Function";
        }
        if ("map".equals(name)) {
            return "Array";
        }
        if ("indexOf".equals(name) || "lastIndexOf".equals(name) || "length".equals(name) || "fromIndex".equals(name)
                || name.endsWith("Number") || name.endsWith("number") || name.startsWith("start") || name.startsWith("end")
                || "limit".equals(name)) {
            return "Number";
        }
        if ("toSource".equals(name) || "toString".equals(name) || "string".equals(name) || "flags" .equals(name) || "separator".equals(name)) {
            return "String";
        }
        if ("regexp".equals(name)) {
            return "RegExp";
        }

        // parent, name specific - Function
        if ("apply".equals(parent) && "argsArray".equals(name)) {
            return "Array";
        }
        if ("bind".equals(parent)) {
            if ("thisArg".equals(name)) {
                return "Object";
            } else if ("arg1,arg2,...".equals(name)) {
                return "Array";
            }
        }

        // parent, name specific - JSON
        if ("stringify".equals(parent)) {
            if ("value".equals(name)) {
                return "Object";
            } else if ("replacer".equals(name)) {
                return "Function";
            } else if ("space".equals(name)) {
                return "Boolean";
            }
        }
        if ("parse".equals(parent)) {
            if ("text".equals(name)) {
                return "String";
            } else if ("reviver".equals(name)) {
                return "Function";
            }
        }

        // parent, name specific - Array
        if ("Array".equals(parent)) {
            if (name.startsWith("reduce") || "pop".equals(name) || "shift".equals(name) ) {
                return "Object";
            } else if ("push".equals(name)) {
                return "Number";
            } else if ("unshift".equals(name)) {
                return "Number";
            } else if ("join".equals(name)) {
                return "String";
            }
        }
        if ("splice".equals(parent) && ("index".equals(name) || "howMany".equals(name))) {
            return "Number";
        }
        if ("slice".equals(parent) && ("begin".equals(name) || "end".equals(name))) {
            return "Number";
        }
        if ("lastIndexOf".equals(parent) && "searchElement".equals(name)) {
            return "Object";
        }
        if ("join".equals(parent) && "separator".equals(name)) {
            return "String";
        }

        // Math specific
        if ("Math".equals(parent) && (name.toUpperCase().equals(name) || "pow".equals(name) || "atan2".equals(name) || "exp".equals(name))) {
            return "Number";
        }
        if ("pow".equals(parent) && "exponent".equals(name)) {
            return "Number";
        }

        // object and its methods
        if ("Object".equals(parent)) {
            if ("defineProperties".equals(name) || "watch".equals(name) || "unwatch".equals(name)) {
                return "undefined";
            }
            if ("getOwnPropertyDescriptor".equals(name) || "seal".equals(name) || "valueOf".equals(name)
                    || "eval".equals(name) || "freeze".equals(name) || "preventExtensions".equals(name)
                    || "defineProperty".equals(name)) {
                return "Object";
            }
            if ("keys".equals(name)) {
                return "Array";
            }

        }
        if ("create".equals(parent) && "propertiesObject".equals(name)) {
            return "Object";
        }

        if ("JSON".equals(parent)) {
            if ("stringify".equals(name)) {
                return "String";
            }
            if ("caller".equals(name) || "bind".equals(name)) {
                return "Function";
            }
            if ("call".equals(name)) {
                return "undefined";
            }
            if ("parse".equals(name)) {
                return "Array|Object";
            }
            if (name.endsWith("INFINITY")) {
                return "Number";
            }

        }
        if ("Date".equals(parent)) {
            if (name.endsWith("String")) {
                return "String";
            } else {
                return "Number";
            }
        }
        if ("Error".equals(parent)) {
            if ("lineNumber".equals(name) || "number".equals(name)) {
                return "Number";
            }
        }
        if (parent.contains("Error")) {
            if ("message".equals(name)) {
                return "String";
            }
        }
 
        // String specific
        if ("String".equals(parent)) {
            if ("search".equals(name)) {
                return "Number";
            }
            if ("substr".equals(name)) {
                return "String";
            }
            if ("match".equals(name)) {
                return "Array";
            }
        }
        if ("replace".equals(parent)) {
            if ("newSubStr".equals(name)) {
                return "String";
            }
        }
        if ("slice".equals(parent) && ("beginSlice".equals(name) || "endSlice".equals(name))) {
            return "Number";
        }

        if ("RegExp".equals(parent)) {
            if ("sticky".equals(name) || "test".equals(name) || "multiline".equals(name)
                    || "global".equals(name) || "ignoreCase".equals(name)) {
                return "Boolean";
            } else if ("source".equals(name)) {
                return "String";
            }
        }
        
        if ("isFinite".equals(parent)) {
            if ("value".equals(name)) {
                return "undefined";
            }
        }

        if ("UTC".equals(parent)) {
            return "Number";
        }

        if ("global".equals(parent)) {
            if ("escape".equals(name) || "unescape".equals(name)) {
                return "String";
            }
        }
        
        if ("executor".equals(name)) {
            return "Function";
        }
        
        if ("parseInt".equals(name) || "parseFloat".equals(name)) {
            return "Number";
        }
        
        if ("eval".equals(name)) {
            return "undefined";
        }
        
        if ("littleEndian".equals(name)) {
            return "Boolean";
        }

        // else
        if (description.matches(".*[nN]ame.*")) {
            return "String";
        } else if (description.matches(".*[Bb]uffer.*") || description.matches(".*[pP]rototype.*")  || description.matches(".*[oO]bject.*")) {
            return "Object";
        } else if (description.matches(".*[Aa]rray.*")) {
            return "Array";
        } else if (description.matches(".*[Bb]oolean.*")) {
            return "Boolean";
        } else if (description.matches(".*[Ss]ize.*") || description.matches(".*[Ii]nteger.*") || description.matches(".*[Nn]umber.*")
                || description.matches(".*[Oo]ffset.*") || description.matches(".*[Nn]umeric.*")) {
            return "Number";
        } else {
            return "String";
        }
    }

    private static class ReturnHolder {

        private String name;
        private String returnType;

        public ReturnHolder(String name, String returnType) {
            this.name = name;
            this.returnType = returnType;
        }

        public String getName() {
            return name;
        }

        public String getReturnType() {
            return returnType;
        }
    }
}
