/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.netbeans.jsstubsgenerator.dom;

import java.io.IOException;
import java.util.List;
import java.util.Map;
import org.netbeans.jsstubsgenerator.stub.GeneratorUtils;
import org.netbeans.jsstubsgenerator.stub.Origin;
import org.netbeans.jsstubsgenerator.stub.Parameter;
import org.netbeans.jsstubsgenerator.stub.Return;
import org.netbeans.jsstubsgenerator.stub.StubObject;
import org.netbeans.jsstubsgenerator.stub.Type;

/**
 *
 * @author Martin Fousek <marfous@netbeans.org>
 */
public class NavigatorInstanceGenerator {

    public static void generate(Map<String, StubObject> allStubs) throws IOException {
        // general definition
        StubObject navigatorInstance = new StubObject(Origin.DOM, "Navigator", StubObject.GLOBAL);
        navigatorInstance.setPrune(true);
        navigatorInstance.setType(Type.OBJECT);
        navigatorInstance.setSummary("Returns a reference to the navigator object, which can be queried for information about the application running the script.");

        // properties definition
        String[][] attributes = new String[][]{
            {"appCodeName", "String", "Returns the internal \"code\" name of the current browser."},
            {"appName", "String", "Returns the name of the browser. The HTML5 specification also allows any browser to return \"Netscape\" here, for compatibility reasons.<br>Note: Do not rely on this property to return the correct browser name. In Gecko-based browsers (like Firefox) and WebKit-based browsers (like Chrome and Safari) the returned string is \"Netscape\"."},
            {"appVersion", "String", "Returns the version of the browser as a string. It may be either a plain version number, like \"5.0\", or a version number followed by more detailed information. The HTML5 specification also allows any browser to return \"4.0\" here, for compatibility reasons."},
            {"buildID", "String", "Returns the build identifier of the browser."},
            {"cookieEnabled", "Boolean", "Returns a Boolean value indicating whether cookies are enabled or not (read-only)."},
            {"doNotTrack", "String", "Returns the user's do-not-track setting. This is \"yes\" if the user has requested not to be tracked by web sites, content, or advertising."},
            {"language", "String", "Returns a string representing the language version of the browser."},
            {"mimeTypes", "String", "Returns a MimeTypeArray object, which contains a list of MimeType objects representing the MIME types recognized by the browser. "},
            {"onLine", "Boolean", "Returns the online status of the browser. The property returns a boolean value, with true for being online and false for being offline. The property sends updates whenever the browser's ability to connect to the network changes. The update occurs when the user follows links or when a script requests a remote page. For example, the property should return false when users click links soon after they lose internet connection.<br>Browsers implement this property differently.<br>In Chrome and Safari, if the browser is not able to connect to a local area network (LAN) or a router, it is offline; all other conditions return true. So while you can assume that the browser is offline when it returns a false value, you cannot assume that a true value necessarily means that the browser can access the internet. You could be getting false positives, such as in cases where the computer is running a virtualization software that has virtual ethernet adapters that are always \"connected.\" Therefore, if you really want to determine the online status of the browser, you should develop additional means for checking. To learn more, see the HTML5 Rocks article, Working Off the Grid.<br>In Firefox and Internet Explorer, switching the browser to offline mode sends a false value. All other conditions return a true value."},
            {"oscpu", "String", "Returns a string that identifies the current operating system."},
            {"platform", "String", "Returns a string representing the platform of the browser."},
            {"plugins", "Array", "Returns a PluginArray object, listing the plugins installed in the application."},
            {"product", "String", "This property returns the product name of the current browser.<br>Note: Do not rely on this property to return a real product name. Both Gecko and WebKit browsers return \"Gecko\" as the value of this property."},
            {"productSub", "String", "productSub returns the build number of the current browser. "},
            {"userAgent", "String", "Returns the user agent string for the current browser."},
            {"vendor", "String", "Returns the name of the browser vendor for the current browser."},
            {"vendorSub", "String", "vendorSub is the substring of the vendor having to do with the vendor version number."},
        };
        List<StubObject> generatedProperties = GeneratorUtils.generateProperties(Origin.DOM, attributes, "Navigator", true);
        for (StubObject instanceProperty : generatedProperties) {
            navigatorInstance.putAttribute(instanceProperty);
        }

        // methods definition
        String[][] methods = new String[][]{
            {"javaEnabled", "Boolean", "This method indicates whether the current browser is Java-enabled or not."},
            {"registerContentHandler", "undefined", "Allows web sites to register themselves as possible handlers for content of a particular MIME type.<br>Note: Web sites may only register content handlers for themselves. For security reasons, it's not possible for an extension or web site to register content handlers targeting other sites."},
            {"registerProtocolHandler", "undefined", "Allows web sites to register themselves as possible handlers for particular protocols.<br>Note: Web sites may only register protocol handlers for themselves. For security reasons, it's not possible for an extension or web site to register protocol handlers targeting other sites."},
        };
        List<StubObject> generatedMethods = generateMethods(methods);
        for (StubObject instanceMethod : generatedMethods) {
            navigatorInstance.putAttribute(instanceMethod);
        }

        allStubs.put("Navigator", navigatorInstance);
    }

    private static List<StubObject> generateMethods(String[][] methods) {
        List<StubObject> methodStubs = GeneratorUtils.generateMethods(methods, "Navigator", true);
        for (StubObject method : methodStubs) {
            if ("javaEnabled".equals(method.getName())) {
                method.setSyntax("result = window.navigator.javaEnabled();");
                method.setReturns(new Return("Boolean", "The return value for this method indicates whether the preference that controls Java is on or off - not whether the browser offers Java support in general."));
            } else if ("registerContentHandler".equals(method.getName())) {
                method.setSyntax("window.navigator.registerContentHandler(mimeType, uri, title);");
                method.addParameter(new Parameter("mimeType", "String", "mimeType is the desired MIME type as a string"));
                method.addParameter(new Parameter("uri", "String", "uri is the URI to the handler as a string"));
                method.addParameter(new Parameter("title", "String", "title is the title of the handler presented to the user as a string"));
            } else if ("registerProtocolHandler".equals(method.getName())) {
                method.setSyntax("window.navigator.registerProtocolHandler(protocol, uri, title);");
                method.addParameter(new Parameter("protocol", "String", "protocol is the protocol the site wishes to handle, specified as a string"));
                method.addParameter(new Parameter("uri", "String", "uri is the URI to the handler as a string. You can include \"%s\" to indicate where to insert the escaped URI of the document to be handled."));
                method.addParameter(new Parameter("title", "String", "title is the title of the handler presented to the user as a string"));
            }
        }
        return methodStubs;
    }

}
