/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.test.uihandler.selenium;

import java.util.Calendar;

/**
 *
 * @author Jindrich Sedek
 */
public class DupsQueryTest extends SeleniumTestBase {
    private static final Calendar cald = Calendar.getInstance();

    public void testComponents() throws Exception {
        selenium.open("analytics/dupsquery.do");
        selenium.type("name=minimum", "30");
        selenium.type("name=maximum", "30");
        selenium.type("name=component", "javaee");
        selenium.type("name=subcomponent", "JSP");
        assertEquals("JSP", selenium.getSelectedLabel("name=subcomponent"));
    }

    public void testCalendar() throws Exception {
        selenium.open("analytics/dupsquery.do");
        int day = cald.get(Calendar.DAY_OF_MONTH);
        String newDay = day > 5 ? String.valueOf(day - 3) : String.valueOf(day + 3);
        verifyTrue(selenium.getValue("name=creationFrom").contains(String.valueOf(day)));
        verifyFalse(selenium.getValue("name=creationFrom").contains(newDay));
        enableDates();
        selenium.click("xpath=//a[contains(@onclick,'creationFrom')]");
        selenium.selectWindow("title=Calendar");
        selenium.click("xpath=//a[text()=" + newDay + "]");
        selenium.selectWindow(null);
        verifyTrue(selenium.getValue("name=creationFrom").contains(newDay));
    }

    public void testCalendar2() throws Exception {
        selenium.open("analytics/dupsquery.do");
        int day = cald.get(Calendar.DAY_OF_MONTH);
        String newDay = day > 5 ? String.valueOf(day - 3) : String.valueOf(day + 3);
        verifyTrue(selenium.getValue("name=creationTo").contains(String.valueOf(day)));
        verifyFalse(selenium.getValue("name=creationTo").contains(newDay));
        enableDates();
        selenium.click("xpath=//a[contains(@onclick,'creationTo')]");
        selenium.selectWindow("title=Calendar");
        selenium.click("xpath=//a[text()=" + newDay + "]");
        selenium.selectWindow(null);
        verifyTrue(selenium.getValue("name=creationTo").contains(newDay));
    }

    public void testAllSubcomponents() throws Exception {
        selenium.open("/analytics/dupsquery.do");
        selenium.addSelection("component", "label=performance");
        selenium.addSelection("subcomponent", "label=Code");
        assertTrue(selenium.isEditable("subcomponent"));

        selenium.click("allsubcomponents");
        assertFalse(selenium.isEditable("subcomponent"));

        selenium.click("allsubcomponents");
        assertTrue(selenium.isEditable("subcomponent"));

        assertTrue(selenium.isSomethingSelected("subcomponent"));
        assertEquals("Code", selenium.getSelectedValue("subcomponent"));
    }

    public void testSimpleSubmit(){
        selenium.open("/analytics/dupsquery.do");
        selenium.type("name=component", "javaee");
        selenium.type("name=subcomponent", "JSP");
        selenium.click("submit");
        waitForPageToLoad();
        assertTrue(selenium.isTextPresent("Exceptions"));
    }

    public void testSubmitComplex(){
        selenium.open("/analytics/dupsquery.do");
	selenium.type("minimum", "10");
	selenium.type("maximum", "20");
	selenium.addSelection("component", "label=platform");
	selenium.click("allsubcomponents");
	selenium.click("status");
	selenium.click("//input[@name='status' and @value='unreported']");
	selenium.click("//input[@name='iz_status' and @value='ALL']");
	selenium.click("useDates");
	selenium.click("showDuplicates");
	selenium.click("submit");
	waitForPageToLoad();
        assertTrue(selenium.isTextPresent("Exceptions"));
    }

    private void enableDates() {
        selenium.click("name=useDates");
        assertTrue(selenium.isChecked("name=useDates"));
        assertTrue(selenium.isEditable("name=creationTo"));
    }
}
