/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package org.netbeans.server.uihandler.statistics;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.LogRecord;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;
import javax.servlet.jsp.PageContext;
import org.netbeans.server.uihandler.Statistics;
import org.netbeans.server.uihandler.statistics.UnitTestLibraryUsage.LibraryUsageBean;
import org.openide.util.lookup.ServiceProvider;

/**
 *
 * @author Theofanis Oikonomou
 */
@ServiceProvider(service=Statistics.class)
public class UnitTestLibraryUsage extends Statistics<LibraryUsageBean> {
    
    private static final LibraryUsageBean EMPTY = new LibraryUsageBean();
    private static final String MESSAGE = "USG_UNIT_TESTS_LIBRARY";
    private static final String UNIT_TESTS_LIBRARY_COUNT = "UNIT_TESTS_LIBRARY_COUNT";

    public UnitTestLibraryUsage() {
        super("UnitTestLibraryUsage");
    }

    @Override
    protected LibraryUsageBean newData() {
        return EMPTY;
    }

    @Override
    protected LibraryUsageBean process(LogRecord rec) {
        
        if (MESSAGE.equals(rec.getMessage())) {
            Object[] params = rec.getParameters();
            if (params != null) {
                return new LibraryUsageBean(params);
            }
        }
        return newData();
    }

    @Override
    protected LibraryUsageBean finishSessionUpload(String userId, int sessionNumber, boolean initialParse, LibraryUsageBean d) {
        return d;
    }

    @Override
    protected void registerPageContext(PageContext page, String name, LibraryUsageBean data) {
        page.setAttribute(name, data.libraryCount);
    }

    @Override
    protected LibraryUsageBean join(LibraryUsageBean one, LibraryUsageBean two) {
        if (EMPTY.equals(one)){
            return two;
        }
        if (EMPTY.equals(two)){
            return one;
        }
        LibraryUsageBean result = new LibraryUsageBean();
        result.libraryCount = new HashMap<String, Integer>(one.libraryCount);
        addAllCounts(result.libraryCount, two.libraryCount);

        return result;
    }

    @Override
    protected void write(Preferences pref, LibraryUsageBean d) throws BackingStoreException {
        writeMap(pref.node(UNIT_TESTS_LIBRARY_COUNT), d.libraryCount);
    }

    @Override
    public LibraryUsageBean read(Preferences pref) throws BackingStoreException {
        LibraryUsageBean bean = new LibraryUsageBean();
        bean.libraryCount = readLibraryCounts(pref.node(UNIT_TESTS_LIBRARY_COUNT));
        return bean;
    }
    
    private Map<String, Integer> readLibraryCounts(Preferences node) throws BackingStoreException {
        String[] keys = node.keys();
        Map<String, Integer> map = new HashMap<String, Integer>(keys.length);
        for (String string : keys) {
            map.put(string, node.getInt(string, 0));
        }
        return map;
    }
    
    public static final class LibraryUsageBean {
        
        Map<String, Integer> libraryCount;

        private LibraryUsageBean() {
        }

        private LibraryUsageBean(Object[] params) {
            String library = params[0].toString();
            libraryCount = Collections.singletonMap(library, 1);
        }

        public Map<String, Integer> getCounts(){
            return libraryCount;
        }
    }
}
