/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2015 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2015 Denis Anisimov
 */
package org.netbeans.server.uihandler.statistics;

import java.util.ArrayList;
import java.util.Collections;
import java.util.EnumMap;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.Set;
import java.util.prefs.AbstractPreferences;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;
import junit.framework.Assert;
import org.netbeans.junit.NbTestCase;

/**
 *
 * @author denis
 */
public class VaadinTest extends NbTestCase{
    
    private Random random = new Random();
    
    public VaadinTest(String name) {
        super(name);
    }
    
    public void testReadWriteIdsOnly() throws BackingStoreException{
        Vaadin vaadin = new Vaadin();
        Preferences prefs = new Prefs(null, "");
        
        Set<String> ids = setUpIds();
        
        Map<Vaadin.DataType,Map<String, Vaadin.StatData>> map = 
                new EnumMap<Vaadin.DataType, Map<String, Vaadin.StatData>>(Vaadin.DataType.class);

        Map<String,Integer> addons = new HashMap<String, Integer>();
        
        Vaadin.VaadinData data = new Vaadin.VaadinData(ids,map, addons);
        
        vaadin.write(prefs, data);
        
        Vaadin.VaadinData read = vaadin.read(prefs);
        Assert.assertEquals("Unexpected idnetifiers after read", ids, read.getIds());
        Assert.assertEquals("Unepxected actions after read", 0, read.getActions().size());
        Assert.assertEquals("Unepxected widgets after read", 0, read.getComponents().size());
        Assert.assertEquals("Unepxected fixesafter read", 0, read.getFixes().size());
        Assert.assertEquals("Unepxected projects after read", 0, read.getProjects().size());
        Assert.assertEquals("Unepxected add-ons after read", 0, read.getAddons().size());
    }
    
        
    public void testReadWriteAddonsData() throws BackingStoreException{
        Vaadin vaadin = new Vaadin();
        Preferences prefs = new Prefs(null, "");
        
        Set<String> ids = setUpIds();
        
        Map<Vaadin.DataType,Map<String, Vaadin.StatData>> map = 
                new EnumMap<Vaadin.DataType, Map<String, Vaadin.StatData>>(Vaadin.DataType.class);

        Map<String,Integer> addons = new HashMap<String, Integer>();
        
        for(String id : ids){
            addons.put(id, random.nextInt());
        }
        
        Vaadin.VaadinData data = new Vaadin.VaadinData(ids,map, addons);
        
        vaadin.write(prefs, data);
        
        Vaadin.VaadinData read = vaadin.read(prefs);
        
        Assert.assertEquals("Unexpected idnetifiers after read", ids, read.getIds());
        Assert.assertEquals("Unepxected actions after read", 0, read.getActions().size());
        Assert.assertEquals("Unepxected widgets after read", 0, read.getComponents().size());
        Assert.assertEquals("Unepxected fixesafter read", 0, read.getFixes().size());
        Assert.assertEquals("Unepxected projects after read", 0, read.getProjects().size());
             
        Assert.assertEquals("Unepxected add-ons after read",addons,  data.getAddons());
    }
    
    public void testReadWriteStatData() throws BackingStoreException{
        Vaadin vaadin = new Vaadin();
        Preferences prefs = new Prefs(null, "");
        
        Set<String> ids = setUpIds();
        List<String> orderedIds = new ArrayList<String>(ids);
        
        Map<Vaadin.DataType,Map<String, Vaadin.StatData>> map = 
                new EnumMap<Vaadin.DataType, Map<String, Vaadin.StatData>>(Vaadin.DataType.class);
        
        Map<String, Vaadin.StatData> fixesMap = makeFixes(orderedIds);
        map.put(Vaadin.DataType.FIX, fixesMap);
        
        Map<String, Vaadin.StatData> actionsMap = makeActions(orderedIds);
        map.put(Vaadin.DataType.ACTION, actionsMap);
        
        Map<String, Vaadin.StatData> projectsMap = makeProjects(orderedIds);
        map.put(Vaadin.DataType.PROJECT, projectsMap);
        
        Map<String, Vaadin.StatData> widgetsMap = makeWidgets(orderedIds);
        map.put(Vaadin.DataType.COMPONENT, widgetsMap);

        Map<String,Integer> addons = new HashMap<String, Integer>();
        
        Vaadin.VaadinData data = new Vaadin.VaadinData(ids,map, addons);
        
        vaadin.write(prefs, data);
        
        Vaadin.VaadinData read = vaadin.read(prefs);
        
        Assert.assertEquals("Unepxected add-ons after read", 0, read.getAddons().size());
        
        checkStatData(ids, read, fixesMap, orderedIds, actionsMap, projectsMap, widgetsMap);
    }
    
    public void testReadWriteAll() throws BackingStoreException{
        Vaadin vaadin = new Vaadin();
        Preferences prefs = new Prefs(null, "");
        
        Set<String> ids = setUpIds();
        List<String> orderedIds = new ArrayList<String>(ids);
        
        Map<Vaadin.DataType,Map<String, Vaadin.StatData>> map = 
                new EnumMap<Vaadin.DataType, Map<String, Vaadin.StatData>>(Vaadin.DataType.class);
        
        Map<String, Vaadin.StatData> fixesMap = makeFixes(orderedIds);
        map.put(Vaadin.DataType.FIX, fixesMap);
        
        Map<String, Vaadin.StatData> actionsMap = makeActions(orderedIds);
        map.put(Vaadin.DataType.ACTION, actionsMap);
        
        Map<String, Vaadin.StatData> projectsMap = makeProjects(orderedIds);
        map.put(Vaadin.DataType.PROJECT, projectsMap);
        
        Map<String, Vaadin.StatData> widgetsMap = makeWidgets(orderedIds);
        map.put(Vaadin.DataType.COMPONENT, widgetsMap);

        Map<String,Integer> addons = new HashMap<String, Integer>();
        
        for(String id : ids){
            addons.put(id, random.nextInt());
        }
        
        Vaadin.VaadinData data = new Vaadin.VaadinData(ids,map, addons);
        
        vaadin.write(prefs, data);
        
        Vaadin.VaadinData read = vaadin.read(prefs);
        
        Assert.assertEquals("Unepxected add-ons after read",addons,  data.getAddons());
        
        checkStatData(ids, read, fixesMap, orderedIds, actionsMap, projectsMap, widgetsMap);
    }

    private void checkStatData(Set<String> ids, Vaadin.VaadinData read, Map<String, Vaadin.StatData> fixesMap, List<String> orderedIds, Map<String, Vaadin.StatData> actionsMap, Map<String, Vaadin.StatData> projectsMap, Map<String, Vaadin.StatData> widgetsMap) {
        Assert.assertEquals("Unexpected idnetifiers after read", ids, read.getIds());
        
        Assert.assertEquals("Unepxected fixes for the first user after read", 
                fixesMap.get(orderedIds.get(0)).getData(), read.getFixes().get(orderedIds.get(0)).getData());
        Assert.assertEquals("Unepxected fixes for the second user after read", 
                fixesMap.get(orderedIds.get(1)).getData(), read.getFixes().get(orderedIds.get(1)).getData());
        
        Assert.assertEquals("Unepxected actions for the first user after read", 
                actionsMap.get(orderedIds.get(0)).getData(), read.getActions().get(orderedIds.get(0)).getData());
        Assert.assertEquals("Unepxected actions for the second user after read", 
                actionsMap.get(orderedIds.get(1)).getData(), read.getActions().get(orderedIds.get(1)).getData());
        
        Assert.assertEquals("Unepxected projects for the first user after read", 
                projectsMap.get(orderedIds.get(0)).getData(), read.getProjects().get(orderedIds.get(0)).getData());
        Assert.assertEquals("Unepxected projects for the second user after read", 
                projectsMap.get(orderedIds.get(1)).getData(), read.getProjects().get(orderedIds.get(1)).getData());
        
        Assert.assertEquals("Unepxected components for the first user after read", 
                widgetsMap.get(orderedIds.get(0)).getData(), read.getComponents().get(orderedIds.get(0)).getData());
        Assert.assertEquals("Unepxected components for the second user after read", 
                widgetsMap.get(orderedIds.get(1)).getData(), read.getComponents().get(orderedIds.get(1)).getData());
    }

    private Map<String, Vaadin.StatData> makeFixes(List<String> orderedIds) {
        Map<String, Vaadin.StatData> mapData = new HashMap<String, Vaadin.StatData>();
        Map<String, Integer> amountMap1 = Collections.singletonMap("fix1",random.nextInt());
        Map<String, Integer> amountMap2 = new HashMap<String, Integer>(amountMap1);
        amountMap2.put("fix2",random.nextInt());
        mapData.put(orderedIds.get(0), new Vaadin.StatData(amountMap1));
        mapData.put(orderedIds.get(1), new Vaadin.StatData(amountMap2));
        return mapData;
    }
    
    private Map<String, Vaadin.StatData> makeActions(List<String> orderedIds) {
        Map<String, Vaadin.StatData> mapData = new HashMap<String, Vaadin.StatData>();
        Map<String, Integer> amountMap1 = Collections.singletonMap("actions1",random.nextInt());
        Map<String, Integer> amountMap2 = new HashMap<String, Integer>();
        amountMap2.put("action2",random.nextInt());
        amountMap2.put("action3",random.nextInt());
        mapData.put(orderedIds.get(0), new Vaadin.StatData(amountMap1));
        mapData.put(orderedIds.get(1), new Vaadin.StatData(amountMap2));
        return mapData;
    }
    
    private Map<String, Vaadin.StatData> makeProjects(List<String> orderedIds) {
        Map<String, Vaadin.StatData> mapData = new HashMap<String, Vaadin.StatData>();
        
        Map<String, Integer> amountMap1 = new HashMap<String, Integer>();
        amountMap1.put("project1",random.nextInt());
        amountMap1.put("project2",random.nextInt());
        Map<String, Integer> amountMap2 = Collections.singletonMap("project3",random.nextInt());
        mapData.put(orderedIds.get(0), new Vaadin.StatData(amountMap1));
        mapData.put(orderedIds.get(1), new Vaadin.StatData(amountMap2));
        return mapData;
    }
    
    private Map<String, Vaadin.StatData> makeWidgets(List<String> orderedIds) {
        Map<String, Vaadin.StatData> mapData = new HashMap<String, Vaadin.StatData>();
        Map<String, Integer> amountMap1 = Collections.singletonMap("widget1",random.nextInt());
        Map<String, Integer> amountMap2 = Collections.singletonMap("widget1",random.nextInt());
       
        mapData.put(orderedIds.get(0), new Vaadin.StatData(amountMap1));
        mapData.put(orderedIds.get(1), new Vaadin.StatData(amountMap2));
        return mapData;
    }

    private Set<String> setUpIds() {
        Set<String> ids = new HashSet<String>();
        ids.add("a");
        ids.add("b");
        return ids;
    }
    
    private static final class Prefs extends AbstractPreferences {
        
        Prefs(Prefs parent, String name){
            super(parent, name);
            myData = new HashMap<String, String>();
            myChildren = new HashMap<String, Prefs>();
        }
        
        @Override
        public Prefs parent() {
            return (Prefs)super.parent(); 
        }

        @Override
        protected void putSpi(String key, String value) {
            myData.put(key, value);
        }

        @Override
        protected String getSpi(String key) {
            return myData.get(key);
        }

        @Override
        protected void removeSpi(String key) {
            myData.remove(key);
        }

        @Override
        protected void removeNodeSpi() throws BackingStoreException {
            parent().removeChild(name());
        }

        @Override
        protected String[] keysSpi() throws BackingStoreException {
            Set<String> keySet = myData.keySet();
            return keySet.toArray(new String[keySet.size()]);
        }

        @Override
        protected String[] childrenNamesSpi() throws BackingStoreException {
            Set<String> childrenNames = myChildren.keySet();
            return childrenNames.toArray(new String[childrenNames.size()]);
        }

        @Override
        protected AbstractPreferences childSpi(String name) {
            AbstractPreferences child = myChildren.get(name);
            if ( child == null ){
                Prefs prefs = new Prefs(this , name);
                myChildren.put(name, prefs);
                child = prefs;
            }
            return child;
        }

        @Override
        protected void syncSpi() throws BackingStoreException {
            
        }

        @Override
        protected void flushSpi() throws BackingStoreException {
            
        }
        
        private void removeChild(String name){
            myChildren.remove(name);
        }

        private final Map<String,String> myData;
        private final Map<String,Prefs> myChildren;
    }
}
