/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.mdrant;

import java.util.Iterator;

import org.apache.tools.ant.BuildException;

import javax.jmi.model.MofPackage;
import javax.jmi.model.ModelPackage;

/** Instantiates metamodel into given extent.
 *
 * @author  Petr Hrebejk
 */
public class Instantiate extends MdrTask.Sub {

    private String name;
    private String extent;
    private String packageName;

    /** Creates a new instance of PrintExtentsNames */
    public Instantiate() {
    }

    public void execute() throws Exception {

        if ( name == null ) {
            throw new BuildException( "Name for newly instantiated package has to be sppecified. Use the \"name\" attribute" );
        }
        
        if ( extent == null && packageName == null ) {
            getRepository().createExtent( name );
        }
        else {
            if ( extent == null ) {
                throw new BuildException( "Extent containing the package has to be specified. Use the \"extent\" attribute" );
            }
            if ( packageName == null ) {
                throw new BuildException( "The package has to be specified. Use the \"package\" attribute" );
            }
            
            MofPackage mp = getModelPackage();
            
            if ( mp == null ) {
                throw new BuildException( packageName + " does not exist in extent " + extent );
            }
            
            getRepository().createExtent( name, mp ); 
        }
        
    }
    
    // ANT task attributes -----------------------------------------------------
    
    public void setName( String name ) {
        this.name = name;
    }
    
    public void setExtent( String extent ) {
        this.extent = extent;
    }
    
    public void setPackage( String packageName ) {
        this.packageName = packageName;
    }
    
    // private methods ---------------------------------------------------------
    
    
    private MofPackage getModelPackage() {
        
        ModelPackage mofPackage = (ModelPackage)getRepository().getExtent( extent );
        MofPackage result;
        
        for (Iterator it = mofPackage.getMofPackage().refAllOfClass().iterator(); it.hasNext();) {
            result = (MofPackage) it.next();
            if (result.getName().equals( packageName )) {
                return result;
            }
        }
        
        return null;
    }
    
    
}
