/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.mdr.persistence.btreeimpl.btreeindex;

import java.io.*;
import org.netbeans.mdr.persistence.*;

/**
 * Encapsulates the behavior of an entry of a given type.
 *
 * @author	Dana Bergen
 * @version	1.0
 */
public abstract class EntryTypeInfo extends Object {

    static final byte EQUAL = 0;
    static final byte GREATER = 1;
    static final byte LESS = -1;

    /**
     * Returns an IntInfo, MOFIDInfo, or StringInfo.
     *
     * @param	type	type to return Info object for
     *
     * @return	a new IntInfo, MOFIDInfo, or StringInfo
     */
    public static EntryTypeInfo getEntryTypeInfo(
    	Storage.EntryType type, Storage storage) {

        if (type == Storage.EntryType.INT) {
	    return new IntInfo();
	} else if (type == Storage.EntryType.MOFID) {
	    return new MOFIDInfo(storage);
	} else if (type == Storage.EntryType.STRING) {
	    return new StringInfo();
	} else {
	    return null;
	}
    }
    
    /**
     * Store the value of the object in a byte array.
     * @param	object
     * @return	newly allocated byte array
     */
    public abstract byte[] toBuffer(Object object);
    /**
     * Create an object from the contents of the byte array.
     * @param buffer
     * @return new object
     */
    public abstract Object fromBuffer(byte[] buffer);
    /**
     * Like fromBuffer, except that if this type is a key, returns object
     * associated with the key
     * @param buffer
     * @return new object
     */
    public Object objectFromBuffer(byte[] buffer, SinglevaluedIndex repos) {
        return fromBuffer(buffer);
    }
    
    /**
     * Compare two items of this type which are stored in byte arrays.
     *
     * @param key
     * @param key2
     * @param	offset		offset into key2 of target key
     * @param	length		length in key2 of target key
     *
     * @return	Returns one of:
     * <p>EQUAL if the two keys are equal
     * <p>GREATER if key greater than key2
     * <p>LESS if key less than key2
     */
    public abstract byte compare(byte[] key, byte[] key2, int offset, int length);
    /**
     * Returns the length of an item of this type, or 0 if variable length.
     */
    public abstract int getLength();
    /**
     * Returns true if this is a fixed length type, otherwise returns false;
     */
    public abstract boolean isFixedLength();
    /**
     * Returns the name of this type as a String.
     */
    public abstract String typeName();

}
