/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.mdr.persistence.btreeimpl.btreeindex;

/**
 * EntryTypeInfo implementation for type String
 *
 * @author	Dana Bergen
 * @version	1.0
 */

public class StringInfo extends EntryTypeInfo {
    public String typeName() {
        return "String";
    }

    public byte[] toBuffer(Object object) {

        if (!(object instanceof String)) {
	    return null;
	} else {
            String s = (String)object;
            int l = s.length();
            char[] arr = new char[l];
            int utflen = 0;
            char c;
            
            s.getChars(0, l, arr, 0);
            for (int i = 0; i < l; i++) {
                c = arr[i];
                if (c < 0x7f) {
                    utflen++;
                } else if (c > 0x7ff) {
                    utflen += 3;
                } else {
                    utflen += 2;
                }
            }
            byte[] encoded = new byte[utflen];
            int offset = 0;
            for (int i = 0; i < l; i++) {
                c = arr[i];
                if (c < 0x007f) {
                    encoded[offset++] = (byte)c;
                } else if (c > 0x07ff) {
                    encoded[offset++] = (byte)(0xe0 | ((c >> 12) & 0x0f));
                    encoded[offset++] = (byte)(0x80 | ((c >> 6) & 0x3f));
                    encoded[offset++] = (byte)(0x80 | (c & 0x3f));
                } else {
                    encoded[offset++] = (byte)(0xc0 | ((c >> 6) & 0x1f));
                    encoded[offset++] = (byte)(0x80 | (c & 0x3f));
                }
            }
            return encoded;
	}
    }
    
    static int fromBufferCount = 0;

    public Object fromBuffer(byte[] buffer) {
        int length = buffer.length;
        int offset = 0;
        StringBuilder sb = new StringBuilder(length);
        if (length > 0) {
            do {
                int b = buffer[offset++] & 0xff;
                length--;
                if (b >= 0xe0) {
                    b = (b & 0x0f) << 12;
                    b |= (buffer[offset++] & 0x3f) << 6;
                    b |= buffer[offset++] & 0x3f;
                    length--;
                    length--;
                } else if (b >= 0xc0) {
                    b = (b & 0x1f) << 6;
                    b |= buffer[offset++] & 0x3f;
                    length--;
                }
                sb.append((char)b);
            } while (length > 0);
        }
        return new String(sb);
    }
    
    final int compareImpl(byte[] key1buffer, byte[] key2buffer, int offset, int length) {
        int key1len = key1buffer.length;
        byte c1, c2;
        int key1offset = 0;

        int n = Math.min(key1len, length);
        while (n-- > 0) {
            c1 = key1buffer[key1offset++];
            c2 = key2buffer[offset++];
            if (c1 != c2)
                return c1 - c2;
        }
        return key1len - length;
    }

    public byte compare(byte[] key1buffer, byte[] key2buffer, int offset, int length) {
	int result = compareImpl(key1buffer, key2buffer, offset, length);
	if (result < 0) {
	    return LESS;
	} else if (result > 0) {
	    return GREATER;
	} else {
	    return EQUAL;
	}
    }

    public int getLength() {
        // length is variable
	return 0;
    }

    public boolean isFixedLength() {
        return false;
    }
}
