/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.mdr.persistence.memoryimpl;

import org.netbeans.mdr.persistence.*;
import java.util.ArrayList;
import java.util.List;

/** Default memory implementation of the MultivaluedOrderedIndex using Hashtable.
 * @author  Pavel Buzek, Martin Matula
 * @version
 */
public class MultivaluedOrderedIndexImpl extends MultivaluedIndexImpl implements MultivaluedOrderedIndex, Streamable {

    /** Creates new MultivaluedOrderedIndexImpl */
    public MultivaluedOrderedIndexImpl(String name, StorageImpl storage, Storage.EntryType keyType, Storage.EntryType valueType, boolean unique) {
        super (name, storage, keyType, valueType, unique);
    }

    public MultivaluedOrderedIndexImpl() {
    }

    public synchronized java.util.List getItemsOrdered(Object key) throws StorageException {
        return (List) getItems (key);
    }

      /** Like getItemsOrdered, but if the index contains keys, this returns the objects 
     * corresponding to the key
     * @return
     * @param key
     * @throws StorageException
     */
    public synchronized java.util.Collection getObjectsOrdered (Object key, SinglevaluedIndex repos) throws StorageException {
    	return (List) getObjects(key, repos);
    }

    /** Removes the element at the specified position in the list of values
     * associated with the specified key.
     */
    public synchronized boolean remove(Object key,int index) throws StorageException {
        List vals = (List) entries.get (key);
        Object orig;
        if (vals != null && (orig = vals.remove(index)) != null) {
            transLog.logRemove(key, orig, index);
            return true;
        } else {
            return false;
        }
    }
    
    /** Inserts the specified element at the specified position in the list of values
     * associated with the specified key.
     */
    public synchronized void add(Object key,int index,Object value) throws StorageException {
        List vals = (List) entries.get (key);
        if (vals == null) {
            vals = new ArrayList ();
            entries.put(key, vals);
        } else if (unique && vals.contains(value)) {
            throw createValueAlreadyContainedExc(key, value);
        }
        vals.add(index, value);
        transLog.logAdd(key, value, index);
    }
    
    /** Replaces the element at the specified position in the list of values
     * associated with the specified key with the specified element.
     * Throws StorageBadRequestException if the index is out of range.
     * @param key
     * @param index
     * @param element
     * @throws StorageException
     */
    public synchronized void replace(Object key,int index,Object element) throws StorageException {
        List vals = (List) entries.get (key);
        try {
            Object orig;
            if (vals != null) {
                if (unique && !isUniqueValue(vals, element, index)) {
                    throw createValueAlreadyContainedExc(key, element);
                }
                if ((orig = vals.set(index, element)) != null) {
                    transLog.logReplace(key, orig, index);
                    return;
                }
            }
        } catch (IndexOutOfBoundsException e) {
        }
        throw new StorageBadRequestException ("Index out of range: " + index + " for key: " + key);
    }
}
