/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.mdr.storagemodel;

import java.util.*;
import java.lang.reflect.Array;

import org.netbeans.mdr.util.DebugException;
import org.netbeans.mdr.persistence.StorageException;
import org.netbeans.mdr.util.Logger;

/**
 * Read-only collection. Wraps an inner collection all entries of which are
 * MOF IDs. The accessor methods of this collection return the objects from
 * storage instead of their IDs.
 *
 * @author  Petr Hrebejk, Martin Matula
 */
class CachedCollection implements Collection {
    private final Collection innerCollection;
    private final MdrStorage storage;
    
    /** Creates a new CachedCollection.
     *
     * @param storage the storage to resolve MOF IDs
     * @param innerCollection the collection of MOF IDs
     */
    public CachedCollection(MdrStorage storage, Collection innerCollection) {
        this.innerCollection = innerCollection;
        this.storage = storage;
    }

    public boolean retainAll(Collection collection) {
        throw new UnsupportedOperationException();
    }

    public boolean contains(Object obj) {
        return innerCollection.contains(obj);
    }

    public Object[] toArray(Object[] obj) {
        Object[] value = toArray();
        Object[] result = obj;
        if (value.length > result.length) {
            if (value.getClass() == result.getClass()) {
                return value;
            } else {
                result = (Object[]) Array.newInstance(obj.getClass(), value.length);
            }
        }
        for (int i = 0; i < result.length; i++) {
            result[i] = (i < value.length) ? value[i] : null;
        }
        return result;
    }

    public Iterator iterator() {
        return new CachedIterator(innerCollection.iterator());
    }

    public boolean removeAll(Collection collection) {
        throw new UnsupportedOperationException();
    }

    public Object[] toArray() {
        Object[] array = innerCollection.toArray();

        try {
            for (int i = 0; i < array.length; i++) {
                array[i] = storage.getObject((org.netbeans.mdr.persistence.MOFID) array[i]);
            }
        } catch (Exception e) {
            throw (DebugException) Logger.getDefault().annotate(new DebugException(), e);
        }

        return array;
    }

    public boolean remove(Object obj) {
        throw new UnsupportedOperationException();
    }

    public void clear() {
        throw new UnsupportedOperationException();
    }

    public boolean addAll(Collection collection) {
        throw new UnsupportedOperationException();
    }

    public int size() {
        return innerCollection.size();
    }

    public boolean containsAll(Collection collection) {
        return innerCollection.containsAll(collection);
    }

    public boolean add(Object obj) {
        throw new UnsupportedOperationException();
    }

    public boolean isEmpty() {
        return innerCollection.isEmpty();
    }
    
    private class CachedIterator implements Iterator {
        private final Iterator innerIterator;
        
        private CachedIterator(Iterator innerIterator) {
            this.innerIterator = innerIterator;
        }
        
        public boolean hasNext() {
            return innerIterator.hasNext();
        }

        public Object next() {
            try {
                return storage.getObject((org.netbeans.mdr.persistence.MOFID) innerIterator.next());
            } catch (StorageException e) {
                throw (DebugException) Logger.getDefault().annotate(new DebugException(), e);
            }
        }

        public void remove() {
            throw new UnsupportedOperationException();
        }
    }
}
