/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.mdr.test;

import java.io.*;
import java.util.*;
import java.net.*;

import junit.extensions.*;
import junit.framework.*;

import org.netbeans.api.mdr.*;
import org.openide.util.Lookup;

import org.netbeans.mdr.util.*;
import org.netbeans.mdr.NBMDRepositoryImpl;
import org.netbeans.lib.jmi.xmi.*;
import org.netbeans.lib.jmi.mapping.*;

import org.xml.sax.*;

import javax.jmi.reflect.*;
import javax.jmi.model.*;

/**
 * Test case verifying functionality of package instantiation, especially cases when a package
 * is a subclass of another package.
 */
public class PackageInheritanceTest extends MDRTestCase {
    
    public PackageInheritanceTest(String testName) {
        super (testName);
    }
    
    public static void main (String[] args) {
        junit.textui.TestRunner.run (suite ());
    }
    
    public static Test suite () {
        TestSuite suite = new TestSuite ();
        suite.addTestSuite (PackageInheritanceTest.class);
        
        TestSetup setup = new TestSetup (suite) {
            public void setUp () {
//                org.netbeans.mdr.handlers.BaseObjectHandler.setDefaultClassLoader (this.getClass ().getClassLoader ());
            }
            public void tearDown () {
            }
        };        
        return setup;        
    }

    public void test () {
        ModelPackage pkg = loadMOFModel ("pkg_inher.xml", "PureMOF");
        MofPackage mofPkg = findMofPackage (pkg, "Package_1");
        RefPackage pkg_1 = createExtent (mofPkg, "Package_1");
        mofPkg = findMofPackage (pkg, "Package_2");
        RefPackage pkg_2 = createExtent (mofPkg, "Package_2");
        mofPkg = findMofPackage (pkg, "Package_3");
        RefPackage pkg_3 = createExtent (mofPkg, "Package_3");
        
        try {
            for (char x = 'A'; x <= 'E'; x++)
                pkg_3.refClass ("Class_" + x);            
            for (char x = 'A'; x <= 'D'; x++)
                pkg_2.refClass ("Class_" + x);
            
            new RandomDataGenerator ().generate (pkg_3, 0, 5);
            new RandomDataGenerator ().generate (pkg_2, 0, 5);
            new RandomDataGenerator ().generate (pkg_1, 0, 5);
            
            pkg_1.refDelete ();            
            pkg_2.refDelete ();
            pkg_3.refDelete ();
            pkg.refDelete ();
        } catch (Exception e) {
            System.out.println(e);
            e.printStackTrace ();
            fail (e.getMessage ());
        }        
    }
    
}
