/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2015 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2015 Sun Microsystems, Inc.
 */
package org.netbeans.modules.php.editor.cup.parser.cutter;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Scanner;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;

/*
<taskdef name="parsercutter"
classname="org.netbeans.modules.php.editor.cup.parser.cutter.ParserCutter"
classpath="/opt/parser.cutter/dist/parser.cutter.jar" />
<parsercutter parserpath="src/org/netbeans/modules/php/editor/parser/ASTPHP5Parser.java" />
 */
/**
 *
 * @author Ondrej Brejla <obrejla@netbeans.org>
 */
public class ParserCutter extends Task {

    private static final double CASES_PER_FAKE_METHOD = 100.0;
    private static final int LINES_PER_ENCODED_ACTION = 4500;
    private static final String ACTION_TABLE_BASE_CLASS_NAME = "EncodedActionTable";
    private File parserPath;
    private StringBuilder text = new StringBuilder(10 * 1024);
    private final Pattern casePattern = Pattern.compile("\\s+case\\s+(\\d+)\\:\\s+//\\s+.+"); // case 429: // string_st ::= T_DEFINE
    private final Map<Integer, String> caseMap = new HashMap<>();
    private final List<EncodedActionTable> actionTables = new LinkedList<>();
    private int fakeMethodsCount;


    public static void main(String... args) {
        if (args.length == 0) {
            System.err.println("File to be cut must be provided as a parameter");
            System.exit(1);
        }
        ParserCutter parserCutter = new ParserCutter();
        parserCutter.setParserpath(args[0]);
        parserCutter.execute();
    }

    public void setParserpath(String parserPath) {
        System.out.println("Setting parser path to: " + parserPath);
        this.parserPath = new File(parserPath);
    }

    @Override
    public void execute() {
        try {
            System.out.println("Reading...");
            read();
            System.out.println("Cutting...");
            cut();
            System.out.println("Writing...");
            write();
            System.out.println("Done.");
        } catch (IOException ex) {
            throw new BuildException(ex);
        }
    }

    public void read() throws FileNotFoundException {
        String newLineSeparator = System.getProperty("line.separator");
        Scanner s = new Scanner(new FileInputStream(parserPath));
        boolean isInActionsTable = false;
        boolean isInDoAction = false;
        boolean possibleLastCase = false;
        Integer currentCaseKey = null;
        int actionTablesCounter = 0;
        int lineCounter = 0;
        EncodedActionTable currentActionTable = null;
        try {
            while (s.hasNextLine()) {
                String nextLine = s.nextLine();
                if (nextLine.trim().equals("protected static final short[][] _action_table =")) {
                    isInActionsTable = true;
                    text.append(nextLine).append(" getActionTable();").append(newLineSeparator);
                    continue;
                }
                if (isInActionsTable && lineCounter == LINES_PER_ENCODED_ACTION) {
                    currentActionTable = new EncodedActionTable(parserPath.getParent(), ++actionTablesCounter);
                    actionTables.add(currentActionTable);
                    currentActionTable.addRow(nextLine);
                    lineCounter = 1;
                    if (nextLine.trim().endsWith("});")) {
                        isInActionsTable = false;
                    }
                    continue;
                }
                if (isInActionsTable && nextLine.trim().equals("unpackFromStrings(new String[] {")) {
                    currentActionTable = new EncodedActionTable(parserPath.getParent(), ++actionTablesCounter);
                    actionTables.add(currentActionTable);
                    lineCounter = 0;
                    continue;
                }
                if (isInActionsTable) {
                    currentActionTable.addRow(nextLine);
                    lineCounter++;
                    if (nextLine.trim().endsWith("});")) {
                        isInActionsTable = false;
                    }
                    continue;
                }
                if (!isInDoAction && nextLine.trim().equals("/** Method with the actual generated action code. */")) {
                    isInDoAction = true;
                }
                if (isInDoAction) {
                    Matcher matcher = casePattern.matcher(nextLine);
                    if (matcher.matches()) {
                        Integer newCaseKey = Integer.parseInt(matcher.group(1));
                        currentCaseKey = newCaseKey;
                        caseMap.put(newCaseKey, nextLine + newLineSeparator);
                    } else if (currentCaseKey != null) {
                        if (currentCaseKey == 0) {
                            if (nextLine.trim().equals("/* . . . . . .*/")) {
                                possibleLastCase = true;
                            } else if (possibleLastCase && nextLine.trim().equals("default:")) {
                                //the end
                                break;
                            }
                        }
                        String oldCaseValue = caseMap.get(currentCaseKey);
                        caseMap.put(currentCaseKey, oldCaseValue + nextLine + newLineSeparator);
                    }
                } else {
                    text.append(nextLine).append(newLineSeparator);
                }
            }
        } finally {
            s.close();
        }
    }

    public void cut() throws IOException {
        fakeMethodsCount = (int) Math.ceil(caseMap.size() / CASES_PER_FAKE_METHOD);
        appendDoAction();
        for (int i = 1; i <= fakeMethodsCount; i++) {
            appendFakeMethod(i);
        }
        appendFooter();
        replaceGetActionTables();
    }

    private void replaceGetActionTables() throws IOException {
        StringBuilder instantiation = new StringBuilder();
        for (Iterator<EncodedActionTable> i = actionTables.iterator(); i.hasNext();) {
            EncodedActionTable actionTable = i.next();
            instantiation.append("new ").append(actionTable).append("().getTableData()");
            actionTable.generateClassFile();
            if (i.hasNext()) {
                instantiation.append(",");
            }
            instantiation.append("\n");
        }
        StringBuilder methodSource = new StringBuilder();
        methodSource.append("private static short[][] getActionTable() {\n")
                .append("return unpackFromStrings(new String[]{\n")
                .append(instantiation)
                .append("});\n")
                .append("}\n");
        text = new StringBuilder(text.toString().replace("private static short[][] getActionTable() {}", methodSource));
    }


    private void appendDoAction() { //java_cup.runtime.Symbol CUP$ASTPHP5Parser$result;
        text.append("public final java_cup.runtime.Symbol CUP$ASTPHP5Parser$do_action(\n"
                + "int                        CUP$ASTPHP5Parser$act_num,\n"
                + "java_cup.runtime.lr_parser CUP$ASTPHP5Parser$parser,\n"
                + "java.util.Stack            CUP$ASTPHP5Parser$stack,\n"
                + "int                        CUP$ASTPHP5Parser$top)\n"
                + "throws java.lang.Exception\n"
                + "{\n");
        text.append("if (CUP$ASTPHP5Parser$act_num < ").append((int) CASES_PER_FAKE_METHOD)
                .append(") {\n" + "    return fakeMethod0to").append((int) CASES_PER_FAKE_METHOD - 1)
                .append("(CUP$ASTPHP5Parser$act_num, CUP$ASTPHP5Parser$parser, CUP$ASTPHP5Parser$stack, CUP$ASTPHP5Parser$top);\n"
                + "}\n");
        if (fakeMethodsCount > 1) {
            for (int i = 2; i <= fakeMethodsCount; i++) {
                text.append("else if (CUP$ASTPHP5Parser$act_num < ").append((int) CASES_PER_FAKE_METHOD * i)
                        .append(") {\n" + "    return fakeMethod").append((int) CASES_PER_FAKE_METHOD * (i - 1))
                        .append("to").append(((int) CASES_PER_FAKE_METHOD * i) - 1)
                        .append("(CUP$ASTPHP5Parser$act_num, CUP$ASTPHP5Parser$parser, CUP$ASTPHP5Parser$stack, CUP$ASTPHP5Parser$top);\n"
                        + "}\n");
            }
        }
        text.append("throw new Exception(\"Invalid action number found in internal parse table\");\n");
        text.append("}\n");
    }

    private void appendFakeMethod(int methodNumber) {
        text.append("public final java_cup.runtime.Symbol fakeMethod").append((int) CASES_PER_FAKE_METHOD * (methodNumber - 1))
                .append("to").append(((int) CASES_PER_FAKE_METHOD * methodNumber) - 1).append("(\n"
                + "int                        CUP$ASTPHP5Parser$act_num,\n"
                + "java_cup.runtime.lr_parser CUP$ASTPHP5Parser$parser,\n"
                + "java.util.Stack            CUP$ASTPHP5Parser$stack,\n"
                + "int                        CUP$ASTPHP5Parser$top)\n"
                + "throws java.lang.Exception\n"
                + "{\n"
                + "java_cup.runtime.Symbol CUP$ASTPHP5Parser$result;\n"
                + "switch (CUP$ASTPHP5Parser$act_num) {\n");
        appendCases(methodNumber);
        text.append("default: \n"
                + "throw new Exception(\"Invalid action number found in internal parse table\");\n");
        text.append("}\n}\n");
    }

    private void appendCases(int methodNumber) {
        int start = (int) CASES_PER_FAKE_METHOD * (methodNumber - 1);
        int max = methodNumber == fakeMethodsCount ? caseMap.size() : methodNumber * (int) CASES_PER_FAKE_METHOD;
        for (int i = start; i < max; i++) {
            text.append(caseMap.get(i));
        }
    }

    private void appendFooter() {
        text.append("\n}");
    }

    public void write() throws IOException {
        if (parserPath.exists()) {
            parserPath.delete();
        }
        Writer out = new OutputStreamWriter(new FileOutputStream(parserPath));
        try {
          out.write(text.toString());
        } finally {
          out.close();
        }
    }

    private class EncodedActionTable {

        List<String> rows = new LinkedList<String>();
        private static final String FILE_EXTENSION = ".java";
        private final String className;
        private final String fileName;

        public EncodedActionTable(String directoryPath, int tableNumber) {
            className = String.format("%s%d", ACTION_TABLE_BASE_CLASS_NAME, tableNumber);
            fileName = String.format("%s/%s%s", directoryPath, className, FILE_EXTENSION);
        }

        public void addRow(String row) {
            rows.add(row);
        }

        public void generateClassFile() throws IOException {
            StringBuilder sb = new StringBuilder();
            appendClassHeader(sb);
            appendRows(sb);
            appendClassFooter(sb);
            write(sb);
        }

        private void appendClassHeader(StringBuilder sb) {
            int year = Calendar.getInstance().get(Calendar.YEAR);
            sb.append("/*\n"
                    + " * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.\n"
                    + " *\n"
                    + " * Copyright ").append(year).append(" Oracle and/or its affiliates. All rights reserved.\n"
                    + " *\n"
                    + " * Oracle and Java are registered trademarks of Oracle and/or its affiliates.\n"
                    + " * Other names may be trademarks of their respective owners.\n"
                    + " *\n"
                    + " * The contents of this file are subject to the terms of either the GNU\n"
                    + " * General Public License Version 2 only (\"GPL\") or the Common\n"
                    + " * Development and Distribution License(\"CDDL\") (collectively, the\n"
                    + " * \"License\"). You may not use this file except in compliance with the\n"
                    + " * License. You can obtain a copy of the License at\n"
                    + " * http://www.netbeans.org/cddl-gplv2.html\n"
                    + " * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the\n"
                    + " * specific language governing permissions and limitations under the\n"
                    + " * License.  When distributing the software, include this License Header\n"
                    + " * Notice in each file and include the License file at\n"
                    + " * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this\n"
                    + " * particular file as subject to the \"Classpath\" exception as provided\n"
                    + " * by Oracle in the GPL Version 2 section of the License file that\n"
                    + " * accompanied this code. If applicable, add the following below the\n"
                    + " * License Header, with the fields enclosed by brackets [] replaced by\n"
                    + " * your own identifying information:\n"
                    + " * \"Portions Copyrighted [year] [name of copyright owner]\"\n"
                    + " *\n"
                    + " * If you wish your version of this file to be governed by only the CDDL\n"
                    + " * or only the GPL Version 2, indicate your decision by adding\n"
                    + " * \"[Contributor] elects to include this software in this distribution\n"
                    + " * under the [CDDL or GPL Version 2] license.\" If you do not indicate a\n"
                    + " * single choice of license, a recipient has the option to distribute\n"
                    + " * your version of this file under either the CDDL, the GPL Version 2 or\n"
                    + " * to extend the choice of license to its licensees as provided above.\n"
                    + " * However, if you add GPL Version 2 code and therefore, elected the GPL\n"
                    + " * Version 2 license, then the option applies only if the new code is\n"
                    + " * made subject to such option by the copyright holder.\n"
                    + " *\n"
                    + " * Contributor(s):\n"
                    + " *\n"
                    + " * Portions Copyrighted ").append(year).append(" Sun Microsystems, Inc.\n"
                    + " */\n"
                    + "package org.netbeans.modules.php.editor.parser;\n"
                    + "\n"
                    + "/**\n * Class generated by Parser Cutter\n */\n"
                    + "public class ").append(className).append(" {\n"
                    + "    protected final StringBuilder sb;\n"
                    + "    public String getTableData() {\n"
                    + "        return sb.toString();\n"
                    + "    }\n"
                    + "    protected ").append(className).append("() {\n"
                    + "        sb = new StringBuilder();\n");
        }

        private void appendRows(StringBuilder sb) {
            for (String row : rows) {
                row = row.replace("    \"\\", "        sb.append(\"\\");
                row = row.replace("    \"\"", "        sb.append(\"\"");
                row = row.replace("\" +", "\");");
                row = row.replace("\",", "\");");
                row = row.replace("\" });", "\");");
                sb.append(row).append("\n");
            }
        }

        private void appendClassFooter(StringBuilder sb) {
            sb.append("    }\n}");
        }

        private void write(StringBuilder sb) throws IOException {
            File newFile = new File(fileName);
            if (newFile.exists()) {
                newFile.delete();
            }
            try (Writer out = new OutputStreamWriter(new FileOutputStream(newFile))) {
              out.write(sb.toString());
            }
        }

        @Override
        public String toString() {
            return className;
        }

    }

}
