/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 */

package org.foo.wavsupport.ui;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import javax.swing.SwingUtilities;
import org.foo.wavsupport.WavDataObject;
import org.foo.wavsupport.api.WavCookie;
import org.foo.wavutils.AudioUI;
import org.foo.wavutils.AudioUI.Selection;
import org.foo.wavutils.Player;
import org.foo.wavutils.Recorder;
import org.foo.wavutils.SamplingGraph;
import org.foo.wavutils.WrappedAudioInputStream;
import org.openide.DialogDisplayer;
import org.openide.NotifyDescriptor;
import org.openide.loaders.DataObject;
import org.openide.util.Task;

/**
 *
 * @author  runger
 */
public class WavPanel extends javax.swing.JPanel implements AudioUI {
    private DataObject wav;
    private Player player;
    private Recorder recorder;

    private WavFileModificationListener fileListener = new WavFileModificationListener();
    private WavDataModificationListener dataListener = new WavDataModificationListener();

    public WavPanel(DataObject dobj)
    {
        initComponents();
        initialize(dobj);
    }

    DataObject getWavDataObject()
    {
        return wav;
    }

    public void setAudioInputStream(WrappedAudioInputStream is)
    {
        WavCookie c = (WavCookie)wav.getCookie(WavCookie.class);
        assert(c != null);
        c.setAudioInputStream(is);
    }

    public WrappedAudioInputStream getAudioInputStream() {
        WavCookie c = (WavCookie)wav.getCookie(WavCookie.class);
        assert (c != null);
        return c.getAudioInputStream();
    }

    public SamplingGraph getSamplingGraph()
    {
        return samplingGraph;
    }

    /** Call only when initializing: in constructor or readExternal() */
    private void initialize(DataObject wavDO)
    {
        assert SwingUtilities.isEventDispatchThread() :
            "Attempting to populate prompt editor from outside the AWT event thread";

        player = new Player(this);
        recorder = new Recorder(this);

        wav = wavDO;
        wav.addPropertyChangeListener(fileListener);

        WavCookie c = (WavCookie)wav.getCookie(WavCookie.class);
        assert(c != null);
        
        c.addPropertyChangeListener(dataListener);
        WrappedAudioInputStream stream = c.getAudioInputStream();
        
        // force a PropertyChangeEvent for drawing the sample graph
        c.setAudioInputStream(null);
        c.setAudioInputStream(stream);
        wav.setModified(false);
        
        playButton.setEnabled(stream != null);
    }

    /**
     * Called when the Player finishes playing or the
     * Recorder finishes recording.
     */
    public void taskFinished( Task task )
    {
        playButton.setEnabled(true);
        recordButton.setEnabled(true);
        cropButton.setEnabled(true);
        stopButton.setEnabled(false);
        pauseButton.setEnabled(false);
    }

    public void setVisible(boolean visible) {
        super.setVisible(visible);

        if (!visible && wav != null)
        {
            if (samplingGraph.isRecording())
            {
                recorder.stop();
                samplingGraph.stop();
                samplingGraph.setRecording(false);
                wav.setModified(true);
            }
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        btnPanel = new javax.swing.JPanel();
        recordButton = new javax.swing.JButton();
        playButton = new javax.swing.JButton();
        pauseButton = new javax.swing.JButton();
        stopButton = new javax.swing.JButton();
        cropButton = new javax.swing.JButton();
        revertButton = new javax.swing.JButton();
        samplingGraph = new org.foo.wavutils.SamplingGraph();

        recordButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/foo/wavutils/recordButton.gif")));
        recordButton.setText(org.openide.util.NbBundle.getMessage(WavPanel.class, "WavPanel.recordButton.text"));
        recordButton.setToolTipText(org.openide.util.NbBundle.getMessage(WavPanel.class, "WavPanel.recordButton.toolTipText"));
        recordButton.setBorder(null);
        recordButton.setBorderPainted(false);
        recordButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                recordButtonActionPerformed(evt);
            }
        });

        btnPanel.add(recordButton);

        playButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/foo/wavutils/playButton.gif")));
        playButton.setText(org.openide.util.NbBundle.getMessage(WavPanel.class, "WavPanel.playButton.text"));
        playButton.setToolTipText(org.openide.util.NbBundle.getMessage(WavPanel.class, "WavPanel.playButton.toolTipText"));
        playButton.setBorder(null);
        playButton.setBorderPainted(false);
        playButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                playButtonActionPerformed(evt);
            }
        });

        btnPanel.add(playButton);

        pauseButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/foo/wavutils/pauseButton.gif")));
        pauseButton.setText(org.openide.util.NbBundle.getMessage(WavPanel.class, "WavPanel.pauseButton.text"));
        pauseButton.setToolTipText(org.openide.util.NbBundle.getMessage(WavPanel.class, "WavPanel.pauseButton.toolTipText"));
        pauseButton.setBorder(null);
        pauseButton.setBorderPainted(false);
        pauseButton.setEnabled(false);
        pauseButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                pauseButtonActionPerformed(evt);
            }
        });

        btnPanel.add(pauseButton);

        stopButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/foo/wavutils/stopButton.gif")));
        stopButton.setText(org.openide.util.NbBundle.getMessage(WavPanel.class, "WavPanel.stopButton.text"));
        stopButton.setToolTipText(org.openide.util.NbBundle.getMessage(WavPanel.class, "WavPanel.stopButton.toolTipText"));
        stopButton.setBorder(null);
        stopButton.setBorderPainted(false);
        stopButton.setEnabled(false);
        stopButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                stopButtonActionPerformed(evt);
            }
        });

        btnPanel.add(stopButton);

        cropButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/foo/wavutils/cropButton.gif")));
        cropButton.setText(org.openide.util.NbBundle.getMessage(WavPanel.class, "WavPanel.cropButton.text"));
        cropButton.setToolTipText(org.openide.util.NbBundle.getMessage(WavPanel.class, "WavPanel.cropButton.toolTipText"));
        cropButton.setBorder(null);
        cropButton.setBorderPainted(false);
        cropButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cropButtonActionPerformed(evt);
            }
        });

        btnPanel.add(cropButton);

        revertButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/foo/wavutils/revertButton.gif")));
        revertButton.setText(org.openide.util.NbBundle.getMessage(WavPanel.class, "WavPanel.revertButton.text"));
        revertButton.setToolTipText(org.openide.util.NbBundle.getMessage(WavPanel.class, "WavPanel.revertButton.toolTipText"));
        revertButton.setBorder(null);
        revertButton.setBorderPainted(false);
        revertButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                revertButtonActionPerformed(evt);
            }
        });

        btnPanel.add(revertButton);

        javax.swing.GroupLayout samplingGraphLayout = new javax.swing.GroupLayout(samplingGraph);
        samplingGraph.setLayout(samplingGraphLayout);
        samplingGraphLayout.setHorizontalGroup(
            samplingGraphLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 396, Short.MAX_VALUE)
        );
        samplingGraphLayout.setVerticalGroup(
            samplingGraphLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 202, Short.MAX_VALUE)
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(btnPanel, javax.swing.GroupLayout.DEFAULT_SIZE, 396, Short.MAX_VALUE)
            .addComponent(samplingGraph, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addComponent(samplingGraph, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(btnPanel, javax.swing.GroupLayout.PREFERRED_SIZE, 47, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
    }// </editor-fold>//GEN-END:initComponents

    private void revertButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_revertButtonActionPerformed
        setAudioInputStream(null);
    }//GEN-LAST:event_revertButtonActionPerformed

    private void cropButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cropButtonActionPerformed
        Selection curSelection = Selection.getSelection(this);
        WavCookie c = (WavCookie)wav.getCookie(WavCookie.class);
        assert(c != null);
        WrappedAudioInputStream pcmStream = c.getAudioInputStream();

        if( curSelection == null || pcmStream == null )
        {
            // nothing to crop
            NotifyDescriptor nd = new NotifyDescriptor.Message(
            "Nothing is selected to crop. Use the mouse to select a portion of the waveform by clicking the left mouse button and dragging.",
            NotifyDescriptor.INFORMATION_MESSAGE);
            DialogDisplayer.getDefault().notify(nd);
            return;
        }

        try
        {
            pcmStream.crop( curSelection.offset, curSelection.length );
            
            // it's the same stream instance, so force a PropretyChangeEvent
            // by setting to null first
            c.setAudioInputStream(null);
            c.setAudioInputStream(pcmStream);
        }
        catch( IOException ex )
        {
            NotifyDescriptor nd = new NotifyDescriptor.Message(
                    "Error Cropping Prompt: " + ex);
            DialogDisplayer.getDefault().notify(nd);
        }
    }//GEN-LAST:event_cropButtonActionPerformed

    private void playButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_playButtonActionPerformed
        playButton.setEnabled(false);
        recordButton.setEnabled(false);
        cropButton.setEnabled(false);
        stopButton.setEnabled(true);
        pauseButton.setEnabled(true);

        player.start();
        samplingGraph.start();
    }//GEN-LAST:event_playButtonActionPerformed

    private void stopButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stopButtonActionPerformed
        try
        {
            if( samplingGraph.isRecording() )
            {
                recorder.stop();
                samplingGraph.stop();
                samplingGraph.setRecording(false);
            }
            else
            {
                player.stop();
                samplingGraph.stop();
            }
        }
        finally
        {
            playButton.setEnabled(true);
            recordButton.setEnabled(true);
            cropButton.setEnabled(true);
            stopButton.setEnabled(false);
            pauseButton.setEnabled(false);
        }
    }//GEN-LAST:event_stopButtonActionPerformed

    private void pauseButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_pauseButtonActionPerformed
        try
        {
            if( samplingGraph.isRecording() )
            {
                //TODO: support pause during record
            }
            else
            {
                player.pause();
            }
        }
        finally
        {
            playButton.setEnabled(false);
            recordButton.setEnabled(false);
            cropButton.setEnabled(false);
            stopButton.setEnabled(true);
            pauseButton.setEnabled(true);
        }
    }//GEN-LAST:event_pauseButtonActionPerformed

    private void recordButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_recordButtonActionPerformed
        if( !samplingGraph.isRecording() )
        {
            // start recording
            playButton.setEnabled(false);
            recordButton.setEnabled(true);
            cropButton.setEnabled(false);
            stopButton.setEnabled(true);
            pauseButton.setEnabled(true);

            samplingGraph.setRecording(true);
            wav.setModified(true);

            recorder.start();
            samplingGraph.start();
        }
        else
        {
            // stop recording
            stopButtonActionPerformed(evt);
        }
    }//GEN-LAST:event_recordButtonActionPerformed

    private class WavFileModificationListener implements PropertyChangeListener
    {
        public void propertyChange(PropertyChangeEvent pce)
        {
            String propertyName = pce.getPropertyName();
            if(DataObject.PROP_MODIFIED.equals(propertyName))
            {
                if(pce.getNewValue() == Boolean.TRUE)
                {
                    // The prompt is modified by a record, crop, normalize, etc.
                    playButton.setEnabled(true);
                    revertButton.setEnabled(true);
                }
                else
                {
                    // the prompt is unmodified by save or revert
                    revertButton.setEnabled(false);
                    
                    WavCookie c = (WavCookie)wav.getCookie(WavCookie.class);
                    assert(c != null);
                    playButton.setEnabled(c.getAudioInputStream() != null);
                }
            }
        }
    }
    
    private class WavDataModificationListener implements PropertyChangeListener
    {
        public void propertyChange(PropertyChangeEvent pce)
        {
            String propertyName = pce.getPropertyName();
            if(WavCookie.PROP_WAVEFORM.equals(propertyName))
            {
                refreshGraphData();
            }
        }
    }
    
    private void refreshGraphData()
    {
        WrappedAudioInputStream wais = getAudioInputStream();
        SamplingGraph graph = getSamplingGraph();
        if (wais == null)
            graph.clearWaveForm();
        else
            graph.createWaveForm(wais);
    }



    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel btnPanel;
    private javax.swing.JButton cropButton;
    private javax.swing.JButton pauseButton;
    private javax.swing.JButton playButton;
    private javax.swing.JButton recordButton;
    private javax.swing.JButton revertButton;
    private org.foo.wavutils.SamplingGraph samplingGraph;
    private javax.swing.JButton stopButton;
    // End of variables declaration//GEN-END:variables

}
