/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2008 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.examples.cardsui;

import java.awt.BorderLayout;
import java.awt.Font;
import javax.swing.ActionMap;
import javax.swing.JSplitPane;
import javax.swing.text.DefaultEditorKit;
import org.openide.explorer.ExplorerManager;
import org.openide.explorer.ExplorerUtils;
import org.openide.explorer.view.BeanTreeView;
import org.openide.nodes.AbstractNode;
import org.openide.util.NbBundle;
import org.openide.util.Utilities;
import org.openide.windows.TopComponent;

/**
 *
 * @author Tim Boudreau
 */
public class CardGameTopComponent extends TopComponent implements ExplorerManager.Provider {

    private final ExplorerManager manager = new ExplorerManager();
    private final JSplitPane split1;
    private final JSplitPane split2;

    public CardGameTopComponent() {
        ActionMap map = this.getActionMap();
        map.put(DefaultEditorKit.copyAction,
                ExplorerUtils.actionCopy(manager));
        map.put(DefaultEditorKit.cutAction,
                ExplorerUtils.actionCut(manager));
        map.put(DefaultEditorKit.pasteAction,
                ExplorerUtils.actionPaste(manager));

        associateLookup(ExplorerUtils.createLookup(manager, map));

        setLayout(new BorderLayout());
        BeanTreeView view = new BeanTreeView();
        view.setRootVisible(false);
        manager.setRootContext(new AbstractNode(new GameChildren()));
        CardComponent comp = new CardComponent();
        split1 = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
                view, comp);

        split2 = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
                split1, new CardsDetailView());

        add(split2, BorderLayout.CENTER);
        if (Utilities.isWindows()) {
            view.getViewport().getView().setFont(new Font("SansSerif",
                    Font.PLAIN, 12));
        }
    }

    public ExplorerManager getExplorerManager() {
        return manager;
    }
    static int activeGames = 0;

    @Override
    protected void componentActivated() {
        ExplorerUtils.activateActions(manager, true);
    }

    @Override
    protected void componentDeactivated() {
        ExplorerUtils.activateActions(manager, false);
    }

    @Override
    public int getPersistenceType() {
        return PERSISTENCE_NEVER;
    }

    @Override
    protected void componentOpened() {
        split1.setDividerLocation(-1);
        split2.setDividerLocation(-1);
        if ("".equals(getDisplayName()) || getDisplayName() == null) {
            Integer gameNumber = new Integer(activeGames++);
            String displayName = NbBundle.getMessage(CardGameTopComponent.class,
                    "LBL_GAME", gameNumber);
            setDisplayName(displayName);
        }
    }
}
