/*
 * PileTest.java
 * JUnit based test
 *
 * Created on December 10, 2006, 11:06 PM
 */

package org.netbeans.examples.cards.api;

import java.util.Collections;
import java.util.List;
import junit.framework.*;
import org.netbeans.examples.cards.api.Pile.VisibilityPolicy;

/**
 *
 * @author Tim Boudreau
 */
public class PileTest extends TestCase {
    
    public PileTest(String testName) {
        super(testName);
    }

    public void testGetVisibilityPolicy() {
        Deck deck = new Deck (2);
        Pile pile = deck.createPile(VisibilityPolicy.ALL_VISIBLE, Collections.<Card>emptyList());
        assertSame (VisibilityPolicy.ALL_VISIBLE, pile.getVisibilityPolicy());
    }

    public void testIsEmpty() {
        System.out.println("testIsEmpty");
        Pile pile = new Pile();
        assertTrue (pile.isEmpty());
        Deck deck = new Deck(0);
        pile.add(deck.getCards().get(0));
        assertFalse (pile.isEmpty());
        Pile other = new Pile();
        other.add(deck.getCards().get(0));
        assertTrue (pile.isEmpty());
        assertFalse (other.isEmpty());
    }

    public void testPeekTopCard() {
        System.out.println("testPeekTopCard");
        Pile pile = new Pile();
        Deck deck = new Deck(0);
        pile.add(deck.getCards().get(0));
        assertSame (deck.getCards().get(0), pile.peekTopCard());
        pile.add(deck.getCards().get(1));
        assertSame (deck.getCards().get(1), pile.peekTopCard());
    }

    public void testRemoveTopCard() {
        System.out.println("testRemoveTopCard");
        Pile pile = new Pile();
        Deck deck = new Deck (0);
        pile.add(deck.getCards().get(0));
        pile.add(deck.getCards().get(1));
        pile.add(deck.getCards().get(2));
        assertSame (deck.getCards().get(2), pile.removeTopCard());
        assertSame (deck.getCards().get(0), pile.removeBottomCard());
        assertEquals (pile.size(), 1);
        assertSame (deck.getCards().get(1), pile.peekTopCard());
        assertSame (deck.getCards().get(1), pile.removeTopCard());
        assertTrue (pile.isEmpty());
    }

    public void testSplit() {
        System.out.println("testSplit");
        Pile pile = new Pile();
        Deck deck = new Deck(0);
        for (int i=0; i < deck.getCards().size(); i++) {
            pile.add (deck.getCards().get(i));
        }
        int sz = pile.size();
        Pile nue = pile.split();
        assertEquals (pile.size(), nue.size());
        assertEquals (sz / 2, pile.size());
    }

    public void testIsCardVisible() {
        System.out.println("testIsCardVisible");
        Pile visiblePile = new Pile(VisibilityPolicy.ALL_VISIBLE, Collections.<Card>emptyList());
        Pile invisiblePile = new Pile(VisibilityPolicy.TOP_CARD_VISIBLE, Collections.<Card>emptyList());
        Deck deck = new Deck(0);
        for (int i=0; i < deck.getCards().size(); i++) {
            if (i % 2 == 0) {
                visiblePile.add (deck.getCards().get(i));
            } else {
                invisiblePile.add (deck.getCards().get(i));
            }
        }
        for (int i=0; i < invisiblePile.getCards().size(); i++) {
            Card c = invisiblePile.getCards().get(i);
            if (i == 0) {
                assertFalse (c.isFaceDown());
            } else {
                assertTrue ("Failed at " + i + " with " + c, c.isFaceDown());
            }
        }
        
        for (int i=0; i < visiblePile.getCards().size(); i++) {
            Card c = visiblePile.getCards().get(0);
            assertFalse (c.isFaceDown());
        }
    }

    public void testCanAdd() {
        System.out.println("testCanAdd");
        Pile clubs = new Pile (VisibilityPolicy.ALL_VISIBLE, 
                AddPolicy.createAddPolicy (AddPolicy.ASCENDING, 
                AddPolicy.ACE_LOW, new AddPolicy.SuitAddPolicy (Suits.CLUBS),
                AddPolicy.ADJACENT, AddPolicy.ACE_MUST_START));
        Deck deck = new Deck(0);
        Suit cs = deck.clubs;
        List <Card> allClubs = cs.getCards();
        List <Card> allHearts = deck.hearts.getCards();
        
        Card aceOfClubs = allClubs.get (0);
        assertSame (Rank.ACE, aceOfClubs.identity());
        Card twoOfClubs = allClubs.get (1);
        assertSame (Rank.TWO, twoOfClubs.identity());
        
        assertFalse (clubs.canAdd(allHearts.get(0)));
        assertFalse (clubs.canAdd(allHearts.get(allHearts.size() - 1)));
        assertFalse (clubs.canAdd(allClubs.get (10)));
        assertTrue (clubs.canAdd (aceOfClubs));
        clubs.add (aceOfClubs);
        assertFalse (clubs.canAdd(allHearts.get(0)));
        assertTrue (clubs.canAdd (allClubs.get(1)));
    }
    
    public void testRedBlack() {
        System.out.println("testRedBlack");
        Deck deck = new Deck(0);
        Pile pile = new Pile (VisibilityPolicy.ALL_VISIBLE,
                AddPolicy.createAddPolicy(AddPolicy.DESCENDING,
                AddPolicy.RED_BLACK, AddPolicy.ADJACENT));
        assertTrue (pile.canAdd(deck.getCard(Rank.JACK, Suits.HEARTS)));
        pile.add (deck.getCard(Rank.JACK, Suits.HEARTS));
        assertFalse (pile.canAdd(deck.getCard (Rank.THREE, Suits.CLUBS)));
        assertFalse (pile.canAdd(deck.getCard(Rank.TEN, Suits.HEARTS)));
        assertFalse (pile.canAdd(deck.getCard(Rank.TEN, Suits.DIAMONDS)));
        assertTrue (pile.canAdd(deck.getCard(Rank.TEN, Suits.CLUBS)));
        assertTrue (pile.canAdd(deck.getCard(Rank.TEN, Suits.SPADES)));
        pile.add (deck.getCard(Rank.TEN, Suits.SPADES));
        assertTrue (pile.canAdd(deck.getCard(Rank.NINE, Suits.DIAMONDS)));
    }
}
