package org.netbeans.modules.manifestcodecompletion;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;
import javax.swing.text.html.HTMLDocument;
import junit.framework.TestCase;
import javax.swing.text.StyledDocument;
import org.openide.text.FilterDocument;

/**
 *
 * @author Jaroslav Tulach
 */
public class ManifestCompletionProviderTest extends TestCase {
    
    public ManifestCompletionProviderTest(String testName) {
        super(testName);
    }

    protected void setUp() throws Exception {
    }

    protected void tearDown() throws Exception {
    }
    public void testGetRowFirstNonWhite() throws BadLocationException {
        StyledDocument d = new HTMLDocument();
        
        int ret = ManifestCompletionProvider.getRowFirstNonWhite(d, 0);
        assertEquals("Zero", 0, ret);
        
        d.insertString(0, "Ahoj\nJardo\n", null);
        int len = d.getLength();
        
        ret = ManifestCompletionProvider.getRowFirstNonWhite(d, len);
        assertEquals("End", len, ret);
        
    }
    public void testEachCharacter() throws BadLocationException {
        StyledDocument d = new FilterDocument(new PlainDocument());
        int[] arr = new int[7];
        arr[0] = 0;
        d.insertString(0, "Ahoj\n", null);
        arr[1] = d.getLength();
        d.insertString(d.getLength(), "Kuk\n", null);
        arr[2] = d.getLength();
        d.insertString(d.getLength(), "Message\n", null);
        arr[3] = d.getLength();
        d.insertString(d.getLength(), "\n", null);
        arr[4] = d.getLength();
        d.insertString(d.getLength(), "\n", null);
        arr[5] = d.getLength();
        d.insertString(d.getLength(), "\n", null);
        arr[6] = d.getLength();
           
        int line = 0;
        for (int i = 0; i < d.getLength(); i++) {
            int res = ManifestCompletionProvider.getRowFirstNonWhite(d, i);
            if (res == arr[line]) {
                continue;
            }
            line++;
            assertEquals("First non white shall be " + arr[line], arr[line], res);
        }
    }
    public void testEachCharacterWithSpaces() throws BadLocationException {
        StyledDocument d = new FilterDocument(new PlainDocument());
        int[] arr = new int[7];
        arr[0] = 0;
        d.insertString(0, "Ahoj\n  ", null);
        arr[1] = d.getLength();
        d.insertString(d.getLength(), "Kuk\n           ", null);
        arr[2] = d.getLength();
        d.insertString(d.getLength(), "Message\n  ", null);
        arr[3] = d.getLength();
        d.insertString(d.getLength(), "\n   ", null);
        arr[4] = d.getLength();
        d.insertString(d.getLength(), "\n  ", null);
        arr[5] = d.getLength();
        d.insertString(d.getLength(), "\n  ", null);
        arr[6] = d.getLength();
           
        int line = 0;
        for (int i = 0; i < d.getLength(); i++) {
            int res = ManifestCompletionProvider.getRowFirstNonWhite(d, i);
            if (res == arr[line]) {
                continue;
            }
            line++;
            assertEquals("First non white shall be " + arr[line], arr[line], res);
        }
    }
    
    public void testYetAnotherBLE() throws BadLocationException {
        String content =
        "Manifest-Version: 1.0\nOpenIDE-Module: org.myorg.feedreader\nOpenIDE-Module-Layer: org/myorg/feedreader/layer.xml\n" +
        "OpenIDE-Module-Localizing-Bundle: org/myorg/feedreader/Bundle.properties\nOpenIDE-Module-Specification-Version: 1.0\n" +
        "  ";
        
        StyledDocument d = new FilterDocument(new PlainDocument());
        d.insertString(0, content, null);
        int res = ManifestCompletionProvider.getRowFirstNonWhite(d, 194);
    }
    
}
