/*
 * ManifestHyperlinkProvider.java
 *
 * Created on November 6, 2006, 12:27 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package org.netbeans.modules.manifesthyperlinking;

import javax.swing.JOptionPane;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.Element;
import javax.swing.text.JTextComponent;
import javax.swing.text.StyledDocument;
import org.netbeans.lib.editor.hyperlink.spi.HyperlinkProvider;
import org.openide.ErrorManager;
import org.openide.awt.StatusDisplayer;
import org.openide.cookies.EditCookie;
import org.openide.cookies.OpenCookie;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.text.NbDocument;
import org.openide.util.RequestProcessor;

/**
 *
 * @author Geertjan Wielenga
 */
public class ManifestHyperlinkProvider implements HyperlinkProvider {
    
    private Document lastDocument;
    private int startOffset;
    private int endOffset;
    private String identifier;
    private static RequestProcessor OPEN = new RequestProcessor("ManifestOpen");
    
    /** Creates a new instance of ManifestHyperlinkProvider */
    public ManifestHyperlinkProvider() {
        
    }
    
    public boolean isHyperlinkPoint(Document doc, int offset) {
        
        if (!(doc instanceof StyledDocument)) {
            JOptionPane.showMessageDialog(null, "1");
            return false;
        }
        
        StyledDocument bdoc = (StyledDocument) doc;
        JTextComponent target = Utilities.getFocusedComponent();
        
        // We want to work only with the open editor and
        // the editor has to be the active component:
        if ((target == null) || (target.getDocument() != bdoc)) {
            return false;
        }

        
        Element lines = NbDocument.findLineRootElement(bdoc);
        int cnt = NbDocument.findLineNumber(bdoc, offset);
        Element line = lines.getElement(cnt);
        String text;
        try {
            text = doc.getText(line.getStartOffset(), line.getEndOffset() - line.getStartOffset());
        } catch (BadLocationException ex) {
            ErrorManager.getDefault().notify(ex);
            return false;
        }
        int after = text.indexOf(':');
        if (after == -1) {
            return false;
        }
        identifier = text.substring(after + 1).trim();
        
        if (identifier.endsWith("layer.xml")
        || identifier.endsWith(".properties")
        || identifier.endsWith(".class")) {
            startOffset = line.getStartOffset() + after + 1;
            endOffset = line.getEndOffset() - 1;
            lastDocument = bdoc;
            return true;
        } else {
            return false;
        }
    }
    
    public int[] getHyperlinkSpan(Document doc, int offset) {
        
        // First check that we are working with BaseDocument:
        if (!(doc instanceof StyledDocument)) {
            return null;
        }
        
        StyledDocument bdoc = (StyledDocument) doc;
        JTextComponent target = Utilities.getFocusedComponent();
        
        // We want to work only with the open editor
        // and the editor has to be the active component and
        // the document has to be the same as was used in the isHyperlinkPoint method:
        if ((target == null) || (lastDocument != bdoc)) {
            return null;
        }
        
        // Return the position that we defined in the isHyperlinkPoint method:
        return new int[] { startOffset+1, endOffset };
        
    }
    
    public void performClickAction(Document doc, int offset) {
        
        // First check that we are working with BaseDocument:
        if (!(doc instanceof Document)) {
            return;
        }
        
        JTextComponent target = Utilities.getFocusedComponent();
        
        // We want to work only with the open editor and
        // the editor has to be active component and
        // the document has to be the same as was used in the isHyperlinkPoint method:
        if ((target == null) || (lastDocument != doc)) {
            return;
        }
        
        //Open the file:
        String fileName = identifier;
        
        String fileName1 = fileName.replace(".class", ".java");

        Object obj = doc.getProperty(Document.StreamDescriptionProperty);
        
        if (obj instanceof DataObject) {
            OpenFile run = new OpenFile((DataObject)obj, fileName1);
            OPEN.post(run);
        }
    }
    
    class OpenFile implements Runnable {
        
        private DataObject obj;
        private String identifier;
        
        public OpenFile(DataObject obj, String identifier) {
            
            super();
            this.obj = obj;
            this.identifier = identifier;
            
        }
        
        public void run() {
            FileObject root = obj.getPrimaryFile().getParent();
            FileObject fo = root.getFileObject("src/" + identifier);
            if (fo == null) {
                StatusDisplayer.getDefault().setStatusText("Cannot open src/" + identifier + " from " + root);
                return;
            }
            DataObject dObject;
            
            try {
                dObject = DataObject.find(fo);
                
                OpenCookie o = (OpenCookie)dObject.getCookie(OpenCookie.class);
                if (o != null) {
                    o.open();
                    return;
                }
                
                EditCookie ec = (EditCookie)dObject.getCookie(EditCookie.class);
                if (ec != null) {
                    ec.edit();
                    return;
                }
            } catch (DataObjectNotFoundException ex) {
                ex.printStackTrace();
            }
            StatusDisplayer.getDefault().setStatusText("Cannot open " + identifier);
        }
        
    }
    
}
