/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2008 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.myfirstpalette;

import java.awt.Image;
import java.awt.datatransfer.DataFlavor;
import java.beans.BeanInfo;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import javax.swing.Action;
import org.netbeans.modules.myfirstpalette.palette.MyItemData;
import org.netbeans.spi.palette.PaletteActions;
import org.netbeans.spi.palette.PaletteController;
import org.netbeans.spi.palette.PaletteFactory;
import org.netbeans.spi.palette.PaletteFilter;
import org.openide.nodes.Node;
import org.openide.util.Lookup;

/**
 *
 * @author Geertjan Wielenga
 */
public class PaletteSupport {

    public static final DataFlavor MY_DATA_FLAVOR = new DataFlavor(MyItemData.class, "Data");
    private static PaletteController controller;

    /** Creates a new instance of PaletteSupport */
    public PaletteSupport() {
    }

    public static PaletteController createPalette() {
        try {
            controller = PaletteFactory.createPalette("Cards", new MyActions(), new MyPaletteFilter(), null);
        } catch (IOException ex) {
            ex.printStackTrace();
        }

        controller.addPropertyChangeListener(new PropertyChangeListener() {

            public void propertyChange(PropertyChangeEvent evt) {
                if (PaletteController.PROP_SELECTED_ITEM.equals(evt.getPropertyName())) {
                    Lookup selItem = controller.getSelectedItem();
                    if (null != selItem) {
                        Node selNode = (Node) selItem.lookup(Node.class);
                        if (null != selNode) {
                            Image selectedImage = selNode.getIcon(BeanInfo.ICON_COLOR_32x32);
                            String selectedName = selNode.getDisplayName();
                            TestTopComponent.findInstance().setCard(selectedName, selectedImage);
                        }
                    }
                }
            }
        });
        return controller;
    }

    private static class MyActions extends PaletteActions {

        public Action[] getImportActions() {
            return null;
        }

        public Action[] getCustomPaletteActions() {
            return null;
        }

        public Action[] getCustomCategoryActions(Lookup lookup) {
            return null;
        }

        public Action[] getCustomItemActions(Lookup lookup) {
            return null;
        }

        public Action getPreferredAction(Lookup lookup) {
            return null;
        }
    }

    public static class MyPaletteFilter extends PaletteFilter {

        static boolean isCategoryExpertVisible = false;

        public static void setCategoryExpertVisible(boolean isCheckedOrNot) {
            if (isCategoryExpertVisible != isCheckedOrNot) {
                isCategoryExpertVisible = isCheckedOrNot;
                controller.refresh();
            }
        }

        public boolean isValidCategory(Lookup lookup) {
            Node categoryNode = (Node) lookup.lookup(Node.class);
            String categoryDisplayName = categoryNode.getDisplayName();
            if (categoryDisplayName.equals("Expert") && !isCategoryExpertVisible) {
                return false;
            }
            return true;
        }

        public boolean isValidItem(Lookup lookup) {
            return true;
        }
    }
}
