/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2008 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.examples.slownode;

import java.util.List;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.ChildFactory;
import org.openide.nodes.Children;
import org.openide.nodes.Node;

/**
 * Well behaved ChildrFactory object which creates its keys on a background
 * thread since they will be slow to create.
 *
 * This also demonstrates batching child results.  createKeys() will return
 * false the first two times it is called;  this allows some nodes to be
 * returned, while computation of more of them continues.  So each node with
 * one of these children will have 15 children, produced 5 at a time.
 *
 * @author Tim Boudreau
 */
final class SlowChildrenWithCancel extends ChildFactory<ExpensiveToCreateModelObject> {
    private int callCount = 0;

    static Children createChildren() {
        //Since we pass true, our keys will be created on a background thread
        return Children.create(new SlowChildrenWithCancel(), true);
    }

    @Override
    protected Node createNodeForKey(ExpensiveToCreateModelObject key) {
        AbstractNode result = new AbstractNode (createChildren());
        result.setDisplayName (key.getName());
        return result;
    }


    @Override
    protected boolean createKeys(List<ExpensiveToCreateModelObject> toPopulate) {
        for (int i=0; i < 5; i++) {
            toPopulate.add (new ExpensiveToCreateModelObject());
            if (Thread.interrupted()) { //removeNotify() was called - we are cancelled
                callCount = 0;
                return true;
            }
        }
        callCount++;
        //Until we've been called 3 times we're not done;  if false is returned,
        //then our most recent batch of nodes will be shown and we will be called
        //again for more.  So check callCount mod 3 to see if the number of times
        //we've been called is a multiple of 3.
        //We could also use ChildFactory.detachable(), reset callCount to 0
        //in removeNotify() and just check == 3 here.
        boolean result = (callCount % 3) == 0;
        return result;
    }
}
