/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2008 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.web.wicket.framework;

import java.io.File;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import org.netbeans.api.project.libraries.LibraryManager;
import org.netbeans.modules.j2ee.dd.api.web.DDProvider;
import org.netbeans.modules.j2ee.dd.api.web.ServletMapping;
import org.netbeans.modules.j2ee.dd.api.web.WebApp;
import org.netbeans.modules.j2ee.dd.api.web.Servlet;
import org.openide.ErrorManager;
import org.openide.filesystems.FileLock;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileSystem;
import org.openide.filesystems.FileUtil;
import org.openide.filesystems.Repository;

/**
 *
 * @author Petr Pisl
 */
public class WicketConfigUtilities {

    public static final String LIB_WICKET_NAME = "Wicket"; //NOI18N
    private static final String WICKET_JAR_PREFIX = "wicket";  //NOI18N
    private static final String LIBS_FOLDER = "org-netbeans-api-project-libraries/Libraries"; //NOI18N
    private static final String LIB_WICKET_FILE = "wicket.xml";

    /** Returns the Wicket action servlet.
     */
    public static Servlet getActionServlet(FileObject dd) {
        // PENDING - must be more declarative.
        if (dd == null) {
            return null;
        }
        try {
            WebApp webApp = DDProvider.getDefault().getDDRoot(dd);
            return (Servlet) webApp.findBeanByName("Servlet", "ServletClass", // NOI18N
                    "wicket.protocol.http.WicketServlet"); //NOI18N;
        } catch (IOException e) {
            ErrorManager.getDefault().notify(ErrorManager.WARNING, e);
            return null;
        }
    }

    /** Returns the mapping for the Wicket action servlet.
     */
    public static String getActionServletMapping(FileObject dd) {
        Servlet servlet = getActionServlet(dd);
        if (servlet != null) {
            try {
                WebApp webApp = DDProvider.getDefault().getDDRoot(dd);
                ServletMapping[] mappings = webApp.getServletMapping();
                for (int i = 0; i < mappings.length; i++) {
                    if (mappings[i].getServletName().equals(servlet.getServletName())) {
                        return mappings[i].getUrlPattern();
                    }
                }
            } catch (IOException e) {
                ErrorManager.getDefault().notify(ErrorManager.WARNING, e);
            }
        }
        return null;
    }

    /**
     * Returns true if the given wicketFolder contains a Wicket installation.
     */
    public static boolean isWicketInstallFolder(File wicketFolder) {
        return findWicketJarFile(wicketFolder) != null;
    }

    private static File findWicketJarFile(File wicketFolder) {
        File wicketJar = null;
        if (wicketFolder.exists() && wicketFolder.isDirectory()) {
            File[] child = wicketFolder.listFiles();
            for (int i = 0; (i < child.length) && wicketJar == null; i++) {
                if (child[i].getName().startsWith(WICKET_JAR_PREFIX) && child[i].getName().endsWith(".jar")) { // NOI18N
                    wicketJar = child[i];
                }
            }
        }
        return wicketJar;
    }

    /** Returns true if the Wicket library is registered in Library Manager.
     */
    public static boolean wicketLibraryExists() {
        return LibraryManager.getDefault().getLibrary(LIB_WICKET_NAME) != null;
    }

    // XXX: Replace this when API for managing libraries is available
    /** Creates a Wicket library in Library Manager.
     */
    public static boolean createWicketLibrary(File wicketFolder) throws IOException {
        final File wicketJar = findWicketJarFile(wicketFolder);
        if (wicketJar == null || !wicketJar.exists()) {
            return false;
        }
        final File wicketLibFolder = new File(wicketFolder, "lib"); // NOI18N
        if (!wicketLibFolder.exists() || !wicketLibFolder.isDirectory()) {
            return false;
        }

        final FileSystem sysFs = Repository.getDefault().getDefaultFileSystem();
        final FileObject libsFolder = sysFs.findResource(LIBS_FOLDER);
        if (libsFolder == null || !libsFolder.isFolder()) {
            return false;
        }

        sysFs.runAtomicAction(new FileSystem.AtomicAction() {

            public void run() throws IOException {
                FileObject wicketLibFO = libsFolder.getFileObject(LIB_WICKET_FILE);
                if (wicketLibFO == null) {
                    wicketLibFO = libsFolder.createData(LIB_WICKET_FILE);
                }
                FileLock lock = wicketLibFO.lock();
                try {
                    PrintWriter out = new PrintWriter(new OutputStreamWriter(wicketLibFO.getOutputStream(lock)));
                    try {
                        out.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>"); //NOI18N
                        out.println("<!DOCTYPE library PUBLIC \"-//NetBeans//DTD Library Declaration 1.0//EN\" \"http://www.netbeans.org/dtds/library-declaration-1_0.dtd\">"); //NOI18N
                        out.println("<library version=\"1.0\">"); //NOI18N
                        out.println("\t<name>" + LIB_WICKET_NAME + "</name>"); //NOI18N
                        out.println("<localizing-bundle>org.netbeans.modules.web.wicket.framework.Bundle</localizing-bundle>"); // NOI18N
                        out.println("\t<type>j2se</type>");                 //NOI18N
                        out.println("\t<volume>");                          //NOI18N
                        out.println("\t\t<type>classpath</type>");          //NOI18N
                        out.println("\t\t<resource>" + FileUtil.getArchiveRoot(wicketJar.toURI().toURL()).toString() + "</resource>"); //NOI18N
                        File[] child = wicketLibFolder.listFiles();
                        for (int i = 0; i < child.length; i++) {
                            if (child[i].getName().endsWith(".jar")) { // NOI18N
                                out.println("\t\t<resource>" + FileUtil.getArchiveRoot(child[i].toURI().toURL()).toString() + "</resource>"); //NOI18N
                            }
                        }
                        out.println("\t</volume>");                         //NOI18N
                        out.println("\t<volume>");                          //NOI18N
                        out.println("\t\t<type>src</type>");                //NOI18N
                        out.println("\t</volume>");                         //NOI18N
                        out.println("\t<volume>");                          //NOI18N
                        out.println("\t\t<type>javadoc</type>");            //NOI18N
                        out.println("\t</volume>");                         //NOI18N
                        out.println("</library>");                          //NOI18N                                                       
                    } finally {
                        out.close();
                    }
                } finally {
                    lock.releaseLock();
                }
            }
        });
        return true;
    }
}
