/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2008 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.web.wicket.framework;

import java.awt.CardLayout;
import java.io.File;
import javax.swing.JFileChooser;
import javax.swing.event.DocumentListener;
import javax.swing.text.JTextComponent;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;

public class WicketConfigurationPanelVisual extends javax.swing.JPanel implements HelpCtx.Provider, DocumentListener {

    private WicketConfigurationPanel panel;
    private boolean componentsEnabled = true;
    private String currentCard;

    /** Creates new form WicketConfigurationPanelVisual */
    public WicketConfigurationPanelVisual(WicketConfigurationPanel panel) {
        this.panel = panel;

        initComponents();
        jTextFieldAppClass.getDocument().addDocumentListener(this);
        ((JTextComponent) jComboBoxURLPattern.getEditor().getEditorComponent()).getDocument().addDocumentListener(this);

        String cardName = WicketConfigUtilities.wicketLibraryExists() ? "setup" : "library"; // NOI18N
        showCard(cardName);
    }

    private void showCard(String cardName) {
        currentCard = cardName;
        ((CardLayout) getLayout()).show(this, currentCard);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        jPanelSetup = new javax.swing.JPanel();
        jLabelServletName = new javax.swing.JLabel();
        jTextFieldServletName = new javax.swing.JTextField();
        jLabelURLPattern = new javax.swing.JLabel();
        jComboBoxURLPattern = new javax.swing.JComboBox();
        jLabelAppClass = new javax.swing.JLabel();
        jTextFieldAppClass = new javax.swing.JTextField();
        jLabelHomePage = new javax.swing.JLabel();
        jTextFieldHomePage = new javax.swing.JTextField();
        jLabelDummyPojo = new javax.swing.JLabel();
        jTextFieldDummyPojo = new javax.swing.JTextField();
        jLabelMainPackage = new javax.swing.JLabel();
        jTextFieldMainPackage = new javax.swing.JTextField();
        jCheckBoxExampleHeader = new javax.swing.JCheckBox();
        jPanelLibrary = new javax.swing.JPanel();
        jTextFieldWicketFolder = new javax.swing.JTextField();
        jButtonBrowse = new javax.swing.JButton();
        jTextArea1 = new javax.swing.JTextArea();
        jTextArea2 = new javax.swing.JTextArea();

        setLayout(new java.awt.CardLayout());

        jLabelServletName.setLabelFor(jComboBoxURLPattern);
        jLabelServletName.setText("Wicket Servlet Name:");

        jTextFieldServletName.setEditable(false);
        jTextFieldServletName.setText("WicketApplication");
        jTextFieldServletName.getAccessibleContext().setAccessibleDescription(null);

        jLabelURLPattern.setLabelFor(jComboBoxURLPattern);
        jLabelURLPattern.setText(org.openide.util.NbBundle.getMessage(WicketConfigurationPanelVisual.class, "LBL_ConfigPanel_URLPattern"));

        jComboBoxURLPattern.setEditable(true);
        jComboBoxURLPattern.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "/wicket" }));
        jComboBoxURLPattern.getAccessibleContext().setAccessibleDescription(null);

        jLabelAppClass.setLabelFor(jTextFieldAppClass);
        jLabelAppClass.setText(org.openide.util.NbBundle.getMessage(WicketConfigurationPanelVisual.class, "LBL_ConfigPanel_ApplicationResource"));

        jTextFieldAppClass.getAccessibleContext().setAccessibleDescription(null);

        jLabelHomePage.setText("Wicket Home Page:");

        jLabelDummyPojo.setText("Dummy POJO:");

        jLabelMainPackage.setLabelFor(jComboBoxURLPattern);
        jLabelMainPackage.setText("Main Package:");

        jCheckBoxExampleHeader.setSelected(true);
        jCheckBoxExampleHeader.setText("Add Example Formatting");
        jCheckBoxExampleHeader.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jCheckBoxExampleHeader.setMargin(new java.awt.Insets(0, 0, 0, 0));

        org.jdesktop.layout.GroupLayout jPanelSetupLayout = new org.jdesktop.layout.GroupLayout(jPanelSetup);
        jPanelSetup.setLayout(jPanelSetupLayout);
        jPanelSetupLayout.setHorizontalGroup(
            jPanelSetupLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanelSetupLayout.createSequentialGroup()
                .add(jPanelSetupLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jLabelAppClass)
                    .add(jLabelServletName)
                    .add(jLabelURLPattern)
                    .add(jLabelHomePage)
                    .add(jLabelDummyPojo)
                    .add(jLabelMainPackage))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanelSetupLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jTextFieldServletName, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 259, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jComboBoxURLPattern, 0, 259, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jTextFieldAppClass, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 259, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jTextFieldHomePage, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 259, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jTextFieldDummyPojo, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 259, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jTextFieldMainPackage, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 259, Short.MAX_VALUE)))
            .add(jCheckBoxExampleHeader)
        );
        jPanelSetupLayout.setVerticalGroup(
            jPanelSetupLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanelSetupLayout.createSequentialGroup()
                .add(jPanelSetupLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jLabelServletName)
                    .add(jTextFieldServletName, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanelSetupLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabelURLPattern)
                    .add(jComboBoxURLPattern, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanelSetupLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jLabelAppClass)
                    .add(jTextFieldAppClass, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanelSetupLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jLabelHomePage)
                    .add(jTextFieldHomePage, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanelSetupLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabelDummyPojo)
                    .add(jTextFieldDummyPojo, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanelSetupLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabelMainPackage)
                    .add(jTextFieldMainPackage, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jCheckBoxExampleHeader)
                .addContainerGap())
        );

        jPanelSetupLayout.linkSize(new java.awt.Component[] {jTextFieldAppClass, jTextFieldMainPackage}, org.jdesktop.layout.GroupLayout.VERTICAL);

        add(jPanelSetup, "setup");

        jTextFieldWicketFolder.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldWicketFolderActionPerformed(evt);
            }
        });
        jTextFieldWicketFolder.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                jTextFieldWicketFolderKeyPressed(evt);
            }
        });

        jButtonBrowse.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/web/wicket/framework/Bundle").getString("LBL_Browse"));
        jButtonBrowse.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonBrowseActionPerformed(evt);
            }
        });

        jTextArea1.setEditable(false);
        jTextArea1.setFont(new java.awt.Font("Dialog", 1, 12));
        jTextArea1.setLineWrap(true);
        jTextArea1.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/web/wicket/framework/Bundle").getString("MSG_Missing_Library"));
        jTextArea1.setWrapStyleWord(true);
        jTextArea1.setAutoscrolls(false);
        jTextArea1.setOpaque(false);

        jTextArea2.setEditable(false);
        jTextArea2.setLineWrap(true);
        jTextArea2.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/web/wicket/framework/Bundle").getString("MSG_Missing_Library_Result"));
        jTextArea2.setWrapStyleWord(true);
        jTextArea2.setAutoscrolls(false);
        jTextArea2.setOpaque(false);

        org.jdesktop.layout.GroupLayout jPanelLibraryLayout = new org.jdesktop.layout.GroupLayout(jPanelLibrary);
        jPanelLibrary.setLayout(jPanelLibraryLayout);
        jPanelLibraryLayout.setHorizontalGroup(
            jPanelLibraryLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jTextArea1)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanelLibraryLayout.createSequentialGroup()
                .add(jTextFieldWicketFolder, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 329, Short.MAX_VALUE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jButtonBrowse))
            .add(jTextArea2)
        );
        jPanelLibraryLayout.setVerticalGroup(
            jPanelLibraryLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanelLibraryLayout.createSequentialGroup()
                .add(jTextArea1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanelLibraryLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jButtonBrowse)
                    .add(jTextFieldWicketFolder, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jTextArea2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(122, Short.MAX_VALUE))
        );
        add(jPanelLibrary, "library");

    }// </editor-fold>//GEN-END:initComponents

    private void jTextFieldWicketFolderActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldWicketFolderActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextFieldWicketFolderActionPerformed

    private void jButtonBrowseActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonBrowseActionPerformed
        JFileChooser chooser = new JFileChooser();
        chooser.setDialogTitle(NbBundle.getMessage(WicketConfigurationPanelVisual.class, "LBL_SelectLibraryLocation")); //NOI18N
        chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        chooser.setSelectedFile(new File(jTextFieldWicketFolder.getText()));
        if (JFileChooser.APPROVE_OPTION == chooser.showOpenDialog(this)) {
            File wicketFolder = chooser.getSelectedFile();
            jTextFieldWicketFolder.setText(wicketFolder.getAbsolutePath());
            panel.fireChangeEvent();
        }
    }//GEN-LAST:event_jButtonBrowseActionPerformed

    private void jTextFieldWicketFolderKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_jTextFieldWicketFolderKeyPressed
        panel.fireChangeEvent();
    }//GEN-LAST:event_jTextFieldWicketFolderKeyPressed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButtonBrowse;
    private javax.swing.JCheckBox jCheckBoxExampleHeader;
    private javax.swing.JComboBox jComboBoxURLPattern;
    private javax.swing.JLabel jLabelAppClass;
    private javax.swing.JLabel jLabelDummyPojo;
    private javax.swing.JLabel jLabelHomePage;
    private javax.swing.JLabel jLabelMainPackage;
    private javax.swing.JLabel jLabelServletName;
    private javax.swing.JLabel jLabelURLPattern;
    private javax.swing.JPanel jPanelLibrary;
    private javax.swing.JPanel jPanelSetup;
    private javax.swing.JTextArea jTextArea1;
    private javax.swing.JTextArea jTextArea2;
    private javax.swing.JTextField jTextFieldAppClass;
    private javax.swing.JTextField jTextFieldDummyPojo;
    private javax.swing.JTextField jTextFieldHomePage;
    private javax.swing.JTextField jTextFieldMainPackage;
    private javax.swing.JTextField jTextFieldServletName;
    private javax.swing.JTextField jTextFieldWicketFolder;
    // End of variables declaration//GEN-END:variables

    boolean valid() {
        // we don't display any warnings for the "setup" panel, nor if the components
        // are not enabled
        if (!"library".equals(currentCard) || !componentsEnabled) { // NOI18N
            return true;
        }
        boolean valid = WicketConfigUtilities.isWicketInstallFolder(getWicketFolder());
        if (!valid) {
            panel.setErrorMessage(NbBundle.getMessage(WicketConfigurationPanelVisual.class, "MSG_PathIsNotWicketFolder"));
        } else {
            panel.setErrorMessage(null);
            showCard("setup"); // NOI18N
        }
        return valid;
    }

    void enableComponents(boolean enable) {
        // jPanelSetup
        jLabelServletName.setEnabled(enable);
        jTextFieldServletName.setEnabled(enable);
        jLabelURLPattern.setEnabled(enable);
        jComboBoxURLPattern.setEnabled(enable);
        jLabelAppClass.setEnabled(enable);
        jTextFieldAppClass.setEnabled(enable);
        jLabelHomePage.setEnabled(enable);
        jTextFieldHomePage.setEnabled(enable);
        jLabelDummyPojo.setEnabled(enable);
        jTextFieldDummyPojo.setEnabled(enable);
        jLabelMainPackage.setEnabled(enable);
        jTextFieldMainPackage.setEnabled(enable);
        jCheckBoxExampleHeader.setEnabled(enable);
        // jPanelLibrary
        jTextFieldWicketFolder.setEnabled(enable);
        jButtonBrowse.setEnabled(enable);
        // we don't want to display any error messages when we are in
        // fact disabled and thus the user can't do anything to remove them
        componentsEnabled = enable;
        if (!enable) {
            panel.setErrorMessage(null);
        }
    }

    public String getServletName() {
        return jTextFieldServletName.getText();
    }

    public void setServletName(String name) {
        jTextFieldServletName.setText(name);
    }

    public String getURLPattern() {
        return (String) jComboBoxURLPattern.getSelectedItem();
    }

    public void setURLPattern(String pattern) {
        jComboBoxURLPattern.setSelectedItem(pattern);
    }

    public String getAppClass() {
        return jTextFieldAppClass.getText();
    }

    public void setAppClass(String appClass) {
        jTextFieldAppClass.setText(appClass);
    }

    public String getHomePage() {
        return jTextFieldHomePage.getText();
    }

    public void setHomePage(String homePage) {
        jTextFieldHomePage.setText(homePage);
    }

    public String getDummyPojo() {
        return jTextFieldDummyPojo.getText();
    }

    public void setDummyPojo(String dummyPojo) {
        jTextFieldDummyPojo.setText(dummyPojo);
    }

    public String getMainPackage() {
        return jTextFieldMainPackage.getText();
    }

    public void setMainPackage(String mainPackage) {
        jTextFieldMainPackage.setText(mainPackage);
    }

    public Boolean isExampleHeader() {
        return jCheckBoxExampleHeader.isSelected();
    }

    public File getWicketFolder() {
        return new File(jTextFieldWicketFolder.getText());
    }

    /** Help context where to find more about the paste type action.
     * @return the help context for this action
     */
    public HelpCtx getHelpCtx() {
        return new HelpCtx(WicketConfigurationPanelVisual.class);
    }

    public void removeUpdate(javax.swing.event.DocumentEvent e) {
        panel.fireChangeEvent();
    }

    public void insertUpdate(javax.swing.event.DocumentEvent e) {
        panel.fireChangeEvent();
    }

    public void changedUpdate(javax.swing.event.DocumentEvent e) {
        panel.fireChangeEvent();
    }
}
