/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.examples.modules.minicomposer;

import java.awt.Image;
import java.awt.event.ActionEvent;
import java.beans.BeanInfo;
import java.io.IOException;
import java.io.Serializable;
import javax.swing.AbstractAction;
import javax.swing.JComponent;
import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.text.Document;
import org.netbeans.core.spi.multiview.CloseOperationHandler;
import org.netbeans.core.spi.multiview.CloseOperationState;
import org.netbeans.core.spi.multiview.MultiViewDescription;
import org.netbeans.core.spi.multiview.MultiViewElement;
import org.netbeans.core.spi.multiview.MultiViewElementCallback;
import org.netbeans.core.spi.multiview.MultiViewFactory;
import org.openide.ErrorManager;
import org.openide.cookies.EditorCookie;
import org.openide.cookies.OpenCookie;
import org.openide.cookies.PrintCookie;
import org.openide.cookies.SaveCookie;
import org.openide.filesystems.FileLock;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObject;
import org.openide.text.CloneableEditor;
import org.openide.text.CloneableEditorSupport;
import org.openide.text.DataEditorSupport;
import org.openide.text.NbDocument;
import org.openide.util.HelpCtx;
import org.openide.util.Utilities;
import org.openide.windows.CloneableOpenSupport;
import org.openide.windows.CloneableTopComponent;
import org.openide.windows.TopComponent;

public class ScoreEditorSupport extends DataEditorSupport implements EditorCookie.Observable, OpenCookie, PrintCookie {
    public ScoreEditorSupport(ScoreDataObject obj) {
        super(obj, new ScoreEnv(obj));
    }
    protected boolean notifyModified() {
        if (!super.notifyModified()) {
            return false;
        }
        ScoreDataObject obj = (ScoreDataObject)getDataObject();
        if (obj.getCookie(SaveCookie.class) == null) {
            obj.addSaveCookie(new Save());
            obj.setModified(true);
        }
        return true;
    }
    protected void notifyUnmodified() {
        super.notifyUnmodified();
        ScoreDataObject obj = (ScoreDataObject)getDataObject();
        SaveCookie save = (SaveCookie)obj.getCookie(SaveCookie.class);
        if (save != null) {
            obj.removeSaveCookie(save);
            obj.setModified(false);
        }
    }
    private class Save implements SaveCookie {
        public void save() throws IOException {
            saveDocument();
            getDataObject().setModified(false);
        }
    }
    
    protected Pane createPane() {
        String id = "score-closer"; // XXX what is this used for??
        CloseOperationState cos = MultiViewFactory.createUnsafeCloseState(id, new ProceedAction(), new DiscardAction());
        MultiViewDescription textual = new TextualView(cos, getDataObject());
        MultiViewDescription visual = new VisualView(cos, getDataObject());
        CloseOperationHandler handler = new ScoreCloser();
        return (Pane)MultiViewFactory.createCloneableMultiView(new MultiViewDescription[] {textual, visual}, textual, handler);
    }
    private final class ProceedAction extends AbstractAction {
        public ProceedAction() {}
        public void actionPerformed(ActionEvent e) {
            try {
                saveDocument();
            } catch (IOException ex) {
                ErrorManager.getDefault().notify(ex);
            }
        }
    }
    private final class DiscardAction extends AbstractAction {
        public DiscardAction() {}
        public void actionPerformed(ActionEvent e) {
            close(false);
        }
    }
    
    private static final class ScoreCloser implements CloseOperationHandler, Serializable {
        private static final long serialVersionUID = 856396626443L;
        public ScoreCloser() {}
        public boolean resolveCloseOperation(CloseOperationState[] elements) {
            //TODO - show a dialog to ask user if the current changes shall be saved or discarded.
            // if saved, perform proceed actions, otherwise perform discard actions
            for (int i = 0; i < elements.length; i++) {
                elements[i].getProceedAction().actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, "xxx"));
            }
            return true;
        }
    }
    private static class ScoreEnv extends DataEditorSupport.Env {
        private static final long serialVersionUID = 85639662675875L;
        public ScoreEnv(ScoreDataObject obj) {
            super(obj);
        }
        protected FileObject getFile() {
            return getDataObject().getPrimaryFile();
        }
        protected FileLock takeLock() throws IOException {
            return ((ScoreDataObject)getDataObject()).getPrimaryEntry().takeLock();
        }
        public CloneableOpenSupport findCloneableOpenSupport() {
            return (ScoreEditorSupport)getDataObject().getCookie(ScoreEditorSupport.class);
        }
    }
    private static final class TextualView implements MultiViewDescription, Serializable {
        private static final long serialVersionUID = 85639662644335L;
        private transient TextualEditor editor;
        private final transient CloseOperationState cos;
        private DataObject object;
        public TextualView(CloseOperationState cos, DataObject dobj) {
            this.cos = cos;
            object = dobj;
        }
        public String preferredID() {
            return object.getPrimaryFile().getNameExt();
        }
        public int getPersistenceType() {
            return TopComponent.PERSISTENCE_ONLY_OPENED;
        }
        public Image getIcon() {
            return object.getNodeDelegate().getIcon(BeanInfo.ICON_COLOR_16x16);
            // XXX listen to changes and call what.... callback.topComponent.setIcon??
        }
        public HelpCtx getHelpCtx() {
            return object.getHelpCtx();
        }
        public String getDisplayName() {
            return "Source"; // XXX I18N
        }
        public MultiViewElement createElement() {
            if (editor == null) {
                ScoreEditorSupport ces = (ScoreEditorSupport)object.getCookie(ScoreEditorSupport.class);
                editor = new TextualEditor(cos, ces);
            }
            return editor;
        }
    }
    
    private static final class TextualEditor extends CloneableEditor implements MultiViewElement {
        private transient JComponent toolbar;
        private transient MultiViewElementCallback callback;
        private transient final CloseOperationState cos;
        
        
        public TextualEditor() {
            super();
            cos = CloseOperationState.STATE_OK;
        }
        
        public TextualEditor(CloseOperationState cos, CloneableEditorSupport supp) {
            super(supp);
            this.cos = cos;
        }
        public void setMultiViewCallback(MultiViewElementCallback callback) {
            this.callback = callback;
        }
        public CloseOperationState canCloseElement() {
            if (cloneableEditorSupport().isModified()) {
                return cos;
            } else {
                return CloseOperationState.STATE_OK;
            }
        }
        public void componentActivated() {
            super.componentActivated();
        }
        public void componentClosed() {
            super.componentClosed();
        }
        public void componentDeactivated() {
            super.componentDeactivated();
        }
        public void componentHidden() {
            super.componentHidden();
        }
        public void componentOpened() {
            super.componentOpened();
            callback.updateTitle(getDisplayName());
        }
        public void componentShowing() {
            super.componentShowing();
        }
        public JComponent getToolbarRepresentation() {
            if (toolbar == null) {
                JEditorPane pane = getEditorPane();
                if (pane != null) {
                    Document doc = pane.getDocument();
                    if (doc instanceof NbDocument.CustomToolbar) {
                        toolbar = ((NbDocument.CustomToolbar) doc).createToolbar(pane);
                    }
                }
                if (toolbar == null) { // ???
                    toolbar = new JPanel();
                }
            }
            return toolbar;
        }
        public JComponent getVisualRepresentation() {
            return this;
        }
    }
    private final static class VisualView implements MultiViewDescription, Serializable {
        private static final long serialVersionUID = 856396622758455L;
        private transient final CloseOperationState cos;
        private DataObject object;
        
        public VisualView(CloseOperationState cos, DataObject dobj) {
            this.cos = cos;
            object = dobj;
        }
        public String preferredID() {
            return ".visual"; // NOI18N
        }
        public int getPersistenceType() {
            return TopComponent.PERSISTENCE_ONLY_OPENED;
        }
        public Image getIcon() {
            return object.getNodeDelegate().getIcon(BeanInfo.ICON_COLOR_16x16);
        }
        public HelpCtx getHelpCtx() {
            return object.getHelpCtx();
        }
        public String getDisplayName() {
            return "Compose"; // XXX I18N
        }
        public MultiViewElement createElement() {
            return new ScorePanel(((ScoreDataObject) object).getPrimaryEntry(), cos);
        }
    }
}
