/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.examples.modules.povproject;

import java.awt.Toolkit;
import java.io.File;
import org.netbeans.modules.examples.api.povray.ViewService;
import org.openide.ErrorManager;
import org.openide.cookies.OpenCookie;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;

/**
 * Implementation of ViewService, which can show the image associated with
 * a file.
 *
 * @author Tim Boudreau
 */
public class ViewServiceImpl implements ViewService {
    private final PovrayProject proj;
    public ViewServiceImpl(PovrayProject proj) {
        this.proj = proj;
    }

    public boolean isRendered(FileObject file) {
        return getImageFor (file) != null;
    }

    public boolean isUpToDate(FileObject scene) {
        FileObject image = getImageFor (scene);
        boolean result;
        if (image != null) {
            result = scene.lastModified().before(image.lastModified());
            System.err.println("Up to date - " + scene.getName() + " - " + result);
        } else {
            System.err.println("[no image for " + scene.getName() + " - not up to date");
            result = false;
        }
        return result;
    }

    public void view(FileObject scene) {
        FileObject image = getImageFor(scene);
        if (image != null) {
            DataObject dob;
            try {
                dob = DataObject.find(image);
                OpenCookie open = 
                    dob.getNodeDelegate().getLookup().lookup (OpenCookie.class);
                if (open != null) {
                    open.open();
                    return;
                }
            } catch (DataObjectNotFoundException ex) {
                ErrorManager.getDefault().notify(ex);
            }
        }
        Toolkit.getDefaultToolkit().beep();
    }

    FileObject getImageFor (FileObject scene) {
        FileObject imagesDir = proj.getImagesFolder(false);
        FileObject result;
        if (imagesDir != null) {
            File sceneFile = FileUtil.toFile (scene);
            if (sceneFile != null) {
                String imageName = Povray.stripExtension(sceneFile) + ".png";
                //Will be null if it doesn't exist:
                result = imagesDir.getFileObject (imageName);
            } else {
                result = null;
            }
        } else {
            //No images dir, there can't be an image
            result = null;
        }
        return result;
    }



}
