/**
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 **/

package org.netbeans.pluginportal.system.impl;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.NoResultException;
import javax.persistence.Query;
import org.netbeans.pluginportal.data.CommentImpl;
import org.netbeans.pluginportal.data.OtherCategoryImpl;
import org.netbeans.pluginportal.data.PluginImpl;
import org.netbeans.pluginportal.data.RatingImpl;
import org.netbeans.pluginportal.system.Comment;
import org.netbeans.pluginportal.system.OtherCategory;
import org.netbeans.pluginportal.system.Plugin;
import org.netbeans.pluginportal.system.PluginAuthenticationException;
import org.netbeans.pluginportal.system.PluginException;
import org.netbeans.pluginportal.system.impl.ModifyPlugin;
import org.netbeans.pluginportal.system.Rating;

/**
 * This class manages any updates to the plugin repository like adds, updated, and removes.
 *
 * @author David Botterill
 */
public class UpdateManager implements ModifyPlugin {
    
    private ArrayList<UpdateListener> listeners = new ArrayList();
    private EntityManagerFactory emf;
    
    /** Creates a new instance of UpdateManager */
    public UpdateManager(EntityManagerFactory inEmf) {
        emf = inEmf;
    }
    
    public void addUpdateListener(UpdateListener inListener) {
        if(null != inListener) {
            listeners.add(inListener);
        }
    }
    
    private void notifyListenersOfAdd(Plugin plugin) {
        Iterator<UpdateListener> listenerIterator = listeners.iterator();
        while(listenerIterator.hasNext()) {
            listenerIterator.next().pluginAdded(plugin);
        }
    }
    
    private void notifyListenersOfChange(Plugin plugin) {
        Iterator<UpdateListener> listenerIterator = listeners.iterator();
        while(listenerIterator.hasNext()) {
            listenerIterator.next().pluginChanged(plugin);
        }
    }
    
    private void notifyListenersOfRemove(Plugin plugin) {
        Iterator<UpdateListener> listenerIterator = listeners.iterator();
        while(listenerIterator.hasNext()) {
            listenerIterator.next().pluginRemoved(plugin);
        }
    }
    
    
    
    public Plugin addPlugin(Plugin plugin, String authenticationToken, String userid) throws PluginException {
        Plugin managedPlugin = null;
        if(PluginSystemFacade.authenticator.isAuthenticated(userid,authenticationToken)) {
            try {
                EntityManager em = PluginSystemFacade.getEntityManager();
                em.getTransaction().begin();
                
                managedPlugin = em.merge((PluginImpl)plugin);
                
                em.getTransaction().commit();
                em.close();
            } catch(Exception e) {
                throw new PluginException(e);
            }
            notifyListenersOfAdd(plugin);
        } else {
            throw new PluginAuthenticationException("Not Authenticated");
        }
        
        
        
        return managedPlugin;
    }
    
    public Plugin addRating(Plugin plugin, Rating newRating, String authenticationToken, String userid) throws PluginException {
        Plugin newPlugin = null;
        if(PluginSystemFacade.authenticator.isAuthenticated(userid,authenticationToken)) {
            if(null == plugin) {
                throw new PluginException("Error adding Rating!. Plugin can not be null.");
            }
  
            if(null == newRating) {
                throw new PluginException("Error adding Rating!. Rating can not be null.");
            }
  
            /**
             * If the rating exists for the userid, update it otherwise add it.
             */
            try {
                boolean recordNotFound=false;
                
                EntityManager em = PluginSystemFacade.getEntityManager();
                
                String select = "select r from RatingImpl r where r.userid='" + newRating.getUserid() + "'" +
                        " and r.plugin.pluginid =" + plugin.getPluginid();
                Query query = em.createQuery(select);
                RatingImpl returnRating=null;
                try {
                    returnRating = (RatingImpl) query.getSingleResult();
                } catch(NoResultException nre) {
                    recordNotFound = true;
                }
                
                
                if(recordNotFound) {
                    em.getTransaction().begin();
                    
                    /**
                     * Add the relationship
                     */
                    ((RatingImpl)newRating).setPlugin((PluginImpl)plugin);
                    plugin.getRatings().add(newRating);
                    
                    em.persist((RatingImpl)newRating);
                    
                    
                    newPlugin = em.merge((PluginImpl)plugin);
                    em.getTransaction().commit();
                    
                } else {
                    newPlugin = plugin;
                }
                
                /**
                 * Now we need to update the static rating on the pluginrecord.
                 */
                if(null != newPlugin.getRatings()) {
                    em.getTransaction().begin();
                    Iterator<Rating> iterator = newPlugin.getRatings().iterator();
                    Rating currentRating=null;
                    int totalRatings = newPlugin.getRatings().size();
                    int rating=0;
                    while (iterator.hasNext()) {
                        rating += iterator.next().getRating_level().intValue();
                    }
                    double avgRating = 0;
                    if(totalRatings != 0) {
                        avgRating = rating/totalRatings;
                    }
                    newPlugin.setAverage_rating(new Double(avgRating));
                    em.merge((PluginImpl)newPlugin);
                    em.getTransaction().commit();
                }
                
                
                em.close();
            } catch(Exception e) {
                throw new PluginException(e);
            }
            notifyListenersOfChange(plugin);
        } else {
            throw new PluginAuthenticationException("Not Authenticated");
        }
        
        return newPlugin;
    }
    
    public Plugin addComment(Plugin plugin, Comment newComment, String authenticationToken, String userid) throws PluginException {
        Plugin mergedPlugin =null;
        if(PluginSystemFacade.authenticator.isAuthenticated(userid,authenticationToken)) {
            if(null == plugin) {
                throw new PluginException("Error adding Comment!. Plugin can not be null.");
            }
            if (null == newComment) {
                throw new PluginException("Error adding Comment!. Comment can not be null.");
            }
            
            try {
                EntityManager em = PluginSystemFacade.getEntityManager();
                em.getTransaction().begin();
                
                /**
                 * Add the relationship
                 */
                ((CommentImpl)newComment).setPlugin((PluginImpl)plugin);
                plugin.getComments().add(newComment);
                
                em.persist((CommentImpl)newComment);
                mergedPlugin = em.merge((PluginImpl)plugin);
                
                em.getTransaction().commit();
                em.close();
            } catch(Exception e) {
                throw new PluginException(e);
            }
        } else {
            throw new PluginAuthenticationException("Not Authenticated");
        }
        
        return mergedPlugin;
    }
    
    public void removePlugin(Plugin plugin, String authenticationToken, String userid) throws PluginException {
        if(PluginSystemFacade.authenticator.isAuthenticated(userid,authenticationToken)) {
            if(null == plugin) {
                throw new PluginException("Error removing plugin! Plugin can not be null.");
            }
            try {
                EntityManager em = PluginSystemFacade.getEntityManager();
                em.getTransaction().begin();
                
                /**
                 * First merge so we can get a managed entity to remove.
                 */
                PluginImpl removePlugin = em.merge((PluginImpl)plugin);
                
                /**
                 * There are some cases where there are bogus othercategorimpl records
                 * left over for this pluginid so make sure they are gone so a constraint
                 * is not violated.
                 *
                 * Fixes issue: 99998
                 *
                 * David Botterill April 17,2007
                 *
                 */
                
                /**
                 * @TODO - find out the condition where extra othercategoryimpls get added to the
                 * plugin.
                 */
                Query query = em.createNativeQuery("delete from othercategoryimpl where plugin_pluginid=" + removePlugin.getPluginid());
                query.executeUpdate();
                /**
                 * Now remove the plugin.
                 */
                em.remove(removePlugin);
                
                em.getTransaction().commit();
                em.close();
            } catch(Exception e) {
                throw new PluginException(e);
            }
            notifyListenersOfRemove(plugin);
        } else {
            throw new PluginAuthenticationException("Not Authenticated");
        }
        
        
    }
    
    public Plugin modifyPlugin(Plugin plugin, String authenticationToken, String userid) throws PluginException {
        Plugin managedPlugin=null;
        if(PluginSystemFacade.authenticator.isAuthenticated(userid,authenticationToken)) {
            if(null == plugin) {
                throw new PluginException("Error modifying plugin! Plugin can not be null.");
            }
            try {
                EntityManager em = PluginSystemFacade.getEntityManager();
                em.getTransaction().begin();
                managedPlugin=em.merge((PluginImpl)plugin);
                em.getTransaction().commit();
                em.close();
            } catch(Exception e) {
                throw new PluginException(e);
            }
            notifyListenersOfChange(plugin);
        } else {
            throw new PluginAuthenticationException("Not Authenticated");
        }
        
        return managedPlugin;
    }
    
    public void addOtherCategory(Plugin plugin, OtherCategory othercategory, String authenticationToken, String userid) throws PluginException {
        if(PluginSystemFacade.authenticator.isAuthenticated(userid,authenticationToken)) {
            if(null == plugin) {
                throw new PluginException("Error adding Other category! Plugin can not be null.");
            }
            
            if(null == othercategory) {
                throw new PluginException("Error adding Other category! OtherCategory can not be null.");
            }
            
            if(null != plugin.getOthercategories()) {
                
                /**
                 * Now we need to make sure this plugin doesn't already belong to this other category before we add it.
                 */
                Iterator<OtherCategory> categories = plugin.getOthercategories().iterator();
                boolean found = false;
                while(null != categories && categories.hasNext()) {
                    OtherCategory currentCategory = categories.next();
                    if(currentCategory.getCategoryname().equalsIgnoreCase(othercategory.getCategoryname())) {
                        found = true;
                        break;
                    }
                }
                if(!found) {
                    
                    /**
                     * Now we also need to make sure we don't create an OtherCategory with a name like another
                     * but with a different case.
                     */
                    
                    OtherCategory newOtherCategory = getOtherCategory(othercategory.getCategoryname());
                    if(null == newOtherCategory) {
                        newOtherCategory = othercategory;
                    }
                    try {
                        EntityManager em = PluginSystemFacade.getEntityManager();
                        em.getTransaction().begin();
                        
                        /**
                         * Add the relationship
                         */
                        ((OtherCategoryImpl)newOtherCategory).setPlugin((PluginImpl)plugin);
                        
                        plugin.getOthercategories().add(newOtherCategory);
                        
                        em.persist((OtherCategoryImpl)newOtherCategory);
                        em.merge((PluginImpl)plugin);
                        
                        em.getTransaction().commit();
                        em.close();
                        notifyListenersOfAdd(plugin);
                    } catch(Exception e) {
                        throw new PluginException(e);
                    }
                }
                
            }
            
        } else {
            throw new PluginAuthenticationException("Not Authenticated");
        }
    }
    
    public void removeRating(Rating newRating, String authenticationToken, String userid) throws PluginException {
        throw new PluginException("removeRating method not implemented!");
    }
    
    public void removeComment(Comment comment, String authenticationToken, String userid) throws PluginException {
        throw new PluginException("removeComment method not implemented!");
    }
    
    public void removeOtherCategories(Plugin plugin, String authenticationToken, String userid) throws PluginException {
        if(PluginSystemFacade.authenticator.isAuthenticated(userid,authenticationToken)) {
            if(null == plugin) {
                throw new PluginException("Error removing plugin! Plugin can not be null.");
            }
            /**
             *
             * Since JPA won't delete the OtherCategory unless we delete the plugin (Cascade DELETE) we
             * must manage the relationship ourselves and remove the OtherCategory.
             *
             */
            
            /**
             * Since the plugin.OtherCategories collection will be modified.  We need to save off a
             * different collection so we won't get a ConcurrentModificationException while iterating
             * over it.
             */
            ArrayList<OtherCategory> categories = new ArrayList(plugin.getOthercategories());
            Iterator<OtherCategory> catIterator = categories.iterator();
            OtherCategory currentOtherCategory = null;
            while(null != catIterator && catIterator.hasNext()) {
                currentOtherCategory = catIterator.next();
                
                try {
                    EntityManager em = PluginSystemFacade.getEntityManager();
                    em.getTransaction().begin();
                    
                    /**
                     * remove the othercategory from the plugin list.
                     */
                    plugin.getOthercategories().remove(currentOtherCategory);
                    em.merge((PluginImpl)plugin);
                    
                    /**
                     * Now remove this OtherCategory.  Note that this will only delete
                     * the entry in the many-to-many relationship that matches this plugin.  The
                     * OtherCategory will be deleted based on its ID and not its name.
                     */
                    OtherCategoryImpl managedOtherCategory = em.merge((OtherCategoryImpl)currentOtherCategory);
                    em.remove(managedOtherCategory);
                    
                    em.getTransaction().commit();
                    em.close();
                } catch(Exception e) {
                    throw new PluginException(e);
                }
            }
            notifyListenersOfChange(plugin);
        } else {
            throw new PluginAuthenticationException("Not Authenticated");
        }
    }
    /**
     * This method needs to be executed by a non-authenticated use when a plugin download
     * is executed.
     */
    public void incrementPluginDownloads(Plugin plugin) throws PluginException {
        Plugin managedPlugin=null;
        
        if(null == plugin) {
            throw new PluginException("Error modifying plugin! Plugin can not be null.");
        }
        if(null != plugin.getDownloads()) {
            plugin.setDownloads(plugin.getDownloads() +1);
        } else {
            plugin.setDownloads(1L);
        }
        try {
            EntityManager em = PluginSystemFacade.getEntityManager();
            em.getTransaction().begin();
            managedPlugin=em.merge((PluginImpl)plugin);
            em.getTransaction().commit();
            em.close();
        } catch(Exception e) {
            throw new PluginException(e);
        }
        
        notifyListenersOfChange(plugin);
        
    }
    
    private OtherCategory getOtherCategory(String inCat) {
        OtherCategory returnOtherCategory = null;
        
        String select = "select o from OtherCategoryImpl o where upper(o.categoryname) ='" + inCat.toUpperCase() + "'";
        EntityManager em = emf.createEntityManager();
        Query query = em.createQuery(select);
        List<OtherCategoryImpl> resultList = query.getResultList();
        /**
         * We could already have multiple entries of the same category with different case but we don't
         * want to make the situation worse so we'll just return the first one.
         */
        if(null != resultList && resultList.size() > 0) {
            returnOtherCategory = new OtherCategoryImpl();
            OtherCategoryImpl foundCategory = resultList.get(0);
            returnOtherCategory.setCategoryname(foundCategory.getCategoryname());
        }
        
        return returnOtherCategory;
    }
    
    
    
    
}
