/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.qa.reports;

import java.util.*;
import java.text.SimpleDateFormat;
import java.text.ParseException;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * SAX parser of the XML export of issues.
 *
 * @author Jan Stola
 */
public class XMLExportHandler extends DefaultHandler {
    /** Parsed issues. */
    private List issues;
    /** Currently parsed issue. */
    private Issue issue;
    /** Character data in the current tag. */
    private StringBuffer chars;
    /** Comments of the current issue. */
    private List comments;
    /** Currecly read comment. */
    private Comment comment;
        
    /**
     * Returns parsed issues.
     *
     * @return parsed issues.
     */
    public List getIssues() {
        return issues;
    }
    
    /**
     * Handles start of the document.     
     */
    public void startDocument()
    throws SAXException {
        issues=new LinkedList();
    }
    
    /**
     * Handles start of some element.
     */
    public void startElement(String namespaceURI, String sName, String qName, Attributes attrs)
    throws SAXException {        
        chars=new StringBuffer();
        if (qName.equals("issue")) {
            issue=new Issue();
            for (int i=0; i<attrs.getLength(); i++) {
                String key=attrs.getQName(i);
                String value=attrs.getValue(i);
                if (key.equals("issue_status")) {
                    if (value.equals(IssueQuery.STATUS_NEW)) issue.status=Issue.STATUS_NEW;
                    if (value.equals(IssueQuery.STATUS_ASSIGNED)) issue.status=Issue.STATUS_ASSIGNED;
                    if (value.equals(IssueQuery.STATUS_REOPENED)) issue.status=Issue.STATUS_REOPENED;
                    if (value.equals(IssueQuery.STATUS_RESOLVED)) issue.status=Issue.STATUS_RESOLVED;
                    if (value.equals(IssueQuery.STATUS_VERIFIED)) issue.status=Issue.STATUS_VERIFIED;
                    if (value.equals(IssueQuery.STATUS_CLOSED)) issue.status=Issue.STATUS_CLOSED;
                } else if (key.equals("priority")) {
                    if (value.equals(IssueQuery.PRIORITY_P1)) issue.priority=Issue.PRIORITY_P1;
                    if (value.equals(IssueQuery.PRIORITY_P2)) issue.priority=Issue.PRIORITY_P2;
                    if (value.equals(IssueQuery.PRIORITY_P3)) issue.priority=Issue.PRIORITY_P3;
                    if (value.equals(IssueQuery.PRIORITY_P4)) issue.priority=Issue.PRIORITY_P4;
                    if (value.equals(IssueQuery.PRIORITY_P5)) issue.priority=Issue.PRIORITY_P5;
                } else if (key.equals("resolution")) {
                    if (value.equals(IssueQuery.RESOLUTION_DUPLICATE)) issue.resolution=Issue.RESOLUTION_DUPLICATE;
                    if (value.equals(IssueQuery.RESOLUTION_FIXED)) issue.resolution=Issue.RESOLUTION_FIXED;
                    if (value.equals(IssueQuery.RESOLUTION_INVALID)) issue.resolution=Issue.RESOLUTION_INVALID;
                    if (value.equals(IssueQuery.RESOLUTION_LATER)) issue.resolution=Issue.RESOLUTION_LATER;
                    if (value.equals(IssueQuery.RESOLUTION_MOVED)) issue.resolution=Issue.RESOLUTION_MOVED;
                    if (value.equals(IssueQuery.RESOLUTION_REMIND)) issue.resolution=Issue.RESOLUTION_REMIND;
                    if (value.equals(IssueQuery.RESOLUTION_WONTFIX)) issue.resolution=Issue.RESOLUTION_WONTFIX;
                    if (value.equals(IssueQuery.RESOLUTION_WORKSFORME)) issue.resolution=Issue.RESOLUTION_WORKSFORME;
                }
            }
        } else if (qName.equals("long_desc")) {
            if (comments==null) comments=new LinkedList();
            comment=new Comment();
        }
    }
    
    /**
     * Handles character data.     
     */
    public void characters(char buf[], int offset, int len)
    throws SAXException {        
        chars.append(buf, offset, len);
    }
    
    /**
     * Handles end of some element.
     */
    public void endElement(String namespaceURI, String sName, String qName)
    throws SAXException {        
        String text=chars.toString();
        if (qName.equals("issue_id")) {
            try {
                issue.number=Integer.parseInt(text);
            } catch (NumberFormatException e) {
                throw new SAXException(e);
            }
        } else if (qName.equals("component")) {
            issue.product=text;
        } else if (qName.equals("version")) {
            issue.version=text;
        } else if (qName.equals("rep_platform")) {
            issue.platform=text;
        } else if (qName.equals("assigned_to")) {
            issue.assignedTo=text;
        } else if (qName.equals("subcomponent")) {
            issue.component=text;
        } else if (qName.equals("reporter")) {
            issue.reporter=text;
        } else if (qName.equals("target_milestone")) {
            issue.target=text;
        } else if (qName.equals("issue_type")) {
            if (text.equals(IssueQuery.TYPE_DEFECT)) issue.type=Issue.TYPE_DEFECT;
            if (text.equals(IssueQuery.TYPE_ENHANCEMENT)) issue.type=Issue.TYPE_ENHANCEMENT;
            if (text.equals(IssueQuery.TYPE_FEATURE)) issue.type=Issue.TYPE_FEATURE;
            if (text.equals(IssueQuery.TYPE_PATCH)) issue.type=Issue.TYPE_PATCH;
            if (text.equals(IssueQuery.TYPE_TASK)) issue.type=Issue.TYPE_TASK;
        } else if (qName.equals("creation_ts")) {
            SimpleDateFormat sdf=new SimpleDateFormat("yyyy-MM-dd HH:mm");
            try {
                issue.creation=sdf.parse(text);
            } catch (ParseException e) {
                throw new SAXException(e);
            }
        } else if (qName.equals("qa_contact")) {
            issue.QAContact=text;
        } else if (qName.equals("status_whiteboard")) {
            issue.statusWhiteboard=text;
        } else if (qName.equals("op_sys")) {
            issue.os=text;
        } else if (qName.equals("short_desc")) {
            issue.summary=text;
        } else if (qName.equals("keywords")) {
            Set keywords=new HashSet();
            while (text.indexOf(',')!=-1) {
                String keyword=text.substring(0,text.indexOf(',')).trim();
                if (keyword.length()!=0) {
                    keywords.add(keyword);
                }
                text=text.substring(text.indexOf(',')+1);
            }
            text=text.trim();
            if (text.length()!=0) {
                keywords.add(text);
            }
            issue.keywords=keywords;
        } else if (qName.equals("long_desc")) {
            comments.add(comment);
            comment=null;
        } else if (qName.equals("who")) {
            comment.who=text;
        } else if (qName.equals("issue_when")) {
            SimpleDateFormat sdf=new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            try {
                comment.when=sdf.parse(text);
            } catch (ParseException e) {
                throw new SAXException(e);
            }
        } else if (qName.equals("thetext")) {
            comment.text=text;
        } else if (qName.equals("issue")) {            
            issue.comments=comments;
            issues.add(issue);            
            comments=null;
            issue=null;
        }
    }
    
    
}
