/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.beans;

import java.util.ArrayList;
import org.netbeans.testspeceditor.*;
import org.netbeans.testspeceditor.events.Bean;
import org.netbeans.testspeceditor.events.BeanChangeSupport;
import org.netbeans.testspeceditor.events.Persistent;
import org.netbeans.testspeceditor.undo.UndoAnnotation;
import org.netbeans.testspeceditor.undo.UndoManager;
import org.netbeans.testspeceditor.undo.UndoMethods;
import org.netbeans.testspeceditor.utils.Utils;
/**
 *
 * @author ehucka
 */
public class TestCase implements Bean, Persistent, TextData {
    
    private String title="";
    private String case_id="";
    private ArrayList<TestAction> actions;
    private Comments comments;
    private Teardown teardown;
    private Setup setup;
    private String attributes;
    private String xrepository;
    private String xmodule;
    private String xtesttype;
    private String xmethod;
    
    transient BeanChangeSupport support = new BeanChangeSupport(this);
    transient int id = -1;
    
     
    public TestCase() {
        actions=new ArrayList(24);
        comments=new Comments(this);
        teardown=new Teardown(this);
        setup=new Setup(this);
    }
    
    public TestCase(String title) {
        this();
        id=Application.getInstance().getDocument().getTestSpec().caseid++;
        UndoManager.getInstance().setLogProperties(false);
        setTitle(title);
        actions.add(new TestResult());
        UndoManager.getInstance().setLogProperties(true);
        setComments("TODO");
    }
    
    public String getTitle() {
        return title;
    }
    
    public String getCase_id() {
        return case_id;
    }
    
    public TestAction[] getActions() {
        return actions.toArray(new TestAction[actions.size()]);
    }
    
    public TestResult getResult() {
        return (TestResult)(actions.get(actions.size()-1));
    }
    
    public String getComments() {
        return comments.getContent();
    }
    
    public Comments getCommentsBean() {
        return comments;
    }
    
    public String getTeardown() {
        return teardown.getContent();
    }
    
    public Teardown getTeardownBean() {
        return teardown;
    }
    
    public String getSetup() {
        return setup.getContent();
    }
    
    public Setup getSetupBean() {
        return setup;
    }
    
    public String getAttributes() {
        return attributes;
    }
    
    public String getXrepository() {
        return xrepository;
    }
    
    public String getXmodule() {
        return xmodule;
    }
    
    public String getXtesttype() {
        return xtesttype;
    }
    
    public String getXmethod() {
        return xmethod;
    }
    
    public void setTitle(String title) {
        String old = getTitle();
        this.title = title;
        setCase_id(Utils.getID(title, id));
        support.propertyChanged("title", old, getTitle());
    }
    
    public void setCase_id(String case_id) {
        String old = getCase_id();
        this.case_id = case_id;
        support.propertyChanged("case_id", old, getCase_id());
    }
    
    public void addAction(TestAction action) {
        int res=-1;
        for (int i=actions.size()-1; i >= 0;i--) {
            if (actions.get(i) instanceof TestResult) {
                res=i;
                break;
            }
        }
        this.actions.add(res, action);
        support.childrenChanged(action, -1, res);
    }

    public void addAction(TestAction after, TestAction action) {
        int res=-1;
        for (int i=actions.size()-1; i >= 0;i--) {
            if (actions.get(i) == after) {
                break;
            }
            res=i;
        }
        this.actions.add(res, action);
        support.childrenChanged(action, -1, res);
    }
    
    public void removeAction(TestAction action) {
        int old=actions.indexOf(action);
        actions.remove(action);
        if (old >= 0) {
            support.childrenChanged(action, old, -1);
        } else {
            throw new IllegalStateException("illegal index");
        }
    }
    
    public void moveUp(TestAction action) {
        int index=actions.indexOf(action);
        actions.remove(index);
        index--;
        actions.add(index, action);
        support.childrenChanged(action, index+1, index);
    }
    
    public void moveDown(TestAction action) {
        int index=actions.indexOf(action);
        actions.remove(index);
        index++;
        actions.add(index, action);
        support.childrenChanged(action, index-1, index);
    }
    
    @UndoAnnotation(method=UndoMethods.GET)
    public TestAction get(int index) {
        return actions.get(index);
    }
    
    @UndoAnnotation(method=UndoMethods.REMOVE)
    public TestAction remove(int index) {
        TestAction act=actions.get(index);
        removeAction(act);
        return act;
    }
    
    @UndoAnnotation(method=UndoMethods.ADD)
    public void add(TestAction act, int index) {
        actions.add(index, act);
        support.childrenChanged(act, -1, index);
    }
    
    public void setComments(String comments) {
        this.comments.setContent(comments);
    }
    
    public void setTeardown(String teardown) {
        this.teardown.setContent(teardown);
    }
    
    public void setSetup(String setup) {
        this.setup.setContent(setup);
    }
    
    public void setAttributes(String attributes) {
        String old = getAttributes();
        this.attributes = attributes;
        support.propertyChanged("attributes", old, getAttributes());
    }
    
    public void setXrepository(String xrepository) {
        String old = getXrepository();
        this.xrepository = xrepository;
        support.propertyChanged("xrepository", old, getXrepository());
    }
    
    public void setXmodule(String xmodule) {
        String old = getXmodule();
        this.xmodule = xmodule;
        support.propertyChanged("xmodule", old, getXmodule());
    }
    
    public void setXtesttype(String xtestType) {
        String old = getXtesttype();
        this.xtesttype = xtestType;
        support.propertyChanged("xtesttype", old, getXtesttype());
    }
    
    public void setXmethod(String xmethod) {
        String old = getXmethod();
        this.xmethod = xmethod;
        support.propertyChanged("xmethod", old, getXmethod());
    }
    
    public BeanChangeSupport getBeanSupport() {
        return support;
    }
    
    public String toString() {
        return title;
    }
    
    public Bean copy() {
        TestCase tc=new TestCase(getTitle());
        tc.setAttributes(getAttributes());
        tc.setComments(getComments());
        tc.setSetup(getSetup());
        tc.setTeardown(getTeardown());
        tc.setXmethod(getXmethod());
        tc.setXmodule(getXmodule());
        tc.setXrepository(getXrepository());
        tc.setXtesttype(getXtesttype());
        for (TestAction ta:actions) {
            if (!(ta instanceof TestResult))
                tc.addAction((TestAction)(ta.copy()));
        }
        tc.getResult().setContent(getResult().getContent());
        return tc;
    }
    
    public void initObject() {
        comments.setParent(this);
        setup.setParent(this);
        teardown.setParent(this);
    }
    
    public String[] getContents() {
        String[] ret=new String[3+actions.size()];
        ret[0]=setup.getContent();
        ret[1]=comments.getContent();
        ret[2]=teardown.getContent();
        for (int i = 0; i < actions.size(); i++) {
            ret[2+i]=actions.get(i).getContent();
        }
        return ret;
    }
}
