/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.ui.actions;

import java.awt.Component;
import org.netbeans.testspeceditor.Application;
import org.netbeans.testspeceditor.ui.edits.HTMLEditor;
import org.netbeans.testspeceditor.ui.edits.MatchingWords;
import org.netbeans.testspeceditor.utils.Timer;

/**
 *
 * @author ehucka
 */
public class NextMatchingWordAction  implements StandaloneAction, Runnable {
    
    HTMLEditor invoked = null;
    String context = null;
    String[] matching = null;
    int index=0;
    int startinsert=-1;
    int insertlength=-1;
    
     
    public NextMatchingWordAction() {
        
    }
    
    public void perform() {
        try {
            if (invoked == null || Application.getInstance().getUi().getFocusOwner() != invoked) {
                if (invoked != null) {
                    run();
                }
                Component comp=Application.getInstance().getUi().getFocusOwner();
                if (comp instanceof HTMLEditor) {
                    invoked=(HTMLEditor)comp;
                    context=invoked.getContext(false);
                    matching=MatchingWords.getInstance().getMatchings(context);
                }
            }
            if (matching.length > 0) {
                String word=matching[index++];
                if (index >= matching.length) {
                    index=0;
                }
                word=word.substring(context.length());
                if (insertlength > -1) { //remove first
                    invoked.getDocument().remove(startinsert, insertlength);
                }
                if (startinsert < 0)
                    startinsert=invoked.getCaretPosition();
                insertlength=word.length();
                if (insertlength > 0) {
                    invoked.getDocument().insertString(startinsert, word, null);
                }
                invoked.setCaretPosition(startinsert+insertlength);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        Timer.getInstance().delayOnce(this, 1000);
    }
    
    public void run() {
        invoked=null;
        context=null;
        matching=null;
        index=0;
        startinsert=-1;
        insertlength=-1;
    }
    
    public boolean isEnabled() {
        return true;
    }
    
    public String getName() {
        return "Next Matching";
    }
    
    public String getHelpCtx() {
        return "Next Matching Word.";
    }
    
    public char getMnemonic() {
        return 'n';
    }
    
    public javax.swing.KeyStroke getKeyStroke() {
        return javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_L, java.awt.event.InputEvent.CTRL_MASK);
    }
    
    public int compareTo(Object o) {
        return getName().compareTo(((Action)o).getName());
    }
    
}
