/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.ui.edits;

import javax.swing.text.Document;

/**
 *
 * @author ehucka
 */
public class InsertionFormatTag extends FormatTag {
    
    public static final String INSERTION_POINT = "|";
    
    private String text;
    private boolean special;
    
     
    public InsertionFormatTag(String name, String text) {
        this.setText(text);
        setName(name);
        setSpecial(true);
    }
    
    /**
     ** for common html tags
     */
    public InsertionFormatTag(String name) {
        this.setText("<"+name+">"+INSERTION_POINT+"</"+name+">");
        setName(name);
        setSpecial(false);
    }
    
    public int compareTo(Object o) {
        if (o instanceof InsertionFormatTag) {
            InsertionFormatTag t=(InsertionFormatTag)o;
            if (t.isSpecial() && !isSpecial()) {
                return 1;
            } else if (!t.isSpecial() && isSpecial()) {
                return -1;
            }
            return toString().compareTo(o.toString());
        } else {
            return -1;
        }
    }
    
    public boolean matchContext(String context) {
        if (context == null || context.length() == 0) return true;
        return getName().startsWith(context);
    }
    
    public String getText() {
        return text;
    }
    
    public void setText(String text) {
        this.text = text;
    }
    
    public boolean isSpecial() {
        return special;
    }
    
    public void setSpecial(boolean special) {
        this.special = special;
    }
    
    public int insertInto(Document document, String context, int caret, int selStart, int selEnd) throws Exception {
        if (caret != selStart || caret != selEnd) { //selection
            String text=document.getText(selStart, selEnd-selStart);
            document.remove(selStart, selEnd-selStart);
            StringBuilder sb=new StringBuilder();
            int c = getText().indexOf(INSERTION_POINT);
            String tt=new String(getText());
            while (c > -1) {
                String pre=tt.substring(0, c);
                sb.append(pre);
                sb.append(text);
                if (c < tt.length()-1) {
                    tt=tt.substring(c+1);
                    c = tt.indexOf(INSERTION_POINT);
                } else {
                    c=-1;
                }
            }
            if (tt.length() > 0)
                sb.append(tt);
            document.insertString(selStart, sb.toString(), null);
            return selStart+sb.length();
        } else { //not selected
            if (context.length() > 0)
                document.remove(caret-context.length(), context.length());
            StringBuilder sb=new StringBuilder();
            int c = getText().indexOf(INSERTION_POINT);
            String tt=new String(getText());
            int last=0;
            while (c > -1) {
                String pre=tt.substring(0, c);
                sb.append(pre);
                if (c < tt.length()-INSERTION_POINT.length()) {
                    tt=tt.substring(c+INSERTION_POINT.length());
                    c = tt.indexOf(INSERTION_POINT);
                    last=sb.length();
                } else {
                    c=-1;
                }
            }
            if (tt.length() > 0)
                sb.append(tt);
            document.insertString(caret-context.length(), sb.toString(), null);
            return caret-context.length()+last;
        }
    }
}
