/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.ui.status;

import java.util.ArrayList;
import javax.swing.SwingUtilities;
/**
 *
 * @author  eh103527
 */
public class StatusManager {
    
    static StatusManager instance=null;
    
    ArrayList<StatusTarget> statusTargets;
    String text;
    double percent;
    
    ArrayList intervals;
    
    WaitThread wait;
    
     
    public StatusManager() {
        statusTargets=new ArrayList(4);
        intervals=new ArrayList(4);
    }
    
    public static StatusManager getInstance() {
        if (instance == null) {
            instance=new StatusManager();
        }
        return instance;
    }
    
    public void registerStaturTarget(StatusTarget trg) {
        statusTargets.add(trg);
    }
    
    public void setStatusText(String text) {
        this.text=text;
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                for (int i=0;i < statusTargets.size();i++) {
                    StatusTarget st=(StatusTarget)(statusTargets.get(i));
                    st.setStatusLineText(StatusManager.this.text);
                }
            }
        });
    }
    
    public void setWaitCursor(final boolean wait) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                for (int i=0;i < statusTargets.size();i++) {
                    StatusTarget st=(StatusTarget)(statusTargets.get(i));
                    st.setWaitCursor(wait);
                }
            }
        });
    }
    
    /**
     *
     * @param percent percent value 0-100
     */
    public void setProgressValue(double percent) {
        this.percent=getAbsoluteValue(percent);
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                for (int i=0;i < statusTargets.size();i++) {
                    StatusTarget st=(StatusTarget)(statusTargets.get(i));
                    st.setProgressValue(StatusManager.this.percent);
                }
            }
        });
    }
    
    public void clearStatusLine() {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                for (int i=0;i < statusTargets.size();i++) {
                    StatusTarget st=(StatusTarget)(statusTargets.get(i));
                    st.setStatusLineText(" ");
                }
            }
        });
    }
    
    public void setStatusText(String text,int seconds) {
        if (wait != null) {
            wait.interrupt();
        }
        while (wait != null) {
            try {
                Thread.currentThread().sleep(1);
            } catch (Exception ex) {
            }
        }
        wait=new WaitThread(text,seconds);
        wait.start();
    }
    
    public void resetProgressBar() {
        intervals.clear();
        percent=0.0;
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                for (int i=0;i < statusTargets.size();i++) {
                    StatusTarget st=(StatusTarget)(statusTargets.get(i));
                    st.setProgressValue(percent);
                }
            }
        });
    }
    
    public void addInterval(double start, double length) {
        intervals.add(new Interval(start, length));
    }
    
    public void removeInterval() {
        if (intervals.size() > 0) {
            intervals.remove(intervals.size()-1);
        }
    }
    
    public double getAbsoluteValue(double relative) {
        double rel=relative;
        for (int i=intervals.size()-1;i >= 0;i--) {
            Interval in=(Interval)(intervals.get(i));
            rel=in.getAbsoluteValue(rel);
        }
        return rel;
    }
    
    public class WaitThread extends Thread {
        String text;
        int time;
        
        public WaitThread(String text, int seconds) {
            super("StatusLine Wait Thread");
            this.text=text;
            this.time=seconds;
        }
        
        public void run() {
            try {
                setStatusText(text);
                sleep(time*1000);
            } catch (InterruptedException e) {
            } finally {
                clearStatusLine();
                StatusManager.this.wait=null;
            }
        }
    }
}
