/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.ui.tree;

import java.util.ArrayList;
import java.util.Collections;
import javax.swing.Icon;
import javax.swing.event.TreeModelEvent;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import org.netbeans.testspeceditor.events.BeanChangeListener;
/**
 *
 * @author ehucka
 */
public abstract class Node implements TreeNode, BeanChangeListener {
    
    protected Node parent=null;
    protected ArrayList<Node> children;
    
    public Node(boolean allowsChildren) {
        if (allowsChildren)
            children=new ArrayList();
    }
    
    public Node(Node parent, boolean allowsChildren) {
        this(allowsChildren);
        this.parent=parent;
    }
    
    public abstract Icon getIcon();
    
    public java.util.Enumeration children() {
        return Collections.enumeration(children);
    }
    
    public void addChild(Node child) {
        children.add(child);
    }
    
    public void addChild(Node child, int index) {
        children.add(index, child);
    }
    
    public void remove(int index) {
        children.remove(index);
    }
    
    public int remove(Node child) {
        int ret=children.indexOf(child);
        children.remove(child);
        return ret;
    }
    
    public boolean isLeaf() {
        return (!getAllowsChildren() || (children.size() == 0));
    }
    
    public TreeNode getParent() {
        return parent;
    }
    
    public boolean getAllowsChildren() {
        return (children != null);
    }
    
    public javax.swing.tree.TreeNode getChildAt(int childIndex) {
        return children.get(childIndex);
    }
    
    public int getIndex(javax.swing.tree.TreeNode node) {
        return children.indexOf(node);
    }
    
    public int getChildCount() {
        return children.size();
    }
    
    public void clearChildren() {
        children.clear();
    }
    
    public Node getRoot() {
        Node n=this;
        while (n.getParent() != null) {
            n=(Node)n.getParent();
        }
        return n;
    }
    
    public AbstractTreeModel getTreeModel() {
        return ((Node)getRoot()).getTreeModel();
    }
    
    public void treeStructureChanged(javax.swing.event.TreeModelEvent e) {
        if (getTreeModel() != null) {
            getTreeModel().treeStructureChanged(e);
        } else if (getParent() != null) {
            ((Node)getParent()).treeStructureChanged(e);
        }
    }
    
    public void treeNodesRemoved(javax.swing.event.TreeModelEvent e) {
        if (getTreeModel() != null) {
            getTreeModel().treeNodesRemoved(e);
        } else if (getParent() != null) {
            ((Node)getParent()).treeNodesRemoved(e);
        }
    }
    
    public void treeNodesInserted(javax.swing.event.TreeModelEvent e) {
        if (getTreeModel() != null) {
            getTreeModel().treeNodesInserted(e);
        } else if (getParent() != null) {
            ((Node)getParent()).treeNodesInserted(e);
        }
    }
    
    public void treeNodesChanged(javax.swing.event.TreeModelEvent e) {
        if (getTreeModel() != null) {
            getTreeModel().treeNodesChanged(e);
        } else if (getParent() != null) {
            ((Node)getParent()).treeNodesChanged(e);
        }
    }
    
    protected TreeModelEvent getNodeEvent() {
        if (getParent() != null) {
            return new TreeModelEvent(getTreeModel(), ((Node)getParent()).getTreePath(),
                    new int[] {getParent().getIndex(this)}, new Object[] {this});
        } else {
            return new TreeModelEvent(this, getTreePath());
        }
    }
    
    public TreePath getTreePath() {
        ArrayList<Node> nodes=new ArrayList();
        Node n=this;
        nodes.add(n);
        while (n.getParent() != null) {
            n=(Node)(n.getParent());
            nodes.add(0, n);
        }
        return new TreePath(nodes.toArray(new Node[nodes.size()]));
    }
}
