/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.ui.tree;

import java.awt.Toolkit;
import java.awt.datatransfer.StringSelection;
import javax.swing.Icon;
import javax.swing.event.TreeModelEvent;
import org.netbeans.testspeceditor.beans.TestAction;
import org.netbeans.testspeceditor.beans.TestResult;
import org.netbeans.testspeceditor.events.Bean;
import org.netbeans.testspeceditor.ui.actions.ActionAnnotation;
import org.netbeans.testspeceditor.ui.actions.ActionsManager;
import org.netbeans.testspeceditor.ui.actions.CopyAction;
import org.netbeans.testspeceditor.ui.actions.CopyAsTextAction;
import org.netbeans.testspeceditor.ui.actions.CutAction;
import org.netbeans.testspeceditor.ui.actions.DeleteAction;
import org.netbeans.testspeceditor.ui.actions.MoveDownAction;
import org.netbeans.testspeceditor.ui.actions.MoveUpAction;
import org.netbeans.testspeceditor.ui.actions.NewAction;
import org.netbeans.testspeceditor.ui.actions.NewTestActionAction;
import org.netbeans.testspeceditor.ui.actions.PasteAction;

/**
 *
 * @author ehucka
 */
public class TestActionNode<A extends TestAction> extends DataNode<A> {
    
    public static Icon icon;
    
    static {
        icon = new javax.swing.ImageIcon(TestActionNode.class.getResource("/org/netbeans/testspeceditor/ui/icons/action.png"));
        ActionsManager.getInstance().registerActions(TestActionNode.class,
                new NewAction(new NewTestActionAction()), null,
                new MoveUpAction(), new MoveDownAction(), null,
                new CopyAction(), new CopyAsTextAction(), new CutAction(), new PasteAction(), new DeleteAction());
    }
    
    public TestActionNode(TestCaseNode root, A data) {
        super(data, root, false);
        getTestAction().getBeanSupport().addBeanChangeListener(this);
    }
    
    public TestAction getTestAction() {
        return getData();
    }
    
    @ActionAnnotation(actionClass=CopyAsTextAction.class)
    public void copyAsText() {
        String cont = ((TestAction)getTestAction()).getContent();
        StringSelection ss = new StringSelection(cont);
        Toolkit.getDefaultToolkit().getSystemClipboard().setContents(ss, null);
    }
    
    @ActionAnnotation(actionClass=DeleteAction.class)
    public void delete() {
        ((TestCaseNode)getParent()).getTestCase().removeAction(getTestAction());
    }
    
    @ActionAnnotation(actionClass=CopyAction.class)
    public void copyTo(Node target) {
        TestCaseNode tc=(TestCaseNode)target;
        tc.getTestCase().addAction((TestAction)(getTestAction().copy()));
    }
    
    @ActionAnnotation(actionClass=CutAction.class)
    public void cutTo(Node target) {
        TestCaseNode tc=(TestCaseNode)target;
        TestCaseNode par=((TestCaseNode)getParent());
        par.getTestCase().removeAction(getTestAction());
        tc.getTestCase().addAction(getTestAction());
    }
    
    @ActionAnnotation(actionClass=MoveUpAction.class)
    public void moveUp() {
        TestCaseNode par=((TestCaseNode)getParent());
        par.getTestCase().moveUp(getTestAction());
    }
    
    @ActionAnnotation(actionClass=MoveDownAction.class)
    public void moveDown() {
        TestCaseNode par=((TestCaseNode)getParent());
        par.getTestCase().moveDown(getTestAction());
    }
    
    public String toString() {
        if (getData() instanceof TestResult) {
            return getCatString("Result", getTestAction().getContent());
        }
        int index=getParent().getIndex(this)+1;
        return getCatString(String.valueOf(index), getTestAction().getContent());
    }
    
    static String getCatString(String pre, String s) {
        StringBuilder sb=new StringBuilder();
        sb.append(pre);
        sb.append(": ");
        sb.append(s);
        return sb.toString();
    }
    
    public void childrenChanged(Bean source, Bean child, int oldIndex, int newIndex) {
    }
    
    public void propertyChanged(Bean source, String propertyName, Object oldValue, Object newValue) {
        if (propertyName.equals("content")) {
            //getTreeModel().nodeChanged(this);
            TreeModelEvent ev=getNodeEvent();
            try {
                treeNodesChanged(ev);
            } catch (Exception ex) {
                System.err.println("error");
                System.err.println("parent "+getParent());
            }
        }
    }
    
    public Icon getIcon() {
        return icon;
    }
    
    public void release() {
        getTestAction().getBeanSupport().removeBeanChangeListener(this);
    }
}
