/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.ui.tree.menu;

import java.util.ArrayList;
import java.util.HashMap;
import javax.swing.JComponent;
import javax.swing.JMenu;
import javax.swing.JPopupMenu;
import javax.swing.JSeparator;
import org.netbeans.testspeceditor.ui.actions.Action;
import org.netbeans.testspeceditor.ui.actions.MultiAction;
import org.netbeans.testspeceditor.ui.actions.NodeAction;
import org.netbeans.testspeceditor.ui.actions.StandaloneAction;
import org.netbeans.testspeceditor.ui.tree.Node;


/**
 *
 * @author ehucka
 */
public abstract class PopupMenu {
    
    public JPopupMenu menu;
    Action[] actions;
    
    HashMap<Action, Performer> performers;
    ArrayList<JSeparator> separators;
    ArrayList<Performer> activePerformers;
    HashMap<String, JMenu> submenus;
    
    int indexseparator = 0;
    
    public PopupMenu(JPopupMenu menu, Action[] actions) {
        this.menu = menu;
        this.actions = actions;
        performers = new HashMap(32);
        separators = new ArrayList(16);
        submenus = new HashMap(16);
        activePerformers = new ArrayList(16);
    }
    
    public Action[] getActions() {
        return actions;
    }
    
    public abstract void generateMenu();
    
    public JComponent getMenu(Action action) {
        if (action == null) {
            return getSeparator();
        }
        if (action instanceof MultiAction) {
            return getSubmenu((MultiAction) action);
        }
        Performer perf = performers.get(action);
        if (perf == null) {
            if (action instanceof NodeAction)
                perf = new NodePerformer((NodeAction)action);
            else if (action instanceof StandaloneAction)
                perf = new StandaloneActionPerformer((StandaloneAction)action);
            performers.put(action, perf);
        }
        activePerformers.add(perf);
        return perf.getItem();
    }
    
    protected void activateActions(Node[] selected) {
        for (Performer perf : activePerformers) {
            if (perf instanceof NodePerformer)
                ((NodePerformer)perf).setSelected(selected);
            if (perf instanceof StandaloneActionPerformer)
                ((StandaloneActionPerformer)perf).update();
        }
    }
    
    public void clear()    {
        indexseparator=0;
        menu.removeAll();
        activePerformers.clear();
    }
    
    protected JSeparator getSeparator() {
        JSeparator sep;
        if (indexseparator < separators.size())
            sep=separators.get(indexseparator++); else {
            sep = new JSeparator();
            separators.add(sep);
            indexseparator++;
            }
        return sep;
    }
    
    protected JMenu getSubmenu(MultiAction action) {
        JMenu men = submenus.get(action.getName());
        if (men == null) {
            men = new JMenu(action.getName());
            submenus.put(action.getName(), men);
        }
        men.removeAll();
        NodeAction[] acts = action.getActions();
        for (int i = 0; i < acts.length; i++) {
            men.add(getMenu(acts[i]));
        }
        return men;
    }
}
