/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.xmlizable;


import org.netbeans.testspeceditor.xmlparser.Parser;
import org.netbeans.testspeceditor.xmlparser.Tag;
import org.netbeans.testspeceditor.xmlparser.TagFactory;
import java.io.File;
import java.io.FileInputStream;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.MissingResourceException;
import java.util.Properties;
import java.util.ResourceBundle;


/**
 *
 * @author  ehucka
 */
public class XMLUtils {
    
    public static String BUNDLE_PATH="";
    
    public static final String OBJECT_NAME="object";
    public static final String CLASS_ATTRIBUTE="class";
    public static final String ID_ATTRIBUTE="xml_id";
    public static final String NAME_ATTRIBUTE="name";
    public static final String LINKS_ATTRIBUTE="links";
    
    public static final String GETTER_METHOD="getTag";
    public static final String SETTER_METHOD="setObject";
    public static final String OBJECT_INIT_METHOD="initObject";
    
    
    //cache of mappings
    static HashMap<String, String> beans;
    static HashMap<String, String> classes;
    static boolean aggregate = false;
    
    static HashMap classLoaders = new HashMap();
    
    static boolean initialized=false;
    
    static {
        classLoaders.put("system", java.lang.ClassLoader.getSystemClassLoader());
    }
    
    HashMap<String, String> beansMap;
    HashMap<String, String> classesMap;
    
    HashSet<Field> singleFields = new HashSet(128);
    HashMap<Class, ArrayList<Field>> allFields = new HashMap(128);
    
    boolean aggregateOnly;
    
    int id=0;
    
    public static void registerClassLoader(String name, ClassLoader loader) {
        ClassLoader ld = (ClassLoader)(classLoaders.get(name));
        if (ld == null) {
            classLoaders.put(name, loader);
            return;
        }
        throw new IllegalStateException("Class loader "+name+" already exists.");
    }
    
    public static ClassLoader getClassLoader(String name) {
        return (ClassLoader)(classLoaders.get(name));
    }
    
    public static Class findClass(String className) throws ClassNotFoundException {
        Object[] lds=classLoaders.values().toArray();
        for (int i=0;i < lds.length;i++) {
            ClassLoader cl=(ClassLoader)lds[i];
            try {
                return Class.forName(className, true, cl);
            } catch (ClassNotFoundException ex) {
                //ex.printStackTrace();
            }
        }
        throw new ClassNotFoundException(className+" class not found, register your classloader by XMLUtils.registerClassLoader method");
    }
    
    private XMLUtils() {
        beansMap=(HashMap)(beans.clone());
        classesMap=(HashMap)(classes.clone());
        this.aggregateOnly=aggregate;
    }
    
    public static XMLUtils getUtils() {
        if (!initialized) {
            throw new IllegalStateException("XML mappings are not initialized yet.");
        }
        return new XMLUtils();
    }
    
    public static void init() {
        if (beans == null)
            beans=new HashMap(64);
        if (classes == null)
            classes=new HashMap(64);
        try {
            if (BUNDLE_PATH != null && BUNDLE_PATH.length() > 0) {
                ResourceBundle rb=java.util.ResourceBundle.getBundle(BUNDLE_PATH);
                for (Enumeration en=rb.getKeys();en.hasMoreElements();) {
                    String bean=(String)(en.nextElement());
                    String clazz=rb.getString(bean);
                    beans.put(bean, clazz);
                    classes.put(clazz, bean);
                }
            }
        } catch (MissingResourceException ex) {
            System.err.println("There is not xmlizable mappings on (BUNDLE_PATH "+ex.getStackTrace()[1].toString()+") "+BUNDLE_PATH+".");
        }
        initialized=true;
    }
    
    /**
     * init mappings by resources - path = e.g. org.netbeans.testspeceditor.xmlizable.mappings.beans (for beans.properties)
     */
    public static void init(String path) {
        if (beans == null)
            beans=new HashMap(64);
        if (classes == null)
            classes=new HashMap(64);
        try {
            ResourceBundle rb=java.util.ResourceBundle.getBundle(path);
            for (Enumeration en=rb.getKeys();en.hasMoreElements();) {
                String bean=(String)(en.nextElement());
                String clazz=rb.getString(bean);
                beans.put(bean, clazz);
                classes.put(clazz, bean);
            }
        } catch (MissingResourceException ex) {
            System.err.println("There is not xmlizable mappings on (path "+ex.getStackTrace()[1].toString()+") "+path+".");
        }
        initialized=true;
    }
    
    public static void init(String path, boolean aggregateOnly) {
        init(path);
        aggregate=aggregateOnly;
    }
    
    public static void init(File mappingsFile) {
        if (beans == null)
            beans=new HashMap(64);
        if (classes == null)
            classes=new HashMap(64);
        Properties props=new Properties();
        try {
            FileInputStream fis=new FileInputStream(mappingsFile);
            props.load(fis);
            fis.close();
            for (Enumeration en=props.keys();en.hasMoreElements();) {
                String bean=(String)(en.nextElement());
                String clazz=props.getProperty(bean);
                beans.put(bean, clazz);
                classes.put(clazz, bean);
            }
        } catch (Exception ex) {
            System.err.println("There is not xmlizable mappings on (file "+ex.getStackTrace()[1].toString()+") "+mappingsFile.getAbsolutePath()+".");
        }
        initialized=true;
    }
    
    /**
     * set xml parser to parse only common tags - object and from init
     */
    public static void initParser(String... otherTags) {
        ArrayList<String> list=new ArrayList(beans.size()+1);
        list.addAll(classes.values());
        list.add(OBJECT_NAME);
        for (String t:otherTags) {
            list.add(t);
        }
        Parser.setSchemaTags(list.toArray(new String[list.size()]));
    }
    
    //ides
    public void initIdGenerator() {
        id=0;
    }
    
    public synchronized int getNextID() {
        return id++;
    }
    
    //mappings
    public void addMapping(String name, String className) {
        beansMap.put(name, className);
        classesMap.put(className, name);
    }
    
    public String getMapping(Class clazz) {
        return classesMap.get(clazz.getName());
    }
    
    public String getSuperMapping(Class clazz) {
        String ret=null;
        Class cl=clazz;
        while (ret == null && cl != Object.class) {
            ret=getMapping(cl);
            cl=cl.getSuperclass();
        }
        if (ret == null) {
            ret=OBJECT_NAME;
        }
        return ret;
    }
    
    public Class getMapping(String name) {
        String cl = beansMap.get(name);
        if (cl != null) {
            try {
                return findClass(cl);
            } catch (Exception ex) {
                throw new IllegalStateException("Cannot find a class for mapping "+name, ex);
            }
        }
        return null;
    }
    
    //call XMLizable methods
    public boolean callGetTag(Object obj, Tag tag) {
        try {
            Method gettag=obj.getClass().getMethod(GETTER_METHOD, new Class[] {Tag.class});
            if (gettag != null) {
                gettag.invoke(obj, new Object[] {tag});
                return true;
            }
        } catch (NoSuchMethodException nse) {
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return false;
    }
    
    public boolean callSetObject(Object obj, Tag tag) {
        try {
            Method setobj=obj.getClass().getMethod(SETTER_METHOD, new Class[] {Tag.class});
            if (setobj != null) {
                setobj.invoke(obj, new Object[] {tag});
                return true;
            }
        } catch (NoSuchMethodException nse) {
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return false;
    }
    
    public void callInitMethod(Object obj) {
        try {
            Method met=obj.getClass().getMethod(XMLUtils.OBJECT_INIT_METHOD, new Class[0]);
            met.invoke(obj, new Object[0]);
        } catch (NoSuchMethodException nse) {
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    public boolean isMultiField(Field field, Object obj) throws Exception {
        return (Collection.class.isAssignableFrom(field.getType()) || field.get(obj) != null && field.get(obj) instanceof Collection);
    }
    
    public boolean isSingleField(Field f, Object obj) throws Exception {
        if (singleFields.contains(f)) return true;
        boolean ret=false;
        if (XMLizable.class.isAssignableFrom(f.getType()) || f.getType() == String.class || f.getType().isPrimitive()) {
            ret=true;
        } else {
            Object o=f.get(obj);
            if (o != null && (o instanceof XMLizable || o instanceof String))
                ret=true;
        }
        if (ret) {
            singleFields.add(f);
        }
        return ret;
    }
    
    //fields methods
    public ArrayList<Field> getAllSingleFields(Object obj) throws Exception {
        ArrayList<Field> fields=getAllFields(obj.getClass());
        ArrayList<Field> ret=new ArrayList(fields.size());
        for (Field f:fields) {
            if (isSingleField(f, obj)) {
                ret.add(f);
            }
        }
        return ret;
    }
    
    public ArrayList<Field> getAllTypeFields(Object obj, Class type) throws Exception {
        ArrayList<Field> fields=getAllFields(obj.getClass());
        ArrayList<Field> ret=new ArrayList(fields.size());
        for (Field f:fields) {
            if (type.isAssignableFrom(f.getType())) {
                ret.add(f);
            }
        }
        return ret;
    }
    
    public ArrayList<Field> getAllCollectionFields(Object obj) throws Exception {
        return getAllTypeFields(obj, Collection.class);
    }
    
    public Field getField(Class clazz, String name) {
        if (clazz == null) {
            return null;
        }
        try {
            Field f=clazz.getDeclaredField(name);
            f.setAccessible(true);
            return f;
        } catch (Exception ex) {
        }
        return getField(clazz.getSuperclass(), name);
    }
    
    protected ArrayList<Field> getAllFields(Class cl) throws Exception {
        if (allFields.containsKey(cl)) {
            ArrayList<Field> ret=allFields.get(cl);
            return ret;
        }
        ArrayList<Field> list=new ArrayList(64);
        list=getAllFieldsRec(cl, list);
        allFields.put(cl, list);
        return list;
    }
    
    private ArrayList<Field> getAllFieldsRec(Class cl, ArrayList<Field> list) throws Exception {
        if (cl == Object.class) {
            return list;
        }
        Field[] flds=cl.getDeclaredFields();
        for (Field f: flds) {
            if (!((f.getModifiers()&Modifier.STATIC) > 0 || (f.getModifiers()&Modifier.TRANSIENT) > 0)) {
                f.setAccessible(true);
                list.add(f);
            }
        }
        return getAllFieldsRec(cl.getSuperclass(), list);
    }
    
    public Field hasName(Class clazz) {
        if (clazz == null) return null;
        try {
            Field field=clazz.getDeclaredField("name");
            if (field != null) {
                field.setAccessible(true);
                return field;
            }
        } catch (Exception ex) {
        }
        return hasName(clazz.getSuperclass());
    }
    
    public static Field getBackComposition(Class clazz, Class assoc) {
        if (null == clazz) {
            return null;
        }
        Field[] flds=clazz.getDeclaredFields();
        for (int i=0;i < flds.length;i++) {
            if (flds[i].getType() == assoc) {
                flds[i].setAccessible(true);
                return flds[i];
            }
        }
        return getBackComposition(clazz.getSuperclass(), assoc);
    }
    
    public static String convertToString(Object obj) {
        if (obj == null) {
            return "";
        }
        if (obj instanceof String) {
            return (String)obj;
        } else  {
            return String.valueOf(obj);
        }
    }
    
    public Class getClassForTag(Tag tag) throws Exception {
        Class cl=null;
        String clname=tag.getAttribute(XMLUtils.CLASS_ATTRIBUTE);
        if (clname != null) {
            cl=Class.forName(clname);
        }
        if (cl == null) {
            cl=getMapping(tag.getName());
            if (cl == null) {
                throw new XMLizableException("No mapping for tag "+tag.getName());
            }
        }
        return cl;
    }
    
    public Tag getTagForObject(XMLizable obj) {
        String name=getMapping(obj.getClass());
        Tag main;
        if (name != null) {
            if (name.indexOf(':') > -1) {//with namespace
                main=TagFactory.createTag(name.substring(0,name.indexOf(':')), name.substring(name.indexOf(':')+1));
            } else {
                main=TagFactory.createTag(name);
            }
        } else {//not mapped - class attribute
            name=getSuperMapping(obj.getClass());
            main=TagFactory.createTag(name);
            main.setAttribute(XMLUtils.CLASS_ATTRIBUTE, obj.getClass().getName());
        }
        return main;
    }
    
    public static boolean isSimilar(String s1, String s2, double ratio) {
        int index=-1;
        String c1=s1.toLowerCase();
        String c2=s2.toLowerCase();
        for (int i=0;i < c1.length();i++) {
            if (i >= c2.length()) {
                break;
            }
            if (c1.charAt(i) == c2.charAt(i)) {
                index=i;
            }
        }
        double maxl=(s1.length() > s2.length())?s1.length():s2.length();
        return ((index/maxl) >= ratio);
    }
    
    public static void setField(Field field, Object object, String value) throws Exception {
        if (field.getType() == String.class) {
            field.set(object, value);
        } else if (field.getType() == int.class) {
            try {
                field.set(object, new Integer(value));
            } catch (Exception ex) {
            }
        } else if (field.getType() == long.class) {
            try {
                field.set(object, new Long(value));
            } catch (Exception ex) {
            }
        } else if (field.getType() == short.class) {
            try {
                field.set(object, new Short(value));
            } catch (Exception ex) {
            }
        } else if (field.getType() == float.class) {
            try {
                field.set(object, new Float(value));
            } catch (Exception ex) {
            }
        } else if (field.getType() == double.class) {
            try {
                field.set(object, new Double(value));
            } catch (Exception ex) {
            }
        } else if (field.getType() == boolean.class) {
            try {
                field.set(object, new Boolean(value));
            } catch (Exception ex) {
            }
        }
    }
    
    public boolean isAggregateOnly() {
        return aggregateOnly;
    }
    
    public static String printMappings() {
        StringBuilder sb=new StringBuilder();
        for (Object o:classes.keySet().toArray()) {
            sb.append(classes.get(o));
            sb.append(": ");
            sb.append(o);
            sb.append("\n");
        }
        return sb.toString();
    }
    
    public String printMappingsLoc() {
        StringBuilder sb=new StringBuilder();
        for (Object o:classesMap.keySet().toArray()) {
            sb.append(classesMap.get(o));
            sb.append(": ");
            sb.append(o);
            sb.append("\n");
        }
        return sb.toString();
    }
    
    public static void setFieldsFromTag(Tag tag, XMLizable object) {
        XMLUtils utils = getUtils();
        try {
            ArrayList<Field> flds = utils.getAllTypeFields(object, String.class);
            for (String atr : tag.getAttributes()) {
                for (Field fld : flds) {
                    if (fld.getName().equals(atr)) {
                        fld.set(object, tag.getAttribute(atr));
                    }
                }
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
}
