/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.xmlparser;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.LinkedList;


/**
 *
 * @author  ehucka
 */
public class Parser {
    
    static String[] schemaTags = null;
    
    protected static XMLDocument parse(InputStream input) throws ParserException {
        ArrayList<Token> tokens=new ArrayList(128);
        CharSource source = new CharSource(input);
        
        Token tok=null;
        Token empty=new Token();
        char c=0;
        int ic = 0;
        try {
            while (source.hasNext()) {
                c = source.next();
                if (tok == null) { //between tokens
                    if (c == '<') { //begining of tag - create new tag token and store empty token
                        tok=new Token();
                        tok.add(c);
                        empty.complete();
                        tokens.add(empty);
                        empty=null;
                    } else { //store char between tags
                        empty.add(c);
                    }
                } else { //in tag token
                    tok.add(c);
                    if (c == '>' && !tok.isCDATA() && !tok.isComment() ||
                            tok.isComment() && tok.endComment() ||
                            tok.isCDATA() && tok.endCDATA()) { //finding end of current token
                        tok.complete();
                        tokens.add(tok);
                        tok=null;
                        empty=new Token();
                    }
                }
            }
            if (empty != null && empty.length() > 0)
                tokens.add(empty);
        } catch (Exception ex) {
            throw new ParserException(ex);
        }
        LinkedList<Tag> stack=new LinkedList();
        XMLDocument document=new XMLDocument();
        document.setParsed(true);
        
        for (Token token:tokens) {
            if (token.isTag()) {  //tag
                if (token.isComment()) { //comment
                    addElement(stack, document, new Comment(token.decodeComment()));
                } else if (token.isCDATA()) {  //not parsed data
                    addElement(stack, document, new CDATAValue(token.decodeCDATA()));
                } else if (token.isSpecial()) {  //<!, <?
                    addElement(stack, document, new Special(token.toString()));
                } else if (token.isSingleTag()) { //single tag <aaa/>
                    if (!canParse(token)) {
                        addValue(token, stack, document);
                    } else {
                        addElement(stack, document, parseTag(token));
                    }
                } else if (token.isFinishTag()) { //end tag </aaa>
                    if (!canParse(token)) {
                        addValue(token, stack, document);
                    } else {
                        String[] names=token.getTagName();
                        if (stack.peek().getName().equals(names[0]) &&
                                ((names.length == 2) ? stack.peek().getNameSpace().equals(names[1]) : true)) {
                            stack.poll();
                        } else {
                            throw new ParserException("Bad closing tag: "+token.toString()+" peek= "+stack.peek().getName());
                        }
                    }
                } else { // first tag <aaa>
                    if (!canParse(token)) {
                        addValue(token, stack, document);
                    } else {
                        Tag t=parseTag(token);
                        addElement(stack, document, t, true);
                    }
                }
            } else {
                addValue(token, stack, document);
            }
        }
        if (document.getRoot() == null) {
            throw new ParserException("Wrong XML stream, document hasn't root tag.");
        }
        return document;
    }
    
    protected static void addElement(LinkedList<Tag> stack, XMLDocument document, Element element) {
        addElement(stack, document, element, false);
    }
    
    protected static void addElement(LinkedList<Tag> stack, XMLDocument document, Element element, boolean hasChildren) {
        if (stack.isEmpty()) {
            document.addElement(element);
        } else {
            /*if (element instanceof Tag && !stack.isEmpty()) {
                Tag last = stack.peek();
                if (last.getElementsSize() > 0) {
                    Element el = last.getElement(last.getElementsSize()-1);
                    if (el instanceof Value && ((Value)el).trim()) {
                        last.removeElement(last.getElementsSize()-1);
                    }
                }
            }*/
            stack.peek().addElement(element);
        }
        if (hasChildren && element instanceof Tag) {
            stack.addFirst((Tag)element);
        }
    }
    
    protected static void addValue(Token token, LinkedList<Tag> stack, XMLDocument document) {
        //token.trim(); //maybe not good here
        if (token.length() > 0) {
            if (stack.isEmpty()) {
                document.addElement(new Value(token.toString()));
            } else {
                stack.peek().addValue(token.toString());
            }
        }
    }
    
    protected static boolean canParse(Token token) {
        String[] names=token.getTagName();
        if (schemaTags == null || schemaTags.length == 0) return true;
        //use only first name now
        for (int i = 0; i < schemaTags.length; i++) {
            if (schemaTags[i].equals(names[0])) {
                return true;
            }
        }
        return false;
    }
    
    protected static Tag parseTag(Token token) throws ParserException {
        String[] names=token.getTagName();
        Tag tag=null;
        if (names == null || names.length == 0 || names.length > 2) {
            throw new ParserException("Bad tag name format: "+token.toString());
        } else if (names.length == 2) {
            tag=TagFactory.createTag(names[0], names[1]);
        } else if (names.length == 1) {
            tag=TagFactory.createTag(names[0]);
        }
        token.parseAttributes(tag);
        tag.setParsed(true);
        return tag;
    }
    
    /**
     * sets all available tags names - if the names are set (not null) each tag name is checked
     *    * if it is in the list and if it is not there it is parsed as a value
     */
    public static void setSchemaTags(String[] names) {
        schemaTags=names;
    }
}
