/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.xmlparser;

import java.util.*;

/**
 *
 * @author  ehucka
 */
public class Tag implements Element {
    
    public static final String TAB="    ";
    
    private ArrayList<Tag> children=null;
    private ArrayList<Element> elements=null;
    private HashMap<String, String> attributes=null;
    private ArrayList<String> attributesIndex=null;
    
    protected String name;
    protected String nameSpace;
    boolean parsed = false;
    
    public Tag(String name) {
        this(name, null);
    }
    
    public Tag(String name, String nameSpace) {
        this.name=name;
        this.nameSpace=nameSpace;
    }
    
    //children section
    public void addChild(Tag child) {
        if (child != null) {
            getChildren().add(child);
            getElements().add(child);
        } else {
            throw new NullPointerException("Add tag is null.");
        }
    }
    
    public void addChild(Tag child, int index) {
        if (index > getChildren().size()-1 || index < 0)
            throw new ArrayIndexOutOfBoundsException("Index: "+index+" and children size = "+children.size());
        if (child != null) {
            Tag t=getChildren().get(index);
            getChildren().add(index, child);
            getElements().add(getElements().indexOf(t), child);
        } else {
            throw new NullPointerException("Add tag is null.");
        }
    }
    
    public Tag getChild(int i) {
        return getChildren().get(i);
    }
    
    public Tag getChild(String name) {
        for (Tag tag:getChildren()) {
            if (tag.name.equals(name)) {
                return tag;
            }
        }
        return null;
    }
    
    public Tag getChild(String namespace, String name) {
        if (namespace == null) return getChild(name);
        for (Tag tag:getChildren()) {
            if (tag.nameSpace != null && tag.nameSpace.equals(namespace) && tag.name.equals(name)) {
                return tag;
            }
        }
        return null;
    }
    
    public Tag[] getChildren(String name) {
        ArrayList<Tag> lst=new ArrayList(getChildren().size());
        for (Tag tag:children) {
            if (tag.name.equals(name)) {
                lst.add(tag);
            }
        }
        return lst.toArray(new Tag[lst.size()]);
    }
    
    public Element[] getAllElements() {
        return getElements().toArray(new Element[getElements().size()]);
    }
    
    public int getChildIndex(Tag child) {
        return getChildren().indexOf(child);
    }
    
    /**
     * Removes all children tags, not elements.
     */
    public void removeChildren() {
        getElements().removeAll(getChildren());
        getChildren().clear();
    }
    
    /**
     * Removes all elements.
     */
    public void removeElements() {
        getChildren().clear();
        getElements().clear();
    }
    
    public void removeChild(Tag child) {
        getChildren().remove(child);
        getElements().remove(child);
    }
    
    public Tag removeChild(int i) {
        Tag t=getChildren().remove(i);
        getElements().remove(t);
        return t;
    }
    
    public int getChildrenSize() {
        return getChildren().size();
    }
    
    //elements
    public void addComment(String text) {
        addElement(new Comment(text));
    }
    
    public void addElement(Element element) {
        if (element instanceof Tag) {
            if (!getChildren().contains(element))
                children.add((Tag)element);
        } else if (element instanceof Value) {
            if (getElements().size() > 0 && getElements().get(getElements().size()-1) instanceof Value) {
                ((Value)(getElements().get(getElements().size()-1))).addValue(((Value)element).toString());
                return;
            }
        }
        getElements().add(element);
    }
    
    public Element getElement(int index) {
        return getElements().get(index);
    }
    
    public int getElementsSize() {
        return getElements().size();
    }
    
    public Element removeElement(int index) {
        return getElements().remove(index);
    }
    
    public void addValue(String text) {
        if (getElements().size() > 0 && getElements().get(getElements().size()-1) instanceof Value) {
            ((Value)(getElements().get(getElements().size()-1))).addValue(text);
        } else {
            Value value=new Value(text);
            getElements().add(value);
        }
    }
    
    public void addCDATAValue(String text) {
        CDATAValue value=new CDATAValue(text);
        addElement(value);
    }
    
    public String getValue() {
        Value v=new Value("");
        for (Element el:getElements()) {
            if (el instanceof Value) {
                v.addValue(((Value)el).toString());
            }
        }
        return v.toString().trim();
    }
    
    public void setValue(String s) {
        addValue(s);
    }
    
    public String getCDATA() {
        StringBuilder sb=new StringBuilder();
        for (Element el:getElements()) {
            if (el instanceof CDATAValue) {
                sb.append(((CDATAValue)el).getValue());
            }
        }
        return sb.toString();
    }
    
    public String getName() {
        return name;
    }
    
    public java.lang.String getNameSpace() {
        return nameSpace;
    }
    
    public void setNameSpace(java.lang.String nameSpace) {
        this.nameSpace = nameSpace;
    }
    
    public String getAttribute(String name) {
        return getAttributesMap().get(name);
    }
    
    public void setAttribute(String name, String value) {
        if (getAttributesMap().get(name) == null) {
            getAttributesIndex().add(name);
        }
        getAttributesMap().put(name, value);
    }
    
    /**
     * only for parsing
     */
    protected void setAttributes(HashMap<String, String> attributes, ArrayList<String> order) {
        this.attributes = attributes;
        this.attributesIndex = order;
    }
    
    public Tag[] getChildren(String[] childrenNames) {
        ArrayList<Tag> ret=new ArrayList(getChildren().size());
        for (Tag t : children) {
            for (String s : childrenNames) {
                if (t.getName().equals(s)) {
                    ret.add(t);
                    break;
                }
            }
        }
        return ret.toArray(new Tag[ret.size()]);
    }
    
    public Tag[] getNameSpaceChildren(String nameSpace) {
        ArrayList<Tag> ret=new ArrayList(getChildren().size());
        for (Tag t : children) {
            if (t.getNameSpace() != null && t.getNameSpace().equals(nameSpace)) {
                ret.add(t);
            }
        }
        return ret.toArray(new Tag[ret.size()]);
    }
    
    //finders
    /**
     * @param name
     * @param attributeName
     * @param attributeValue
     * @return
     */
    public List<Tag> findChild(String name, String attributeName, String attributeValue) {
        ArrayList<Tag> ret = new ArrayList(16);
        for (Tag t:getChildren()) {
            List<Tag> lst = t.findChild(name, attributeName, attributeValue);
            if (lst.size() > 0)
                ret.addAll(lst);
            if (t.getName().equals(name)) {
                String attr=t.getAttribute(attributeName);
                if (attr != null && attr.equals(attributeValue)) {
                    ret.add(t);
                }
            }
        }
        return ret;
    }
    
    public void append(StringBuilder sb, String context) {
        if (!parsed && context != null)
            sb.append(context);
        sb.append("<");
        if (nameSpace != null && nameSpace.length() > 0) {
            sb.append(nameSpace);
            sb.append(":");
        }
        sb.append(name);
        //attributes
        for (String key : getAttributesIndex()) {
            String val;
            val=encodeString(getAttribute((String)key));
            if (val != null) {// && val.length() > 0) {
                sb.append(" ");
                sb.append(key.toString());
                sb.append("=\"");
                sb.append(val);
                sb.append("\"");
            }
        }
        if (getElements().size() == 0) {
            sb.append("/>");
        } else { //children and elements - comment, value, CDATA
            sb.append(">");
            boolean alone=true;
            if (!parsed) {
                for (Element el : getElements()) {
                    if (!(el instanceof Value) || ((Value)el).toString().contains("\n")) {
                        alone = false;
                        break;
                    }
                }
                if (!alone && context != null)
                    sb.append("\n");
            }
            for (Element el : getElements()) {
                if (!parsed && !(el instanceof Value) && !(el instanceof Tag)) {
                    if (context != null) {
                        sb.append(context);
                        sb.append(TAB);
                    }
                }
                if (context != null) el.append(sb, context+TAB);
                else el.append(sb, null);
                if (!alone && context != null) {
                    sb.append("\n");
                }
            }
            //end tag
            if (!alone && !parsed && context != null) {
                sb.append(context);
            }
            sb.append("</");
            if (nameSpace != null && nameSpace.length() > 0) {
                sb.append(nameSpace);
                sb.append(":");
            }
            sb.append(name);
            sb.append(">");
        }
    }
    
    //for overide output methods
    protected String encodeString(String text) {
        return text;
    }
    
    public String toString() {
        return (nameSpace != null && nameSpace.length() > 0)?nameSpace+":"+name:name;
    }
    
    public Element copy() {
        Tag ret=TagFactory.createTag(name,nameSpace);
        ret.setParsed(isParsed());
        for (String key:getAttributesIndex()) {
            ret.setAttribute(key,getAttribute(key));
        }
        for (Element el:getElements()) {
            el=el.copy();
            if (el instanceof Tag) {
                ret.addChild((Tag)el);
            } else {
                ret.addElement(el);
            }
        }
        return ret;
    }
    
    public boolean hasChild(Tag child) {
        return getChildren().contains(child);
    }
    
    public String[] getAttributes() {
        return getAttributesIndex().toArray(new String[getAttributesIndex().size()]);
    }
    
    /**
     * compare with another tag
     * tags are equal if have the same subtags and same attributes with its values
     */
    public boolean dataEquals(Tag t) {
        //test attributes
        boolean ret=getAttributesIndex().equals(t.getAttributesIndex());
        for (String k : getAttributesIndex()) {
            ret&=getAttributesMap().get(k).equals(t.getAttributesMap().get(k));
        }
        if (!ret) {
            return ret;
        }
        ret&=(getChildren().size() == t.getChildren().size());
        if (ret) {
            for (int i = 0; i < getChildren().size(); i++) {
                if (!getChildren().get(i).dataEquals(t.getChildren().get(i))) {
                    return false;
                }
            }
        }
        return ret;
    }
    
    protected ArrayList<Tag> getChildren() {
        if (children == null) {
            children=new ArrayList(8);
        }
        return children;
    }
    
    protected ArrayList<Element> getElements() {
        if (elements == null) {
            elements=new ArrayList(8);
        }
        return elements;
    }
    
    protected HashMap<String, String> getAttributesMap() {
        if (attributes == null) {
            attributes=new HashMap(8);
        }
        return attributes;
    }
    
    protected ArrayList<String> getAttributesIndex() {
        if (attributesIndex == null) {
            attributesIndex=new ArrayList(8);
        }
        return attributesIndex;
    }
    
    public String write(String context) {
        StringBuilder sb=new StringBuilder();
        append(sb, context);
        return sb.toString();
    }
    
    public boolean isParsed() {
        return parsed;
    }
    
    public void setParsed(boolean parsed) {
        this.parsed = parsed;
        if (!parsed) { //trim - remove formated values
            for (int i = 0; i < getElements().size(); i++) {
                if (elements.get(i) instanceof Value && ((Value)(elements.get(i))).trim()) {
                    elements.remove(i--);
                } else if (elements.get(i) instanceof Tag) {
                    ((Tag)(elements.get(i))).setParsed(false);
                }
            }
        }
    }
}


