/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.quiz;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.text.DateFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.persistence.EntityManager;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.netbeans.modules.quiz.entities.Category;
import org.netbeans.modules.quiz.entities.Quiz;
import org.netbeans.modules.quiz.entities.QuizQuestion;

/**
 *
 * @author Jindrich Sedek
 */
public class ProcessingServlet extends HttpServlet {
    private static final String QUIZ_DATA_DIR = "/space/quizDataDir";
    private static final String FILE_PREFIX = "data";
    private static final int MAX_QUESTION_COUNT = 10;
    /** 
     * Processes requests for both HTTP <code>GET</code> and <code>POST</code> methods.
     * @param request servlet request
     * @param response servlet response
     */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        String dropId = request.getParameter("dropId");
        EntityManager em = PersistenceUtils.getInstance().createEntityManager();
        try{
            em.getTransaction().begin();
            if (dropId == null){
                Date startDate = getDate(request.getParameter("startdate").trim());
                Date endDate = getDate(request.getParameter("enddate").trim());
                String prize = request.getParameter("prize");
                String idStr = request.getParameter("id");
                Integer id = Integer.parseInt(idStr);
                Quiz quiz = new Quiz(id, startDate, endDate, prize);
                ArrayList<QuizQuestion> questions = new ArrayList();
                for (int i = 1; i < MAX_QUESTION_COUNT; i++){
                    String responseTypeParam = request.getParameter("response_type" + i);
                    String questionText = request.getParameter("question" + i).trim();
                    String responseText = request.getParameter("response" + i).trim();
                    String categoryIdText = request.getParameter("category" + i);
                    if (questionText.length() == 0 || responseText.length() == 0){
                        break;
                    }
                    QuestionType responseType = QuestionType.valueOf(responseTypeParam);
                    QuizQuestion question = new QuizQuestion(quiz, i, responseType, responseText, questionText);
                    Integer catId = Integer.parseInt(categoryIdText);
                    question.setCategory(em.find(Category.class,  catId));
                    questions.add(question);
                }
                // add questions
                quiz.setQuizQuestionCollection(questions);
                String content = request.getParameter("content");
                File dataDir = new File(QUIZ_DATA_DIR);
                File data = new File(dataDir, FILE_PREFIX + quiz.getId());
                Writer wr = new FileWriter(data);
                wr.write(content);
                wr.write('\n');
                wr.close();
                quiz.setContentPath(data.getAbsolutePath());
                em.persist(quiz);
                PersistenceUtils.getInstance().reloadActualQuiz();
            }else{
                Integer id = Integer.parseInt(dropId);
                Quiz quiz = em.find(Quiz.class, id);
                em.remove(quiz);
            }
            em.getTransaction().commit();
        }catch(Exception except){
            em.getTransaction().rollback();
            Logger.getLogger(ProcessingServlet.class.getName()).log(Level.SEVERE, "Problem creating quiz", except);
        }finally{
            em.close();
        }
        request.getRequestDispatcher("showallquizes.jsp").forward(request, response);
    }

    // <editor-fold defaultstate="collapsed" desc="HttpServlet methods. Click on the + sign on the left to edit the code.">
    /** 
     * Handles the HTTP <code>GET</code> method.
     * @param request servlet request
     * @param response servlet response
     */
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /** 
     * Handles the HTTP <code>POST</code> method.
     * @param request servlet request
     * @param response servlet response
     */
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /** 
     * Returns a short description of the servlet.
     */
    public String getServletInfo() {
        return "Admin page processing servlet";
    }

    private Date getDate(String parameter) {
        try{
            DateFormat format = DateFormat.getDateInstance();
            Date result = format.parse(parameter);
            return result;
        }catch(ParseException exc){
            Logger.getLogger(ProcessingServlet.class.getName()).log(Level.INFO, "Processing date parametter", exc);
            return null;
        }
    }
    // </editor-fold>
}



