/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.quiz.entities;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EntityManager;
import javax.persistence.Id;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.NoResultException;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Query;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Version;
import org.netbeans.modules.quiz.PersistenceUtils;

/**
 *
 * @author Jindrich Sedek
 */
@Entity
@Table(name = "QUIZ")
@NamedQueries({
    @NamedQuery(name = "Quiz.findById", query = "SELECT q FROM Quiz q WHERE q.id = :id"),
    @NamedQuery(name = "Quiz.findCorrectResponses",
    query = "SELECT ri.response, COUNT(ri.response) " +
    "FROM ResponseItem ri JOIN ri.response r " +
    "WHERE ri.correct = true AND r.quiz = :quiz " +
    "GROUP BY ri.response " +
    "HAVING COUNT(ri.response) = :count")
})
public class Quiz implements Serializable, Comparable<Quiz> {

    private static final long serialVersionUID = 1L;
    @Id
    @Column(name = "ID", nullable = false)
    private Integer id;
    @Column(name = "STARTDATE", nullable = false)
    @Temporal(TemporalType.DATE)
    private Date startdate;
    @Column(name = "ENDDATE", nullable = false)
    @Temporal(TemporalType.DATE)
    private Date enddate;
    @Column(name = "CONTENT_SOURCE")
    private String contentSource;
    @Column(name = "NOTIFIED")
    private Integer notified;
    @Column(name = "CLOSED")
    private Integer closed;
    @Column(name = "PRIZE")
    private String prize;
    @Version
    protected int version;
    @OneToMany(cascade = CascadeType.ALL, mappedBy = "quiz")
    private Collection<Response> responseCollection;
    @OneToMany(cascade = CascadeType.ALL, mappedBy = "quiz")
    private Collection<QuizQuestion> quizQuestionCollection;
    @OneToOne(mappedBy = "quiz")
    private Winners winners;

    public Quiz() {
    }

    public Quiz(Integer id, Date startDate, Date endDate, String prize) {
        this.id = id;
        this.startdate = startDate;
        this.enddate = endDate;
        this.prize = prize;
        this.notified = 0;
        this.closed = 0;
    }

    public Integer getId() {
        return id;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    public Date getStartdate() {
        return startdate;
    }

    public void setStartdate(Date startdate) {
        this.startdate = startdate;
    }

    public Date getEnddate() {
        return enddate;
    }

    public void setEnddate(Date enddate) {
        this.enddate = enddate;
    }

    public String getContentSource() {
        return contentSource;
    }

    public void setContentSource(String contentSource) {
        this.contentSource = contentSource;
    }

    public Collection<Response> getResponseCollection() {
        return responseCollection;
    }

    public void setResponseCollection(Collection<Response> responseCollection) {
        this.responseCollection = responseCollection;
    }

    public Collection<QuizQuestion> getQuizQuestionCollection() {
        return quizQuestionCollection;
    }

    public void setQuizQuestionCollection(Collection<QuizQuestion> quizQuestionCollection) {
        this.quizQuestionCollection = quizQuestionCollection;
    }

    public Winners getWinners() {
        return winners;
    }

    public void setWinners(Winners winners) {
        this.winners = winners;
    }

    @Override
    public int hashCode() {
        int hash = 0;
        hash += (id != null ? id.hashCode() : 0);
        return hash;
    }

    @Override
    public boolean equals(Object object) {
        // TODO: Warning - this method won't work in the case the id fields are not set
        if (!(object instanceof Quiz)) {
            return false;
        }
        Quiz other = (Quiz) object;
        if ((this.id == null && other.id != null) || (this.id != null && !this.id.equals(other.id))) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        return "org.netbeans.modules.quiz.entities.Quiz[id=" + id + "]";
    }

    public int compareTo(Quiz o) {
        return this.startdate.compareTo(o.startdate);
    }

    public String getContentPath() {
        return contentSource;
    }

    public void setContentPath(String content) {
        this.contentSource = content;
    }

    public int getQuestionCount() {
        return quizQuestionCollection.size();
    }

    public static Quiz findById(EntityManager em, Integer id) {
        Query query = em.createNamedQuery("Quiz.findById").setParameter("id", id);
        Quiz quiz = null;
        try {
            quiz = (Quiz) query.getSingleResult();
        } catch (NoResultException exc) {
            // no item
        }
        return quiz;
    }

    public Integer getNotified() {
        return notified;
    }

    public void setNotified(Integer notified) {
        this.notified = notified;
    }

    public Integer getClosed() {
        return closed;
    }

    public void setClosed(Integer closed) {
        this.closed = closed;
    }

    public static Integer suppressId() {
        EntityManager em = PersistenceUtils.getInstance().createEntityManager();
        Query query = em.createQuery("SELECT MAX(e.id) FROM Quiz e");
        Integer maxId = (Integer) query.getSingleResult();
        if (maxId == null) {
            maxId = 0;
        }
        em.close();
        return maxId + 1;
    }

    public String getPrize() {
        return prize;
    }

    public void setPrize(String prize) {
        this.prize = prize;
    }

    public List<NbUser> getWinnerCandidatesList() {
        EntityManager em = PersistenceUtils.getInstance().createEntityManager();
        Query query = em.createNamedQuery("Quiz.findCorrectResponses");
        query.setParameter("quiz", this);
        query.setParameter("count", getQuestionCount());
        List<Object[]> result = query.getResultList();
        ArrayList<NbUser> candidates = new ArrayList<NbUser>(result.size());
        for (Object[] row : result) {
            candidates.add(((Response) row[0]).getNbuser());
        }
        return candidates;
    }
}
