/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.quiz;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import javax.persistence.EntityManager;
import org.junit.BeforeClass;
import org.junit.Test;
import org.netbeans.modules.quiz.entities.NbUser;
import org.netbeans.modules.quiz.entities.Quiz;
import org.netbeans.modules.quiz.entities.QuizQuestion;
import org.netbeans.modules.quiz.entities.Response;
import org.netbeans.modules.quiz.entities.ResponseItem;
import org.netbeans.modules.quiz.entities.Winners;
import static org.junit.Assert.*;

/**
 *
 * @author Jindrich Sedek
 */
public class PersistenceUtilsTest {

    private PersistenceUtils pu;

    public PersistenceUtilsTest() {
    }

    @BeforeClass
    public static void setUpClass() throws Exception {
        PersistenceUtils.setDefaultPu(PersistenceUtils.TEST_PU);
    }

    @Test
    public void testEntityManager() {
        pu = PersistenceUtils.getInstance();
        EntityManager em = pu.createEntityManager();

        NbUser user = new NbUser("nbId", "name", "email", "address", "university", "ip");
        Quiz quiz = new Quiz(Quiz.suppressId(), new Date(2008, 11, 15), new Date(2006, 10, 14), "Prize");
        QuizQuestion question = new QuizQuestion(quiz, 1, QuestionType.CHOICE, "1");
        QuizQuestion question2 = new QuizQuestion(quiz, 2, QuestionType.CHOICE, "3");
        QuizQuestion question3 = new QuizQuestion(quiz, 3, QuestionType.CHOICE, "2");
        Response resp = new Response(user, quiz);
        ResponseItem response = new ResponseItem(resp, 1, "3", true);
        ResponseItem response2 = new ResponseItem(resp, 2, "4", false);
        Winners winner = new Winners(quiz, user);
        em.getTransaction().begin();
        em.persist(user);
        em.persist(quiz);
        em.persist(resp);
        em.persist(winner);
        em.persist(question);
        em.persist(question2);
        em.persist(question3);
        em.persist(response2);
        em.persist(response);
        em.getTransaction().commit();
        em.close();
        em = pu.createEntityManager();

        load(em, NbUser.class, 1);
        load(em, Quiz.class, 1);
        load(em, Winners.class, 1);
        load(em, Response.class, 1);
        load(em, QuizQuestion.class, 3);
        load(em, ResponseItem.class, 2);
        em.close();
    }

    @Test
    public void testAcceptingServlet() throws IOException {
        Date today = new Date();
        Quiz quiz = new Quiz(Quiz.suppressId(), today, new Date(today.getTime() + 3600 * 24 * 7 * 1000),"Prize");
        File content = File.createTempFile("quiz", "quiz");
        FileWriter writer = new FileWriter(content);
        writer.write("TEST");
        writer.close();

        content.createNewFile();
        quiz.setContentPath(content.getAbsolutePath());
        PersistenceUtils.persist(quiz);
        String ip = "123.23.42.23";
        ArrayList<String> params = new ArrayList<String>(5);
        params.add(0, "id");
        params.add(1, "1");
        params.add(2, "name");
        params.add(3, "email");
        params.add(4, "mail");
        params.add(5, "university");
//        int result = AcceptingServlet.insertRequest(params.toArray(), ip);
//        assertEquals(result, AcceptingServlet.STATUS_OK);
//        result = AcceptingServlet.insertRequest(params.toArray(), ip);
//        assertEquals(result, AcceptingServlet.STATUS_ALREADY_SUBMITTED);

    }

    private <T> T load(EntityManager em, Class<T> cls, int count) {
        List<T> result = pu.getAll(em, cls);
        assertEquals(count, result.size());
        assertNotNull(result.get(0));
        return result.get(0);
    }

    @Test
    public void testcompareToNowInIndia() throws ParseException {
        Date localDate = new Date(108, 3, 9, 13, 00);

        Date indiaDate = new Date(108, 3, 9, 16, 30);
        assertEquals(0, PersistenceUtils.compareIndianToLocal(indiaDate, localDate));

        indiaDate = new Date(108, 3, 9, 16, 20);
        assertTrue(PersistenceUtils.compareIndianToLocal(indiaDate, localDate) < 0); // earlier in INDIA

        indiaDate = new Date(108, 3, 9, 16, 50);
        assertTrue(PersistenceUtils.compareIndianToLocal(indiaDate, localDate) > 0); // later in INDIA
        
        indiaDate = new Date(108, 03, 28, 00, 00);//in  14 days
        assertTrue(PersistenceUtils.compareIndianToLocal(indiaDate, localDate) > 0);

        indiaDate = new Date(108, 03, 10, 00, 00);//in  tomorow
        assertTrue(PersistenceUtils.compareIndianToLocal(indiaDate, localDate) > 0);
    }
    
}