/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.quiz.stats;

import java.util.ArrayList;
import java.util.Date;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import javax.persistence.EntityManager;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.netbeans.modules.quiz.PersistenceUtils;
import org.netbeans.modules.quiz.QuestionType;
import org.netbeans.modules.quiz.entities.Category;
import org.netbeans.modules.quiz.entities.NbUser;
import org.netbeans.modules.quiz.entities.Quiz;
import org.netbeans.modules.quiz.entities.QuizQuestion;
import org.netbeans.modules.quiz.entities.Response;
import org.netbeans.modules.quiz.entities.ResponseItem;
import static org.junit.Assert.*;

/**
 *
 * @author pblaha
 */
public class QuizStatsDAOTest {

    private static PersistenceUtils pu;
    private static EntityManager em;

    public QuizStatsDAOTest() {
    }

    @BeforeClass
    public static void setUpClass() {
        PersistenceUtils.setDefaultPu(PersistenceUtils.TEST_PU);
        pu = PersistenceUtils.getInstance();
        em = pu.createEntityManager();

        em.getTransaction().begin();

        Category cat1 = new Category(1);
        cat1.setName("Basic");
        em.persist(cat1);

        Hashtable<Integer, Quiz> quizzes = new Hashtable<Integer, Quiz>();
        // 3 quizzes
        for (int i = 1; i < 4; i++) {
            Quiz quiz = new Quiz(i, new Date(2008, 11, 15), new Date(2006, 10, 14), "Prize" + i);

            ArrayList<QuizQuestion> questions = new ArrayList();
            questions.add(new QuizQuestion(quiz, 1, QuestionType.CHOICE, "a", "Q" + i, cat1));
            questions.add(new QuizQuestion(quiz, 2, QuestionType.CHOICE, "b", "Q" + i, cat1));
            quiz.setQuizQuestionCollection(questions);
            quizzes.put(new Integer(i), quiz);
            em.persist(quiz);
        }

        // 3 users
        for (int k = 1; k < 4; k++) {

            NbUser user = new NbUser("nbId" + k, "User" + k, "email" + k, "address", "university", "ip");
            em.persist(user);

            // response for 2 quizzes
            for (int j = 1; j < 3; j++) {
                Quiz quiz = quizzes.get(new Integer(j));
                Response resp = new Response(user, quiz);
                System.out.println(resp);

                ArrayList<ResponseItem> responses = new ArrayList();
                responses.add(new ResponseItem(resp, 1, "correct" + j, true));
                responses.add(new ResponseItem(resp, 2, "false" + j, false));
                resp.setResponseItemCollection(responses);
                em.persist(resp);
            }
        }

        NbUser user = new NbUser("nbId4", "User4", "email4", "address", "university", "ip");
            em.persist(user);
        Quiz quiz = quizzes.get(new Integer(3));
        Response resp = new Response(user,quiz);
        System.out.println(resp);

        ArrayList<ResponseItem> responses = new ArrayList();
        responses.add(new ResponseItem(resp, 1, "correct", true));
        responses.add(new ResponseItem(resp, 2, "correct", true));
        resp.setResponseItemCollection(responses);
        em.persist(resp);


        em.getTransaction().commit();
        em.close();
    }

    @AfterClass
    public static void tearDownClass() throws Exception {
    }

    @Before
    public void setUp() {
    }

    @After
    public void tearDown() {
    }
    
    /**
     * 
     *  Results
     *  Q1 u1 c/f u2 c/f u3 c/f
     *  1,1 3 1,2 0
     *  1 3
     * 
     *  Q2 u1 c/f u2 c/f u3 c/f
     *  2,1 3 2,2 0 
     *  2 3
     * 
     *  Q3 u4 c/c
     *  3,1 1 3,2 1
     *  3 1
     * 
     */

    /**
     * Test of getCorrectQuestions method, of class QuizStatsDAO.
     */
    @Test
    public void testGetCorrectQuestions() {
        System.out.println("getCorrectQuestions");
        QuizStatsDAO instance = new QuizStatsDAO();
        List<QuestionItem> expected = new ArrayList<QuestionItem>();
        expected.add(new QuestionItem(1, 1, "Q1", 3L));
        expected.add(new QuestionItem(2, 1, "Q2", 3L));
        expected.add(new QuestionItem(3, 1, "Q3", 1L));
        expected.add(new QuestionItem(3, 2, "Q3", 1L));
        List<QuestionItem> result = instance.getCorrectQuestions();

    }
    
    
     private void compare(List<QuestionItem> list1, List<QuestionItem> list2) {
         QuestionItem[] list1Array = (QuestionItem[]) list1.toArray();
         QuestionItem[] list2Array = (QuestionItem[]) list2.toArray();
         for (int i = 0; i < list2Array.length; i++) {
             QuestionItem questionItem2 = list2Array[i];
             QuestionItem questionItem1 = list1Array[i];
             assertEquals(questionItem1.getQuiz(), questionItem2.getQuiz());
             assertEquals(questionItem1.getRecord(), questionItem2.getRecord());
             assertEquals(questionItem1.getQuestion(), questionItem2.getQuestion());
             assertEquals(questionItem1.getCount(), questionItem2.getCount());
         }
     }

    /**
     * Test of getTotalResponses method, of class QuizStatsDAO.
     */
    @Test
    public void testGetTotalResponses() {
        System.out.println("getTotalResponses");
        QuizStatsDAO instance = new QuizStatsDAO();
        Map<Integer, Long> result = instance.getTotalResponses();
        assertEquals(result.get(1).intValue(), 3);
        assertEquals(result.get(2).intValue(), 3);
        assertEquals(result.get(3).intValue(), 1);
    }
}