/*
                    Sun Public License Notice
 
The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/
 
The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.
 
Contributor(s): Petr Slegr.
 */

package org.netbeans.modules.regsup.auted;

import org.netbeans.modules.regsup.auted.actions.*;
import org.netbeans.modules.regsup.logic.aut2expr.ResolveResult;
import org.netbeans.modules.regsup.logic.LogicAutomaton;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import org.netbeans.modules.regsup.debug.RexDebugger;

/**
 *
 * @author  Petr Slegr
 * @version 25.04.02
 */
public class AutomatonEditorPanel extends javax.swing.JPanel {
    
    private AutomatonEditor rootEditor;
    
    // !!!!! docasne
    AutomatonPanel currentPanel;
    
    private AutomatonPanel rootPanel;
    
    private ArrayList editStack = new ArrayList ();
    
    private int editStackIndex;
    
    private static final int EVALUATE_TAB_INDEX = 0;
    
    private static final int GENERATE_TAB_INDEX = 1;
    
    /** Index of first ("cloneable") debug tab
     */
    private static final int FIRST_DEBUG_TAB_INDEX = 2;
    
    // assignment due to NPE
    private String title = "";
    
    /** Number (id) of last open debugger
     */
    private int debuggerCount;
    
    private int debuggerLabelCount;
    
    private ViewHistoryEntry standardViewHistory;
    
    private Hashtable debugViewHistories = new Hashtable ();
    
    private ViewHistoryEntry currentViewHistory;
    
    /** Creates new form AutomatonEditorPanel */
    public AutomatonEditorPanel () {
        initComponents ();
        prepareMenu ();
        prepareOther ();
    }
    
    public AutomatonEditorPanel (AutomatonEditor editor) {
        this ();
        this.rootEditor = editor;
        rootPanel = AutomatonPanel.createFor (editor.getContent ());
        setAutomatonPanel (rootPanel);
        /*
        baseViewHistory = new ViewHistoryEntry ();
        currentViewHistory = baseViewHistory;
         */
        currentViewHistory = new ViewHistoryEntry ();
        standardViewHistory = currentViewHistory;
        currentViewHistory.changeTo (rootPanel);
        updateGUI ();
        //updateTitle ();
    }
    
    public AutomatonEditor getRootEditor () {
        return rootEditor;
    }
    
    // used in add action
    public AutomatonPanel getCurrentView () {
        return currentPanel;
    }
    
    public String getTitle () {
        return title;
    }
    
    private void prepareOther () {
        evaluatePanel = new EvaluatePanel (this);
        generatePanel = new GeneratePanel ();
        // unicode char for >
        jTabbedPane1.addTab ("Graph " + (char)0x22B3 + " Expression", evaluatePanel);
        jTabbedPane1.addTab ("Expression  " + (char)0x22B3 + " Graph", generatePanel);
        debugPanel = new DebugPanel (this);
        debugPanel.parent = this;
        jTabbedPane1.addTab ("Debugging", debugPanel);
        automatonEditorStatusBarCheckItem.setState (true);
        //
        backButton.addActionListener (new ActionListener () {
            public void actionPerformed (ActionEvent e) {
                backButtonActionPerformed (e);
            }}
        );
        //
        forwardButton.addActionListener (new ActionListener () {
            public void actionPerformed (ActionEvent e) {
                forwardButtonActionPerformed (e);
            }}
        );
        //
        rootButton.addActionListener (new ActionListener () {
            public void actionPerformed (ActionEvent e) {
                rootButtonActionPerformed (e);
            }}
        );
        //
        jTabbedPane1.addChangeListener (new TabbedPaneListener ());
    }
    
    private void prepareMenu () {
        boolean ide = MainManager.getDefault ().getRunMode () == MainManager.RUN_IDE;
        jMenuBar1 = new javax.swing.JMenuBar ();
        //
        /*
        insertMenu = new javax.swing.JMenu ();
        insertMenu.setText ("Insert");
         */
        
        editMenu = new javax.swing.JMenu ();
        editMenu.addMenuListener (new EditMenuListener ());
        viewMenu = new javax.swing.JMenu ();
        // !!! dat do nazvu menu...kvuli vyhledavani
        automatonEditorStatusBarCheckItem = new javax.swing.JCheckBoxMenuItem ();
        debugMenu = new javax.swing.JMenu ();
        openDebuggerMenuItem = new javax.swing.JMenuItem ();
        finishDebuggerMenuItem = new javax.swing.JMenuItem ();
        finishDebuggerMenuItem.addActionListener (new ActionListener () {
            public void actionPerformed (ActionEvent evt) {
                finishDebuggerItemActionPerformed (evt);
            }
        });
        jSeparator3 = new javax.swing.JSeparator ();
        continueDebuggerMenuItem = new javax.swing.JMenuItem ();
        helpMenu = new javax.swing.JMenu ();
        aboutItem = new javax.swing.JMenuItem ();
        //
        if (! ide) {
            fileMenu = new javax.swing.JMenu ();
            fileMenu.setText ("File");
            jSeparator1 = new javax.swing.JSeparator ();
            saveItem = new javax.swing.JMenuItem ();
            jSeparator2 = new javax.swing.JSeparator ();
            exitItem = new javax.swing.JMenuItem ();
            newAutomatonItem = new javax.swing.JMenuItem ();
            
            newAutomatonItem.setText ("New automaton");
            newAutomatonItem.addActionListener (new java.awt.event.ActionListener () {
                public void actionPerformed (ActionEvent evt) {
                    newAutomatonItemActionPerformed (evt);
                }
            });
            
            fileMenu.add (newAutomatonItem);
            fileMenu.add (jSeparator1);
            saveItem.setText ("Save");
            saveItem.addActionListener (new java.awt.event.ActionListener () {
                public void actionPerformed (ActionEvent evt) {
                    saveItemActionPerformed (evt);
                }
            });
            
            fileMenu.add (saveItem);
            fileMenu.add (jSeparator2);
            exitItem.setText ("Exit");
            fileMenu.add (exitItem);
            jMenuBar1.add (fileMenu);
        }
        editMenu.setText ("Edit");
        jMenuBar1.add (editMenu);
        /*
        jMenuBar1.add (insertMenu);
         */
        viewMenu.setText ("View");
        automatonEditorStatusBarCheckItem.setText ("Automaton Editor Status Bar");
        automatonEditorStatusBarCheckItem.addActionListener (new java.awt.event.ActionListener () {
            public void actionPerformed (ActionEvent evt) {
                automatonEditorStatusBarCheckItemActionPerformed (evt);
            }
        });
        
        viewMenu.add (automatonEditorStatusBarCheckItem);
        jMenuBar1.add (viewMenu);
        debugMenu.setText ("Debug");
        openDebuggerMenuItem.setText ("Start");
        openDebuggerMenuItem.addActionListener (new java.awt.event.ActionListener () {
            public void actionPerformed (ActionEvent evt) {
                openDebuggerMenuItemActionPerformed (evt);
            }
        });
        
        debugMenu.add (openDebuggerMenuItem);
        finishDebuggerMenuItem.setText ("Finish");
        debugMenu.add (finishDebuggerMenuItem);
        debugMenu.add (jSeparator3);
        continueDebuggerMenuItem.setText ("Continue");
        continueDebuggerMenuItem.setEnabled (false);
        debugMenu.add (continueDebuggerMenuItem);
        jMenuBar1.add (debugMenu);
        
        if (! ide) {
            windowMenu = new javax.swing.JMenu ();
            windowMenu.setText ("Window");
            jMenuBar1.add (windowMenu);
        }
        helpMenu.setText ("Help");
        aboutItem.setText ("About");
        aboutItem.addActionListener (new java.awt.event.ActionListener () {
            public void actionPerformed (ActionEvent evt) {
                aboutItemActionPerformed (evt);
            }
        });
        
        helpMenu.add (aboutItem);
        jMenuBar1.add (helpMenu);
        // hack due to adding menubar into panel
        jMenuBar1.setPreferredSize (new Dimension (400, 23));
        jMenuBar1.setMinimumSize (new Dimension (400, 23));
        // end of hack
        
        GridBagConstraints gridBagConstraints = new GridBagConstraints ();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 0;
        add (jMenuBar1, gridBagConstraints);
        
    }
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        navigationPanel = new javax.swing.JPanel();
        rootButton = new javax.swing.JButton();
        backButton = new javax.swing.JButton();
        forwardButton = new javax.swing.JButton();
        containerHierarchyCombo = new javax.swing.JComboBox();
        mainSplitPane = new javax.swing.JSplitPane();
        mainPanel = new javax.swing.JPanel();
        jTabbedPane1 = new javax.swing.JTabbedPane();

        setLayout(new java.awt.GridBagLayout());

        setPreferredSize(new java.awt.Dimension(32767, 32767));
        navigationPanel.setLayout(new java.awt.GridBagLayout());

        navigationPanel.setBackground(java.awt.Color.lightGray);
        rootButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/netbeans/modules/regsup/resources/icons/root.gif")));
        rootButton.setToolTipText("Jump to root container");
        rootButton.setMaximumSize(new java.awt.Dimension(30, 30));
        rootButton.setMinimumSize(new java.awt.Dimension(30, 30));
        rootButton.setPreferredSize(new java.awt.Dimension(30, 30));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 2);
        navigationPanel.add(rootButton, gridBagConstraints);

        backButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/netbeans/modules/regsup/resources/icons/back_1.gif")));
        backButton.setToolTipText("Move to upper container");
        backButton.setPreferredSize(new java.awt.Dimension(30, 30));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(4, 2, 4, 2);
        navigationPanel.add(backButton, gridBagConstraints);

        forwardButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/netbeans/modules/regsup/resources/icons/forward.gif")));
        forwardButton.setToolTipText("Move to (previously selected) nested container.");
        forwardButton.setMaximumSize(new java.awt.Dimension(63, 27));
        forwardButton.setMinimumSize(new java.awt.Dimension(63, 27));
        forwardButton.setPreferredSize(new java.awt.Dimension(30, 30));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(4, 2, 4, 2);
        navigationPanel.add(forwardButton, gridBagConstraints);

        containerHierarchyCombo.setMinimumSize(new java.awt.Dimension(200, 26));
        containerHierarchyCombo.setPreferredSize(new java.awt.Dimension(200, 26));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(4, 2, 4, 4);
        navigationPanel.add(containerHierarchyCombo, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        add(navigationPanel, gridBagConstraints);

        mainSplitPane.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        mainSplitPane.setResizeWeight(1.0);
        mainPanel.setLayout(new java.awt.GridBagLayout());

        mainSplitPane.setTopComponent(mainPanel);

        mainSplitPane.setBottomComponent(jTabbedPane1);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        add(mainSplitPane, gridBagConstraints);

    }//GEN-END:initComponents
    
    private void newAutomatonItemActionPerformed (ActionEvent evt) {
        VisualAutomatonContainer container = new VisualAutomatonContainer (null, new Point ());
        //AutomatonEditor editor = new AutomatonEditor (container);
        AutomatonEditor editor = MainManager.getDefault ().createEditor (container);
        /*
        EditorWindow window = new EditorWindow (editor);
        MainManager.getDefault ().setCurrentEditor (editor);
        MainManager.getDefault ().setCurrentWindow (window);
        window.show ();
         */
    }
    
    private void openDebuggerMenuItemActionPerformed (ActionEvent evt) {
        /*
        // !!!!???? testovat jestli je akce enabled
        MainManager.getDefault ().getAction (StartDebuggerAction.class).performAction ();
         */
        startDebugger ();
    }
    
    private void finishDebuggerItemActionPerformed (ActionEvent evt) {
        finishDebugger ();
    }
    
    private void saveItemActionPerformed (ActionEvent evt) {
        rootEditor.save ();
    }
    
    private void automatonEditorStatusBarCheckItemActionPerformed (ActionEvent evt) {
        boolean state = automatonEditorStatusBarCheckItem.getState ();
        //prepareEditorStatusBar (state);
        currentPanel.setStatusBar (state);
    }
    
    private void aboutItemActionPerformed (ActionEvent evt) {
        // Add your handling code here:
    }
    
    /** Sets specified panel as a current.
     */
    private void setAutomatonPanel (AutomatonPanel panel) {
        System.out.println ("param is: " + panel.hashCode ());
        if (panel == currentPanel) {
            throw new IllegalStateException ("panel is the same as current: " + panel);
        }
        GridBagConstraints constr = new GridBagConstraints ();
        constr.fill = constr.BOTH;
        constr.gridx = 0;
        constr.gridy = 0;
        constr.weightx = 1;
        constr.weighty = 1;
        if (currentPanel != null) {
            mainPanel.remove (currentPanel);
        }
        mainPanel.add (panel, constr);
        panel.getEditor ().getContent ().addContainerListener (new ContainerListenPoint ());
        if (currentPanel != null) {
            System.out.println ("current panel: " + currentPanel.hashCode ());
        }
        currentPanel = panel;
        System.out.println ("current panel changed to: " + currentPanel.hashCode ());
        // !!! mozna redundantni
        revalidate ();
        repaint ();
    }
    
    /**
     * It should check whether panel is sub...sub of root panel.
     */
    public void changeViewTo (AutomatonPanel panel) {
        /*
        // current panel is the same as new one
        if (panel == currentPanel) {
            return;
        }
        //currentPanel = panel;
        // "tree behaviour"
        if (editStackIndex < editStack.size ()) {
            for (int i = editStackIndex; i < editStack.size (); i++) {
                editStack.remove (i);
            }
        }
        editStack.add (editStackIndex++, panel);
         */
        currentViewHistory.changeTo (panel);
        setAutomatonPanel (panel);
        updateGUI ();
        repaint ();
        System.out.println ("view changed");
    }
    
    /**
     */
    public void changeViewBack () {
        /*
        if (editStackIndex < 1) {
            return;
        }
        editStackIndex--;
        AutomatonPanel panel;
        if (editStackIndex == 0) {
            panel = rootPanel;
        } else {
            panel = (AutomatonPanel) editStack.get (editStackIndex - 1);
        }
         */
        AutomatonPanel panel = currentViewHistory.toPrevious ();
        setAutomatonPanel (panel);
        updateGUI ();
    }
    
    /**
     */
    public void changeViewForward () {
        /*
        //System.out.println ("index: " + editStackIndex);
        if (editStackIndex > editStack.size () - 1) {
            // for security
            return;
        }
        AutomatonPanel panel = (AutomatonPanel) editStack.get (editStackIndex);
        editStackIndex++;
         */
        AutomatonPanel panel = currentViewHistory.toNext ();
        setAutomatonPanel (panel);
        updateGUI ();
    }
    
    /**
     * Does not add a new entry into history.
     */
    public void changeViewToRoot () {
        /*
        editStackIndex = 0;
        setAutomatonPanel (rootPanel);
         */
        if (currentViewHistory.canFirst ()) {
            setAutomatonPanel (currentViewHistory.toFirst ());
            updateGUI ();
        }
    }
    
    /**
     */
    private void updateGUI () {
        /*
        if (editStackIndex < 1) { // root panel is not in stack
            backButton.setEnabled (false);
        } else {
            backButton.setEnabled (true);
        }
        if (editStackIndex > editStack.size () - 1) {
            forwardButton.setEnabled (false);
        } else {
            forwardButton.setEnabled (true);
        }
        rootButton.setEnabled (! rootPanel.equals (currentPanel));
         */
        backButton.setEnabled (currentViewHistory.canPrevious ());
        forwardButton.setEnabled (currentViewHistory.canNext ());
        rootButton.setEnabled (currentViewHistory.canFirst ());
        //
        finishDebuggerMenuItem.setEnabled (canFinishDebugger ());
        // !!!!! volat to samostatne?
        updateTitle ();
    }
    
    private void updateTitle () {
        VisualAutomatonContainer root = rootPanel.getEditor ().getContent ();
        VisualAutomatonContainer current = currentPanel.getEditor ().getContent ();
        ArrayList reversePath = new ArrayList ();
        VisualAutomatonContainer upper = current;
        while (upper != null) {
            reversePath.add (upper);
            upper = upper.getContainer ();
        }
        StringBuffer path = new StringBuffer ();
        for (int i = reversePath.size () - 1; i >= 0; i--) {
            path.append (((VisualAutomatonContainer) reversePath.get (i)).getContentName ());
            if (i != 0) {
                path.append (TITLE_PATH_DELIMITER);
            }
        }
        // !!?? mozna zase v samostatne metode?
        path.append (" ");
        if (rootPanel.getEditor ().getContent ().isModified ()) {
            if (rootPanel.getEditor ().getContent ().isStructModified ()) {
                path.append (TITLE_STRUCT_DIRTY_CHAR);
            } else {
                path.append (TITLE_DIRTY_CHAR);
            }
        }
        title = path.toString ();
        //setTitle (title);
    }
    
    /** Evaluates CURRENT automaton in this panel.
     */
    public ResolveResult evaluateAutomaton () {
        VisualAutomatonContainer content = currentPanel.getEditor ().getContent ();
        LogicAutomaton automaton = content.getLogicAutomaton ();
        ResolveResult result = automaton.resolve ();
        evaluatePanel.setResultData (result);
        return result;
    }
    
    /** Starts new debugger for root container of this panel
     * @returns created debugger or null if debugger could not be started
     * (i.e. automaton contains error)
     */
    public RexDebugger startDebugger () {
        changeViewToRoot ();
        ResolveResult result = evaluateAutomaton ();
        if (! result.isOk ()) {
            jTabbedPane1.setSelectedIndex (EVALUATE_TAB_INDEX);
            return null;
        }
        RexDebugger debugger = MainManager.getDefault ().createDebugger (rootEditor);
        AutomatonPanel panel = AutomatonPanel.createFor (rootEditor.getContent (), debugger);
        ViewHistoryEntry history = new ViewHistoryEntry ();
        history.changeTo (panel);
        DebugPanel debugPanel = new DebugPanel (debugger);
        debugPanel.parent = this;
        debugViewHistories.put (debugPanel, history);
        //changeViewTo (panel);
        currentViewHistory = history;
        setAutomatonPanel (history.getCurrentView ());
        if (debuggerCount == 0) {
            jTabbedPane1.setTitleAt (FIRST_DEBUG_TAB_INDEX, "Debugging (0)");
            jTabbedPane1.setComponentAt (FIRST_DEBUG_TAB_INDEX, debugPanel);
        } else {
            jTabbedPane1.add ("Debugging (" + String.valueOf (debuggerLabelCount) + ")", debugPanel);
        }
        jTabbedPane1.setSelectedIndex (FIRST_DEBUG_TAB_INDEX + debuggerCount);
        debuggerLabelCount++;
        debuggerCount++;
        debugger.start ();
        repaint ();
        return debugger;
    }
    
    /** Finishes the current debugger i.e. debugger in current tab. If other than debugging
     * is selected does nothing.
     */
    public void finishDebugger () {
        int index = jTabbedPane1.getSelectedIndex ();
        if (index < FIRST_DEBUG_TAB_INDEX) {
            return;
        }
        if (debuggerCount == 0) {
            return;
        }
        DebugPanel panel = (DebugPanel) jTabbedPane1.getComponentAt (index);
        debuggerCount--;
        if (debuggerCount == 0) {
            jTabbedPane1.setComponentAt (FIRST_DEBUG_TAB_INDEX, new DebugPanel (this));
        } else {
            jTabbedPane1.remove (panel);
        }
        //
        if (index == FIRST_DEBUG_TAB_INDEX) {
            jTabbedPane1.setSelectedIndex (EVALUATE_TAB_INDEX); // was bug: wrong constant :-))
        } else {
            jTabbedPane1.setSelectedIndex (index - 1);
        }
    }
    
    /**
     */
    public boolean canFinishDebugger () {
        return debuggerCount > 0 && jTabbedPane1.getSelectedIndex () >= FIRST_DEBUG_TAB_INDEX;
    }
    
    /**
     */
    private void prepareEditMenu () {
        if (null == undoMenuItem) {
            undoMenuItem = new JMenuItem ();
            editMenu.add (undoMenuItem);
        }
        if (null == redoMenuItem) {
            redoMenuItem = new JMenuItem ();
            editMenu.add (redoMenuItem);
        }
        UndoAction undo = (UndoAction) MainManager.getDefault ().getAction (UndoAction.class);
        undoMenuItem.setText (undo.getName ());
        undoMenuItem.setAction (undo);
        undoMenuItem.setEnabled (undo.isEnabled ());
        //
        /*
        RedoAction redo = (RedoAction) MainManager.getDefault ().getAction (RedoAction.class);
        redoMenuItem.setText (redo.getName ());
        redoMenuItem.setAction (redo);
        redoMenuItem.setEnabled (redo.isEnabled ());
         */
        //
        editMenu.show ();
    }
    
    /** Registers AutomatonEditorPanelListener to receive events.
     * @param listener The listener to register.
     */
    public synchronized void addAutomatonEditorPanelListener (AutomatonEditorPanelListener listener) {
        if (listenerList == null ) {
            listenerList = new javax.swing.event.EventListenerList ();
        }
        listenerList.add (AutomatonEditorPanelListener.class, listener);
    }
    
    /** Removes AutomatonEditorPanelListener from the list of listeners.
     * @param listener The listener to remove.
     */
    public synchronized void removeAutomatonEditorPanelListener (AutomatonEditorPanelListener listener) {
        listenerList.remove (AutomatonEditorPanelListener.class, listener);
    }
    
    /** Notifies all registered listeners about the event.
     *
     * @param event The event to be fired
     */
    private void fireAutomatonEditorPanelActivated () {
        if (listenerList == null) return;
        Object[] listeners = listenerList.getListenerList ();
        for (int i = listeners.length-2; i>=0; i-=2) {
            if (listeners[i]==AutomatonEditorPanelListener.class) {
                ((AutomatonEditorPanelListener)listeners[i+1]).activated (this);
            }
        }
    }
    
    /** Notifies all registered listeners about the event.
     *
     * @param event The event to be fired
     */
    private void fireAutomatonEditorPanelDeactivated () {
        if (listenerList == null) return;
        Object[] listeners = listenerList.getListenerList ();
        for (int i = listeners.length-2; i>=0; i-=2) {
            if (listeners[i]==AutomatonEditorPanelListener.class) {
                ((AutomatonEditorPanelListener)listeners[i+1]).deactivated (this);
            }
        }
    }
    
    private void backButtonActionPerformed (ActionEvent evt) {
        changeViewBack ();
    }
    
    private void forwardButtonActionPerformed (ActionEvent evt) {
        changeViewForward ();
    }
    
    private void rootButtonActionPerformed (ActionEvent evt) {
        changeViewToRoot ();
    }
    
    ////////////////////////////////////////////////////////////////////////////
    private class ContainerListenPoint implements VisualAutomatonContainerListener {
        
        public void elementRemoved (VisualAutomatonContainerEvent event) {
        }
        
        public void containerAdded (VisualAutomatonContainerEvent event) {
            //prepareHierarchyCombo ();
        }
        
        public void containerRemoved (VisualAutomatonContainerEvent event) {
            //prepareHierarchyCombo ();
        }
        
        public void elementAdded (VisualAutomatonContainerEvent event) {
        }
        
        public void contentChanged (VisualAutomatonContainerEvent event) {
            //prepareStatusTitle ();
            // !!?? co kreslit jenom currentPanel?
            currentPanel.repaint (); // was BUG - was missing
        }
        
    }
    
    ////////////////////////////////////////////////////////////////////////////
    private class EditMenuListener implements javax.swing.event.MenuListener {
        
        public void menuCanceled (MenuEvent e) {
            //System.out.println("menu canceled");
        }
        
        public void menuDeselected (MenuEvent e) {
            //System.out.println("menu deselected");
        }
        
        public void menuSelected (MenuEvent e) {
            //System.out.println("menu selected");
            prepareEditMenu ();
        }
        
    }
    
    ////////////////////////////////////////////////////////////////////////////
    /** Class holding "stack" of open panels and index to current panel
     */
    private class ViewHistoryEntry {
        
        private int stackIndex;
        
        private ArrayList stack = new ArrayList ();
        
        AutomatonPanel getCurrentView () {
            return (AutomatonPanel) stack.get (stackIndex - 1);
        }
        
        boolean canFirst () {
            //System.out.println ("index: " + stackIndex);
            return stackIndex > 1;
        }
        
        boolean canPrevious () {
            //System.out.println ("index: " + stackIndex);
            return stackIndex > 1;
        }
        
        boolean canNext () {
            //System.out.println ("index: " + stackIndex);
            return stackIndex < stack.size ();
        }
        
        AutomatonPanel toFirst () {
            stackIndex = 1;
            return getCurrentView ();
        }
        
        AutomatonPanel toPrevious () {
            if (canPrevious ()) {
                stackIndex--;
                return (AutomatonPanel) stack.get (stackIndex - 1);
            }
            throw new IllegalStateException ("Impossible");
        }
        
        AutomatonPanel toNext () {
            if (canNext ()) {
                stackIndex++;
                return getCurrentView ();
            }
            throw new IllegalStateException ("Impossible");
        }
        
        void changeTo (AutomatonPanel panel) {
            if (stackIndex < stack.size ()) {
                for (int i = stackIndex; i < stack.size (); i++) {
                    stack.remove (i);
                }
            }
            stack.add (panel);
            stackIndex++;
        }
        
    }
    
    ////////////////////////////////////////////////////////////////////////////
    private class TabbedPaneListener implements ChangeListener {
        
        /**
         * Invoked when the target of the listener has changed its state.
         *
         * @param e  a ChangeEvent object
         */
        public void stateChanged (ChangeEvent e) {
            //System.out.println("stateChanged: " + e);
        }
        
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JComboBox containerHierarchyCombo;
    private javax.swing.JButton backButton;
    private javax.swing.JButton rootButton;
    private javax.swing.JSplitPane mainSplitPane;
    private javax.swing.JButton forwardButton;
    private javax.swing.JTabbedPane jTabbedPane1;
    private javax.swing.JPanel navigationPanel;
    private javax.swing.JPanel mainPanel;
    // End of variables declaration//GEN-END:variables
    
    private javax.swing.JMenuItem openDebuggerMenuItem;
    
    private javax.swing.JMenu fileMenu;
    
    private javax.swing.JMenuItem saveItem;
    
    private javax.swing.JMenu windowMenu;
    
    private javax.swing.JMenu viewMenu;
    
    private javax.swing.JMenu insertMenu;
    
    private javax.swing.JMenuItem newAutomatonItem;
    
    private javax.swing.JMenuItem continueDebuggerMenuItem;
    
    private javax.swing.JCheckBoxMenuItem automatonEditorStatusBarCheckItem;
    
    private javax.swing.JMenu debugMenu;
    
    private javax.swing.JMenuItem exitItem;
    
    private javax.swing.JMenu editMenu;
    
    private javax.swing.JMenuBar jMenuBar1;
    
    private javax.swing.JSeparator jSeparator1;
    
    private javax.swing.JSeparator jSeparator2;
    
    private javax.swing.JSeparator jSeparator3;
    
    private javax.swing.JMenuItem finishDebuggerMenuItem;
    
    private javax.swing.JMenuItem aboutItem;
    
    private javax.swing.JMenu helpMenu;
    
    private JMenuItem undoMenuItem;
    
    private JMenuItem redoMenuItem;
    
    private AutomatonEditorStatusBar statusBar;
    
    private EvaluatePanel evaluatePanel;
    
    private GeneratePanel generatePanel;
    
    private DebugPanel debugPanel;
    
    public static final char TITLE_DIRTY_CHAR = '*';
    
    public static final char TITLE_PATH_DELIMITER = '.';
    
    public static final char TITLE_STRUCT_DIRTY_CHAR = '+';
    
    /** Utility field used by event firing mechanism. */
    private javax.swing.event.EventListenerList listenerList =  null;
    
}
