/*
                    Sun Public License Notice
 
The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/
 
The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.
 
Contributor(s): Petr Slegr.
 */

package org.netbeans.modules.regsup.auted;

import org.netbeans.modules.regsup.logic.LogicAutomaton;
import java.util.*;
import java.awt.Point;
import javax.swing.undo.*;
import javax.swing.event.UndoableEditListener;
import org.netbeans.modules.regsup.logic.*;
import org.netbeans.modules.regsup.util.*;
import org.netbeans.modules.regsup.util.Util;
import java.lang.Double;
import java.awt.geom.RoundRectangle2D;
import java.lang.reflect.Method;
import java.beans.*;
import java.lang.reflect.InvocationTargetException;
import org.netbeans.modules.regsup.logic.CompoundLogicAutomatonNode;
import javax.swing.event.UndoableEditEvent;
import javax.swing.ImageIcon;
import javax.swing.Icon;
import java.util.List;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.RenderingHints;
import java.awt.Color;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import org.netbeans.modules.regsup.logic.aut2expr.ProxyResolveProblem;
import java.util.ArrayList;
import org.netbeans.modules.regsup.logic.aut2expr.ResolveProblem;
import org.netbeans.modules.regsup.logic.aut2expr.DirectResolveProblem;

/**
 *
 * @author  Petr Slegr
 * @version 19.11.01
 */
public class VisualAutomatonContainer extends VisualAutomatonNode {
    
    /**
     */
    protected ImageIcon icon;
    
    private int lastId;
    
    private ArrayList elements;
    
    private ArrayList edges = new ArrayList ();
    
    private ArrayList selfEdges = new ArrayList ();
    
    /**
     */
    private LogicAutomaton automaton;
    
    private transient UndoableEditSupport undoSupport;
    
    private transient UndoManagerPlus undoManager;
    
    private boolean compoundEditStarted;
    
    private CompoundEdit compoundEdit;
    
    private transient ArrayList containerListenerList;
    
    /** Is this container nested in other container?
     */
    // !!! neni zbytecne?
    private boolean root;
    
    /**
     * Not used
     */
    private boolean folded;
    
    /** Maps logicEdge -> visualEdge
     */
    // !!!!???? nemelo by toto mapovani byt spis ve VisualAutomatonEdge?
    private Hashtable visualEdges;
    
    private UndoListenPoint undoListener;
    
    private VisualAutomatonContainerListener containerListener;
    
    /** Display name customizable for user
     */
    private String contentName;
    
    /** Long name to be mainly displayed in tooltip
     */
    private String longHumanName;
    
    /** Name of the template this container has been created from
     */
    private String templateName;
    
    // use case of names:
    // ...
    
    private static int contentCounter;
    
    // !!! otazka zda by arranger nemel byt globalni
    //private LogicAutomatonArranger arranger;
    
    //private boolean modified;
    
    private static int idCounter;
    
    /** For creation templates
     */
    public VisualAutomatonContainer (CompoundLogicAutomatonNode logicNode) {
        super (logicNode);
        automaton = logicNode.getAutomaton ();
        init ();
    }
    
    VisualAutomatonContainer (VisualAutomatonContainer parent, Point position, LogicAutomatonNode logicNode, LogicAutomaton automaton) {
        super (position, logicNode);
        this.automaton = automaton;
        //
        if (null == parent) {
            root = true;
        } else {
            root = false;
        }
        //
        init ();
    }
    
    /**
     * NOTE: Parent (container) is assigned during adding. By default container is treated
     * as root (top-level).
     */
    // ! now used only in default constructor
    VisualAutomatonContainer (Point position, CompoundLogicAutomatonNode logicNode) {
        super (position, logicNode);
        automaton = logicNode.getAutomaton ();
    }
    
    /** Creates container on position (0, 0)
     */
    public VisualAutomatonContainer () {
        // compound logic node passed to super constructor has not importance
        this (new Point (0, 0), new CompoundLogicAutomatonNode (new LogicAutomaton ()));
        init ();
    }
    
    /**
     */
    private void init () {
        // !!! pouze pro ladeni
        folded = true;
        //
        geometry = prepareGeometry ();
        undoSupport = new UndoableEditSupport ();
        undoListener = new UndoListenPoint ();
        undoManager = new UndoManagerPlus ();
        elements = new ArrayList ();
        visualEdges = new Hashtable ();
        contentName = "NonameAutomaton" + String.valueOf (contentCounter++);
        //icon = new ImageIcon ("resource/icons/defaultContainer.gif"); !! does not work
        icon = new ImageIcon (getClass ().getResource ("/org/netbeans/modules/regsup/resources/icons/defaultContainer.gif"));
        // !!! temporarily disabled
        //arranger = new LayerAutomatonArranger ();
        // !!!
        containerListener = new ContainerListenPoint ();
        setHumanName ("Automaton" + String.valueOf (idCounter++));
    }
    
    /** Creates new VisualAutomatonContainer */
    /// !!!??? je potreba parent jako param constr, kdyz se tam pri vkladani stejne dava?
    /// !!!??? roor == container ???
    // !!!!! udelan public jenom z debug duvodu
    public VisualAutomatonContainer (VisualAutomatonContainer parent, Point position) {
        this (parent, position, null, null);
        automaton = new LogicAutomaton ();
        // !!!!!!! proc nevytvaret logicNode automaticky v konstruktoru VANode????
    }
    
    /**
     */
    public LogicAutomatonNode getLogicNode () {
        return logicNode;
    }
    
    /**
     */
    public LogicAutomaton getLogicAutomaton () {
        return automaton;
    }
    
    /** Removes all content of this container and generate new ones according
     * to supplied automaton
     */
    synchronized void setLogicAutomaton (LogicAutomaton automaton) {
        /*
        // !!! pokusit se o "differ" a delat to inteligentne
        startCompoundEdit (new AutomatonChange ());
        Iterator it = new HashSet (elements).iterator ();
        while (it.hasNext ()) {
            VisualAutomatonElement elem = (VisualAutomatonElement) it.next ();
            if (elem instanceof VisualAutomatonNode) {
                removeElement (elem);
            }
        }
        //arrangeElements (automaton);
        ///System.out.println ("layers:\n" + ((LayerAutomatonArranger) arranger).prepareLayers (automaton));
        this.automaton = automaton;
        List list = arranger.createAndArrangeFor (automaton);
        it = list.iterator ();
        while (it.hasNext ()) {
            insertElement ((VisualAutomatonElement) it.next ());
        }
        endCompoundEdit ();
         */
    }
    
    private void arrangeElements (LogicAutomaton automaton) {
        Set nodes = automaton.getNodes ();
        Iterator it = nodes.iterator ();
        int nodeNum = (int) Math.ceil (Math.sqrt (nodes.size ()));
        PointPlus startPos = new PointPlus (30, 30); // !! const
        PointPlus horizInc = new PointPlus (80, 0);
        PointPlus vertiInc = new PointPlus (0, 80);
        PointPlus verti = vertiInc;
        PointPlus horiz;
        PointPlus cur;
        //while (it.hasNext ()) {
        for (int i = 0; i < nodeNum; i++)  {
            horiz = startPos;
            for (int j = 0; j < nodeNum; j++) {
                if (it.hasNext ()) {
                    LogicAutomatonNode logicNode = (LogicAutomatonNode) it.next ();
                    cur = horiz.add (verti);
                    createNode (cur, logicNode);
                    horiz = horiz.add (horizInc);
                }
            }
            verti = verti.add (vertiInc);
        }
        it = nodes.iterator ();
        while (it.hasNext ()) {
            LogicAutomatonNode logicNode = (LogicAutomatonNode) it.next ();
            Set outputEdges = logicNode.getOutputEdges ();
            Iterator edgit = outputEdges.iterator ();
            while (edgit.hasNext ()) {
                LogicAutomatonEdge logicEdge = (LogicAutomatonEdge) edgit.next ();
                createEdge (find (logicNode), find (logicEdge.getTarget ()), logicEdge);
            }
        }
    }
    
    /**
     */
    public boolean isRoot () {
        return root;
    }
    
    /*
    public boolean isFolded () {
        return folded;
    }
     
    public void setFolded (boolean folded) {
        this.folded = folded;
    }
     */
    
    /**
     * @return empty enumeration if this container has no container inside it
     * @return enumeration of direct sub-containers otherwise
     */
    public Enumeration getContainers () {
        ArrayList sel = new ArrayList ();
        for (int i = 0; i < elements.size (); i++) {
            VisualAutomatonElement elem = (VisualAutomatonElement) elements.get (i);
            if (elem instanceof VisualAutomatonContainer) { // ! class name
                sel.add (elem);
            }
        }
        return Collections.enumeration (sel);
    }
    
    /**
     */
    public boolean isOn (Point position) {
        return geometry.isOn (position);
    }
    
    /**
     */
    public boolean isOnForSelection (Point position) {
        return isOn (position);
    }
    
    /**
     */
    public boolean isDragable () {
        return true;
    }
    
    /** Returns copy of elements from this container.
     */
    public Set getElements () {
        return new HashSet (elements);
    }
    
    /** Does not include self-edges
     */
    public Set getEdges () {
        return new HashSet (edges);
    }
    
    // !!!! zrusit a pouzivat getElements ()
    public Set elements () {
        return new HashSet (elements);
    }
    
    /**
     */
    public String getContentName () {
        return contentName;
    }
    
    /**
     */
    public void setContentName (String contentName) {
        // !!! spousta checkovani...jestli jmeno neni uz pouzito a je correct
        // !!! firovani event -> contentName jako property
        this.contentName = contentName;
    }
    
    /**
     */
    public Icon getContentIcon () {
        return icon;
    }
    
    /** Asks if this container is STRUCTURALLY modified.
     */
    public boolean isStructModified () {
        return undoManager.containsStructuralEdit ();
    }
    
    /** Asks if this container is modified.
     */
    public boolean isModified () {
        //return modified;
        return undoManager.containsEdit ();
    }
    
    void notifySave () {
        undoManager.notifySave ();
    }
    
    /** Method that should be called after ALL change of this container
     */
    void someChange () {
        // top-level container has null here
        if (logicNode != null) {
            ((CompoundLogicAutomatonNode) logicNode).clearResolved ();
            //System.out.println ("clearResolved in: " + this);
        }
    }
    
    /** Looks for a VisualAutomatonContainer where ResolveProblem described by
     * specified ResolveProblem is present
     * @return List of VisualAutomatonElements where the last element is problematic (if present)
     * // !!!!
     */
    public List findPlaceOf (ResolveProblem problem) {
        // !!! trochu se pocita se subclassovanim ResolveProblem classes
        List res = new ArrayList ();
        ResolveProblem prb = problem;
        VisualAutomatonContainer cont = this;
        while (prb.isProxy ()) {
            if (prb instanceof ProxyResolveProblem) {
                ProxyResolveProblem proprb = (ProxyResolveProblem) prb;
                LogicAutomatonNode place = proprb.getPlace ();
                VisualAutomatonNode node = cont.find (place);
                if (node != null) {
                    if (node instanceof VisualAutomatonContainer) {
                        res.add (node);
                        cont = (VisualAutomatonContainer) node;
                        prb  = proprb.getProblem ();
                    }
                    else {
                        throw new IllegalStateException ("Expected VisualAutomatonContainer");
                    }
                } else {
                    throw new IllegalStateException ("Node not found");
                }
            }
        }
        //
        if (prb instanceof DirectResolveProblem) {
            DirectResolveProblem diprb = (DirectResolveProblem) prb;
            LogicAutomatonNode cause = diprb.getCause ();
            if (cause != null) {
                VisualAutomatonNode node = cont.find (cause);
                res.add (node);
            }
        }
        return res;
    }
    
    /**
     * @return null if none of nodes has specified logic node
     */
    public VisualAutomatonNode find (LogicAutomatonNode logicNode) {
        for (int i = 0; i < elements.size () ; i++) {
            VisualAutomatonElement elem = (VisualAutomatonElement) elements.get (i);
            if (elem instanceof VisualAutomatonNode) {
                VisualAutomatonNode node = (VisualAutomatonNode) elem;
                if (node.getLogicNode ().equals (logicNode)) {
                    return node;
                }
            }
        }
        return null;
    }
    
    /**
     * @return null if none of nodes has specified logic node
     */
    public VisualAutomatonEdge find (LogicAutomatonEdge logicEdge) {
        for (int i = 0; i < elements.size () ; i++) {
            VisualAutomatonElement elem = (VisualAutomatonElement) elements.get (i);
            if (elem instanceof VisualAutomatonEdge) {
                VisualAutomatonEdge edge = (VisualAutomatonEdge) elem;
                if (edge.getLogicEdge ().equals (logicEdge)) {
                    return edge;
                }
            }
        }
        return null;
    }
    
    /** Core method for addition.
     * @return event refering to addition
     */
    private VisualAutomatonAddition __addElement (VisualAutomatonElement element) {
        boolean res = elements.add (element);
        if (! res) {
            throw new IllegalStateException ("element is already inserted: " + element);
        }
        element.setContainer (this);
        // !!!!! souvisi s rozdelenim
        if (element instanceof VisualAutomatonEdge) {
            edges.add (element);
        }
        VisualAutomatonAddition event = new VisualAutomatonAddition (this, element);
        fireContentChanged (event);
        fireElementAdded (event);
        if (element instanceof VisualAutomatonContainer) { // ! class name
            fireContainerAdded (event);
        }
        //System.out.println ("element added: " + element + " on: " + element.getPosition ());
        someChange ();
        return event;
    }
    
    /** Undo-firing version of core __addElement()
     */
    private void insertElement (VisualAutomatonElement element) {
        VisualAutomatonAddition event = __addElement (element);
        fireUndoableEdit (event);
    }
    
    /**
     */
    VisualAutomatonAddition addElement (VisualAutomatonNode node) {
        VisualAutomatonAddition res = __addElement (node);
        automaton.addNode (node.getLogicNode ());
        return res;
    }
    
    /**
     */
    VisualAutomatonAddition addElement (VisualAutomatonEdge edge) {
        VisualAutomatonAddition res = __addElement (edge);
        LogicAutomatonEdge logicEdge = edge.getLogicEdge ();
        logicEdge.getSource ().addOutputEdge (logicEdge);
        visualEdges.put (logicEdge, edge);
        return res;
    }
    
    /**
     */
    VisualAutomatonAddition addElement (VisualAutomatonContainer container) {
        VisualAutomatonAddition res = __addElement (container);
        container.addUndoableEditListener (undoListener);
        container.addContainerListener (containerListener);
        return res;
    }
    
    /** Common form of addElement (<type>) for use if specific type is
     * unknown.
     */
    VisualAutomatonAddition addElement (VisualAutomatonElement element) {
        if (element instanceof VisualAutomatonContainer) { // must be before VisualAutomatonNode
            return addElement ((VisualAutomatonContainer) element);
            //return;
        }
        if (element instanceof VisualAutomatonEdge) {
            return addElement ((VisualAutomatonEdge) element);
            //return;
        }
        if (element instanceof VisualAutomatonNode) {
            return addElement ((VisualAutomatonNode) element);
        }
        throw new IllegalArgumentException ("Improper type of element");
    }
    
    /** Core delete
     */
    // !!! mazat z prvku tento container (unSetContainer ())
    private void __deleteElement (VisualAutomatonElement element) {
        boolean res = elements.remove (element);
        if (! res) {
            throw new IllegalArgumentException ("Container: " +  this + " does not contain element: " +
            element);
        }
        // !!!!! souvisi s rozdelenim
        if (element instanceof VisualAutomatonEdge) {
            edges.remove (element);
        }
        VisualAutomatonContainerEvent event = new VisualAutomatonRemoval (this, element, null);
        fireContentChanged (event);
        fireElementRemoved (event);
        if (element instanceof VisualAutomatonContainer) {
            fireContainerRemoved (event);
        }
        someChange ();
        //System.out.println ("element deleted: " + element);
    }
    
    /** Non-undo firing delete.
     * Deletes edge.
     */
    private void deleteElement (VisualAutomatonEdge edge) {
        __deleteElement (edge);
        LogicAutomatonEdge logicEdge = edge.getLogicEdge ();
        visualEdges.remove (logicEdge);
        logicEdge.getSource ().removeOutputEdge (logicEdge); //opacna hrana se vymaze "sama"
    }
    
    /** Non-undo firing delete.
     * Deletes node.
     */
    private void deleteElement (VisualAutomatonNode node) {
        __deleteElement (node);
        LogicAutomatonNode logicNode = node.getLogicNode ();
        // removal of depending edges
        Set logicEdges = logicNode.getOutputEdges ();
        logicEdges.addAll (logicNode.getInputEdges ());
        //Set edges = new HashSet ();
        for (Iterator it = logicEdges.iterator (); it.hasNext (); ) {
            Object o = it.next ();
            VisualAutomatonEdge visualEdge = (VisualAutomatonEdge) visualEdges.get (o);
            //edges.add (visualEdge);
            deleteElement (visualEdge);
        }
        // end of removal...
        automaton.removeNode (logicNode);
    }
    
    /** Non-undo firing delete.
     * Deletes container.
     */
    private void deleteElement (VisualAutomatonContainer nestedContainer) {
        __deleteElement (nestedContainer);
        automaton.removeNode (nestedContainer.getLogicNode ()); // was bug: was missing
        nestedContainer.removeUndoableEditListener (undoListener);
    }
    
    /**
     */
    private void deleteElement (VisualAutomatonElement element) {
        if (element instanceof VisualAutomatonContainer) { // must be before VisualAutomatonNode
            deleteElement ((VisualAutomatonContainer) element);
            return;
        }
        if (element instanceof VisualAutomatonEdge) {
            deleteElement ((VisualAutomatonEdge) element);
            return;
        }
        if (element instanceof VisualAutomatonNode) {
            deleteElement ((VisualAutomatonNode) element);
        }
    }
    
    //!!!!????
    /** Adds list of elements "as are", i.e. without any actions except storing
     * in this container itself.
     */
    private void addElementsAsAre (java.util.List elems) {
        Iterator it = elems.iterator ();
        while (it.hasNext ()) {
            __addElement ((VisualAutomatonElement) it.next ());
        }
    }
    
    /**
     */
    public void removeElement (VisualAutomatonEdge edge) {
        deleteElement (edge);
        VisualAutomatonContainerEdit event = new VisualAutomatonRemoval (this, edge, null); // was BUG
        fireContentChanged (event);
        fireUndoableEdit (event);
    }
    
    /**
     */
    public void removeElement (VisualAutomatonNode node) {
        deleteElement (node);
        LogicAutomatonNode logicNode = node.getLogicNode ();
        Set logicEdges = logicNode.getOutputEdges ();
        logicEdges.addAll (logicNode.getInputEdges ());
        final Set removedEdges = new HashSet ();
        for (Iterator it = logicEdges.iterator (); it.hasNext (); ) {
            VisualAutomatonEdge visualEdge = (VisualAutomatonEdge) visualEdges.get (it.next ());
            removedEdges.add (visualEdge);
            deleteElement (visualEdge);
        }
        VisualAutomatonContainerEdit event = new VisualAutomatonRemoval (this, node, removedEdges);
        fireContentChanged (event);
        //undoSupport.postEdit (event);
        fireUndoableEdit (event);
        //lastId--;
    }
    
    /**
     */
    public void removeElement (VisualAutomatonContainer container) {
        deleteElement (container);
        VisualAutomatonContainerEdit event = new VisualAutomatonRemoval (this, container, null);
        fireContentChanged (event);
        fireUndoableEdit (event);
        
    }
    
    /** Common form of removeElement (<type>) for use if specific type is
     * unknown.
     */
    public void removeElement (VisualAutomatonElement element) {
        if (element instanceof VisualAutomatonContainer) { // must be before VisualAutomatonNode
            removeElement ((VisualAutomatonContainer) element);
            return;
        }
        if (element instanceof VisualAutomatonEdge) {
            removeElement ((VisualAutomatonEdge) element);
            return;
        }
        if (element instanceof VisualAutomatonNode) {
            removeElement ((VisualAutomatonNode) element);
        }
    }
    
    //??? je potreba
    /**
     */
    public VisualAutomatonNode createNode (Point position, LogicAutomatonNode logicNode) {
        VisualAutomatonNode node = new VisualAutomatonNode (position, logicNode);
        //node.setPositionAsSystem (position); // !!! neni zbytecne??
        //node.setId (lastId++); je zbytecne, vyresit...rozdelit interni a prezentacni node ID !!!
        VisualAutomatonAddition event = __addElement (node);
        fireUndoableEdit (event);
        return node;
    }
    
    /** Creates node within container (main method).
     * @return created node
     */
    public VisualAutomatonNode createNode (Point position) {
        LogicAutomatonNode logicNode = automaton.createNode ();
        return createNode (position, logicNode);
    }
    
    /** Creates node within this container on position (0, 0).
     * Used in reading persistent form.
     */
    public VisualAutomatonNode createNode () {
        return createNode (new Point (0, 0), automaton.createNode ());
    }
    
    /** Creates new empty sub-container within this container (main method)
     */
    public VisualAutomatonContainer createContainer (Point position) {
        //System.out.println ("position: " + position);
        LogicAutomaton nestedAutomaton = new LogicAutomaton ();
        CompoundLogicAutomatonNode logicNode = automaton.createCompoundNode (nestedAutomaton);
        VisualAutomatonContainer cont = new VisualAutomatonContainer (this, position, logicNode, nestedAutomaton);
        VisualAutomatonAddition event = addElement (cont);
        fireUndoableEdit (event);
        return cont;
    }
    
    //!!!! zrevidovat smysl pouziti - add-akce
    public VisualAutomatonContainer createContainer (VisualAutomatonContainer container, Point position) {
        VisualAutomatonAddition event = addElement (container);
        container.setPositionAsSystem (position); // was BUG - missed
        automaton.addNode (container.getLogicNode ());
        fireUndoableEdit (event);
        return container;
    }
    
    public VisualAutomatonContainer createContainer () {
        return createContainer (new Point (0, 0));
    }
    
    /**
     */
    public VisualAutomatonEdge createEdge (
    VisualAutomatonNode from,
    VisualAutomatonNode to,
    LogicAutomatonEdge logicEdge) {
        VisualAutomatonEdge edge = new VisualAutomatonEdge (from, to, logicEdge);
        VisualAutomatonAddition event = __addElement (edge);
        visualEdges.put (logicEdge, edge);
        fireUndoableEdit (event);
        return edge;
    }
    
    /**
     * @return null if edge from->to already exists
     */
    public VisualAutomatonEdge createEdge (
    VisualAutomatonNode from,
    VisualAutomatonNode to,
    RexEdgeElement rex) {
        if (from.equals (to)) {
            throw new IllegalArgumentException ("The same 'from' and 'to' node. If desired use createSelfEdge ()");
        }
        LogicAutomatonNode logicFromNode = from.getLogicNode ();
        LogicAutomatonNode logicToNode = to.getLogicNode ();
        LogicAutomatonEdge logicEdge = logicFromNode.createOuptputEdgeTo (logicToNode, rex);
        if (null == logicEdge)
            return null;    // impossible to create the edge
        // !!!!! lepsi vyhazovat vyjimku
        return createEdge (from, to, logicEdge);
    }
    
    /**
     * @return null if specified node already have some self edge
     */
    public VisualAutomatonEdge createSelfEdge (VisualAutomatonNode from, RexEdgeElement rexValue) {
        LogicAutomatonNode logicNode = from.getLogicNode ();
        if (logicNode.hasSelfEdge ()) {
            return null;
        }
        LogicAutomatonEdge logicEdge = logicNode.setSelfEdge (rexValue);
        VisualAutomatonEdge edge = new VisualAutomatonEdge (from, from, null);
        selfEdges.add (edge);
        VisualAutomatonAddition event = __addElement (edge);
        fireUndoableEdit (event);
        fireElementAdded (event);
        return edge;
    }
    
    /**
     * NOTE: It returns first reached element but it's not said anything
     * whether curent impl allows to place two elements on the same position.
     *
     * @return element on relative position from this container
     * @return null if this container does not contain any element
     * on the position
     */
    public VisualAutomatonElement findElementOn (Point position) {
        for (int i=0; i<elements.size (); i++) {
            VisualAutomatonElement elem = (VisualAutomatonElement) elements.get (i);
            if (elem.isOnForSelection (position)) {
                return elem;
            }
        }
        return null;
    }
    
    /**
     * @see findElementOn
     */
    public boolean isElementOn (Point position) {
        return findElementOn (position) != null ? true : false;
    }
    
    /**
     * @return true if this container contains specified element
     */
    public boolean contains (VisualAutomatonElement element) {
        // !!!!!! pri rozdeleni predelat
        return elements.contains (element);
    }
    
    /**
     */
    public synchronized void addUndoableEditListener (UndoableEditListener listener) {
        //!!! kdyz neni root melo by byt povoleno pridani pouze jednoho listeneru
        //!!! a to nadrazeneho containeru
        undoSupport.addUndoableEditListener (listener);
    }
    
    /**
     */
    public synchronized void removeUndoableEditListener (UndoableEditListener listener) {
        undoSupport.removeUndoableEditListener (listener);
    }
    
    /**
     */
    void startCompoundEdit (CompoundEdit edit) {
        if (compoundEditStarted) {
            throw new IllegalStateException ("Compound edit already started");
        }
        compoundEditStarted = true;
        //compoundEdit = new CompoundEdit ();
        compoundEdit = edit;
    }
    
    void endCompoundEdit () {
        if (! compoundEditStarted) {
            throw new IllegalStateException ("Compound edit not started");
        }
        compoundEdit.end ();
        undoManager.addEdit (compoundEdit);
        undoSupport.postEdit (compoundEdit);
        compoundEditStarted = false;
    }
    
    /** To be called from this class and from other VisualAutomatonElement subclasses.
     */
    void fireUndoableEdit (UndoableEdit edit) {
        if (! compoundEditStarted) {
            undoManager.addEdit (edit); // was terrible BUG - must be first
            //undoManager.end ();
            undoSupport.postEdit (edit);
        } else {
            compoundEdit.addEdit (edit);
            //System.out.println ("added edit to compound: " + edit);
        }
    }
    
    /**
     */
    void fireContentChanged (VisualAutomatonContainerEvent event) {
        ArrayList list;
        synchronized (this) {
            if (containerListenerList == null) {
                return;
            }
            list = (ArrayList)containerListenerList.clone ();
        }
        for (int i = 0; i < list.size (); i++) {
            ((VisualAutomatonContainerListener)list.get (i)).contentChanged (event);
        }
        // !!!! zrusit, bude se delat samo
        someChange ();
        //System.out.println ("firing: contentChanged");
    }
    
    /**
     */
    public synchronized void addContainerListener (VisualAutomatonContainerListener listener) {
        if (containerListenerList == null ) {
            containerListenerList = new java.util.ArrayList ();
        }
        containerListenerList.add (listener);
    }
    
    /**
     */
    void fireElementAdded (VisualAutomatonContainerEvent event) {
        ArrayList list;
        synchronized (this) {
            if (containerListenerList == null) {
                return;
            }
            list = (ArrayList)containerListenerList.clone ();
        }
        for (int i = 0; i < list.size (); i++) {
            ((VisualAutomatonContainerListener)list.get (i)).elementAdded (event);
        }
        //System.out.println ("firing: element added");
    }
    
    /**
     */
    void fireContainerAdded (VisualAutomatonContainerEvent event) {
        ArrayList list;
        synchronized (this) {
            if (containerListenerList == null) {
                return;
            }
            list = (ArrayList)containerListenerList.clone ();
        }
        for (int i = 0; i < list.size (); i++) {
            ((VisualAutomatonContainerListener)list.get (i)).containerAdded (event);
        }
        //System.out.println ("firing: container added");
    }
    
    /**
     */
    void fireElementRemoved (VisualAutomatonContainerEvent event) {
        ArrayList list;
        synchronized (this) {
            if (containerListenerList == null) {
                return;
            }
            list = (ArrayList)containerListenerList.clone ();
        }
        for (int i = 0; i < list.size (); i++) {
            ((VisualAutomatonContainerListener)list.get (i)).elementRemoved (event);
        }
        //System.out.println ("firing: element removed");
    }
    
    /**
     */
    void fireContainerRemoved (VisualAutomatonContainerEvent event) {
        ArrayList list;
        synchronized (this) {
            if (containerListenerList == null) {
                return;
            }
            list = (ArrayList)containerListenerList.clone ();
        }
        for (int i = 0; i < list.size (); i++) {
            ((VisualAutomatonContainerListener)list.get (i)).containerRemoved (event);
        }
        //System.out.println ("firing: container removed");
    }
    
    /**
     */
    /*
    protected ElementGeometry prepareGeometry () {
        ElementGeometry res;
        if (folded) {
            res = new ContainerGeometry (this);
        } else {
            // it is not used
            res = new ProxyGeometry ();
        }
        return res;
    }
     */
    
    /**
     */
    public String toString () {
        StringBuffer res = new StringBuffer ();
        res.append ("vcontainer ");
        res.append (contentName);
        res.append (" : {");
        for (int i=0; i<elements.size (); i++) {
            res.append (elements.get (i).toString ());
            res.append (" ");
        }
        res.append ("}");
        return res.toString ();
    }
    
    
    //////////////////////////////////////////////////////////////////////////
    private class UndoListenPoint implements UndoableEditListener {
        
        /**
         * An undoable edit happened
         */
        public void undoableEditHappened (UndoableEditEvent e) {
            //undoSupport.postEdit (e.getEdit ());
            fireUndoableEdit (e.getEdit ());
        }
        
    }
    
    ///////////////////////////////////////////////////////////////////////////
    /** For future use.
     */
    class ProxyGeometry extends ElementGeometry {
        
        /**
         */
        public void paint (Graphics2D g) {
            if (true) throw new UnsupportedOperationException ();
            for (int i=0; i<elements.size (); i++) {
                ((VisualAutomatonElement) elements.get (i)).getGeometry ().paint (g);
            }
        }
        
        public boolean isOn (Point position) {
            throw new UnsupportedOperationException ();
        }
        
        public boolean isOnForSelection (Point position) {
            throw new UnsupportedOperationException ();
        }
        
        /**
         */
        void paintSelected (Graphics2D g) {
            throw new UnsupportedOperationException ();
        }
        
        public Point[] getSnapPoints () {
            throw new UnsupportedOperationException ();
        }
        
    }
    
    ////////////////////////////////////////////////////////////////////////////
    static class ContainerGeometry extends ConnectableGeometry {
        
        protected GeneralPath path;
        
        protected PointPlus[] snapPoints;
        
        protected VisualAutomatonContainer container;
        
        protected Color activeInColor = Color.pink;
        
        private static double BASE = 5;
        
        private static double LONG_SIDE = 90;
        
        private static double SHORT_SIZE = 24;
        
        private static final int HOR_SNAPS_COUNT = 15;
        
        private static final int VER_SNAPS_COUNT = 10;
        
        private static double WIDTH = 2 * BASE + LONG_SIDE;
        
        private static double HEIGHT = 2 * BASE + SHORT_SIZE;
        
        protected static final int SNAP_POINTS_COUNT = 2 * (HOR_SNAPS_COUNT + VER_SNAPS_COUNT);
        // was bug: missing braces
        
        protected static final PointPlus[] relativeSnapPoints = new PointPlus [SNAP_POINTS_COUNT];
        
        // !! snap points, count jako field v prekdovi, obecnou podporu pro pocitani
        // absolute z relative
        
        static {
            prepareRelativeSnapPoints ();
        }
        
        private static void prepareRelativeSnapPoints () {
            double ELEM = LONG_SIDE / HOR_SNAPS_COUNT;
            double x = ELEM;
            // there were a lot of terrible bugs: indexes... :-( :o)
            // !! perf
            for (int i = 0; i < 2 * HOR_SNAPS_COUNT; i++) {
                relativeSnapPoints[i] = new PointPlus (x, 0);
                //System.out.println ("relative: " + relativeSnapPoints[i] + " in " + i);
                relativeSnapPoints[++i] = new PointPlus (x, HEIGHT);;
                //System.out.println ("relative: " + relativeSnapPoints[i] + " in " + i);
                x += ELEM;
            }
            ELEM = SHORT_SIZE / VER_SNAPS_COUNT;
            double y = ELEM;
            for (int i = 2 * HOR_SNAPS_COUNT; i < SNAP_POINTS_COUNT; i++) {
                relativeSnapPoints[i] = new PointPlus (0, y);
                //System.out.println ("relative: " + relativeSnapPoints[i] + " in " + i);
                relativeSnapPoints[++i] = new PointPlus (WIDTH, y);
                //System.out.println ("relative: " + relativeSnapPoints[i] + " in " + i);
                y += ELEM;
            }
        }
        
        /**
         */
        public ContainerGeometry (VisualAutomatonContainer container) {
            this.container = container;
            //
            float x = (float) container.position.getX ();
            float y = (float) container.position.getY ();
            path = new GeneralPath ();
            float a = x + (float) BASE;
            float aa = a + (float) LONG_SIDE;
            float aaa = aa + (float) BASE;
            float b = y + (float) BASE;
            float bb = b + (float) SHORT_SIZE;
            float bbb = bb + (float) BASE;
            // ! doesn't work - don't know why
            //shape = path.getBounds ();
            // complemental solution: but not neccessary
            //shape = new Rectangle2D.Double (x, y, WIDTH, HEIGHT);
            //
            snapPoints = new PointPlus [SNAP_POINTS_COUNT];
            for (int i = 0; i < snapPoints.length; i++) {
                snapPoints[i] = relativeSnapPoints[i].add (container.position);
                //System.out.println("i: " + i + " " + snapPoints[i]);
            }
            path.moveTo (a, y);
            path.lineTo (aa, y);
            path.lineTo (aaa, b);
            path.lineTo (aaa, bb);
            path.lineTo (aa, bbb);
            path.lineTo (a, bbb);
            path.lineTo (x, bb);
            path.lineTo (x, b);
            path.closePath ();
            //
            //activeColor = Color.PINK;
            shape = path;
        }
        
        /**
         */
        public void paint (Graphics2D g) {
            g.draw (path);
            //paintSnapPoints (snapPoints, g);
            Icon icon = container.getContentIcon ();
            int height = icon.getIconHeight ();
            int width = icon.getIconWidth ();
            int posX = (int) container.position.getX ();
            int posY = (int) container.position.getY ();
            int x = posX + 5; // ! const
            int y = posY + (int) (HEIGHT - height) / 2;
            icon.paintIcon (null, g, x, y);
            x = x + width + 4;
            int fontHeight = g.getFontMetrics ().getHeight ();
            g.drawString (container.getHumanName (), x, y + fontHeight - 4);
        }
        
        /**
         */
        void paintActiveIn (Graphics2D g) {
            Color oldColor = g.getColor ();
            g.setColor (activeInColor);
            g.fill (shape);
            g.setColor (oldColor);
            paint (g);
        }
        
        void paintActiveOut (Graphics2D g) {
        }
        
        /**
         */
        public boolean isOn (Point position) {
            return path.contains (position);
        }
        
        public boolean isOnForSelection (Point position) {
            return isOn (position);
        }
        
        public int getConnectPolicy () {
            return ConnectableGeometry.CONNECT_POLICY_SNAPS;
        }
        
        public Point[] getSnapPoints () {
            // !!!??? radsi klonovat
            return snapPoints;
        }
        
    }
    
    ///////////////////////////////////////////////////////////////////
    class VisualAutomatonRemoval extends VisualAutomatonContainerEdit {
        
        protected VisualAutomatonElement element;
        
        protected Set edges;
        
        /**
         * @param dependentEdges null is allowed
         */
        public VisualAutomatonRemoval (
        VisualAutomatonContainer container,
        VisualAutomatonElement element,
        Set dependentEdges) {
            super (container);
            // !!! check null
            this.element = element;
            this.edges = dependentEdges;
        }
        
        /**
         */
        public void redo () throws CannotRedoException {
            super.redo ();
            container.deleteElement (element);
            if (null != edges) {
                for (Iterator it = edges.iterator (); it.hasNext (); ) {
                    VisualAutomatonEdge edge = (VisualAutomatonEdge) it.next ();
                    container.deleteElement (edge);
                }
            }
        }
        
        /**
         */
        public void undo () throws CannotUndoException {
            super.undo ();
            container.addElement (element);
            if (null != edges) {
                for (Iterator it = edges.iterator (); it.hasNext (); ) {
                    VisualAutomatonEdge edge = (VisualAutomatonEdge) it.next ();
                    container.addElement (edge);
                }
            }
        }
        
        public String getPresentationName () {
            return "Removal of " + element.getHumanName (); // was BUG
        }
        
        public String getUndoPresentationName () {
            return "Undo " + getPresentationName ();
        }
        
        public String getRedoPresentationName () {
            return "Redo " + getPresentationName ();
        }
        
        public VisualAutomatonElement getElement () {
            return element;
        }
        
        public VisualAutomatonContainer getPlace () {
            return container;
        }
        
    }
    
    class VisualAutomatonAddition extends VisualAutomatonContainerEdit {
        
        protected VisualAutomatonElement element;
        
        public VisualAutomatonAddition (VisualAutomatonContainer container, VisualAutomatonElement element) {
            // !!! checkovat oboji na null
            super (container);
            this.element = element;
        }
        
        /**
         */
        public void redo () throws CannotRedoException {
            super.redo ();
            container.addElement (element);
        }
        
        /**
         */
        public void undo () throws CannotUndoException {
            super.undo ();
            container.deleteElement (element);
        }
        
        /**
         */
        public String getPresentationName () {
            return "addition of " + element.getHumanName (); // was BUG - missing element
        }
        
        /**
         */
        public String getUndoPresentationName () {
            return "Undo " + getPresentationName ();
        }
        
        /**
         */
        public String getRedoPresentationName () {
            return "Redo " +  getPresentationName ();
        }
        
        public VisualAutomatonElement getElement () {
            return element;
        }
        
        public VisualAutomatonContainer getPlace () {
            return container;
        }
        
    }
    
    /////////////////////////////////////////////////////////////////
    
    /** "Envelope" undo class.
     */
    private class AutomatonChange extends CompoundEdit /* VisualAutomatonContainerEdit */
    implements VisualAutomatonContainerEvent {
        
        public String getPresentationName () {
            return "Change of automaton";
        }
        
        public String getUndoPresentationName () {
            return "Undo " + getPresentationName ();
        }
        
        public String getRedoPresentationName () {
            return "Redo " + getPresentationName ();
        }
        
    }
    
    /////////////////////////////////////////////////////////////////
    /** Class for propagation of container events to enclosing container
     */
    private class ContainerListenPoint implements VisualAutomatonContainerListener {
        
        public void elementRemoved (VisualAutomatonContainerEvent event) {
            fireElementRemoved (event);
        }
        
        public void containerAdded (VisualAutomatonContainerEvent event) {
            fireContainerAdded (event);
        }
        
        public void containerRemoved (VisualAutomatonContainerEvent event) {
            fireContainerRemoved (event);
        }
        
        public void elementAdded (VisualAutomatonContainerEvent event) {
            fireElementAdded (event);
        }
        
        public void contentChanged (VisualAutomatonContainerEvent event) {
        }
        
    }
    
}

