/*
                    Sun Public License Notice

The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/

The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.

Contributor(s): Petr Slegr.
*/

package org.netbeans.modules.regsup.debug;

import org.netbeans.modules.regsup.logic.*;
import org.netbeans.modules.regsup.auted.*;
import java.util.*;

/**
 *
 * @author  Petr Slegr
 * @version 28.12.01
 */
public class RexDebuggerState {
    
    /**
     * Ordinal number of char from input. It is set only for states reached AFTER
     * reading of char.
     */
    private int number;
    
    /**
     * Has this state been reached by reading of char?
     */
    private boolean read;
    
    /** Flag marking that this state was first after user action
     */
    private boolean user;

    /** Debugger was "breaked" (some breakpoint applied) in this state.
     */
    private boolean breaked;
    
    /** All nodes except FINAL nodes and "to be steped-in" containers (i.e.
     *  includes "steped-out" containers).
     */
    private Set pureNodes;
    
    /** Set of "steped-out" VisualAutomatonContainers
     */
    //private Set closed;
    
    /** Set of VisualAutomatonEdges.
     */
    private Set edges;
    
    /** Set of "to be steped-in" containers
     */
    private Set containers;
    
    /**
     */
    private Set closedContainers = new HashSet ();
    
    /** Set of VisualAutomatonNodes which ARE NOT VisualAutomatonContainer and ARE final
     */
    private Set finalNodes;
    
    /** Set of final nodes from top level container
     */
    private Set topFinalNodes;
    
 
    /**
     */
    RexDebuggerState () {
        //number = num;
        pureNodes = new HashSet ();
        finalNodes = new HashSet ();
        topFinalNodes = new HashSet ();
        containers = new HashSet ();
        edges = new HashSet ();
    }
    
    /**
     */
    public boolean isRead () {
        return read;
    }
    
    /** Sets number of character from input that has been read and sets the read
     * flag.
     * @side
     */
    void setNumber (int number) {
        this.number = number;
        System.out.println("number seted: " + number);
        read = true;
    }
    
    /**
     */
    public int getNumber () {
        return number;
    }
    
    /** For adding of new (unknown) node
     */
    void addNode (VisualAutomatonNode node) {
        // !!! check duplicity
        if (node instanceof VisualAutomatonContainer) {
            containers.add (node);
            //System.out.println("state: " + this + " added into containers: " + node);
        } else {
            if (node.isFinalNode ()) {
                if (node.getSuperContainer () != null) {
                    // node is final && is not within top-level container
                    finalNodes.add (node);
                    // !!!!!!!!!!!!!!!!!!! BUG BUG BUG
                    // !!!!!!!!!!!!!!!!!!! nema se taky pridavat do pureNodes?
                    // !!!!!!!!!!!!!!!!!!! BUG BUG BUG
                    // na final a topFinal nodes nemit asi vlastni mnoziny a testovat, usetri pamet
                    //System.out.println("state: " + this + " added into finalNodes: " + node);
                } else {
                    topFinalNodes.add (node);
                    pureNodes.add (node); // was bug: final node need not be last...:-)
                    //System.out.println("state: " + this + " added into pureNodes/topFinalNodes: " + node);
                }
            } else {
                pureNodes.add (node);
                //System.out.println("state: " + this + " added into pureNodes: " + node);
            }
        }
    }
    
    /**
     */
    /*
    void addClosedContainer (VisualAutomatonContainer container) {
        closed.add (container);
    }
     */
    
    /** For adding "steped-out" containers and non-final nodes
     */
    void addPureNode (VisualAutomatonNode node) {
        pureNodes.add (node);
    }
    
    void addClosedContainer (VisualAutomatonContainer container) {
        closedContainers.add (container);
    }
    
    /** For copying of set of pure nodes from previous state
     */
    void addPureNodes (Set set) {
        pureNodes.addAll (set);
    }
    
    void addEdge (VisualAutomatonEdge edge) {
        // !!! check duplicity
        boolean res = edges.add (edge);
    }
    
    void setUser (boolean user) {
        this.user = user;
    }
    
    public boolean isUser () {
        return user;
    }
    
    /**
     */
    // !!! used in AutomatonInnerPanel
    public Set getPureNodes () {
        return new HashSet (pureNodes);
    }
    
    /**
     */
    Set getEdges () {
        return new HashSet (edges);
    }
    
    /**
     */
    Set getContainers () {
        return new HashSet (containers);
    }
    
    Set getClosedContainers () {
        return new HashSet (closedContainers);
    }
    
    /**
     */
    Set getFinalNodes () {
        return new HashSet (finalNodes);
    }
    
    /** ALL
     */
    public Set getNodes () {
        Set res = new HashSet (pureNodes);
        res.addAll (containers);
        res.addAll (finalNodes);
        return res;
    }
    
    /** Contains this state "un-steped in" containers?
     */
    boolean containsContainers () {
        return containers.size () > 0;
    }
    
    public boolean containsClosedContainers () {
        return closedContainers.size () > 0;
    }
    
    /**
     */
    boolean containsFinalNodes () {
        return finalNodes.size () > 0;
    }
    
    public boolean containsTopFinalNode () {
        return topFinalNodes.size () > 0;
    }
    
    boolean containsPureNode (VisualAutomatonNode node) {
        return pureNodes.contains (node);
    }
    
    // !!?? used in StepIntoAction.isEnabled
    public boolean containsContainer (VisualAutomatonContainer container) {
        return containers.contains (container);
    }
    
    // !!?? used in StepAction.isEnabled
    public boolean containsPureNodesOnly () {
        return ! containsFinalNodes () && ! containsContainers ();
    }
    
    boolean containsClosedContainer (VisualAutomatonContainer container) {
        return containsPureNode (container);
    }
    
    public String toString () {
        return getNodes ().toString ();
    }
    
}