/*
                    Sun Public License Notice

The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/

The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.

Contributor(s): Petr Slegr.
*/

package org.netbeans.modules.regsup.ide;

import org.xml.sax.*;
import java.util.ArrayList;
import org.netbeans.modules.regsup.auted.*;
import java.awt.Point;
import java.util.Hashtable;
import org.netbeans.modules.regsup.logic.*;

/** 
 * @author Petr Slegr
 * @version 28.04.02
 */
public class AutomatonHandlerImpl implements AutomatonHandler {
    
    public static final boolean DEBUG = false;
    
    private ArrayList containerStack = new ArrayList ();
    
    private VisualAutomatonContainer root;
    
    /** Holds the currently created element
     */
    private VisualAutomatonNode current;
    
    /** Holds the current coords
     */
    private int positionX, positionY;
    
    /** Mapping "nodeId string" -> VisualAutomatonNode
     */
    private Hashtable nodes = new Hashtable ();
    
    /** Flag that "node" tag is currently processed
     */
    private boolean nodeProc;
    
    private boolean sourceProc;
    
    private boolean targetProc;
    
    private VisualAutomatonNode source;
    
    private VisualAutomatonNode target;
    
    private RexEdgeElement edgeValue;
    
    public static final String STRING_TRUE = "true";
    
    public static final String STRING_FALSE = "false";
    
    
    /** Returns root VisualAutomatonContainer created by parsing persistence
     * form
     */
    public VisualAutomatonContainer getResult () {
        return root;
    }
    
    ////////////////////////////////////////////////////////////////////////////
    // generated methods
    public void start_source (final AttributeList meta) throws SAXException {
        if (DEBUG) System.err.println ("start_source: " + meta);
        sourceProc = true;
    }
    
    public void end_source () throws SAXException {
        if (DEBUG) System.err.println ("end_source()");
        sourceProc = false;
    }
    
    public void start_container (final AttributeList meta) throws SAXException {
        if (DEBUG) System.err.println ("start_container: " + meta);
        VisualAutomatonContainer cont;
        if (containerStack.size () == 0) {
            root = cont = new VisualAutomatonContainer ();
        } else {
            VisualAutomatonContainer container = (VisualAutomatonContainer)
            containerStack.get (containerStack.size () - 1);
            cont = container.createContainer ();
        }
        containerStack.add (cont);
        current = cont;
        nodeProc = true;
    }
    
    public void end_container () throws SAXException {
        if (DEBUG) System.err.println ("end_container()");
        containerStack.remove (containerStack.size () - 1);
        nodeProc = false;
    }
    
    public void start_edge (final AttributeList meta) throws SAXException {
        if (DEBUG) System.err.println ("start_edge: " + meta);
    }
    
    public void end_edge () throws SAXException {
        if (DEBUG) System.err.println ("end_edge()");
        VisualAutomatonContainer cont = (VisualAutomatonContainer) containerStack.
        get (containerStack.size () - 1);
        VisualAutomatonEdge edge = cont.createEdge (source, target, edgeValue);
    }
    
    public void handle_nodeId (final java.lang.String data, final AttributeList meta) throws SAXException {
        if (DEBUG) System.err.println ("handle_nodeId: " + data);
        String str = data.trim ();
        if (nodeProc) {
            Object res = nodes.put (str, current);
            if (res != null) { // was bug: was ==
                throw new IllegalStateException ("nodeId already used: " + str);
            }
            try {
                int nodeId = Integer.parseInt (data.trim ());
                current.setID (nodeId);
            }
            catch (NumberFormatException e) {
                throw new SAXException (e.toString ());
            }
        }
        else if (sourceProc || targetProc) {
            Object o = nodes.get (str);
            if (o == null) {
                throw new IllegalStateException ("nodeId not defined");
            }
            VisualAutomatonNode node = (VisualAutomatonNode) o;
            if (sourceProc) {
                source = node;
            }
            if (targetProc) {
                target = node;
            }
        } else {
            throw new IllegalStateException ("wrong structure");
        }
    }
    
    public void start_node (final AttributeList meta) throws SAXException {
        if (DEBUG) System.err.println ("start_node: " + meta);
        VisualAutomatonNode node = ((VisualAutomatonContainer) containerStack.
        get (containerStack.size () - 1)).createNode ();
        current = node;
        nodeProc = true;
    }
    
    public void end_node () throws SAXException {
        if (DEBUG) System.err.println ("end_node()");
        nodeProc = false;
    }
    
    public void start_position (final AttributeList meta) throws SAXException {
        if (DEBUG) System.err.println ("start_position: " + meta);
    }
    
    public void end_position () throws SAXException {
        if (DEBUG) System.err.println ("end_position()");
        Point position = new Point (positionX, positionY);
        if (! current.equals (root)) {
            // has not sense to set position for top-level container
            current.setPosition (position);
        }
    }
    
    public void handle_longHumanName (final java.lang.String data, final AttributeList meta) throws SAXException {
        if (DEBUG) System.err.println ("handle_longHumanName: " + data);
    }
    
    public void start_target (final AttributeList meta) throws SAXException {
        if (DEBUG) System.err.println ("start_target: " + meta);
        targetProc = true;
    }
    
    public void end_target () throws SAXException {
        if (DEBUG) System.err.println ("end_target()");
        targetProc = false;
    }
    
    public void start_content (final AttributeList meta) throws SAXException {
        if (DEBUG) System.err.println ("start_content: " + meta);
    }
    
    public void end_content () throws SAXException {
        if (DEBUG) System.err.println ("end_content()");
    }
    
    public void handle_humanName (final java.lang.String data, final AttributeList meta) throws SAXException {
        if (DEBUG) System.err.println ("handle_humanName: " + data);
        current.setHumanName (data.trim ());
    }
    
    public void handle_y (final java.lang.String data, final AttributeList meta) throws SAXException {
        if (DEBUG) System.err.println ("handle_y: " + data);
        try {
            positionY = Integer.parseInt (data.trim ());
        } catch (NumberFormatException e) {
            throw new SAXException ("Wrong integer as positionX");
        }
    }
    
    public void handle_x (final java.lang.String data, final AttributeList meta) throws SAXException {
        if (DEBUG) System.err.println ("handle_x: " + data);
        try {
            positionX = Integer.parseInt (data.trim ());
        } catch (NumberFormatException e) {
            throw new SAXException ("Wrong integer as positionX");
        }
    }
    
    public void handle_value (final java.lang.String data, final AttributeList meta) throws SAXException {
        if (DEBUG) System.err.println ("handle_value: " + data);
        try {
            // ! was BUG: edge value assigned to LOCAL!!! variable :o(
            edgeValue = RegexpEdgeParser.createRexEdgeElement (data.trim ());
            // ! was bug: missing trim....:o)
        }
        catch (RexException e) {
            e.printStackTrace();
            throw new SAXException (e.toString ());
        }
    }
    
    public void start_automaton (final AttributeList meta) throws SAXException {
        if (DEBUG) System.err.println ("start_automaton: " + meta);
    }
    
    public void end_automaton () throws SAXException {
        if (DEBUG) System.err.println ("end_automaton()");
    }
    
    public void handle_templateName (final java.lang.String data, final AttributeList meta) throws SAXException {
        if (DEBUG) System.err.println ("handle_templateName: " + data);
    }
    
    public void handle_starting (final java.lang.String data, final AttributeList meta) throws SAXException {
        if (DEBUG) System.err.println ("handle_starting: " + data);
        if (data.trim ().equalsIgnoreCase (STRING_TRUE)) {
            current.setStarting (true);
        }
        else if (! data.trim ().equalsIgnoreCase (STRING_FALSE)) {
            throw new SAXException ("wrong string as starting");
        }
    }
    
    public void handle_final (final java.lang.String data, final AttributeList meta) throws SAXException {
        if (DEBUG) System.err.println ("handle_final: " + data);
        if (DEBUG) System.err.println ("handle_starting: " + data);
        if (data.trim ().equalsIgnoreCase (STRING_TRUE)) {
            current.setFinalNode (true);
        }
        else if (! data.trim ().equalsIgnoreCase (STRING_FALSE)) {
            throw new SAXException ("wrong string as final");
        }
    }
    
}

