/*
                    Sun Public License Notice

The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/

The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.

Contributor(s): Petr Slegr.
*/

package org.netbeans.modules.regsup.logic;

import org.apache.regexp.*;
import java.io.*;
import java.util.ArrayList;
import org.netbeans.modules.regsup.logic.ComplexRexEdgeElement;

/**
 *
 * @author  Petr Slegr
 * @version 01.05.02
 */
public class RegexpEdgeParser {
    
    private String str;
    
    /** Creates a new instance of RegexpEdgeParser */
    private RegexpEdgeParser (String str) {
        this.str = str;
    }
    
    private RexEdgeElement parse () throws RESyntaxException {
        REDebugCompiler comp = new REDebugCompiler ();
        comp.compile (str);
        // hack for a single char element like "a"
        if (str.length () == 1) {
            return new ComplexRexEdgeElement (str.charAt (0));
        }
        // end of hack
        CharArrayWriter w = new CharArrayWriter ();
        comp.dumpProgram (new PrintWriter (w));
        char[] dump = w.toCharArray ();
        //
        int i = 0;
        while (dump[i] != '[') i++; // ! was bug: usable only for dumps containing "["
        i++; // move behind "["
        /*
        System.out.println("dump.length: " + dump.length);
        while (true) {
            if (dump[i] != '[') {
                System.out.println("i: " + i + " dump[i]: " + dump[i]);
                i++;
            } else {
                System.out.println("i: " + i + " dump[i]: " + dump[i]);
                break;
            }
        }
         */
        StringBuffer stb = new StringBuffer ();
        ParsedEdgeElement res = new ParsedEdgeElement (str);
        ArrayList chars = new ArrayList ();
        boolean interval = false;
        char second = 0;
        //System.out.println("dump start");
        //System.out.println(dump);
        //System.out.println("dump end");
        while (dump[i] != ']') {
            if (dump[i] == '\\') {
                i++;
                while (dump[i] != '\\' && dump[i] != '-' && dump[i] !=']') {
                    stb.append (dump[i]);
                    i++;
                }
                String strch = stb.toString ();
                int intch = Integer.parseInt (strch);
                if (intch > Character.MAX_VALUE || intch < Character.MIN_VALUE) {
                    throw new IllegalStateException ("too large char: " + intch);
                }
                if (interval) {
                    Character ch = (Character) chars.remove (chars.size () - 1);
                    res.addInterval (ch.charValue (), (char) intch);
                    interval = false;
                } else {
                    chars.add (new Character ((char) intch));
                    //System.out.println("char added: " + new Character ((char) intch));
                }
            }
            else if (dump[i] == '-') {
                interval = true;
                i++;
            } else {
                if (interval) {
                    Character ch = (Character) chars.remove (chars.size () - 1);
                    res.addInterval (ch.charValue (), dump[i]);
                    interval = false;
                } else {
                    chars.add (new Character (dump[i]));
                    //System.out.println("char added: " + new Character (dump[i]));
                }
                i++;
            }
        }
        for (int j = 0; j < chars.size (); j++) {
            res.addSingle (((Character) chars.get (j)).charValue ());
        }
        return res;
    }
    
    // !!! hazet Rex nebo Regexp exception?
    public static RexEdgeElement createRexEdgeElement (String str) throws RexException {
        try {
            RegexpEdgeParser parser = new RegexpEdgeParser (str);
            return parser.parse ();
        }
        catch (RESyntaxException e) {
            throw new RexException (e.toString ());
        }
    }
    
    private static void test (String str) {
        try {
            System.out.println ("testing input: " + str);
            RexEdgeElement res = createRexEdgeElement (str);
            ComplexRexEdgeElement cres = (ComplexRexEdgeElement) res;
            System.out.println("singles: " + cres.getSingles ());
            System.out.println("intervals: " + cres.getIntervals ());
            System.out.println ("output: " + cres);
            System.out.println("testing end..........................");
        }
        catch (Exception e) {
            e.printStackTrace ();
        }
    }
    
    public static void main (String[] args) {
        test ("a");
        test ("[a]");
        test ("[a-b]");
        test ("[a-c]");
        test ("[^a]");
        test ("[^a-b]");
        test ("[^a-c]");
        test ("[ka-ce-h]"); // bug in regexp ???
        test ("[^ka-ce-h]"); // bug in regexp ???
        test ("[^bd]"); // bug in regexp ???
        test ("[a-h^b-e]");
        test ("[a-h^d]");
        //
        test ("[^be]");
        test ("[ka-be-h]");
        test ("[^ka-be-h]");
    }
}
