/*
                    Sun Public License Notice

The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/

The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.

Contributor(s): Petr Slegr.
*/

package org.netbeans.modules.regsup.logic.aut2expr;

import org.netbeans.modules.regsup.logic.*;
import java.util.Hashtable;
import java.util.Set;
import java.util.HashSet;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collection;

/**
 *
 * @author Petr Slegr
 * @version 29.7.01
 */
public class RegEquationSet {

    private Hashtable eqs;
    
    private int maxOrder;
    
    public RegEquationSet () {
        eqs = new Hashtable ();
    }
    
    public void addEquation (RegEquation eq) {
        Object res = eqs.put (eq.getState (), eq);
        if (null != res)
            throw new IllegalStateException ("Equation already added: " + eq);
        int order = eq.getOrder ();
        if (order > maxOrder)
            maxOrder = order;
    }
    
    private boolean check () {
        if (eqs.size () == 0) {
            //System.out.println("check: empty set");
            return false;
        }
        if (eqs.size () < maxOrder) {
            //System.out.println("check: incomplete set");
            return false;
        }
        return true;
    }
    
    public RexElement solve () {
        if (! check ())
            return null;
        Set set = new HashSet (eqs.size ());
        // !!!! neklonuje se
        ArrayList list = new ArrayList (eqs.values ());
        RegEquation equ;
        int size = list.size ();
        int i = 0;
        equ = (RegEquation) list.get (i);
        equ.eliminate ();
        set.add (equ);
        while (i < size - 1) {
            equ = (RegEquation) list.get (i + 1);
            Iterator it = set.iterator ();
            while (it.hasNext ()) {
                RegEquation e = (RegEquation) it.next ();
                equ.substitute (e);
            }
            //
            equ.eliminate ();
            //
            it = set.iterator ();
            while (it.hasNext ()) {
                RegEquation e = (RegEquation) it.next ();
                e.substitute (equ);
            }
            //
            set.add (equ);
            i++;
        }
        Iterator it = set.iterator ();
        RexAlterElement res = new RexAlterElement ();
        while (it.hasNext ()) {
            RegEquation eqa = (RegEquation) it.next ();
            if (eqa.isFin ()) {
                res.add (eqa.getSolutionPart ());
            }
        }
        return res;
    }
    
    public String toString () {
        StringBuffer buf = new StringBuffer ();
        Collection c = eqs.values ();
        Iterator it = c.iterator ();
        while (it.hasNext ()) {
            buf.append (it.next ().toString ());
        }
        return buf.toString ();
    }

    public static void main (String[] args) {
        /*
        RegEquationSet es = new RegEquationSet ();
        RegEquation eq1 = new RegEquation (new NodeID (0), true, false);
        RegEquation eq2 = new RegEquation (new NodeID (1), false, false);
        RegEquation eq3 = new RegEquation (new NodeID (2), false, true);
        eq1.addEmptyMember ();
        //eq1.addMember (new NodeID (0), new RexCharElement ('b'));
        eq2.addMember (0, 'a');
        eq2.addMember (2, 'a');
        eq3.addMember (0, 'c');
        eq3.addMember (1, 'b');
        es.addEquation (eq1);
        es.addEquation (eq2);
        es.addEquation (eq3);
        System.out.println("set: " + es.eqs);
         */ 
        /* OLDER
        Set s = es.solve ();
        System.out.println("solution: " + s);
        Iterator it = s.iterator ();
        while (it.hasNext ()) {
            RegEquation equ = (RegEquation) it.next ();
            System.out.println("equ.empty: " + equ.members.get (RegEquation.EmptyNodeID.getDefault ()));
        }
         */
    }
}
